!******************************************************************************
!
! File : atom_data.f90
!   by : Alan Tackett
!   on : 7/27/98
!  for : PAW method
!
! This file contains the data structures that are required for loading and 
! storing the fixed and variable portions of the atom data.  
!
!  Modified for spin dependence by Ping Tang
!    Last change 5/23/05
!  
!******************************************************************************


Module atom_data

Use strings
Use anderson_mixing
Use gaussfunc_data
Use mathlib

Type atom_info_fixed       !** Define the Fixed atom information 
  character*10     :: Atom_Name         !** Name of Atom, ie, Ca, C, etc
  character*10     :: Shape_Type        !** Type of shape func used for proj.
  type(GridInfo)   :: RadGrid           !** Used for log radial grid
  Integer          :: Nofthistype       !** Number of atoms of this type 
  Integer          :: Basis_Size        !** Number of Basis functions for atom
  Integer          :: Mesh_Size         !** Num mesh points for each basis fn
  Integer          :: Coretail_points   !** Num mesh points for CoreTail_Density
  Integer          :: DenVhat_Size      !** Size of the Density and V_hat mat's
  Integer          :: Hartree_Size      !** Number of V_Hartree terms
  Integer          :: Atomic_Charge     !** Atomic Z value
  Integer          :: Overlap_Size      !** Overlap, Kinetic, V_ion size
  Integer          :: LCAO_Size         !** Size of TPhi_LCAO arrays
  Integer          :: nlm_Size          !** Size of nlm_LUT
  Integer          :: Hat_MaxL          !** MAx L-value for the n^
  Integer, Pointer :: L_Value(:)        !** L value for each orbital
  Integer, Pointer :: nlm_LUT(:,:)      !** nilimi Look up table
  Integer, Pointer :: nl_Base(:)        !** nlm_LUT index for start of (NiLi)
  Real             :: Mass              !** Mass of atom
  Real             :: LCAO_Step         !** LCAO step size
!  Real             :: V_local           !** Local atom potential coefficient
  Real             :: Core_charge       !** Core charge of atom
  Real             :: Qeffion           !** -Z+Qcore-Qcore~
  Real             :: Rc                !** Basis fn cutoff radius.
  Real             :: Rc_RS_Scale       !** Scale factor for RS Proj's
  Real             :: Gcut_Proj         !** Projector Gcut 
  Integer          :: Gpnt_Size
  Real             :: Rbox              !** Local grid enhancement radius
  Real             :: Mesh_Step         !** Mesh step size (Bohr)
  Real             :: Atom_Energy       !** Total Energy of Atom
  Real             :: CoreTail_SelfEnergy!** CoreTail Self-energy
  Real             :: CoreTail_HatEnergy!** CoreTail-Hat interaction energy
  Real, pointer    :: Hat_SelfEnergy(:) !** Hat self-energy 
  Real,    Pointer :: RadR_Vlocal(:)    !** Rad Vlocal fn (rad mesh)
  Real,    Pointer :: Rad_Vlocal(:)     !** Rad FFT of Vlocal fn
  Real,    Pointer :: Shape_Func(:)     !** Shape for Vlocal and Hat (rad mesh)
  Real,    Pointer :: Tphi_LCAO(:,:)    !** |phi~> for use in LCAO construction
  Real,    Pointer :: RadG_LCAO(:,:)    !** |phi~> Radial G form LCAO 
  Real,    Pointer :: Phi(:,:)          !** |phi>
  Real,    Pointer :: TPhi(:,:)         !** |phi~>
  Real,    Pointer :: TP(:,:)           !** |p~>  <-- twiddle Projector
 !Real,    Pointer :: TP_deriv(:,:)     !** Radial derivative of projector
  Real,    Pointer :: List_Oij(:)       !** Overlap matrix (Loading use only)
  Real,    Pointer :: Oij(:,:)          !** Overlap or Oij matrix (27)
  Real,    Pointer :: Core_Density(:)   !** Core density
  Real,    Pointer :: CoreTail_Density(:)!** Core tail density (coretail_points)
  Real,    Pointer :: FTCoreTail(:)     !** FT of radial Core density
  Real,    Pointer :: GradCoreTail(:)   !** Gradient of Core Tail density
  Real,    Pointer :: Core_Xchange(:)   !** Exchange Correlation Functional
  Real,    Pointer :: Core_dDendr(:)    !** Grad core density needed for GGA
  Real,    Pointer :: Kinetic(:)        !** Kinetic energy Matrix
  Real,    Pointer :: V_ion(:)          !** ??(eq. 38, WN)??
  Real,    Pointer :: Density(:)        !** Density
  Real,    Pointer :: V_hat(:)          !** E-S pot. corr. to Gauss. Form
  Integer, Pointer :: LUT_DenVhat(:)    !** LUT for density and V_hat
  Real,    Pointer :: V_Hartree(:)      !** Hartree Potential
  !Real,    Pointer :: TV_Hartree(:)     !** Hartree energy for scself estimate
  Integer, Pointer :: LUT_V_Hartree(:)  !** LUT for V_Hartree
  Integer          :: QVlm_Size         !** Size of aQlm and aVlm
  Real,    Pointer :: aQlm(:)           !** G^(LM)_limiljmj,lkmkllml * 
                                        !**     V^(aL)_njljnili term in WN46
  Real,    Pointer :: aVlm(:)           !** Contains the coeff - A47
  Integer, Pointer :: LUT_Orb(:)        !** Encoded indices for aQlm and aVlm
  !Complex, Pointer :: Hatij(:,:,:)      !** G-space terms for self-energy
  Real,    Pointer :: Cijkl(:)          !** Coeff of W^a_kl in WN48(coulomb)
  !Real,    Pointer :: TCijkl(:)         !** Coulomb term for scself-energy
  !Complex, Pointer :: Wijkl(:)          !** Occupation matrix for self-energy
  !Real,    Pointer :: EXFij(:,:)        !** Extra term for self_energy
  Integer, Pointer :: LUT_Cijkl(:)      !** Encoded indices for Cij
  Integer          :: Cijkl_Size        !** Size of Cijkl
  Integer, Pointer :: Rad_Size          !** Vxc size 
  Integer, Pointer :: Rad_Skip          !** Vxc skip factor
  Real,    Pointer :: R2(:)             !** R^2 for Vxc
  Real,    Pointer :: Phi_ij_R2(:,:)    !** Phi_i*Phi_J/R^2 for Vxc
  Real,    Pointer :: TPhi_ij_R2(:,:)   !** Phi~_i*Phi~_J/R^2 for Vxc
  Real,    Pointer :: dPhi_ijdr(:,:)    !** Deriv of Phi_i*Phi_j/R^2 for Vxc
  Real,    Pointer :: dTPhi_ijdr(:,:)   !** Deriv of TPhi_i*.Phi_j/R^2 for Vxc
  Real,    Pointer :: Proj_Scale(:)     !** P~ scale 
  Real,    Pointer :: RadHat(:,:)       !** Radial n^ fn's (gmag, n)
  Integer, Pointer :: Proj_Fn(:)        !** P~ Kummer function
  Complex, Pointer :: Ylm(:,:,:)        !** Ylm's for Vxc
  Complex, Pointer :: dYlmdtheta(:,:,:) !** Deriv of Ylm's for GGA Vxc
  Complex, Pointer :: dYlmdphi(:,:,:)   !** Deriv of Ylm's for GGA Vxc
  Real,    Pointer :: FnRad(:,:), FnRadspin(:,:)   
                                        !** Work storage for Vxc
  Real,    Pointer :: Cij(:,:), Cijspin(:,:)          
                                        !** Work Storage for Vxc
  Real,    Pointer :: dCijdtheta(:,:),dCijdthetaspin(:,:)    
                                        !** Work Storage for Vxc
  Real,    Pointer :: dCijdphi(:,:),dCijdphispin(:,:)      
                                        !** Work Storage for Vxc

  Integer           :: Orbitals_Size   !** Size of Valence orbitals list
  Integer, POINTER  :: Valence_Orbitals(:) !** Valence Orbitals
  Real,    Pointer  :: Init_Occ(:), Init_Occspinup(:), Init_Occspindn(:)     !** Initial Occupancy

  !*** These are only used for the norm conserving PP ***
  Character*10      :: icorr
  Character*10      :: irel
  Character*10      :: nicore
  Real              :: Alpha
  Real,  Pointer    :: Pot_local(:)

  !*** These are only used for semi-empirical PP **
  Integer           :: Poly_Degree     !** Degree of G~0 polynomial for V
  Real, Pointer     :: Poly(:)         !** Poly for G~0
  Real              :: Poly_Gcut       !** Switch for poly use
  Real              :: Gaussian(2)     !** Gaussian Fudge potential
  Type (GaussFunc)  :: V_sepm          !** SEPM V in sum of gaussians
End Type

Type Verlet_Data   !*** Contains the Verlet intergration data
    Real    :: Pos(3,3)           !** Positions
    Real    :: Velocity(3,3)      !** Velocities
    Real    :: Acceleration(3,3)  !** Acceleration
    Real    :: CurrForce(3)       !** Current Force calculated by CalcForce
    Integer :: NumMoves           !** Number of valid moves
    Logical :: Predictor_Phase    !** Phase of the Verlet algorithm
End Type

Type Enhance_Type  !** Defines grid enhancement around an atom **
    Integer :: Level   !** Min grid point level of enhancement **
    Real    :: Radius  !** Enhancement radius 
End Type

Type Specific_Atom      !** Contains the specifc atom information
  Character*50      :: Name            !** Atom Name
  Integer           :: TypeIndex       !** AtomType Index
  Integer           :: Enhance_Size    !** Number of enhance data structs
  Integer           :: Orbitals_Size   !** Size of Valence orbitals list
  Integer, POINTER  :: Valence_Orbitals(:) !** Valence Orbitals
  Real              :: Frac_Pos(3)     !** Position in fractional coordinates
  Real              :: Pos(3)          !** Position in Cartesian coordinates
  Real,    Pointer  :: Init_Occ(:), Init_Occspinup(:), Init_Occspindn(:)     !** Initial Occupancy
  Real              :: Nuclear_Params(7)  !** Nulcear charge paramters
  Type(Verlet_Data) :: Force           !**Force information for moving the atom
  !Complex, Pointer  :: TWij(:,:)       !**Temporary Wij Coeff for self-energy
  Complex, Pointer  :: Wij(:,:),Wijspin(:,:)          
                                       !** Projected occupation coeff. WN45
  Complex, Pointer  :: Fij(:,:,:),Fijspin(:,:,:)      !** Force Wij coeff
  Complex, Pointer  :: Feij(:,:,:),Feijspin(:,:,:)    !** Force Wij Pulay coeff
  Complex, Pointer  :: Dij(:,:),Dijspin(:,:)          !** Dij matrix (A8)
  Complex, Pointer  :: cDij(:,:),cDijspin(:,:) !** Dij matrix (A8) -- current
               ! not subject to Anderson mix
  Complex, Pointer  :: Dij_old(:),Dij_oldspin(:) 
                                      !** Previous iterations Dij-mapped to 1d
  Complex, Pointer  :: Wij_old(:,:), Wij_oldspin(:,:) !** Previous iterations Wij matrix (A8)
  Complex, Pointer  :: Qlm(:,:)        !** Comp. charge Multipole moments. WN46
  Complex, Pointer  :: dEdQlm(:,:)     !** dE/dQlm appears in WN49.
  Logical           :: Freeze          !** Determines if an atom is moved
  Real              :: Friction        !** Friction coef
  type (Enhance_Type), Pointer :: Enhance(:) !** Ptr to enhancement structs
  Type (Anderson_Context), Pointer :: Dij_AC, Dijspin_AC
  Type (Anderson_Context), Pointer :: Wij_AC, Wijspin_AC

End Type

Type Shell_Map                     !** Map atomic positions to parameters
  Integer :: MapParams             !** Total parameters of this atom (<=3)
  Real              :: Origin(3)   !** Constant position vector for origin
  Integer, Pointer     :: Map(:)   !** Map to parameter indices
  Real, Pointer     :: V(:,:)      !** MapParams unit vectors to define pos
  Real, Pointer     :: IV(:,:)     !** Inverse unit vectors
End Type

Type Shell_Structure               !** Atom position parametrization
  Integer :: NPARAMS               !** Total number of shell parameters
  Real, Pointer :: C(:)            !** List of parameter values
  Real, Pointer :: ShellForce(:)   !** Generalized forces
  Type (Shell_Map), Pointer :: AtomMap(:) !Parameters map to atoms
End Type

Integer :: Atom_Types              !** Number of different Atom types
Integer :: Max_Atom_Types          !** Max number of different types allowed
Integer :: Specific_Atoms          !** Number of specific Atoms
Integer :: Max_Specific_Atoms      !** Max number of specific atoms allowed
Logical :: UseShells               !** Flag for using shell map

Type (Atom_Info_Fixed), Pointer ,save :: AtomType_Info(:)  !** Array of atom info
Type (Specific_Atom),   Pointer ,save :: Atom_List(:)     !** Array of specific atoms
Type (Shell_Structure) :: Shell
Integer, Pointer :: AtomType_MAP(:)     !** LUT for mapping Type->atom
Integer, Pointer :: AtomType_Range(:,:) !** Range of types/atom
Type (Anderson_Context), Pointer,save :: Move_AC

Type (StringList_Struct) :: AtomType_LUT      !** LUT for AtomTypes
Type (StringList_Struct) :: Atom_LUT          !** LUT for specific Atoms

Real,    Pointer  :: Occupancy(:,:)       !** Running Occupancy(band,K-pnt)
Integer, Pointer  :: BKMap(:,:)           !** BKMap(band,K-pnt)=PsiIndex
Integer           :: NumBands             !** Number of Bands
Integer           :: NumKpnts             !** Number of PSI k-points
Real              :: TotalElectrons       !** Total Number of electrons
Real              :: NetCharge            !** Net charge of system
Real              :: TotalEnergy          !** Total Energy
Real              :: CohesiveEnergy       !** Cohesive energy of the system
Real              :: BestCohesiveEnergy   !** Stored Cohesive energy 
Real              :: AtomicEnergy         !** Atomic Energy due to core
Real              :: NuclearEnergy        !** Nuclear Energy due to core approx

Real  :: Netforce(3)   !** Net force on unit cell before drift removal
Logical           :: Anchor               !** True if one atom is held fixed
Integer           :: AnchorIndex          !** "specific atom" index of anchor


End Module
