!*****************************************************************************
!
! File : bz_data.f90
!   by : Alan Tackett
!   on : 8/18/95
!  for : DOS and NOS Calculation
!
! This file defines the data structure that contains all the information
! required to calculate the density of states(DOS), number of states(NOS), 
! and other integrals involving the Fermi Surface.
!
!*****************************************************************************

Module bz_data


Type bz_struct
   Integer          :: Method        !** Integration Method
   Integer          :: Bands         !** Total Number of Energy Bands
   Integer          :: TotalKpnts    !** Number of Kpnts in grid
   Integer          :: TotalUniq     !** Number of Unique K points
   Integer          :: TotalCubes    !** Number of cubes
   Integer          :: MaxIter       !** Max number of iterations allowed
   Real             :: Fermi         !** Fermi Energy
   Real             :: Electrons     !** Number of electrons
   Real             :: bz_Low        !** Low estimate of Fermi Energy
   Real             :: bz_High       !** High estimate of Fermi Energy
   Real             :: accuracy      !** Accuracy of Fermi estimate
   Real             :: sigma         !** Gaussian smearing width
   Real             :: CubeVol       !** Cube Volume in k-space
   Real             :: UnitCellVol   !** Unit Cell Volume in k-space
   Real,    Pointer :: Weight(:,:)   !** Int. Wts - Weight(Bands,TotalKpnts)
   Real,    Pointer :: WtUniq(:,:)   !** Uniq int Wts - WtUniq(Bands,TotalUniq)
   Real,    Pointer :: Kp(:,:)       !** List of K-points - Kp(3,TotalKpnts)
   Real,    Pointer :: Ku(:,:)       !** Unique K-pnts list - Ku(3,TotalUniq)
   Real,    Pointer :: Ke(:,:)       !** K-pnt Energies - Ke(Bands,TotalKpnts)
   Real,    Pointer :: KEu(:,:)      !** Unique K-pnt KE's-KUe(Bands,TotalUniq)
   Integer, Pointer :: Kmap(:)       !** Map from Kp->Ku
   Integer, Pointer :: Cube(:,:)     !** Cube list - Cube(TotalCubes,Corners)
End Type

End Module
