!******************************************************************************
!
! File : countbands.f90
!   by : Alan Tackett
!   on : 7/30/98
!  for : PAW project
!
!  CountBands - Counts the number of bands per BZ k-Point this is determined
!               by the Atom_List(i)%Valence_Orbitals which is actually a
!               nili index list determining which basis functions to use
!               as initial valence bands.
!               
!               The number of electrons is also counted.
!
!******************************************************************************

Subroutine CountBands
  Use paw_inout
  Use misc
  Use word
  Use atom_data
  Use options_data

  implicit none
  
  Type (Specific_Atom), Pointer :: A
  Type (Atom_Info_Fixed), Pointer :: AT
  Integer          :: i, j, L, nili, ExtraPsi
  Integer, Pointer :: L_Values(:), Orbitals(:)
  Real, Pointer    :: Occ(:), Occup(:), Occdn(:)

  NumBands = 0
  TotalElectrons = 0
  NetCharge = 0

  Do i=1, Specific_Atoms
     A => Atom_List(i)
     AT => AtomType_Info(A%TypeIndex)

     L_Values => AT%L_Value
     Orbitals => A%Valence_Orbitals
     If (.not.spindependence)Occ => A%Init_Occ
     If (spindependence)Occup => A%Init_Occspinup
     If (spindependence)Occdn => A%Init_Occspindn

     NetCharge = Netcharge + (AT%Atomic_Charge - AT%Core_Charge)

     Do j=1, A%Orbitals_Size
        nili = Orbitals(j)
        L = L_Values(nili)
        
        NumBands = NumBands + 2*L+1
        if(.not.spindependence) TotalElectrons = TotalElectrons + Occ(j)
        if(spindependence) TotalElectrons = TotalElectrons + Occup(j)+Occdn(j)
     End Do
  End Do
  
  Write(Log_Unit, *) 'CountBands: Total number of valence bands:',NumBands
  Write(Log_Unit, *) 'CountBands: Total charge from occupancy:', TotalElectrons

  Write(Log_Unit, *) 'CountBands: Total charge from atoms:', Netcharge

  NetCharge = NetCharge - TotalElectrons
  If (ABS(NetCharge)>1E-6) then
     Write(Log_Unit,*) 'CountBands: Charged SuperCell! Net charge:',NetCharge
     Write(Log_Unit,*) 'CountBands: Compensting with a uniform charge of :',&
          -Netcharge

     Write(Error_Unit,*) 'CountBands: Charged SuperCell! Net charge:',NetCharge
     Write(Error_Unit,*) 'CountBands: Compensting with a uniform charge of :',&
          -Netcharge

  else
     Write(Log_Unit,*) 'CountBands: Neutral SuperCell. No compensation charge required.'
  End If

 
  NumBands = 2*NumBands
!NumBands = 1
!TotalElectrons = 2
  Write(Log_Unit, *) 'CountBands: Using valence bands',NumBands

  Return
End Subroutine



