!****************************************************************************
!
! File : crystal_data.f90
!   by : Alan Tackett
!   on : 9/18/95
!  for : PAW project
!
! Contains the structure pertaining to the crystal.
!
!****************************************************************************

Module crystal_data

Use bzint

Type Crystal_struct    !** Crystal Data structure
   Real    :: Basis(3,3)         !** Lattice Basis Vectors
   Real    :: Recip(3,3)         !** Reciprocal Basis
   Real    :: InverseBasis(3,3)  !** Inverse Basis (Recip / 2*Pi)
   Real, Pointer :: RotMatrix(:,:,:)  !** Array containing symmetry rot mat's
   Real, Pointer :: Trans(:,:)   !** Array containing symmetry rot translations
   Real    :: Volume             !** Unit cell Volume
   Real    :: RecipVol           !** Volume of the reciprocal unit cell
   Real    :: sigma              !** Gaussian Smearing width
   Real, Pointer :: Wt(:)        !** Kpoints weight - Temporary Buffer
   Integer :: Rot_Size           !** Number of rotation matrices
   Integer :: BZ_Method          !** BZ Integration Method
   Integer :: Kpnts_Grid(3)      !** K-points Grid Size
   Logical :: Auto_symmetry      !** Automatically calculate the symmetry
   Integer :: CloneCell(3)       !** Cell replication
End Type

Type (Crystal_Struct) :: xtal         !** Crystal data information
Logical               :: Xtal_Defined !** Determines if I have the Xtal defined
Type (BZ_Struct)     :: BZ            !** BZ data
Real, Pointer         :: angpoints(:,:) !** Gauss-Legendre integration points
Integer               :: Num_AngPoints  !** number of angular points
Integer               :: BLOCH_ZERO

Type Two_Dim   !** Data type for 2-dimensional plot
   Real  :: X(3)  ! X-axis in Cartesian coordinates
   Real  :: Y(3)  ! Y-axis in Cartesian coordinates
   Real  :: Origin(3) ! Coordinate (in Cartesian coordinates) of plot origin
   Integer :: Grid(2)  ! Number of grid points in 2 dimensions
   Integer :: Query    ! Number to check correct setting of parameters
End Type

Type Three_Dim   !** Data type for 3-dimensional plot
   Real  :: X(3)  ! X-axis in Cartesian coordinates
   Real  :: Y(3)  ! Y-axis in Cartesian coordinates
   Real  :: Z(3)  ! Z-axis in Cartesian coordinates
   Real  :: Origin(3) ! Coordinate (in Cartesian coordinates) of plot origin
   Real  :: maxbond   ! Maximum bond length for ball and stick output
   Real  :: bondtol   ! Bond length % slack for ball and stick output
   Integer :: Grid(3)  ! Number of grid points in 3 dimensions
   Integer :: Query    ! Number to check correct setting of parameters
End Type

Type (Two_Dim) :: Planeplot
Type (Three_Dim) :: Volumeplot

End Module
