!******************************************************************************
!
! File : debug.f90
!   by : Alan Tackett
!   on : 08/11/97
!  for : Testing MGPAW
!
! modified for spin dependence by Ping Tang
!     and N. A. W. Holzwarth
! last change: 5/27/05
!
!******************************************************************************

Module debug

Use paw_inout
Use mem_data
Use memmgr
Use psilib
Use fileio
Use paw_inout
Use search_sort
Use hamop
Use word
Use basis_lib
Use gpoints
Use oinverse

Implicit NONE!!!!!!!!!!!!!


!******************************************************************************
Contains
!******************************************************************************


!******************************************************************************
!
!  WriteMatrix_Maple - Prints a matrix to disk for inclusion in maple
!
!    FileName  - FileName 
!    A         - Matrix
!    Rows,Cols - Rows and columns of matrix
!
!******************************************************************************

Subroutine WriteMatrix_Maple(Filename, A, Rows, Cols)
  Character*(*), Intent(IN) :: FileName
  Complex,       Intent(IN) :: A(:,:)
  Integer,       Intent(IN) :: Rows
  Integer,       Intent(IN) :: Cols

  Integer :: i,j,fd
  Character*100 :: text, t1, t2

  fd = 231
  Open(fd, FILE=Filename, RECL=1024)

  Write(fd,*) 'with(linalg);'

  Write(t1, *) Rows
  Write(t2, *) Cols
!  Write(fd,*) 'A := array(1..',TRIM(ADJUSTL(t1)), &
!       ', 1..',TRIM(ADJUSTL(t2)),', ['

  Write(fd,*) 'A := array(1..',Rows, &
       ', 1..',Cols,', ['

  Do i=1, Rows
     Write(fd,*)    '      ['
     Do j=1, Cols
!**        Write(t1,*) Real(A(i,j))
!**        Write(t2,*) Imag(A(i,j))

        IF (j==Cols) then
!**           Write(fd,*) '      ',TRIM(ADJUSTL(t1)),'+',TRIM(ADJUSTL(t2)), '*I'
           Write(fd,*) '      ',Real(A(i,j)),'+',Imag(A(i,j)), '*I'
        else
!**           Write(fd,*) '      ',TRIM(ADJUSTL(t1)),'+',TRIM(ADJUSTL(t2)), '*I,'
           Write(fd,*) '      ',Real(A(i,j)),'+',Imag(A(i,j)), '*I,'
        End If
     End Do
     Write(fd,*)    '      ]'
  End Do

  Write(fd, *) ']);'

  Close(fd)

  Return
End Subroutine

!******************************************************************************
!
!  CheckError_Grid - Calculates the energy and magnitude of the given Grid
!
!  Psi - Wave vector to multiplies band
!
!******************************************************************************

Subroutine CheckError_Grid(Grid, Kpnt,BandEnergy,Band, OutPut, Err)
  Complex,  Intent(IN)    :: Grid(:)
  Integer,          Intent(IN)    :: Kpnt  
  Real,     Intent(IN)    :: bandEnergy
  Integer,          Intent(IN)    :: Band
  Integer, OPTIONAL, Intent(IN)   :: Output
  Real, OPTIONAL,  Intent(OUT)   :: Err
  

  Complex :: energy, MAg, Error, oi_err, hoih_err, Pdot2(PLM_Max)
  Complex, Pointer :: Psi(:), Hx(:), KE(:), PDOT(:), Oi(:), HOiHx(:)
  Complex, Pointer :: Ve(:)
  Integer      :: LSize, Out, i
  Integer :: Vec_toProcess(Mem_MapSize)  
  Logical :: PDOT_Stored
  Real :: Eo, Ew

  If (Present(Output)) then
     out = Output
  else
     out = 6
  End If
  
  if(.NOT.spindependence .OR. PsiInfo(Band)%spinup) THEN
       Ve=>SCFValues%Ve
  else
       Ve=>SCFValues%Vespin
  end if

!Write(*,*) 'CheckError: Start!!!!!!!!!'
  Call Getbuffer( Psi)
  Call GetBuffer( Hx)
  Call GetBuffer( KE)

  Psi = Grid

  Vec_toProcess = MH_Skip
!Write(*,*) 'CheckError: Before Phase_Generic'
  Call Phase_Generic( Vec_toPRocess,  Kpnt)
!Write(*,*) 'CheckError: Before CalcProjProducts'

  PDOT => Psiinfo(Band)%PDOT
  Call CalcHxandOx( Psi, Hx, KE, Ve, PDOT, &
       Kpnt, PsiInfo(Band)%PDOT_Stored)

Eo = 1;  Ew = 1;
hoih_err = -1
If (Eigen_Mode == EIGEN_ENERGY) then
Call GetBuffer( HOiHx)
  Call Calc_HOiHx_Hx_Ox( Psi, HOiHx, Hx, KE, Ve, PDOT, &
       Kpnt, PsiInfo(Band)%PDOT_Stored, Eo, Ew)

hoih_err = Basis_DotProd(Psi, HOiHx, .FALSE.)

energy = Basis_DotProd( Psi,Hx, .FALSE.)
HOiHx = HoiHx - hoih_err*KE
oi_err = Basis_DotProd( HOiHx, HoiHx, .FALSE.)
Call FreeBuffer(HOiHx)

end if

  mag = Basis_DotProd( Psi, KE, .FALSE.)

  Ke = Hx - BandEnergy*Ke
  
  Error = Basis_DotProd( KE, KE, .FALSE.)
  Call FreeBuffer(KE)
 
  Energy = Basis_DotProd( Psi, Hx, .FALSE.)
  Call FreeBuffer(Hx)

  Call FreeBuffer(Psi)

  Write(out,*) 'CheckError: Calculated Energy: ', Real(Energy), &
       ' * Band Energy: ', BandEnergy, ' * Mag(Psi)=',Real(mag)
  if (abs(real(mag)) > 1D-100) Write(out,*)'             Normalized Energy =', Real(Energy/mag)

  Write(out,*) 'CheckError: Error in Hx-eX =', Real(Error), ' * Kpnt=',Kpnt, ' * Band=', Band

If (Eigen_Mode == EIGEN_ENERGY) then
   energy = (Energy - Eo)/Ew
   Write(out,*) 'CheckError: Error in Oinv =',oi_err
   Write(out,*) 'CheckError: Error in HOiH =',hoih_err, ' shift energy=',Energy**2
End If
  If (Present(Err)) Err = Error

  Return
End Subroutine

!******************************************************************************
!
!  CheckError - Calculates the energy and magnitude of the given band
!
!  PsiBand - Wave vector to multiplies band
!
!******************************************************************************
Subroutine CheckError(PsiBand, OutPut, Error)
  Integer,          Intent(IN)    :: PsiBand
  Integer, OPTIONAL, Intent(IN) :: Output
  Real, OPTIONAL,     Intent(OUT) :: Error

  Real :: Err
  Integer :: Out
  Complex, Pointer :: Psi(:)

  If (Present(Output)) then
     out = Output
  else
     out = 6
  End If

  Call Getbuffer( Psi)
  Call GetPsi_fromBuffer( PsiBand, Psi)

  Call CheckError_GRID( Psi, PsiInfo(PsiBand)%Kpnt, &
       PsiInfo(PsiBand)%Energy, PsiBand, out, Err)

  Call FreeBuffer(Psi)

  If (Present(Error)) Error = Err

  PsiInfo(PSiBand)%Error = Err

  Return
End Subroutine


!******************************************************************************
!
!  CheckClusterError - Checks the cluster error
!
!     Output - Output Unit
!     Cluster - Cluster to check
!
!******************************************************************************

Subroutine CheckClusterError(Output, Cluster, DoStoreError)
  Integer, Intent(IN) :: Output
  Integer, Intent(IN) :: Cluster
  Logical, Intent(IN) :: DoStoreError

  Integer :: i,  LSize
  Real :: Error
  
  LSize = PsiArraySize

  Do i=1, Mem_MapSize
     If (ClusterMap(i) == CLuster) then
        Write(Output, *) 'CheckCLusterError: Disk Index=', i, &
             ' * CLuster=', CLuster
        Call CheckError( i, Output, Error)

        If (DoStoreError) PsiInfo(i)%Error = Error
     End If
  End Do

  Return
End Subroutine

!******************************************************************************
!
! DisplayClusterEnergy - Printe the Cluster Energy to the specified device
!  
!    Output - Output device
!    Cluster - Cluster Info to Display
!
!******************************************************************************

Subroutine DisplayClusterEnergy(Output, Cluster,Countup,Countdn)
  Integer, Intent(IN) :: Output
  Integer, Intent(IN) :: Cluster
  Real, INTENT(INOUT) :: Countup
  Real, INTENT(INOUT), OPTIONAL :: Countdn

  Integer :: i
  Character*50 :: tmpstr, Kpnt_String


  Write(Output,*)'Energies for cluster',cluster, &
       ' (DiskRec, Energy, Occ, Kpnt)', ' * Size:',ClusterSize(cluster)

  Do i=1, Mem_MapSize
     If (CLusterMap(i) == Cluster) then
        Write(Kpnt_String, *) PsiInfo(i)%Kpnt,PsiInfo(i)%spinup        

        Write(OutPut, *) i, ' *', PsiInfo(i)%Energy, &
             ' *',PsiInfo(i)%Occupancy, ' * ', TRIM(AdjustL(Kpnt_String))
        If (PsiInfo(i)%spinup) Countup=Countup+PsiInfo(i)%Occupancy
        If (present(Countdn).and..not.PsiInfo(i)%spinup)&
          Countdn=Countdn+PsiInfo(i)%Occupancy                                                
     End if
  End Do

  Return
End Subroutine


!******************************************************************************
!
! DisplayEnergy - Prints the Energy levels to the specified device
!  
!    Output - Output device
!
!******************************************************************************

Subroutine DisplayEnergy(Output,Countup,Countdn)
  Integer, Intent(IN) :: Output
  Real, INTENT(OUT) ::Countup
  Real, INTENT(OUT),optional ::Countdn

  Integer :: i

  Countup=0
  If (present(Countdn)) Countdn=0
  Do i=1, LastCluster
    If(present(Countdn)) then
     Call DisplayCLusterEnergy(Output, i,Countup,Countdn)
    Else
     Call DisplayCLusterEnergy(Output, i,Countup)
    Endif
  End Do

  Return
End Subroutine

!******************************************************************************
!
!  CheckPotErrorl - Checks the error in the potential
!
!   V     - Potential
!   Rho   - Density  
!   Output - Output unit
!
!
!******************************************************************************

Subroutine CheckPotError( Output, Err)
  Integer,          Intent(IN) :: Output
  Real, OPTIONAL, Intent(OUT) :: Err

  If (Present(Err)) Err = 0

  Return
End Subroutine


!******************************************************************************
!
! WriteSlice
!
!******************************************************************************

Subroutine WriteSLice(PM_Slice,Dir,fn1, fn2, V)
  Integer, Intent(IN) :: PM_SLice(:)
  Integer, Intent(IN) :: Dir
  Character*(*), intent(IN) :: Fn1, Fn2
  Complex, Intent(IN) :: V(:)

  Complex, Pointer :: Vr(:)
  Character*1, PARAMETER :: XYZ(3) = (/'X', 'Y', 'Z'/)
  Integer :: P(3), Pf(3), i, fd, offset
  Complex :: Value
  Real    :: mag, R(3)

  write(Log_unit,*) 'Entered subroutine WriteSlice which no longer works'
  stop
  Vr =>SCFvalues%Work
  Vr = 0
  Vr(FFTmap_Low) = V
  
  Call PerformFFT(FFT_TO_R, G_LOW, Vr)

  fd = 98
  Open(fd, FILE=Fn1, RECL=10000)
  Write(fd,*) 'PM, R, Fn(Re), Fn(Im), Fn(mag**2)  Fn(mag)  (Dir=', &
       XYZ(Dir),')'
  P = PM_Slice
  Do i=1, FFT_Grid(Dir, G_Low)    
!Write(*,*) 'WriteSLice: i=',i
     P(dir) = i

     offset = CalcLinearIndex(FFT_Grid(:,G_LOW), P) + 1
     Value = Vr(offset)

     !Call CalcRealSpace( Pf, R)
     mag = CONJG(Value)*Value
     Write(fd, *) i, R(dir), Real(Value), AImag(Value), mag, sqrt(mag)
  End Do

  Close(fd)
 
  Return
End Subroutine


!******************************************************************************
!
! WritePsiSlice
!
!******************************************************************************

Subroutine WritePSiSlice( PM_SLice, Dir, ext)
  Integer, Intent(IN) :: PM_SLice(:)
  Integer, Intent(IN) :: Dir
  Character*(*), intent(IN) :: Ext

  Integer :: i
  Character*100 :: Fname1, fname2
  Complex, Pointer :: Psi(:), Vout(:), KE(:), Ve(:)
  Complex, TARGET  :: PDot(PLM_Max)
  Integer :: Vec_toProcess(Mem_MapSize)  

  write(Log_unit,*) 'Entered WritePSiSlice which no longer works'
  stop
  Call GetBuffer( Psi)
  Call GetBuffer( Vout)
  Call GetBuffer( KE)

  Fname1 = 'pot' // TRIM(EXT)
  Fname2 = 'pot_used' // TRIM(EXT)
  SCFvalues%Work = SCFvalues%Ve
  Call PerformFFT(FFT_TO_G, G_LOW, SCFvalues%Work)
  Psi = SCFvalues%Work(FFTmap_LOW)

  Call WriteSLice( PM_SLice, Dir, Fname1, Fname2, &
         SCFvalues%Work)

  Vec_toProcess = MH_SKip

  Do i=1, Mem_MapSize - 1
     If (ClusterMap(i)>0) then
        Call GetPsi_fromBuffer( i, Psi)
        Write(Fname2, *) i
        Fname1 = 'psi' // TRIM(ADJUSTL(Fname2)) // TRIM(EXT)
        Fname2 = 'psi_used' // TRIM(ADJUSTL(Fname2)) // TRIM(EXT)
        Call WriteSLice(PM_SLice,Dir,Fname1,Fname2, Psi)

        Vout = 0
        Call Phase_Generic( Vec_toProcess,  &
             PsiInfo(i)%Kpnt)

  if(.NOT.spindependence .OR. PsiInfo(i)%spinup) then
         Ve=>SCFValues%Ve
  else
         Ve=>SCFValues%Vespin
  endif
        Call HPsi( Psi, &
             Vout, Ve, PsiInfo(i)%PDot, PsiInfo(i)%Kpnt, PsiInfo(i)%PDOT_Stored)
        Call Opsi( Psi, KE, &
             PsiInfo(i)%PDot, PsiInfo(i)%Kpnt, PsiInfo(i)%PDOT_Stored)

        Vout = Vout - PsiInfo(i)%Energy*KE
Write(Log_Unit,*) 'WPS: i=',i
Fname2 = ' '
        Write(Fname2, *) i
Write(Log_Unit,*) 'AFter Fname2 Store =!',TRIM(Fname2),'!'
        Fname1 = 'psierr' // TRIM(ADJUSTL(Fname2)) // TRIM(EXT)
        Fname2 = 'psierr_used' // TRIM(ADJUSTL(Fname2)) // TRIM(EXT)
        Call WriteSLice(PM_SLice,Dir,Fname1,Fname2, Vout)
     End If
  End Do

  Call FreeBuffer(Psi)
  Call FreeBuffer(Vout)
  Call FreeBuffer(KE)

  Return
End Subroutine




End Module
