!******************************************************************************
!
! File : doprint.f90
!   by : Alan Tackett
!   on : 9/25/95
!  for : PAW Project
!
!  DoPrint - Formats and generates output for the PAW program.
!            Below is the Format for the statement.  The function
!            is line oriented.  So all input on the entire line is processed!
!
!  Input_WC - Input Context to take input from
!
!  Print OUTPUT_DEVICE LITERAL_STRING_1, LITERAL_STRING_2, .. 
!
!  Where OUTPUT_DEVICE is one of the following:
!              LOG - Print to Log File (Default)
!              OUTPUT - Rpints to the output file
!              ERROR  - Prints to the Error File
!              ALL    - Prints to ALL the above
!              
!
!         LITERAL_STRING - A Literal string containing text to be printed.
!              Multiple Strings may be listed
!
!******************************************************************************

Subroutine DoPrint(input_wc)
  Use paw_inout
  implicit none
!*  Use word
  Type (Word_Context), Intent(INOUT) :: Input_WC

  Integer, PARAMETER :: LOG_OUT = 1
  Integer, PARAMETER :: OUT_OUT = 2
  Integer, PARAMETER :: ERR_OUT = 3
  Integer, PARAMETER :: ALL_OUT = 4

  Logical :: NewTok
  Integer :: Out_Dev, tlen
  Character*200 :: token, Token2

  Call GetNextWord(Input_WC, Token2, tlen)
  Token = Token2
  Call UpperCase(Token2)

  NewTok = .TRUE.
  if (Trim(Token2) == "LOG") then
    Out_Dev = LOG_OUT
  else if (Trim(Token2) == "OUTPUT") then
    Out_Dev = OUT_OUT
  else if (Trim(Token2) == "ERROR") then
    Out_Dev = ERR_OUT
  else if (Trim(Token2) == "ALL") then
    Out_Dev = ALL_OUT
  else 
    NewTok = .FALSE.               !** Flag NOT to get another token
    Out_Dev = LOG_OUT
  End If

  if (NewTok) Call GetWord(Input_WC, Token, tlen)

  NewTok = .TRUE.

  Do While (NewTok) 
    Select Case (Out_Dev)
      Case (LOG_OUT)
            Write(Log_Unit, *) Token(1:Tlen)
      Case (OUT_OUT)
            Write(Output_Unit, *) token(1:TLen)
      Case (ERR_OUT)
            Write(Error_Unit, *) token(1:TLen)
      Case (ALL_OUT)
            Write(Log_Unit, *) Token(1:Tlen)
            Write(Output_Unit, *) token(1:TLen)
            Write(Error_Unit, *) token(1:TLen)   
    End Select

    If (W_Error == W_OK) then
      Call GetWord(Input_WC, Token, Tlen)
    else
      NewTok = .FALSE.
    End IF
  End Do
    
  Return
End Subroutine
