!******************************************************************************
!
! File : fftw.f90
!   by : Natalie Holzwarth
!   on : 07/08/99
!  for : PWPAW
!
! Module containing the fft routines needed for pwpaw                     
!  Obtained from  fttp:/theory.lcs.mit.edu/~fftw
!
! Copyright (c) 1997-1999 Massachusetts Institute of Technology
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!

 Module fftw

   Use timing

   implicit none

        ! include "fftw_f77.i"
!     This file contains PARAMETER statements for various constants
!     that can be passed to FFTW routines.  You should include
!     this file in any FORTRAN program that calls the fftw_f77
!     routines (either directly or with an #include statement
!     if you use the C preprocessor).
!
!     integer FFTW_FORWARD,FFTW_BACKWARD
!     parameter (FFTW_FORWARD=-1,FFTW_BACKWARD=1)
  Integer, parameter :: FFTW_FORWARD=-1,FFTW_BACKWARD=1
!
!     integer FFTW_REAL_TO_COMPLEX,FFTW_COMPLEX_TO_REAL
!     parameter (FFTW_REAL_TO_COMPLEX=-1,FFTW_COMPLEX_TO_REAL=1)
  Integer, parameter :: FFTW_REAL_TO_COMPLEX=-1,FFTW_COMPLEX_TO_REAL=1
!
!     integer FFTW_ESTIMATE,FFTW_MEASURE
!     parameter (FFTW_ESTIMATE=0,FFTW_MEASURE=1)
  Integer,  parameter :: FFTW_ESTIMATE=0,FFTW_MEASURE=1
!
!     integer FFTW_OUT_OF_PLACE,FFTW_IN_PLACE,FFTW_USE_WISDOM
!     parameter (FFTW_OUT_OF_PLACE=0)
!     parameter (FFTW_IN_PLACE=8,FFTW_USE_WISDOM=16)
  Integer, parameter :: FFTW_OUT_OF_PLACE=0,FFTW_IN_PLACE=8,FFTW_USE_WISDOM=16
!
!     integer FFTW_THREADSAFE
!     parameter (FFTW_THREADSAFE=128)
  integer,  parameter :: FFTW_THREADSAFE=128
!

Character, PARAMETER :: FFT_TO_G = 'F'   !*** Perform FFT(r->g)
Character, PARAMETER :: FFT_TO_R = 'B'   !*** Perform FFT(g->r)


  Type FFT_Plan
      Integer*8 :: to_R
      Integer*8 :: to_G
      Integer :: Grid(4)
  End Type
     
  Type(FFT_Plan), Pointer, PRIVATE :: Plan_List(:)
  Integer, PRIVATE          :: Plan_Size

!***************************************************************************
contains
!***************************************************************************


!***************************************************************************
!do3dfftw -- driver program for fftw

  Subroutine do3dfftw(Dir,plan,Inout)

   Character, Intent(IN) :: Dir
   Type(FFT_Plan), Intent(IN) :: plan
   Complex, Intent(INOUT) :: Inout(:)

!**   write(6,*) 'entering do3dfftw with',dir,grid

   If (Dir == 'F') then
     call fftwnd_f77_one(plan%to_G,Inout(1),0)
   else if (Dir == 'B') then
     call fftwnd_f77_one(plan%to_R,Inout(1),0)
    
     Inout=Inout/Plan%Grid(4)
   else 
 
      write(6,*) 'Error in fftw -- Dir = ', Dir
      stop
   endif

   return
   end subroutine do3dfftw


!******************************************************************************
!
!  PerformFFT - Performs the 3d FFT   -- version for fftw
!
!  Dir  - Direction(FFT_TO_G or FFT_TO_R)
!  Grid - Grid Dimensions
!  InOut   - In and  Output array
!
!
!******************************************************************************

Subroutine PerformFFT(Dir, Plan, InOut)
  Character, Intent(IN)  :: Dir
  Integer,   Intent(IN)  :: Plan
  Complex,   Intent(INOUT)  :: InOut(:)

!  Include "DXMLDEF.inc"
!  Integer :: err, Grid(3)
!  Integer, EXTERNAL :: zfft_3d
!  Grid = plan_list(plan)%Grid(1:3)
!  err = zfft_3d('C','C', Dir, InOut, InOut, grid(1),grid(2),grid(3), &
!       grid(1), grid(2), 1,1,1)

  Call Start_Timer(Timer(FFT_Timer))

  call do3dfftw(Dir,plan_List(plan),InOut)

  If (Dir==FFT_TO_G) InOut=InOut/plan_list(plan)%Grid(4)
  If (Dir==FFT_TO_R) InOut=InOut*plan_list(plan)%Grid(4)

  Call Stop_Timer(Timer(FFT_Timer))

  Return
End Subroutine


!******************************************************************************
!
!  TestFFT - Test the fft routine
!
!******************************************************************************

Subroutine TestFFT(plan, InOut)
  Integer,   Intent(IN)  :: plan
  Complex,   Intent(INOUT)  :: InOut(:)

  Complex :: csum

  Inout = 1

  csum = DOT_PRODUCT(InOut,inout)
  Write(*,*) 'TestFFT: Grid=',plan_list(plan)%Grid, ' * SUM=',csum
  Call PerformFFT(FFT_TO_R, plan, InOut)
  csum = DOT_PRODUCT(InOut,inout)
  Write(*,*) 'TestFFT: FFT_TO_R Result=',csum

  Call PerformFFT(FFT_TO_G, plan, InOut)
  csum = DOT_PRODUCT(InOut,inout)
  Write(*,*) 'TestFFT: FFT_TO_G Result=',csum

  Return
End Subroutine


!******************************************************************************
!
!  FFT_CreatePlan - Creates a plan for use
!
!      slot - Slot to store the plan
!      grid - Size of FFT grid
!
!******************************************************************************

Subroutine FFT_CreatePlan(slot, grid)
  Integer, Intent(IN) :: slot
  Integer, Intent(IN) :: grid(:)

  Plan_list(slot)%Grid(1:3) = Grid(1:3)
  Plan_list(slot)%Grid(4) = PRODUCT(Grid(1:3))

  call fftwnd_f77_create_plan(plan_list(slot)%to_g, 3, &
       Plan_List(slot)%grid(1), FFTW_FORWARD, FFTW_ESTIMATE+FFTW_IN_PLACE)

  call fftwnd_f77_create_plan(plan_list(slot)%to_R, 3, &
       Plan_List(slot)%grid(1), FFTW_BACKWARD, FFTW_ESTIMATE+FFTW_IN_PLACE)
  
  Return
end Subroutine


!******************************************************************************
!
!  FFT_DestroyPlan - Destroys a plan
!
!      slot - Slot to destroy
!
!******************************************************************************

Subroutine FFT_DestroyPlan(slot)
  Integer, Intent(IN) :: slot

  call fftwnd_f77_destroy_plan(plan_list(slot)%to_g)
  call fftwnd_f77_destroy_plan(plan_list(slot)%to_R)
  
  Return
end Subroutine


!******************************************************************************
!
! InitFFTW - Initializes the FFTW routines for use
!
!    MaxPlans - Max number of different FFT grids to construct plans for
!
!******************************************************************************

Subroutine InitFFTW(MaxPlans)
   Integer, Intent(IN) :: MaxPlans

   Plan_Size = MaxPlans
   Allocate(Plan_List(Plan_size))

   Return
End Subroutine


end module fftw

