!******************************************************************************
!
! File : forces.f90
!   by : Alan Tackett
!   on : 8/1/98
!  for : PAW program
!
! Module for calculating the forces on the atoms
!
! Updatedate for spin by Ping Tang and N. A. W. Holzwarth
!    Last modification 5/23/05
!******************************************************************************

Module forces

Use word
Use paw_inout
Use atom_data
Use crystal_data
Use crystal_symmetry
Use spherical_harmonic
Use misc
Use gpoints
Use projectors
Use psilib
Use options_data
Use mathlib
Use mem_data
Use memmgr
Use search_sort
Use hamiltonian

Implicit NONE!!!!!!!!!!!!!!

Integer, PARAMETER :: FORCE_WIJ = 0   !** Which Fij coeff to calculate
Integer, PARAMETER :: FORCE_UIJ = 1

!******************************************************************************
Contains
!******************************************************************************



!******************************************************************************
!
!  CalcForces - Calculates the Forces on the atoms
!
!******************************************************************************

Subroutine CalcForces

  Real    :: Cohesivee, F(3)
  Integer :: Copy_Vnewmix, Copy_Dijnewmix, i
  Logical :: DoCalcHam

 ! Initialize force array

  Call Start_Timer(Timer(Force_Timer))
  write(log_unit,*) 'inCalcForces'
  call flush(log_unit)
  DoCalcHam = Forces_Always_Calc_H

  Do i=1, Specific_Atoms
     !write(log_unit,*) 'Calcforces', i
     !call flush(log_unit)
     Atom_List(i)%Force%CurrForce = 0
     Atom_List(i)%Fij = 0
     Atom_List(i)%Feij = 0
     !write(log_unit,*) size(Atom_List(i)%Fij),size(Atom_List(i)%Feij)
     !call flush(log_unit)
     if (spindependence) then
        Atom_List(i)%Fijspin = 0
        Atom_List(i)%Feijspin = 0
     !write(log_unit,*) size(Atom_List(i)%Fijspin),size(Atom_List(i)%Feijspin)
     !call flush(log_unit)
     endif
  End Do

 ! recalculate Dij and Qlm matrix for current solution
  If (DoCalcHam) then
     Copy_Vnewmix=V_newMix
     Copy_Dijnewmix=Dij_NewMix
     V_newMix=1
     Dij_NewMix=1
     Call CalcHam(Cohesivee)
     CohesiveEnergy =Cohesivee
     V_newMix=Copy_Vnewmix
     Dij_NewMix=Copy_Dijnewmix

     !Call CalcEigenValues -- Taken out 1-24-01 NAWH
   write(Log_unit,*) 'from CalcForces - Cohesiveenergy = ', Cohesiveenergy
  End If


! DenHat and Vlocal terms

  Call CalcFij

  If (xtal%Rot_Size > 1) then
     Call SymFij(2)
     Call SymFij(3)
  End If

  Call AccumForce1

  Do i=1,Specific_atoms
   write(Log_unit,*) 'Plane wave contribution to force for i',i
   write(Log_unit,*) Atom_list(i)%force%CurrForce
  Enddo
  
  Call AccumForce_PROJ

  Do i=1,Specific_atoms
   write(Log_unit,*) 'Term #2 contribution to force for i',i
   write(Log_unit,*) Atom_list(i)%force%CurrForce
  Enddo

!  Call AccumForce3

!  Do i=1,Specific_atoms
!   write(Log_unit,*) 'Term #3 contribution to force for i',i
!   write(Log_unit,*) Atom_list(i)%force%CurrForce
!  Enddo

! temp write statements
  Do i=1,Specific_atoms
   write(Log_unit,*) 'Before sym  force for i',i
   write(Log_unit,*) Atom_list(i)%force%CurrForce
  Enddo
   
  If (xtal%Rot_size > 1) Call SymForce

  Do i=1,Specific_atoms
   write(Log_unit,*) 'After  sym  force for i',i
   write(Log_unit,*) Atom_list(i)%force%CurrForce
  Enddo

  F = 0
  Do i=1, Specific_Atoms 
     F = F + atom_list(i)%force%currforce
  Enddo
  NetForce = F
  Write(Log_Unit,'("CalcForces: Net Force :", 1p3e15.7)') F

  !** Remove drift in forces **  Removed by NAWH 01/31/00
  !!!F = F/Specific_Atoms
  !!!Do i=1, Specific_Atoms
  !!!   atom_list(i)%Force%CurrForce = atom_list(i)%Force%CurrForce - F
  !!!end Do

!!!!!!! Take out the Anchor shift in this subroutine
 !!! !** Apply Anchor constraint of Anchor=.true.
 !!! If (Anchor) then 
 !!!    F=-atom_list(AnchorIndex)%force%currforce
 !!!    Write(Log_Unit,'("CalcForces: Anchor Force:",1p3e15.7)') F
 !!!    Do i=1, Specific_Atoms
 !!!       atom_list(i)%Force%CurrForce = atom_list(i)%Force%CurrForce + F
 !!!    end Do

 !!!    Do i=1,Specific_atoms
 !!!     write(Log_unit,*) 'After  sym  force for i',i
 !!!     write(Log_unit,*) Atom_list(i)%force%CurrForce
 !!!    Enddo

 !!! EndIf

  Call Stop_Timer(Timer(Force_Timer))
  Return
End Subroutine

!******************************************************************************
!
!  CHeckForces - Checks     the Forces on the atoms
!
!  ForceTol   - magnitude to togle convergence check
!  Converged  - true if force<forcetol 
!
!******************************************************************************

Subroutine CheckForces(ForceTol,Converged)
  Real    , Intent(IN):: ForceTol
  Logical , Intent(OUT):: Converged
  
  Integer :: i
  Real :: mag

  Converged = .TRUE.
  Do i=1, Specific_Atoms
     mag = SQRT(DOT_PRODUCT(Atom_List(i)%Force%CurrForce, &
               Atom_List(i)%Force%CurrForce))
     
     If (mag>ForceTol) Converged = .FALSE.
  End Do

 return
End Subroutine


!******************************************************************************
!
!  AccumForce1 - Accumulates the DenHat and Vlocal contribution
!
!
!******************************************************************************

Subroutine AccumForce1

  Real    :: con
  Integer         :: L,m,lut,gi,atom,HatIndex,Offset_Q,i,atype
  Real    :: theta, Gmag,  RadG, G(3)
  Complex, parameter :: ai = (0,1)
  Complex :: accum(3),Phase,Ylm(13),Ylm_neg(13),DenT,tmp,tmpp,smoothden
  Type (Specific_Atom), Pointer :: A
  Type (Atom_Info_Fixed), Pointer :: AT


! DenHat and Vlocal terms

con=2*Four_Pi/xtal%volume

  HatIndex = 1
  Do gi=2, Gpnt_Size(G_HIGH)
     G = Gpnt(1:3,gi)
     Gmag = Gpnt(4,gi)
     if (spindependence) then
       smoothden=SCFvalues%RhoSmooth(gi)+SCFvalues%RhoSmoothspin(gi)
     else
       smoothden=SCFvalues%RhoSmooth(gi)
     endif
     DenT = con *  Conjg((smoothden &
       + SCFvalues%RhoHat(gi)+SCFvalues%CoreTail(gi)))/(Gmag**2)

     !*** If needed inc the HatIndex ***
     If (G_NewMag(gi)) HatIndex = HatIndex + 1
 
 
        !*** structure factors ***
     Do atom = 1 , Specific_atoms
        A => Atom_List(atom)
        AT => AtomType_Info(A%TypeIndex)
        Offset_Q = AT%Hat_MaxL + 1
 
        theta = DOT_PRODUCT(G, A%Pos)
        Phase = CMPLX(cos(theta), -sin(theta))
        
        !*** Sum over Qlm factors
        accum=0
        Do L=0, AT%Hat_MaxL
           Ylm = Spharm(G(1), G(2), G(3), L, .TRUE.)
           
           RadG = AT%RadHat(HatIndex, L+1)
           tmp=0
           Do m=-L, L
              tmp=tmp+A%Qlm(L+1,Offset_Q+m)*Ylm(L+m+1)
           Enddo

           tmp=tmp*(ai**(-L))
          ! tmpp=tmp*((-1)**L)
           
          ! accum=accum + (RadG*G)*ai*(tmp*Phase*DenT-tmpp*Conjg(Phase*DenT))
           accum=accum - 2*(RadG*G)*Imag(tmp*Phase*DenT)
        Enddo

        !*** Add coretail term
           accum=accum - 2*G*AT%FTCoreTail(HatIndex)*Imag(Phase*DenT)

        !*** Vlocal Contributions
        tmp=Phase*Conjg(smoothden)
        ! accum = accum + ai*G*AT%Rad_Vlocal(HatIndex)*(tmp-Conjg(tmp))
        accum = accum - 2*G*AT%Rad_Vlocal(HatIndex)*Imag(tmp)
        
        !*** VXC term
           accum=accum - 2*G*AT%FTCoreTail(HatIndex)* &
                  Imag(Phase*Conjg(SCFValues%VXC(gi)))
        A%force%CurrForce = A%force%CurrForce + accum
   Enddo
 Enddo

  Return
End Subroutine

!******************************************************************************
!
!  AccumForce_PROJ - Accumulates the Fij contributions 
!
!
!******************************************************************************

Subroutine AccumForce_PROJ

  Type (Specific_Atom), Pointer :: A
  Type (Atom_Info_Fixed), Pointer :: AT
  Integer :: i,j, atom, Total_Basis
  Complex :: accum(3)


  
  Do atom = 1 , Specific_atoms
     A => Atom_List(atom)
     AT => AtomType_Info(A%TypeIndex)
     Total_Basis = AT%nlm_Size
     
     accum = 0
     do i=1,Total_Basis
        do j=1,Total_Basis
           accum = accum - A%Fij(:,i,j)*A%Dij(i,j)&
                + A%Feij(:,i,j)*AT%Oij(i,j)
        Enddo
     Enddo
     
    If (spindependence) then
     do i=1,Total_Basis
        do j=1,Total_Basis
           accum = accum - A%Fijspin(:,i,j)*A%Dijspin(i,j)&
              + A%Feij(:,i,j)*AT%Oij(i,j)
        Enddo
     Enddo
    endif

     A%force%Currforce = A%force%Currforce + accum     
  Enddo

  Return

End Subroutine
           

!******************************************************************************
!
!  SymFij - Symmetrizes the Fij and Feij coefficients
!
!      index = 2 for Fij   or   index = 3 for Feij
!
!******************************************************************************

Subroutine SymFij(Index)
  Integer, INTENT(IN) :: Index

  type Fij_Atom
     Complex, Pointer :: Fij(:,:,:)
  End Type

  Integer :: i,j,k, nili, njlj, Li, Lj,mi1, mj1, mi2, mj2, Basis_Size
  Integer :: ibase, jbase,spin,s
  Real    :: c
  Complex :: zsum(3),zrotated(3)
  Complex, Pointer :: Fij(:,:,:), Fij_Copy(:,:,:)
  Type (Fij_Atom) :: Fij_List(Specific_Atoms)
  Type (Specific_Atom), Pointer :: A
  Type (Atom_Info_Fixed), Pointer :: AT
  Character*100 :: Msg


  If (index.ne.2.and.index.ne.3) then
    write(Msg,*) "SymFIJ: Error in index",index
    Call Check_Error(1, Msg, Error_Unit, .TRUE., IDTEXT="SymFij:")
    Stop
  Endif

  spin=1
  if (spindependence) spin=2
  c = 1.0/XTal%Rot_Size

  Do i=1, Specific_Atoms          !**** Make a temp copy of Fij ****

     A => Atom_List(i)
     AT => AtomType_Info(A%TypeIndex)

     j=AT%nlm_Size
     Allocate(Fij_List(i)%Fij(3, j, j), STAT=k)
     Write(MSg,*) "SymFij: Error Allocating Temp array for atom:",i
     Call Check_Error(k, Msg, Error_Unit, .TRUE., IDTEXT="SymFij:")
  End Do

 Do s=1,spin
  Do i=1, Specific_Atoms  
     A => Atom_List(i)
     If (index.eq.2.and.s==1) Fij_List(i)%Fij = A%Fij
     If (index.eq.2.and.s==2) Fij_List(i)%Fij = A%Fijspin
     If (index.eq.3.and.s==1) Fij_List(i)%Fij = A%Feij
     If (index.eq.3.and.s==2) Fij_List(i)%Fij = A%Feijspin
  End Do
  Do i=1, Specific_Atoms        !*** Symmetrize Fij
     A => Atom_List(i)
     AT => AtomType_Info(A%TypeIndex)
     
     Basis_Size = AT%Basis_Size     
     If (index.eq.2.and.s==1) Fij => A%Fij
     If (index.eq.2.and.s==2) Fij => A%Fijspin
     If (index.eq.3.and.s==1) Fij => A%Feij
     If (index.eq.3.and.s==2) Fij => A%Feijspin

     Do nili=1, Basis_Size
        Li = AT%L_Value(nili)
        ibase = AT%nl_Base(nili)+Li
        Do mi1=-Li, Li
           Do njlj=1, Basis_Size
              Lj = AT%L_Value(njlj)
              jbase = AT%nl_Base(njlj)+Lj
              Do mj1=-Lj, Lj

                 zrotated = 0
                 Do j=1, XTal%Rot_Size
                    zsum = 0
                    Fij_Copy => Fij_List(iatrans(i,j))%Fij

                    Do mi2=-Li, Li
                       Do mj2=-Lj, Lj
                         zsum= zsum + &
                               (zarot(mi2+Li+1,mi1+Li+1,Li+1,j)) &
                               *CONJG(zarot(mj2+Lj+1,mj1+Lj+1,Lj+1,j)) &
                               *Fij_copy(:,ibase+mi2, jbase+mj2)
                       End Do
                    End Do

                    Do k=1,3
                        zrotated = zrotated + XTal%RotMatrix(:,k,j)*zsum(k)
                    Enddo

                 End Do 
                 Fij(:,ibase+mi1, jbase+mj1) = zrotated*c
              End Do
           End Do    !**njlj
        End Do
     End Do    !**nili
  End Do
 End Do    ! spin


  Do i=1, Specific_Atoms        !*** Free Fij Copy
     DeAllocate(Fij_List(i)%Fij)
  End Do

  Return
End Subroutine


!******************************************************************************
!
!  SymForce - Symmetrizes the Forces                    
!
!
!******************************************************************************

Subroutine SymForce       

  Integer :: i,j,k
  Real    :: c
  Complex :: zsum(3),zrotated(3),ztmp(3)
  Complex :: Ftmp(3,Specific_Atoms)          
  Type (Specific_Atom), Pointer :: A

  c = 1.0/XTal%Rot_Size

  Do i=1, Specific_Atoms          !**** Make a temp copy of Forces ***

     A => Atom_List(i)

     Ftmp(1:3,i)=A%Force%CurrForce(1:3)
  End Do

  Do i=1, Specific_Atoms        !*** Symmetrize Forces
     A => Atom_List(i)
     
     zsum = 0
     Do j=1, XTal%Rot_Size
        ztmp = Ftmp(1:3,iatrans(i,j))

        zrotated = MATMUL((XTal%RotMatrix(:,:,j)),ztmp)

        zsum = zsum + zrotated

     End Do 
     A%Force%CurrForce(1:3) = zsum*c
  End Do

  Return
End Subroutine


!******************************************************************************
!
! CalcFij - Calculates  d  SUM_nk  f_nk <Psi_nk|p_i><p_j|Psi_nk>
!                      ---
!                       dR^a 
!   and stores results in Fij  and also calculates Feij
!
!******************************************************************************

Subroutine CalcFij

  Integer :: Prange(2),Base, G_Half, G_Size, atype
  Integer :: i,j, k,Base_DI,  atom, L, m, am, N, RadP, PLM_Index
!  Integer :: Psi_toUse(Mem_MapSize), Flag, PsiBand, PsiK,PsiIn
  Integer :: Flag, PsiBand, PsiK,PsiIn
  Integer ,pointer:: Psi_toUse(:)
  Complex, Pointer :: Ylm(:), Phase(:)
  Complex, Pointer :: Work(:), Glist(:,:), PDOT(:), DPDOT(:,:)
  Real, Pointer :: Proj(:)
  Real :: c1, occ, energy, BlochK(3)
  Complex :: c3, cL,tmp(3), c_p, c_dp, term
  Complex, parameter :: ai = (0,1)
  Complex, Pointer :: Fij(:,:,:), Feij(:,:,:), WF(:)
  Type (Specific_Atom), Pointer :: A
  Type (Mem_handle), Pointer :: Psi

  Allocate(Psi_toUse(Mem_MapSize))

  !write(log_unit,*) 'In CalcFij'
  !call flush(log_unit)

 Do atom=1,Specific_Atoms 
  Atom_List(atom)%Fij=0        ! zero Fij array
  Atom_List(atom)%Feij=0        ! zero Fije array
  if (spindependence) then
     Atom_List(atom)%Fijspin=0        ! zero Fij array
     Atom_List(atom)%Feijspin=0        ! zero Fije array
  endif
 Enddo

  N=Gpnt_Size(G_LOW)-1
  c1 = 4*Pi / sqrt(xtal%Volume)

  Allocate(Glist(3,Gall_Size(G_Low)),DPDOT(3,Plm_Max))
  Call GetBuffer(Work)

  Glist(:,1:Gpnt_Size(G_LOW)) = Gpnt(1:3,1:Gpnt_Size(G_LOW))
  Glist(:,Gpnt_Size(G_LOW)+1:Gall_Size(G_Low)) = -Gpnt(1:3,2:Gpnt_Size(G_LOW))

  Do k=1, NumKpnts
    BlochK = BZ%Ku(:,k)
 
    Psi_toUse = MH_Skip
    Where ((PsiInfo(:)%Kpnt == k) .AND. (PsiInfo(:)%Occupancy >1E-10))
       Psi_toUse = MH_toProcess
    End Where
    
    Call Phase_Generic( Psi_toUse,  k)
    Call GetNextPsi(Flag, Psi)
    
    Do While (Flag > 0)
       
       WF => Psi%Ptr
       
       PsiIn=Psi%Index
       PsiInfo(PsiIn)%DoSave=1
       PsiInfo(PsiIn)%Available=Mem_Used
       occ = PsiInfo(PsiIn)%Occupancy
       energy = PsiInfo(PsiIn)%Energy
       PDOT => PsiInfo(PsiIn)%PDOT
       DPDOT = 0

       PDOT_Dir = -PDOT_Dir
       Call SetStartEnd(Prange)

       Do atom=1, Specific_Atoms
          atype = Atom_List(atom)%typeIndex             

          If (Proj_Mode /= MIN_TIME) then
             Call GetStructFactor( atom, Phase)
             Base_DI = Atomtype_Info(atype)%Basis_Size*(k-1)
                                    !** Calc starting DI for Kpnt
          Else
             Base_DI = RS_PLMMax(atom)*(k-1)
          End If
          
         ! G_Half = Atomtype_Info(atype)%Gpnt_size
         ! G_Size = 2*G_Half - 1

           G_Half = Gpnt_Size(G_LOW)
           G_Size = Gall_Size(G_LOW)
          
          Do j=1, RS_PLMMax(atom)
             RadP = RS_PLM(atom,1,j)
             L = RS_PLM(atom,2,j)
             m = RS_PLM(atom,3,j)
             am = ABS(m)
             
             PLM_Index = RS_PLM(atom,4,j)
             
             If (Run_Mode == Min_MEMORY) then
                Call GetYlm( k, L, am, Ylm)
                Call LRU_GetRec(LRU_RadProj(atype),  Base_DI+radP, Proj)
                
                cL = ai**L
                
!--------
                If (m>=0) then
                   i = 1
                   term = CONJG(Ylm(i))*Proj(i)*Phase(i)*WF(i)
                   Pdot(PLM_Index) = term
                   DPDOT(:,PLM_Index) = Term*Glist(:,i)
                   Do i=2, G_Half
                      term = CONJG(Ylm(i))*Proj(i)*Phase(i)*WF(i)
                      PDOT(PLM_Index) = PDOT(PLM_Index) + term
                      DPDOT(:,PLM_Index) = DPDOT(:,PLM_Index) + Term*Glist(:,i)

                      term = CONJG(Ylm(i+n)) * Proj(i+G_Half-1) * &
                           Phase(i+n) * WF(i+n)

                      PDOT(PLM_INDEX) = PDOT(PLM_Index) + term
                      DPDOT(:,PLM_Index) = DPDOT(:,PLM_Index) + &
                           Term*GList(:,i+n)
                   end Do
                   PDOT(PLM_Index) = c1*cL*PDOT(PLM_Index)
                   DPDOT(:,PLM_Index) = ai*CL*C1*DPDOT(:,Plm_Index)
                Else                                
                   i = 1
                   term = Ylm(i)*Proj(i)*Phase(i)*WF(i)
                   PDOT(PLM_Index) = term
                   DPDOT(:,PLM_Index) = term*GList(:,i)
                   Do i=2, G_Half
                      term = Ylm(i)*Proj(i)*Phase(i)*WF(i)
                      PDOT(PLM_Index) = PDOT(PLM_Index) + term
                      DPDOT(:,PLM_Index) = DPDOT(:,PLM_Index) + term*Glist(:,i)

                      term = Ylm(i+n) * Proj(i+G_Half-1) * &
                           Phase(i+n) * WF(i+n)
                      PDOT(PLM_Index) = PDOT(PLM_Index) + term
                      DPDOT(:,Plm_Index) = DPDOT(:,PLM_Index) + &
                           term*Glist(:,i+n)
                   end Do
                   
                   c3 = (-1)**am
                   PDOT(PLM_Index) = c1*cL*c3*PDOT(PLM_Index)
                   DPDOT(:,PLM_Index) = ai*c1*cL*c3*DPDOT(:,PLM_Index)
                End If
             else
                Call LRU_GetRec(LRU_Proj(atom),  Base_DI+j, Phase)

                i = 1
                term = Phase(i)*WF(i)
                Pdot(PLM_Index) = term
                DPDOT(:,PLM_Index) = Term*Glist(:,i)
                Do i=2, G_Half
                   term = Phase(i)*WF(i)
                   PDOT(PLM_Index) = PDOT(PLM_Index) + term
                   DPDOT(:,PLM_Index) = DPDOT(:,PLM_Index) + Term*Glist(:,i)

                   term = Phase(i+G_Half-1) * WF(i+n)

                   PDOT(PLM_INDEX) = PDOT(PLM_Index) + term
                   DPDOT(:,PLM_Index) = DPDOT(:,PLM_Index) + &
                        Term*GList(:,i+n)
                end Do
                DPDOT(:,PLM_Index) = ai*DPDOT(:,Plm_Index)
             End If
          End Do  !** j
       End Do

      If(spindependence.and.(.not.PsiInfo(PsiIn)%spinup)) then
       Do atom=1,Specific_Atoms  ! Accumulate Fij
          Fij => Atom_List(atom)%Fijspin
          Feij => Atom_List(atom)%Feijspin
          Base = PLM_AtomRange(1,atom)-1
            
          !write(log_unit,*) 'CalcFij spindown', atom,Base
          !call flush(log_unit)
          Do i=PLM_AtomRange(1,atom),PLM_AtomRange(2,atom)
             Do j=PLM_AtomRange(1,atom),PLM_AtomRange(2,atom)
                tmp = occ*(CONJG(PDOT(i))*DPDOT(:,j) + &
                     CONJG(DPDOT(:,i))*PDOT(j))
                Fij(:,i-Base,j-Base) = Fij(:,i-Base,j-Base) + tmp
                Feij(:,i-Base,j-Base) = Feij(:,i-Base,j-Base) + tmp*energy
             EndDo
          Enddo
       Enddo
      else
       Do atom=1,Specific_Atoms  ! Accumulate Fij
          Fij => Atom_List(atom)%Fij
          Feij => Atom_List(atom)%Feij
          Base = PLM_AtomRange(1,atom)-1
            
          !write(log_unit,*) 'CalcFij spinup', atom,Base
          !call flush(log_unit)
          Do i=PLM_AtomRange(1,atom),PLM_AtomRange(2,atom)
             Do j=PLM_AtomRange(1,atom),PLM_AtomRange(2,atom)
                tmp = occ*(CONJG(PDOT(i))*DPDOT(:,j) + &
                     CONJG(DPDOT(:,i))*PDOT(j))
                Fij(:,i-Base,j-Base) = Fij(:,i-Base,j-Base) + tmp
                Feij(:,i-Base,j-Base) = Feij(:,i-Base,j-Base) + tmp*energy
             EndDo
          Enddo
       Enddo
      Endif
     
       Call GetNextPsi(Flag, Psi)
    End do
 End do

 Call FreeBuffer(Work)
 Deallocate(Glist,DPDOT)
 DeAllocate(Psi_toUse)

 Return
End Subroutine


End Module
