!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  Program genkpoints
!   5/16/00  translation of pawbz pgm into Alan Tackett's form
!     program pawbz
!  -- to construct grid for Brillouin Zone integral
!    
! Calling sequence:  genkpoints [input paw file name]
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
Program Genkpoints

  Use Crystal_data
  Use Crystal_symmetry
  Use mathlib
  Use options_data
  Use paw_inout
  Use Word
  Use Strings
  Use Units

  implicit none

  Character*100 :: Fn, token
  Logical    :: ex
  LOGICAL    :: findA=.false.,findB=.false.,findC=.false.
  Integer       :: err, i , j, tlen,nrot
  integer       :: ifsym,ii,jj,kk,iftmp,ifcry,iargc
  integer, parameter :: input_unit=5

  if (iargc() < 1) then
     Write(*,*) 'Format: pwpaw input_file'
     STOP
  End if
  
  Call GetArg(1,Fn)
  write(*,*) 'Filename=',Fn     ! A typical pwpaw input file
  Token = Fn
  Call UpperCase(Token)
  Inquire(file=Trim(fn),exist=ex)
  if (.not.ex) then
    Write(*,*) "Can't access file ",Trim(fn)
    Write(*,*) "Make sure the file exists and has Read permissions."
    stop
  endif
  

  Allocate(xtal%trans(3,48),xtal%rotmatrix(3,3,48))
  Call initconstants

  Write(*,*) 'Scratch_FD=',Scratch_Unit

  Call DetWordConstants(SCRATCH_UNIT)

  Call InitWord(PAW_WC, INPUT_BASE_UNIT, trim(Fn), DELIMS, COMMENT, LIT_CHAR, &
       MAX_INCLUDE, INCLUDE_STR)
  
  Call GetNextWord(PAW_WC,Token,tlen)
  nrot=0; xtal%Rot_Size=100

    Do                                                    
     If (W_Error /= W_EOF ) then                        
        Call UpperCase(Token)                           
                                                        
        If (Trim(Token) == "A") then                    
          Call GetNumbers(PAW_WC,xtal%Basis(1:3,1),3)   
          findA=.true.                                  
          write(*,*) 'A =',xtal%Basis(1:3,1)            
        Else If (Trim(Token) == "B") then               
          Call GetNumbers(PAW_WC,xtal%Basis(1:3,2),3)   
          findB=.true.                                  
          write(*,*) 'B =',xtal%Basis(1:3,2)            
        Else If (Trim(Token) == "C") then               
          Call GetNumbers(PAW_WC,xtal%Basis(1:3,3),3)   
          findC=.true.                                  
          write(*,*) 'C =',xtal%Basis(1:3,3)            
        Else If (Trim(Token) == "ROT_SIZE") then                            
          Call GetNumber(PAW_WC,xtal%Rot_Size)                              
          write(*,*) 'Rot_Size =',xtal%Rot_Size                             
        Else if (Trim(token) == "MATRIX") then                              
          Call GetNumber(PAW_WC, j)                                         
          Call GetNumbers(PAW_WC, xtal%RotMatrix(1,:, j), 3)                
          Call GetNumbers(PAW_WC, xtal%RotMatrix(2,:, j), 3)                
          Call GetNumbers(PAW_WC, xtal%RotMatrix(3,:, j), 3)                
          Call GetNextWord(PAW_WC, Token, tlen)  !** Get the end statement  
          write(*,*) 'Loaded matrix #',j                                    
        Else if (Trim(token) == "TRANSLATION") then                         
          Call GetNumber(PAW_WC, j)                                         
          Call GetNumbers(PAW_WC, xtal%Trans(:,j), 3)                       
          nrot=nrot+1
        EndIf                                           
     Else                                               
        Exit                                            
     EndIf                                              
        If (findA .AND. findB .AND. findC .AND. nrot==xtal%Rot_Size) Exit         
                                                        
    Call GetNextWord(PAW_WC,Token,tlen)                 
   Enddo                                                  
                                                        
   If (.not.findA .OR. .not.findB .OR. .not.findC &
      .OR. nrot /= xtal%Rot_Size) then 
      write(*,*) 'Error -- Basis functions and/or rotations not found'   
      stop                                              
   Endif                                                
                                                        
    xtal%Recip = RecipBasis(xtal%Basis)

    If (Xtal%Rot_Size.le.0) then
       xtal%Rot_Size=1
       xtal%RotMatrix=0
       xtal%Trans=0
       xtal%RotMatrix(1,1,1)=1
       xtal%RotMatrix(2,2,1)=1
       xtal%RotMatrix(3,3,1)=1
    Endif

    Call setsym(0)

      do

      write(*,*) 'input Agrid, Bgrid, Cgrid'
      read(*,*) ii,jj,kk
      call kgrid(ii,jj,kk,input_unit,SCRATCH_UNIT)
      enddo

      stop
      end

      subroutine kgrid(ii,jj,kk,input_unit,iftmp)
!   subroutine to generate k-space grid for Gaussian smoothing 
!      method of dos calculation
!  uses point group rotations and first positive translations

      use crystal_symmetry
      use crystal_data
      use paw_inout 

      implicit none

      real*8 :: v(3),w(3),u(3)
      real*8, allocatable :: rk(:,:),wgt(:)
      real*8, parameter :: eps=1.d-6
      real*8 :: xkk,xjj,xii,dfloat,ww,dabs,wate,xx1,xx2,xx3
      integer :: nkpts,nkpts0,icount,m1,m2,m3,it,n1,n2,n3,ier,iftmp
      integer :: ii,jj,kk,i,j,k,inc,np,nq,mt,nx,l,krep,iq
      integer :: input_unit
      character *100 :: answer, filename

      nkpts=ii*jj*kk

      if (nkpts.lt.1) then
        write(*,*) ' kpoints < 1 -- program exiting '
        stop
      endif
      

      allocate(rk(3,nkpts),wgt(nkpts),stat=ier)
       if (ier.ne.0) then
         write(*,*) 'error in allocating rk',nkpts,ier
         stop
         endif

      

      nkpts0=nkpts
      xkk=1.d0/kk
      xjj=1.d0/jj
      xii=1.d0/ii
      icount=0

      do k=1,kk
      do j=1,jj
      do i=1,ii
      icount=icount+1
      wgt(icount)=1.d0
      rk(1,icount)=(dfloat(i)-0.5d0)*xii
      rk(2,icount)=(dfloat(j)-0.5d0)*xjj
      rk(3,icount)=(dfloat(k)-0.5d0)*xkk
      enddo
      enddo
      enddo

      inc=3
!    generate all equivalent points
      np=1

       mainloop: do

       np=np+1


       if (np.gt.nkpts) exit mainloop

       innerloop: do

       nq=np-1
       do m1=1,3
       do m2=1,3
       do m3=1,3

      do it=1,3
      v(it)=rk(it,np)
      enddo

      v(1)=v(1)+(m1-2)
      v(2)=v(2)+(m2-2)
      v(3)=v(3)+(m3-2)

      do mt=1,Xtal%Rot_Size
       do it=1,3
       w(it)=igrot(it,1,mt)*v(1)+igrot(it,2,mt)*v(2)+igrot(it,3,mt)*v(3)
       enddo


        do  iq=1,nq
         xx1=dmod(dabs(w(1)-rk(1,iq)),1.d0)
         xx2=dmod(dabs(w(2)-rk(2,iq)),1.d0)
         xx3=dmod(dabs(w(3)-rk(3,iq)),1.d0)


         if(xx1+xx2+xx3.le.eps) then    !  w already in list
          wgt(iq)=wgt(iq)+1.d0
!  np not needed
      nx=np+1
      if (nx.le.nkpts)  then

      do  l=nx,nkpts

      do  it=1,3
      rk(it,l-1)=rk(it,l)
      enddo

      enddo
      endif

      nkpts=nkpts-1
      np=np-1

        exit innerloop
         endif

        enddo  !iq

       enddo   !mt

      enddo    !m3
      enddo    !m2
      enddo    !m1

       exit innerloop
      enddo innerloop

      enddo mainloop

!  completed list of k kvectors
      write(*,  &
          "('kgrid -- ',i3,' x',i3,' x',i3,i5,' pts reduced to',i5)")  &
                ii,jj,kk,nkpts0,nkpts

      write(*,*) 'Input two character strings --'
      write(*,*) 'save filename OR try again OR quit now'
      read(*,*) answer,filename

      Call Uppercase(answer)
      If (trim(answer)=="SAVE") then

      Open(unit=SCRATCH_UNIT,file=trim(filename),form='formatted')

      write(SCRATCH_UNIT,'("K-POINTS_LIST",i10)') nkpts
      do  it=1,nkpts
      do  l=1,3
      if (rk(l,it).gt.0.5d0) rk(l,it)=rk(l,it)-1.d0
      enddo
       wate=wgt(it)
      write (SCRATCH_UNIT,'(3f20.16,2x,f10.5)')(rk(j,it),j=1,3),wate
      enddo
      write(SCRATCH_UNIT,'("END")')

      close(SCRATCH_UNIT)
      stop
      else if (trim(answer)=="QUIT") then
      stop
      endif

      return
       end
