!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Program genprepareballandstick
!
!   Stand-alone program to determine atomic positions and bonds
!    Calculate atomic positions and bonds for Ball and Stick model
!         (Data Explorer output and XCrysden output)
!
!   This version is based on Atomplot3d but takes interactive input
!      input and output is bohr units for dx, anstroms for XCrysden
!   This version is slightly different from old prepareballandstickplus
!      in that it inputs original unit cell and plotting cell is given
!      as X= xafrac*a + xafrac*b + xafrac*c
!      as Y= yafrac*a + yafrac*b + yafrac*c
!      as Z= zafrac*a + zafrac*b + zafrac*c
!   This version allows for monoclinic case 
!              (differs from newprepareballandstick)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
Program genprepareballandstick
Use mathlib
Implicit None

 Real(8), allocatable :: ballpoints(:,:),nballpoints(:,:)
 Real(8) :: resx, resy, resz ,plotmat(3,3),solvmat(3,3),v(3),vin(3),vout(3)
 Real(8) :: chk,rtol,tx,ty,tz,XA,YA,ZA,ua,ub,uc,ttx,tty,ttz,ssx,ssy,ssz
 REAL(8) :: alen,blen,clen,alpha,beta,gamm,units
 Real(8), parameter :: atol=2.e-2
 Integer, allocatable :: noballs(:),nnoballs(:)
 Integer :: ncount,ncountmax,many,ierr,npair,i,j,k,n,specific_atoms,types
 Integer :: na,nb,nc,idum(500)
 Character(128) :: atompositionfile,PLOT_NAME, unittype
 Character(2) :: sym
 logical :: angstromunits

 Real(8) :: X(3),Y(3),Z(3),invmat(3,3),&
          volume,maxbond,bondtol,basis(3,3),tmp(3,3),origin(3)
 Real(8), allocatable :: atompos(:,:)
 Integer, allocatable :: typeindex(:),inside(:),inlist(:),typenumber(:)
 Integer, allocatable :: pair1(:),pair2(:),it(:),inner(:)
 Character(2), allocatable :: typemap(:),asym(:),nasym(:)
 integer, parameter :: CHARGEOUT_UNIT = 15

 call initconstants
 angstromunits=.false.; units=1.d0
 if (iargc() > 0) then
    Call GetArg(1,unittype)
    if (TRIM(unittype(1:1))=='a'.or.TRIM(unittype(1:1))=='A') &
            angstromunits=.true.
            units=0.52917720859d0
 endif
 Read(5,*) ((basis(j,i),j=1,3),i=1,3)     ! crystal basis functions in bohr
 write(6,*) 'basis', basis
 Read(5,*) ((solvmat(j,i),j=1,3),i=1,3)
 Plotmat=MatMul(basis,solvmat)
 write(6,*) 'X in original coord', Plotmat(1,1),Plotmat(2,1),Plotmat(3,1)
 write(6,*) 'Y in original coord', Plotmat(1,2),Plotmat(2,2),Plotmat(3,2)
 write(6,*) 'Z in original coord', Plotmat(1,3),Plotmat(2,3),Plotmat(3,3)
 X=0;Y=0;Z=0
 alen=sqrt(DOT_PRODUCT(Plotmat(:,1),Plotmat(:,1)))
 blen=sqrt(DOT_PRODUCT(Plotmat(:,2),Plotmat(:,2)))
 clen=sqrt(DOT_PRODUCT(Plotmat(:,3),Plotmat(:,3)))
 alpha=ACOS(DOT_PRODUCT(Plotmat(:,2),Plotmat(:,3))/&
             (sqrt(DOT_PRODUCT(Plotmat(:,2),Plotmat(:,2))*&
             DOT_PRODUCT(Plotmat(:,3),Plotmat(:,3)))))
 beta=ACOS(DOT_PRODUCT(Plotmat(:,1),Plotmat(:,3))/&
             (sqrt(DOT_PRODUCT(Plotmat(:,1),Plotmat(:,1))*&
             DOT_PRODUCT(Plotmat(:,3),Plotmat(:,3)))))
 gamm=ACOS(DOT_PRODUCT(Plotmat(:,2),Plotmat(:,1))/&
             (sqrt(DOT_PRODUCT(Plotmat(:,2),Plotmat(:,2))*&
             DOT_PRODUCT(Plotmat(:,1),Plotmat(:,1)))))
 write (6,*) 'alpha ', alpha*180/acos(-1.d0)
 write (6,*) 'beta ', beta*180/acos(-1.d0)
 write (6,*) 'gamm ', gamm*180/acos(-1.d0)
 X(1)=alen ; write(6,*) X(1)
 resy=blen ; write(6,*) resy
 Y(1)=resy*cos(gamm);   Y(2)=resy*sin(gamm)
 write(6,*) clen
 Z(1)=clen*cos(beta)
 Z(2)=clen*(cos(alpha)-cos(beta)*cos(gamm))/sin(gamm)
 Z(3)=clen*sqrt(1.d0+ 2*cos(alpha)*cos(beta)*cos(gamm)&
                - cos(alpha)**2-cos(beta)**2-cos(gamm)**2 )/sin(gamm) 

 write(6,*) 'X ', X
 write(6,*) 'Y ', Y
 write(6,*) 'Z ', Z
! X=0.52917720859d0*X; Y=0.52917720859d0*Y;  Z=0.52917720859d0*Z
! write(6,*) 'X angstroms', X
! write(6,*) 'Y angstroms', Y
! write(6,*) 'Z angstroms', Z
 call Calc3x3InverseN(solvmat,invmat,i)
 write(6,*) 'called  Calc3x3InverseN with error ', i
 write(6,*) 'a col in inverse', invmat(1,1),invmat(2,1),invmat(3,1)
 write(6,*) 'b col in inverse', invmat(1,2),invmat(2,2),invmat(3,2)
 write(6,*) 'c col in inverse', invmat(1,3),invmat(2,3),invmat(3,3)
 tmp=MatMul(solvmat,invmat)
 write(6,*) 'check1', ((tmp(j,i),j=1,3),i=1,3)
 tmp=MatMul(invmat,solvmat)
 write(6,*) 'check1', ((tmp(j,i),j=1,3),i=1,3)
 Read(5,*) origin       !  origin of plot in fractional coordinates (basis)
 write(6,*) 'origin', origin
 Read(5,*) maxbond,bondtol     ! in bohr
 write(6,*) 'maxbond,bondtol',maxbond,bondtol
 Read(5,*) types
 write(6,*) 'types',types

 volume = ABS(basis(1,1)*(basis(2,2)*basis(3,3)-basis(3,2)*basis(2,3)) +&
              basis(2,1)*(basis(3,2)*basis(1,3)-basis(1,2)*basis(3,3)) +&
              basis(3,1)*(basis(1,2)*basis(2,3)-basis(2,2)*basis(1,3)))

 Allocate(typemap(types),inlist(types),typenumber(types))
 Read(5,*) typemap,typenumber,inlist
 write(6,*) 'typemap', typemap
 write(6,*) 'typenumber', typenumber
 write(6,*) 'inlist', inlist
 

 read(5,*) atompositionfile, PLOT_NAME
 write(6,*) 'files',atompositionfile, PLOT_NAME

 open(7,file=TRIM(atompositionfile),form='formatted')
 specific_atoms=0
    do
        read(7,*,iostat=i) sym,XA,YA,ZA
        write(6,*) sym,XA,YA,ZA
        if (i/=0) exit
        specific_atoms=specific_atoms+1
        write(6,*) sym,XA,YA,ZA
    enddo

 write(6,*) 'specific_atoms = ', specific_atoms

 allocate(atompos(3,specific_atoms),typeindex(specific_atoms),&
              inside(specific_atoms))

 rewind(7)
    do i=1,specific_atoms
       read(7,*) sym,atompos(1,i),atompos(2,i),atompos(3,i)
       typeindex(i)=0
       do j=1,types
          if (TRIM(typemap(j))==TRIM(sym)) typeindex(i)=j
       enddo
       if (typeindex(i)<1) then
          write(6,*) 'Type not found ', &
              sym,atompos(1,i),atompos(2,i),atompos(3,i)
          exit
       endif
       write(6,*) i,' sym ', typeindex(i)
       inside(i)=inlist(typeindex(i))
     enddo

  rtol=bondtol  !Tolerance of including atoms

  rtol=MAX(atol,rtol)

! Determine length of plotting cell boundaries
    resx=SQRT(DOT_Product(X,X))
    resy=SQRT(DOT_Product(Y,Y))
    resz=SQRT(DOT_Product(Z,Z))
    XA=resx
    YA=resy
    ZA=resz
    ua=sqrt(Dot_Product(Basis(:,1),Basis(:,1)))
    ub=sqrt(Dot_Product(Basis(:,2),Basis(:,2)))
    uc=sqrt(Dot_Product(Basis(:,3),Basis(:,3)))

    n=8

  Open(unit=CHARGEOUT_UNIT,&
   file=Trim(PLOT_Name)//".axes.dx",form="formatted")

   idum=1
   write(CHARGEOUT_UNIT, &
     '("object 1 class array type float rank 0 items ",i4," data follows")') n
   write(CHARGEOUT_UNIT,'(80i4)') (idum(i),i=1,n)
   write(CHARGEOUT_UNIT,'("attribute ""dep"" string ""positions""")')
   write(CHARGEOUT_UNIT,'("#")')
   write(CHARGEOUT_UNIT,&
    '("object 2 class array type float rank 1 shape 3 items ",i4," data follows")') n

   v=0
            write(CHARGEOUT_UNIT,'(1p3e15.7)') v(1:3)*units
            write(CHARGEOUT_UNIT,'(1p3e15.7)') X(1:3)*units
            write(CHARGEOUT_UNIT,'(1p3e15.7)') Y(1:3)*units
            write(CHARGEOUT_UNIT,'(1p3e15.7)') (X(1:3)+Y(1:3))*units
            write(CHARGEOUT_UNIT,'(1p3e15.7)') Z(1:3)*units
            write(CHARGEOUT_UNIT,'(1p3e15.7)') (X(1:3)+Z(1:3))*units
            write(CHARGEOUT_UNIT,'(1p3e15.7)') (Y(1:3)+Z(1:3))*units
            write(CHARGEOUT_UNIT,'(1p3e15.7)') (X(1:3)+Y(1:3)+Z(1:3))*units

   write(CHARGEOUT_UNIT,'("attribute ""dep"" string ""positions""")')
   write(CHARGEOUT_UNIT,'("#")')

   write(CHARGEOUT_UNIT, &
    '("object 3 class array type int rank 1 shape 2 items 12 data follows")')
   write(CHARGEOUT_UNIT, '("  0  1")')
   write(CHARGEOUT_UNIT, '("  0  2")')
   write(CHARGEOUT_UNIT, '("  1  3")')
   write(CHARGEOUT_UNIT, '("  2  3")')
   write(CHARGEOUT_UNIT, '("  4  5")')
   write(CHARGEOUT_UNIT, '("  4  6")')
   write(CHARGEOUT_UNIT, '("  5  7")')
   write(CHARGEOUT_UNIT, '("  6  7")')
   write(CHARGEOUT_UNIT, '("  0  4")')
   write(CHARGEOUT_UNIT, '("  1  5")')
   write(CHARGEOUT_UNIT, '("  2  6")')
   write(CHARGEOUT_UNIT, '("  3  7")')


   write(CHARGEOUT_UNIT,'("attribute ""element type"" string ""lines""")')
   write(CHARGEOUT_UNIT,'("attribute ""ref"" string ""positions""")')
   write(CHARGEOUT_UNIT,'("#")')
   write(CHARGEOUT_UNIT,'("object ""molecule"" class field")')
   write(CHARGEOUT_UNIT,'("component ""data"" value 1")')
   write(CHARGEOUT_UNIT,'("component ""positions"" value 2")')
   write(CHARGEOUT_UNIT,'("component ""connections"" value 3")')
   write(CHARGEOUT_UNIT,'("attribute ""name"" string ""molecule""")')
   write(CHARGEOUT_UNIT,'("#")')
   write(CHARGEOUT_UNIT,'("end")')

   close (CHARGEOUT_UNIT)

! file for atomic positions
  Open(unit=CHARGEOUT_UNIT,&
    file=Trim(PLOT_Name)//".atom.dx",form="formatted")


! Rough estimate of number atoms in plot

    many=10
    
    ncountmax=((2*many+1)**2)*Specific_Atoms 
    plotmat(:,1)=X; plotmat(:,2)=Y; plotmat(:,3)=Z

    write(6,*)   ' many = ', many

  Allocate(noballs(ncountmax),ballpoints(3,ncountmax),asym(ncountmax),&
          nnoballs(ncountmax),nballpoints(3,ncountmax),nasym(ncountmax),&
          pair1(ncountmax),pair2(ncountmax),it(ncountmax),inner(ncountmax))

  noballs=0;nnoballs=0
  ballpoints=0;nballpoints=0


   ncount=0

   do n=1,Specific_Atoms      ! Some assumptions here about shape of cell

     if (inside(n)==0) then
        ttx=atol*tx; tty=atol*ty; ttz=atol*tz
     else 
        ttx=tx;  tty=ty;   ttz=tz
     endif
        ssx=atol*tx; ssy=atol*ty; ssz=atol*tz

     do i = -many, many
        do j = -many, many
           do k = -many, many

           v = atompos(:,n) - origin + (/i,j,k/) 

           vin= MATMUL(invmat,v)
           vout=MATMUL(plotmat,vin)

!  Check if atom is in plotting cell
           if (vin(1).gt.-atol.and.vin(1).lt.1+atol &
               .and.vin(2).gt.-atol.and.vin(2).lt.1+atol &
               .and.vin(3).gt.-atol.and.vin(3).lt.1+atol) then
               
              ncount=ncount+1 
                  write(6,'("atom", i5, 1p3e15.7)') , ncount, vin
             ! check of atom is inside actual cell with atol tolerance
              if (vin(1).gt.-atol.and.vin(1).lt.1+atol &
               .and.vin(2).gt.-atol.and.vin(2).lt.1+atol &
               .and.vin(3).gt.-atol.and.vin(3).lt.1+atol) then
               inner(ncount)=0
              else
               inner(ncount)=1
              endif

              noballs(ncount)=typenumber(typeindex(n))
              asym(ncount)=typemap(typeindex(n))
              ballpoints(:,ncount)=vout
              it(ncount)=inside(n)

             write(6,'(3i5,a5,1p3e15.7)') ncount,inside(n),inner(ncount),&
                     asym(ncount),vout
           Endif

           Enddo
         Enddo     
       Enddo
    Enddo

   write(6,*) 'ncount = ', ncount

         chk=(maxbond)**2
         npair=0  
          do i=1,ncount-1
           if (inner(i)==0) then
           do j=i+1,ncount
            v=ballpoints(:,i)-ballpoints(:,j)
            if(DOT_Product(v,v).le.chk) then
             npair=npair+1
             pair1(npair)=i;pair2(npair)=j
            endif
           enddo
           endif
          enddo

    write(6,*) 'pairs = ', npair

        !  Eliminate unbounded atoms with inside=1

        i=0
          do j=1,ncount
             if(it(j)>0) then
               do k=1,npair
                  if (pair1(k)==j.or.pair2(k)==j) then
                     i=i+1
                     nballpoints(:,i)=ballpoints(:,j)
                     nasym(i)=asym(j)
                     nnoballs(i)=noballs(j)
                     exit
                  endif
               enddo
              else if (it(j)==0) then
                i=i+1
                nballpoints(:,i)=ballpoints(:,j)
                nasym(i)=asym(j)
                nnoballs(i)=noballs(j)
              endif
          enddo

     ncount=i
     write(6,*) 'new number ', ncount

!  output atomic coordinates for Data Explorer
 
    write(CHARGEOUT_UNIT, &
     '("object 1 class array type float rank 0 items ",i5," data follows")') &
      ncount
    write(CHARGEOUT_UNIT,*) (nnoballs(i),i=1,ncount)   

    write(CHARGEOUT_UNIT,'("attribute ""dep"" string ""positions""")')
    write(CHARGEOUT_UNIT,'("#")')
    write(CHARGEOUT_UNIT,&
     '("object 2 class array type float rank 1 shape 3 items ",i5," data follows")') &
      ncount

         do i=1,ncount
          write(CHARGEOUT_UNIT,*) nballpoints(1:3,i)*units
          enddo

        write(CHARGEOUT_UNIT,'("attribute ""dep"" string ""positions""")')
        write(CHARGEOUT_UNIT,'("#")')


         chk=(maxbond)**2
         npair=0  
          do i=1,ncount-1
           do j=i+1,ncount
            v=nballpoints(:,i)-nballpoints(:,j)
            if(DOT_Product(v,v).le.chk) then
            npair=npair+1
            ! write(CHARGEOUT_UNIT,*) i,j
            endif
           enddo
          enddo

   write(CHARGEOUT_UNIT, &
   '("object 3 class array type int rank 1 shape 2 items ",i5," data follows")')&
   npair 
          do i=1,ncount-1
           do j=i+1,ncount
            v=nballpoints(:,i)-nballpoints(:,j)
            if(DOT_Product(v,v).le.chk) then
             write(CHARGEOUT_UNIT,*) i-1,j-1
            endif
           enddo
          enddo

         write(CHARGEOUT_UNIT,'("attribute ""element type"" string ""lines""")')
         write(CHARGEOUT_UNIT,'("attribute ""ref"" string ""positions""")')
         write(CHARGEOUT_UNIT,'("#")')
         write(CHARGEOUT_UNIT,'("object ""molecule"" class field")')
         write(CHARGEOUT_UNIT,'("component ""data"" value 1")')
         write(CHARGEOUT_UNIT,'("component ""positions"" value 2")')
         write(CHARGEOUT_UNIT,'("component ""connections"" value 3")')
         write(CHARGEOUT_UNIT,'("attribute ""name"" string ""molecule""")')
         write(CHARGEOUT_UNIT,'("#")')
         write(CHARGEOUT_UNIT,'("end")')

      Close(CHARGEOUT_UNIT)



!  XCrysDen output
  Open(unit=CHARGEOUT_UNIT,&
    file=Trim(PLOT_Name)//".xsf",form="formatted")
  Write(CHARGEOUT_UNIT,'("CRYSTAL")')
  Write(CHARGEOUT_UNIT,'("PRIMVEC   1")')
  write(CHARGEOUT_UNIT,'(1p3e15.7)') X(1:3)*0.52917720859
  write(CHARGEOUT_UNIT,'(1p3e15.7)') Y(1:3)*0.52917720859
  write(CHARGEOUT_UNIT,'(1p3e15.7)') Z(1:3)*0.52917720859
  Write(CHARGEOUT_UNIT,'("PRIMCOORD   1")')
  Write(CHARGEOUT_UNIT,'(i10  "   1")')  ncount

     do i=1,ncount
        
        write(CHARGEOUT_UNIT,'(a2,2x,1p3e15.7)') nasym(i),  &
                      nballpoints(1:3,i)*0.52917720859
     enddo



End program

