!******************************************************************************
!
! File : gradEnk.f90
!   by : N. A. W. Holzwarth and Yonas Abraham
!   on : 7/24/00
!  for : PAW program
!
! Module for calculating k gradient for Enk       
!
! modified for spin by Ping Tang and N. A. W. Holzwarth
!    Last change 5/23/05
!******************************************************************************

Module GradEnk

Use paw_inout
Use atom_data
Use crystal_data
Use spherical_harmonic
Use gpoints
Use projectors
Use options_data
Use mathlib

Implicit NONE!!!!!!!!!!!!!!

Type Temp_Grad
     Complex,  pointer :: Gij(:,:,:)
End Type

Type Temp_GP
     Real,  pointer    :: Gpnl(:,:)
     Real,  pointer    :: pnl(:,:)
End Type

Type (Temp_Grad) ,private ,pointer :: Temp_atom(:)
Type (Temp_GP) ,private ,pointer :: Temp_type(:)
Complex,  pointer ,private :: Ylm(:,:),Gylm(:,:,:)
 Complex, pointer, private :: GRADPDOT(:,:),WF(:),WW(:)
 Complex, pointer , private :: W1(:),W2(:),W3(:)
 Real, pointer ,private :: Glist(:,:)

Real, private :: YlmCon
Integer, private :: MaxMaxL

!*****************************************************************************
Contains
!******************************************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! InitGradEnk
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
Subroutine InitGradEnk

 Type (Specific_Atom), Pointer :: A
 Type (Atom_Info_Fixed), Pointer :: AT
 Integer :: i,j,k,many,ier,n,L,atype,lut,atom,m,am,Base

 YlmCon=4*Pi/SQRT(xtal%Volume)

 Allocate (Temp_atom(Specific_atoms),Temp_Type(Atom_Types),stat=ier)
       If (ier /= 0) then
          write(Error_Unit,*) 'Error allocating Temp in InitGradEnk'
          stop
       EndIf
 Do i=1,Specific_Atoms
    A=> Atom_List(i)
    AT=> AtomType_Info(A%TypeIndex)
    many=AT%nlm_Size
    Allocate(Temp_Atom(i)%Gij(3,many,many),stat=ier)
       If (ier /= 0) then
          write(Error_Unit,*) 'Errror allocating Gij in InitGradEnk',i,many
          stop
       EndIf
 Enddo

 MaxMaxL=0
 Do i=1,Atom_Types
    many=AtomType_Info(i)%Basis_Size
    MaxMaxL=Max(MaxMaxL,MaxVal(AtomType_Info(i)%L_Value(1:many)))
    Allocate(Temp_Type(i)%Gpnl(Gall_Size(G_LOW),many), &        
             Temp_Type(i)%pnl(Gall_Size(G_LOW),many),stat=ier)
       If (ier /= 0) then
          write(Error_Unit,*) 'Error allocating Gpnl in InitGradEnk',i,many
          stop
       EndIf
 Enddo

 Write(Log_Unit,*) 'InitGradEnk: MaxMaxL = ',MaxMaxL
 Call Flush(Log_Unit)
 If (MaxMaxL.gt.6) then
    Write(Error_Unit,*) 'InitGradEnk: MaxMaxL > 6 ', MaxMaxL
    stop
 EndIf
 Allocate(GRADPDOT(Plm_Max,3),Glist(Gall_Size(G_LOW),4),stat=ier)
   If (ier /= 0) then
      write(Error_Unit,*) 'Error allocating GRADPDOT in CalcGradEnk',Plm_Max
      stop
   EndIf

 Write(Log_Unit,*) 'CalcEnk: MaxMaxl = ', MaxMaxL

 many=((MaxMaxL+2)*(MaxMaxL+1))/2
 Allocate(Ylm(Gall_Size(G_LOW),many),GYlm(Gall_Size(G_Low),3,many),stat=ier)
       If (ier /= 0) then
          write(Error_Unit,*) 'Errror allocating Ylm in CalcGradEnk',many
          stop
       EndIf

!! Finished allocation

Return
End Subroutine 

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Subroutine FinishGradEnk
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

 Subroutine FinishGradEnk
 Integer :: i
 Do i=1,Specific_Atoms
    Deallocate(Temp_Atom(i)%Gij)
 Enddo

 Deallocate(GRADPDOT,Glist,Ylm,GYlm)

 Return
 End Subroutine

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! NewKforGradEnk
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
Subroutine NewKforGradEnk(Kval)
 Real, intent(IN) :: Kval(3)

 Real , parameter :: tol=1.e-11
 Type (Specific_Atom), Pointer :: A
 Type (Atom_Info_Fixed), Pointer :: AT
 Real          :: tmp,tmp1,Gmag
 Complex  :: Ylmtmp(13,7),GYlmtmp(3,13,7)
 Integer :: i,j,k,many,ier,n,L,atype,lut,atom,m


    do i=1,Gpnt_Size(G_LOW)
       Glist(i,1:3)=Kval + Gpnt(1:3,i)
    Enddo

    do i=Gpnt_Size(G_LOW)+1,Gall_Size(G_LOW)
       Glist(i,1:3)=Kval - Gpnt(1:3,i+1-Gpnt_Size(G_LOW))
    Enddo

    Do i=1,Gall_Size(G_LOW)
       Glist(i,4)=SQRT(DOT_PRODUCT(Glist(i,1:3),Glist(i,1:3)))
       If (Glist(i,4) <tol) then
          Glist(i,1:4)=0
       Else
          Glist(i,1:3)=Glist(i,1:3)/Glist(i,4)
       EndIf
    Enddo

    Do i=1,Atom_Types
    AT => AtomType_Info(i)

!! Calculate and store Fourier Transforms of radial term

       Do n=1,AT%Basis_Size
          L=AT%L_Value(n)
          Do j=1,Gall_Size(G_LOW)
                  tmp = RadFourierDeriv(L, Glist(j,4), AT%Mesh_Step, &
                      AT%Mesh_Size, AT%TP(:,n), AT%RadGrid)
                  tmp1 = RadialFourier(L, Glist(j,4), AT%Mesh_Step, &
                      AT%Mesh_Size, AT%TP(:,n), AT%RadGrid)
             Temp_type(i)%Gpnl(j,n)= tmp
             Temp_type(i)%pnl(j,n)= tmp1
          Enddo
       Enddo
   Enddo         

!! Calculate and store Fourier transform of spherical harmonics

 Do j=1,Gall_Size(G_LOW)
    Call Ylmlist(Glist(j,1),Glist(j,2),Glist(j,3),MaxMaxL,Ylmtmp,Error_Unit)
    Call GradYlmlist(Glist(j,1),Glist(j,2),Glist(j,3),MaxMaxL,GYlmtmp)
    many=0
    Do L=0,MaxMaxL
       do m=0,L
          many=many+1
          Ylm(j,many)=YlmTmp(L+1+m,L+1)
          GYlm(j,:,many)=GYlmTmp(:,L+1+m,L+1)
       Enddo
    Enddo
 Enddo

Return
End Subroutine
    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! CalcGradEnk
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
Subroutine CalcGradEnk(Enk, WF, PDOT, GradEnk)
 Real, intent(IN) :: Enk
 Complex, intent(IN) :: WF(:)
 Complex, intent(INOUT) :: PDOT(:)
 Real, intent(OUT) :: GradEnk(3)

 Real , parameter :: tol=1.e-11
 Complex , parameter :: ai=(0,1)
 Type (Specific_Atom), Pointer :: A
 Type (Atom_Info_Fixed), Pointer :: AT
 Complex, pointer :: Phase(:),Dij(:,:)
 Real          :: tmp,tmp1,Gmag
 Complex  :: il,ctmp(3)
Complex :: tempdot(PLM_Max)

 Integer :: i,j,k,many,ier,n,L,atype,lut,atom,m,am,Base
 Integer :: Index,PlmIndex

 tempdot=PDOT

 Call GetBuffer(WW)
 Call GetBuffer(W1)
 Call GetBuffer(W2)
 Call GetBuffer(W3)

     PDOT=0
     GRADPDOT=0
 
     Do atom=1,Specific_Atoms
        Temp_atom(atom)%Gij=0
     Enddo

   !! Prepare projector and gradprojector coefficients
     i=0
     Do atype = 1, Atom_Types
        AT => AtomType_Info(atype)

        Do n=1,AT%Basis_Size
           L = AT%L_Value(n)
           il= YlmCon*(ai**L)
           Do m=-L,L
           am=ABS(m)
           Index=((L*(L+1))/2) + 1 + am
           If (m < 0) then
              il=il*((-1)**am)
              WW=WF*Temp_type(atype)%pnl(:,n)*Ylm(:,Index)
              W1=WF*(Temp_type(atype)%pnl(:,n)*GYlm(:,1,Index) &
                +Temp_type(atype)%Gpnl(:,n)*Ylm(:,Index)*Glist(:,1))
              W2=WF*(Temp_type(atype)%pnl(:,n)*GYlm(:,2,Index) &
                +Temp_type(atype)%Gpnl(:,n)*Ylm(:,Index)*Glist(:,2))
              W3=WF*(Temp_type(atype)%pnl(:,n)*GYlm(:,3,Index) &
                +Temp_type(atype)%Gpnl(:,n)*Ylm(:,Index)*Glist(:,3))
           Else
              WW=WF*Temp_type(atype)%pnl(:,n)*CONJG(Ylm(:,Index))
              W1=WF*(Temp_type(atype)%pnl(:,n)*CONJG(GYlm(:,1,Index)) &
                 +Temp_type(atype)%Gpnl(:,n)*CONJG(Ylm(:,Index))*Glist(:,1))
              W2=WF*(Temp_type(atype)%pnl(:,n)*CONJG(GYlm(:,2,Index)) &
                 +Temp_type(atype)%Gpnl(:,n)*CONJG(Ylm(:,Index))*Glist(:,2))
              W3=WF*(Temp_type(atype)%pnl(:,n)*CONJG(GYlm(:,3,Index)) &
                 +Temp_type(atype)%Gpnl(:,n)*CONJG(Ylm(:,Index))*Glist(:,3))
           EndIf
              
              Do lut=AtomType_Range(1,atype),AtomType_Range(2,atype)
                 atom=AtomType_Map(lut)
                 Call GetStructFactor(atom,Phase)
                 
                 i=i+1
       
                 PlmIndex=PDOT_Map(5,i)

                 PDOT(PlmIndex)=il*SUM(WW(:)*Phase(:))
                 GRADPDOT(PlmIndex,1)=il*SUM(W1(:)*Phase(:))
                 GRADPDOT(PlmIndex,2)=il*SUM(W2(:)*Phase(:))
                 GRADPDOT(PlmIndex,3)=il*SUM(W3(:)*Phase(:))
              EndDo

         EndDo    !m
      Enddo    !n

    Enddo !atype

  ! Do i=1,PLM_Max
  !   write(Log_UNit,*) 'Compare pdot --',i,ABS(pdot(i)-tempdot(i))
  ! Enddo


    Do atom=1,Specific_Atoms
       Base=PLM_AtomRange(1,atom)-1

       Do i=PLM_AtomRange(1,atom),PLM_AtomRange(2,atom)
          Do j=PLM_AtomRange(1,atom),PLM_AtomRange(2,atom)

           Temp_atom(atom)%Gij(1,i-Base,j-Base)=CONJG(GRADPDOT(i,1))*PDOT(j) &
                    + CONJG(PDOT(i))*GRADPDOT(j,1)
           Temp_atom(atom)%Gij(2,i-Base,j-Base)=CONJG(GRADPDOT(i,2))*PDOT(j) &
                    + CONJG(PDOT(i))*GRADPDOT(j,2)
           Temp_atom(atom)%Gij(3,i-Base,j-Base)=CONJG(GRADPDOT(i,3))*PDOT(j) &
                    + CONJG(PDOT(i))*GRADPDOT(j,3)

          Enddo
        Enddo
     Enddo

     GradEnk(1)=2*DOT_PRODUCT(Glist(:,1)*WF(:),Glist(:,4)*WF(:))
     GradEnk(2)=2*DOT_PRODUCT(Glist(:,2)*WF(:),Glist(:,4)*WF(:))
     GradEnk(3)=2*DOT_PRODUCT(Glist(:,3)*WF(:),Glist(:,4)*WF(:))

    Do Atom = 1,Specific_Atoms
       A => Atom_List(atom)
       AT => AtomType_Info(A%TypeIndex)
       Base=AT%nlm_Size
       if(spindependence.and.(.not.Global_spinup)) then
           Dij=>A%Dijspin
       else
           Dij=>A%Dij
       endif

       ctmp=0
       Do i=1,Base
          Do j=1,Base
             ctmp=ctmp+(Dij(i,j)-Enk*AT%Oij(i,j))*Temp_atom(atom)%Gij(:,i,j)
          Enddo
        Enddo

       GradEnk(:)=GradEnk(:)+ctmp(:)
    Enddo

  write(Log_Unit,*) 'CalcGradEnk: Enk, GradEnk = ',Enk,GradEnk
  Call Flush(Log_Unit)
 Call FreeBuffer(WW)
 Call FreeBuffer(W1)
 Call FreeBuffer(W2)
 Call FreeBuffer(W3)

 Return
 End Subroutine

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! RadFourierDeriv - calculates the radial integral necessary to evaluate the 
!             derivative of the Fourier transform of f(r)/r*Ylm
!
!     L  - L value of Ylm
!     k  - Spherical Bessel argument scale factor(see below)
!     h  - Radial spacing between grid points
!     N  - Number of grid points
!     F  - tabulated Function
!
!   input : l, k, h, n, f(:) , where f(r) is tabulated at n points 
!      in intervals of h;   f(1)==f(r=0)=0
!
!   The radial integral is returned:
!       gftf == int(r*f(r)*d j_l(k*r),r=0..(n-1)*h)
!                          _________
!                          d k
!
!
!******************************************************************************

Real Function RadFourierDeriv(l,k,h,n,f,Grid)
  Integer, Intent(IN) :: L
  Real,    Intent(IN) :: k
  Real,    Intent(IN) :: h
  Integer, Intent(IN) :: n
  Real,    Intent(IN) :: f(n)
  Type(GridInfo), Intent(IN) :: Grid

  Integer :: i
  Real, pointer :: x(:),y(:)
  Real, parameter :: tol=1.e-11
  Real :: R0
 
  Allocate(x(n),y(n))

  If (k < tol ) then
    Write(Error_Unit,*) ' Error in radFourierDeriv -- k too small ', k
    stop
  EndIf

  If (Grid%islog) then
      R0=Grid%drdu(1)
      Do i=1,n
         x(i) = Grid%drdu(i) - R0
      End Do
  Else
      Do i=1,n
         x(i) = (i-1)*h
      End Do
  Endif

  y=k*x
  Call DerivSphereBessel(L,n,y)

  Do i=1,n
     y(i)=y(i)*f(i)*(x(i)**2)
  End Do


  RadFourierDeriv = IntSimpson(n,h,y,Grid)        
  
  DeAllocate(x,y)
  return
End Function

!******************************************************************************
!
! gradylmlist: Returns gradient spherical harmonic functions for l in the order
!         m=-l,...0,...l in Condon-Shortley convention and for 
!         l=0,.. lmax
!
! IN : x, y, z- These are the x, y, and z cartesian coordinates
!      lmax
!      error_unit
!
! OUT: GradYlm(3,13,lmax+1) -array containing the spherical harmonic
!               values.
!
!******************************************************************************
Subroutine GradYlmlist(x, y, z, lmax, Gradylm)
  implicit none

  Real, intent(IN)   :: x, y, z
  integer, intent(IN):: lmax
  Complex, INTENT(OUT) :: Gradylm(:,:,:)

! Local declarations

  Real               :: Ylm_Norm, r, costheta, sintheta, cosphi, sinphi , tmp
  Real  :: vtheta(3)
  Complex :: vphi(3)
  Real, parameter    :: tol = 1.e-11
  Integer    :: m,im ,L
  Complex    :: phase

  Gradylm = 0

  if (lmax.lt.0.or.lmax.gt.6) then   
   write(error_unit,*) 'error in GradYlmlist',lmax,x,y,z
   stop
  endif

  r=sqrt(x*x+y*y+z*z)

  If (r.lt.tol.or.lmax.eq.0) return

  costheta=z/r
  cosphi=1
  sinphi=0
  sintheta=sqrt((1-costheta)*(1+costheta))
  if (sintheta.gt.tol) then
     cosphi=x/(r*sintheta)
     sinphi=y/(r*sintheta)
  else
     sintheta=0
  end if

  vtheta(1)=costheta*cosphi/r;vtheta(2)=costheta*sinphi/r;vtheta(3)=-sintheta/r
  vphi(1)=CMPLX(0,-sinphi/r);vphi(2)=CMPLX(0,cosphi/r);vphi(3)=0

  phase=CMPLX(cosphi,sinphi)

  do L=1,lmax

  Ylm_Norm= sqrt((2*L+1)/Four_Pi)
    
     Gradylm(:,L+1,l+1)=Ylm_Norm*Der_Theta_P(l,0,costheta)*vtheta
     tmp=1.0/(L*(L+1))
     Do m=1,L
        Gradylm(:,L+1+m,l+1)=Ylm_Norm*sqrt(tmp)*(phase**m)* &
          (Der_Theta_P(l,m,costheta)*vtheta + Der_Phi_P(l,m,costheta)*vphi)
        Gradylm(:,L+1-m,l+1)=((-1)**m)*CONJG(Gradylm(:,L+1+m,l+1))
        if (m.lt.L) tmp=tmp/((L+m+1)*(L-m))
     End do

  Enddo
  
end subroutine Gradylmlist

End Module GradEnk
