!******************************************************************************
!
! File : grrp.f90
!   by : Alan Tackett
!   on : 11/02/97
!  for : PAW
!
!  This file containsthe routines needed to perform the Generalized Rayleigh
!  Ritz projections for determining the new Psi's and eigenvalues. Also
!  routines are included to perform the Back-Rotations.
!
!******************************************************************************

Module grrp

Use paw_inout
Use mem_data
Use memmgr
Use psilib
Use fileio
Use paw_inout
Use search_sort
Use hamiltonian
Use debug

!USe xlfutility

Implicit NONE!!!!!!!!!!!!!

!******************************************************************************
Contains


!******************************************************************************
!
!  Diagonalizer - Diagonalizes Hbase & OBase matrices
!
!  VecSize      - Number of basis vectors used to construct Hbase, OBase
!  ArraySize    - Dimension of Hbase, OBase
!  NewSize      - Number of EigenValues and EigenVectors returned
!  Eigen        - List of EigenValues
!  Vec          - MAtrix containing the new eigenvectors
!  DoOrthog     - If true, assume Obase is identity matrix
!  Hbase, Obase - Assumed to be Hermitian
!
!******************************************************************************

Subroutine Diagonalizer(DoOrthog, VecSize, ArraySize, NewSize, Hbase, Obase, & 
                        Eigen,  Vec)
  Logical,          Intent(IN)  :: DoOrthog
  Integer,          Intent(IN)  :: VecSize
  Integer,          Intent(IN)  :: ArraySize
  Integer,          Intent(OUT) :: NewSize
  Complex,          Intent(INOUT) :: Hbase(:,:)
  Complex,          Intent(INOUT) :: Obase(:,:)
  Real,     Intent(OUT) :: Eigen(:)
  Complex,  Intent(OUT) :: Vec(:,:)

  Integer :: Kindex, i, j, k, LWork,  LSize
  Integer ::  Info
!  Complex  :: Omat(ArraySize,ArraySize)
!  Complex  :: Hmat(ArraySize, ArraySize), VecR(ArraySize, ArraySize)
!  Complex  :: Work(4*ArraySize), JUNK(ArraySize,ArraySize)
  Complex, allocatable  :: Omat(:,:),Hmat(:,:),VecR(:,:),Work(:),JUNK(:,:)
!  Real     ::  RWork(8*ArraySize), val, Lambda(ArraySize)
  Real ,allocatable    ::  RWork(:), Lambda(:)
  Real     :: tol,val

  Allocate(Hmat(ArraySize, ArraySize), VecR(ArraySize, ArraySize), &
     Omat(ArraySize, ArraySize), Work(4*ArraySize), &
     JUNK(ArraySize,ArraySize),  RWork(8*ArraySize),  Lambda(ArraySize))
  tol=Overlap_tol

Call PrintDate(Log_Unit,'Diagonalizer: START')
!Call flush(Log_Unit)

  LWork = 4*ArraySize
  NewSize=VecSize

!  val = MAXVAL(ABS(CONJG(Hbase)*Hbase))
!  val = SQRT(ABS(val))
!Write(Log_Unit,*) 'GRRP: MAXVAL(Hmat) check=',val
!Call flush(Log_Unit)
!!Write(*,*) 'GRRP: MAXVAL(Hmat) check=',val

!  val = MAXVAL(ABS(CONJG(Obase)*Obase))
!  val = SQRT(ABS(val))
!Write(Log_Unit,*) 'GRRP: MAXVAL(Omat) check=',val
!Call flush(Log_Unit)
!!Write(*,*) 'GRRP: MAXVAL(Omat) check=',val

!val = SUM(ABS(AImag(Hbase)))
!Call flush(Log_Unit)
!Write(Log_Unit,*) 'GRRP: IMAG Hmat check=',val
!val = SUM(ABS(AImag(Obase)))
!Write(Log_Unit,*) 'GRRP: IMAG Omat check=',val
!Call flush(Log_Unit)
!write(LOg_Unit,*) 'in diag',vecsize,arraysize
!Call flush(Log_Unit)

  Work = 0;  Rwork = 0;
  Hmat = Hbase;  Omat = Obase;  VecR = 0

  Hmat = Hbase;  Omat = Obase

  Info =13; NewSize=VecSize

  If (DoOrthog) then   ! Omat is not diagonal

     If (.not.Calc_O_Eigenvalues) then
        Call ZHEGV(1, 'V', 'U', VecSize, Hmat(1,1), ArraySize, &
             Omat(1,1), ArraySize, Eigen, Work, Lwork, Rwork, Info)

        write(Log_Unit,*) ' completed zhegv with Info=',Info
!Call flush(Log_Unit)

        If (Info.ne.0) then
           Hmat=Hbase; Omat=Obase
        EndIf
     EndIf
     
     If (Info.ne.0) then

        Call ZHEEV('V', 'U', VecSize, Omat(1,1), ArraySize, Lambda, &
          Work, LWork, Rwork, Info)

        write(Log_Unit,*) ' completed Omat diagonalization with Info=',Info

        j=0 ; VecR=0
        Do i=1,VecSize
           write(Log_Unit,*) 'i lambda',i,Lambda(i)
           If (Lambda(i) > tol) then
              j=j+1
              VecR(1:VecSize,j)=Omat(1:VecSize,i)/SQRT(Lambda(i))
           EndIf
        Enddo     


        If (j > 0) then
          NewSize=j
        Else
          Write(Log_Unit,*) 'GRRP - O matrix is singular', Lambda
          Stop
        Endif

        Write(Log_Unit,*) 'GRRP - NewSize = ',NewSize
!       Omat=0
!       Omat=MATMUL(Hmat(1:VecSize,1:VecSize),VecR(1:VecSize,1:NewSize))
!!!!!Following code replaces fortran MATMUL which is too slow
        Omat=0
        do k=1,NewSize
           do i=1,VecSize
!             Omat(i,k)=0
              do j=1,VecSize
                 Omat(i,k)=Omat(i,k)+CONJG(Hmat(j,i))*VecR(j,k)
              Enddo
           Enddo
         Enddo
!End Replace MATMUL
!       JUNK=0                  
!       JUNK=TRANSPOSE(CONJG(VecR(1:VecSize,1:NewSize)))
!       Hmat=0
!       Hmat=MATMUL(JUNK(1:NewSize,1:VecSize),Omat(1:VecSize,1:newSize))
!!!!!Following code replaces fortran MATMUL which is too slow
        Hmat=0
        do k=1,NewSize
           do i=1,NewSize
!             Hmat(i,k)=0
              Do j=1,VecSize
                 Hmat(i,k)=Hmat(i,k)+CONJG(VecR(j,i))*Omat(j,k)
              Enddo
           Enddo
        Enddo
!End Replace MATMUL
       
        Call ZHEEV('V', 'U', NewSize, Hmat(1,1), ArraySize, Eigen, &
             Work, LWork, Rwork, Info)
   
        write(Log_Unit,*) ' completed Hmat diagonalization with Info=',Info
   
        Omat=Hmat
!       Hmat=0
!       Hmat=MATMUL(VecR(1:VecSize,1:NewSize),Omat(1:NewSize,1:NewSize))
!!!!!Following code replaces fortran MATMUL which is too slow
        Hmat=0
        JUNK(1:NewSize,1:VecSize)=TRANSPOSE(VecR(1:VecSize,1:NewSize))
        do k=1,NewSize
           do i=1,VecSize
!             Hmat(i,k)=0
              Do j=1,NewSize
                 Hmat(i,k)=Hmat(i,k)+JUNK(j,i)*Omat(j,k)
              Enddo
           Enddo
        Enddo
!End Replace MATMUL
      EndIf
   
   else
        Call ZHEEV('V', 'U', VecSize, Hmat(1,1), ArraySize, Eigen, &
          Work, LWork, Rwork, Info)

        write(Log_Unit,*) ' completed Hmat diagonalization with Info=',Info
   End If

! Write(Log_Unit,*) 'GRRP - Eigen',Eigen(1:NewSize)
                  

  Vec = Hmat

Call PrintDate(Log_Unit,'Diagonalizer: END')
!Call flush(Log_Unit)
  DeAllocate(Hmat,Omat,VecR,Work,JUNK,RWork,Lambda)

  Return 
End Subroutine
 

!******************************************************************************
!
!  MakeNewPsi - Creates the New Psi's from the Old Psi's after a GRRP-BR
!
!  BasisSize    - Number of Basis Vectors
!  KpntBands    - Number of Bands for the K-point
!  Vec          - MAtrix containing the new eigenvectors
!  Eigen        - List of EigenValues
!  LUT          - LUT matching Disk Index to Vec, Eigen position
!
! 05/23/05  NAWH:  updated for spin
!******************************************************************************

Subroutine MakeNewPsi(BasisSize, KpntBands, Vec, Eigen, LUT,Kpnt)
  Integer,  Intent(IN)    :: BasisSize
  Integer,  Intent(IN)    :: KpntBands
  Complex,  Intent(IN) :: Vec(:,:)
  Real,     Intent(IN) :: Eigen(:)
  Integer,  Intent(IN) :: LUT(:,:)
  Integer,  Intent(IN)    :: Kpnt

  Complex :: dot
  Integer :: i, LSize, ioerr, Kindex, CurrentSet(Mem_MapSize), j, LastGMRES, k
 ! Integer :: VecMap(Mem_MapSize), SU1, SU2, KpntIndex
  Integer ::  SU1, SU2, KpntIndex
  Integer ,allocatable:: VecMap(:)
  Real :: Mag
  Character*100 :: msg
 ! Complex :: c, PDot(PLM_Max)
  Complex :: c
  Complex ,allocatable :: PDot(:)
  Complex, Pointer :: NewPSi(:), Ox(:), Work(:)
  Type(MEm_Handle), Pointer :: Psi, FAS, MH(:)
  Logical :: PDOT_Stored


  Allocate(VecMap(Mem_MapSize),PDot(PLM_Max))
  Call Start_Timer(Timer(MakeNewPsi_Timer))

  Call GetBuffer( NewPsi)
  Call GetBuffer( Ox)


  KpntIndex = 0
  j = 0
  Do i=1, KpntBands   !**** Create NEw Psi's


     VecMap = MH_Skip        !**** Make MAp of Psi's to use in construction
     VecMap(LUT(1,1:BasisSize)) = MH_toProcess

     Call Phase_Generic(VecMap,  Kpnt)

     NewPsi = 0
     Call GetNextPsi(KIndex, Psi)
     Do While (KIndex /= 0)
        NewPsi = NewPsi + Vec(LUT(2,Psi%index),i)*Psi%Ptr
        Call GetNExtPsi(Kindex, Psi)
     End Do

     Call GetNextFreeIndex(KpntIndex)
     PsiInfo(KpntIndex)%Kpnt=Kpnt
     PsiInfo(KpntIndex)%spinup=Global_spinup
     PsiInfo(KpntIndex)%DoSave=1
     PsiInfo(KpntIndex)%Available=Mem_Used
     PsiInfo(KpntIndex)%Energy=Eigen(i)
     PsiInfo(KpntIndex)%PDOT_Stored=.false.
     PsiInfo(KpntIndex)%KE_Stored=.false.

     PDOT_Stored = .FALSE.
     call kplusGfilter(NewPsi,Kpnt)
     Call Opsi( NewPsi, Ox, PDot, Kpnt, PDOT_Stored)
     Mag = Basis_DotProd( Ox, NewPsi, .FALSE.)
     mag = SQRT(mag)
 Write(Log_Unit,*) 'MakeNewPsi: Mag=',Mag, ' * DI=',KpntIndex,' E=',Eigen(i)
     NewPsi = NewPsi/mag
     Call SavePsi_toBuffer(KpntIndex,NewPsi)
  End Do
! No longer need BasisSize entries
  PsiInfo(LUT(1,1:BasisSize))%Kpnt=0
  PsiInfo(LUT(1,1:BasisSize))%spinup=.true.
  PsiInfo(LUT(1,1:BasisSize))%DoSave=0
  PsiInfo(LUT(1,1:BasisSize))%Available=Mem_Available
  PsiInfo(LUT(1,1:BasisSize))%PDOT_Stored=.FALSE.
  PsiInfo(LUT(1,1:BasisSize))%KE_Stored=.FALSE.

  Do i = 1 , BasisSize
    j =  PsiInfo(LUT(1,i))%MemBufIndex
    If (j > 0) then
       GlobalMap%Psi_Handle(j)%Index = 0
       PsiInfo(LUT(1,i))%MemBufIndex=0             
    EndIf
!write(log_unit,*) 'MakeNewPsi: released indices',i,LUT(1,i),j
  Enddo

  Call FreeBuffer(NewPsi)
  Call FreeBuffer(Ox)

  Call Stop_Timer(Timer(MakeNewPsi_Timer))
  DeAllocate(VecMap,PDot)

!write(Log_unit,*) 'complete MakeNewPsi'
  Return
End Subroutine
!******************************************************************************
!
!  PerfromGRRP_ALL - Performs a generalized Rayleigh-Ritz operation for current
!                set of wave functions for the given kpoint
!
!  LSize - LEvel Dimensions
!  BlochK - Bloch Phase 
!  Vec_toProcess- List of Psi's to process 
!  VecSize      - Number of Bands beign processed
!  Eigen        - List of EigenValues
!  LUT          - LUT matching Disk Index to Vec, Eigen position
!  Vec          - MAtrix containing the new eigenvectors
!
!
!******************************************************************************

Subroutine PerformGRRP_All( Vec_toProcess, VecSize, &
     Vec, Eigen, LUT, NewSize, Kpnt)
  Integer,          Intent(IN)  :: VecSize
  Integer,          Intent(IN)  :: Vec_toProcess(:)
  Complex,  Intent(OUT) :: Vec(:,:)
  Real,     Intent(OUT) :: Eigen(:)
  Integer,          Intent(OUT) :: LUT(:,:)
  Integer,          Intent(OUT) :: NewSize
  Integer,          Intent(IN)  :: Kpnt

  Integer :: Kindex, i, j, k, LWork, PsiIndex, LUTIndex, LSize
 !Integer :: VecProcessed(Mem_MapSize), Info, ArraySize
  Integer :: Info, ArraySize
  Integer ,allocatable :: VecProcessed(:)
  Complex, Pointer :: Hx(:), KE(:), Ox(:)
  Complex, Pointer  :: PsiWork(:),Ve(:)
 !Complex  :: Hbase(VecSize,VecSize), Obase(VecSize, VecSize)
  Complex ,allocatable :: Hbase(:,:), Obase(:, :),PDot(:)
 !Complex  :: Term, PDot(PLM_Max), E3
  Complex  :: Term, E3
  Real     :: val
  Type(Mem_Handle), Pointer :: Psi2

  Allocate(VecProcessed(Mem_MapSize),Hbase(VecSize,VecSize), &
      Obase(VecSize, VecSize), PDot(PLM_Max))
Call PrintDate(Log_Unit,'PerformGRRP_ALL: START')
  ArraySize = VecSize
!Write(Log_unit,*) 'GRRP: VecSize= ',VecSize, 'Kpnt=',Kpnt,' * Vec_toProcess=',Vec_toProcess
!call flush(Log_unit)
  Call GetBuffer( PsiWork)
  Call GetBuffer( Hx)
  Call GetBuffer( Ox)


  LUT = 0

  Hbase = 0
  Obase = 0
  
  if(Global_spinup) Ve =>SCFValues%Ve
  if(.not.Global_spinup) Ve =>SCFValues%Vespin

  Psi_toProcess = Vec_toProcess

  j = 0          !**** Make the LUT *****
  Do i=1, Mem_MapSize
     If (Psi_toProcess(i) == MH_toProcess) then
        j = j + 1
        LUT(1,j) = i
        LUT(2,i) = j
        write(log_unit,*)'PerformGRRP_ALL: LUT ',i,j
        call flush(log_unit)
     End If
  End Do

  !*** Initialize the Proj config and K-point to use ***
!Write(Log_Unit,*) 'PerformGRRP_All: Kpnt=',Kpnt, ' * VTP=', Vec_toProcess
  Call Phase_Generic( Vec_toProcess,  Kpnt)

  VecProcessed = Vec_toProcess
  Do i=1, VecSize
     Call GetNextPsi_UnProcessed(KIndex, VecProcessed,  &
          PsiIndex, PsiWork)
     call kplusGfilter(PsiWork,Kpnt)
     Call SavePsi_toBuffer(PsiIndex , PsiWork)


     Call CalcHxandOx( PsiWork, Hx, Ox, Ve, &
          PSiInfo(PsiIndex)%PDOT, Kpnt, PSiInfo(PsiIndex)%PDOT_Stored)

     PsiIndex = LUT(2,PsiIndex)    !** Get the LUT index
     Hbase(PsiIndex, PsiIndex) = Basis_DotProd( Hx, PsiWork, .FALSE.)
     Obase(PsiIndex, PsiIndex) = Basis_DotProd( Ox, PsiWork, .FALSE.)
     
     Do j=1, i-1
        Call GetNextPsi(Kindex, Psi2)
        Call kplusGfilter(Psi2%ptr,Kpnt)

         k = LUT(2, Psi2%Index)    !** Get the LUT index
        
        Term = Basis_DotProd( Hx, Psi2%Ptr, .FALSE.)
        Hbase(PsiIndex, k) = Term
        Hbase(K, PsiIndex) = CONJG(Term)

        Term = Basis_DotProd( Ox, Psi2%Ptr, .FALSE.)
        Obase(PsiIndex, k) = Term
        Obase(K, PsiIndex) = CONJG(Term)
     End Do

  End Do


  val = MAXVAL(ABS(CONJG(Hbase)*Hbase))
  val = SQRT(ABS(val))
Write(Log_Unit,*) 'GRRP: MAXVAL(Hmat) check=',val
!Write(*,*) 'GRRP: MAXVAL(Hmat) check=',val

  val = MAXVAL(ABS(CONJG(Obase)*Obase))
  val = SQRT(ABS(val))
Write(Log_Unit,*) 'GRRP: MAXVAL(Omat) check=',val

!val = SUM(ABS(AImag(Hbase)))
!Write(Log_Unit,*) 'GRRP: IMAG Hmat check=',val
!val = SUM(ABS(AImag(Obase)))
!Write(Log_Unit,*) 'GRRP: IMAG Omat check=',val

!Write(Log_Unit,*) 'GRRP: VecSize=',VecSize, ' * Vec_toProcess=',Vec_toProcess
!Write(Log_Unit,*) 'GRRP: LUT(1,:)=',LUT(1,1:VecSize)

!Write(Log_Unit,*) 'GRRP: Printing Hmat-----------------------------------'
!Do i=1, VecSize
!  Write(Log_Unit,*) 'GRRP: Row=',i
!  Write(Log_Unit,*) 'GRRP:   H(i,:)=',Hbase(i,:)
!End Do
!Write(Log_Unit,*) 'GRRP: Printing Omat-----------------------------------'
!Do i=1, VecSize
!  Write(Log_Unit,*) 'GRRP: Row=',i
!  Write(Log_Unit,*) 'GRRP:   O(i,:)=',Obase(i,:)
!End Do

  Call Diagonalizer(.true. , VecSize, VecSize, NewSize, Hbase, Obase, &
      Eigen, Vec)


  Call FreeBuffer(PsiWork)
  Call FreeBuffer(Hx)
  Call FreeBuffer(Ox)

Call PrintDate(Log_Unit,'PerformGRRP_ALL: End')
  DeAllocate(VecProcessed,Hbase,Obase,PDot)

  Return 
End Subroutine

!******************************************************************************
!
! UpdateKpnt - Updates the Wave functions for a given Kpnt
!
!  Kpnt  - Kpnt index
!  VecSize - Number of Psi's for the kpoint
!  Basis_toProcess - List of Psi's to process
!
!******************************************************************************

Subroutine UpdateKpnt( Kpnt, VecSize, Basis_toProcess)
  Integer, Intent(IN) :: Kpnt
  Integer, Intent(IN) :: VecSize
  Integer, Intent(IN) :: Basis_toProcess(:)

 ! Complex  :: Vec(VecSize, VecSize)
  Complex ,allocatable :: Vec(:,:)
 !Real     :: Eigen(VecSize)
  Real ,allocatable    :: Eigen(:)
 !Integer          :: LUT(2,Mem_MapSize), KpntBands, i, NewSize
  Integer          :: KpntBands, i, NewSize
  Integer ,allocatable :: LUT(:,:)

  Allocate(Vec(VecSize, VecSize),Eigen(VecSize),LUT(2,Mem_MapSize))
  KpntBands = 0
  Do i=1, Mem_MapSize
     If ((PsiInfo(i)%Kpnt == Kpnt) .AND. (CLusterMap(i) > 0) &
          .and.(PsiInfo(i)%spinup.eqv.Global_spinup)) then
        KpntBands = KpntBands + 1
     End If
  End Do

  Call PerformGRRP_ALL( Basis_toProcess, KpntBands, &
       Vec, Eigen, LUT, NewSize, Kpnt)
  Call MakeNewPsi( KpntBands, NewSize, Vec, Eigen, LUT, Kpnt)

  DeAllocate(Vec,Eigen,LUT)
write(Log_Unit,*) 'completed updatekpnt'
  Return
End Subroutine


!******************************************************************************
!
! MiniDiag -- Recalculate basis for given k and spin
!
!  Kpnt  - Kpnt index
!  VecSize - Number of Psi's for the kpoint
!  Basis_toProcess - List of Psi's to process
!
!******************************************************************************

Subroutine MiniDiag( Kpnt, VecSize, Basis_toProcess)
  Integer, Intent(IN) :: Kpnt
  Integer, Intent(IN) :: VecSize
  Integer, Intent(IN) :: Basis_toProcess(:)

 ! Complex  :: Vec(VecSize, VecSize)
  Complex ,allocatable :: Vec(:,:)
 !Real     :: Eigen(VecSize)
  Real ,allocatable    :: Eigen(:)
 !Integer          :: LUT(2,Mem_MapSize), KpntBands, i, NewSize
  Integer          :: KpntBands, i, NewSize
  Integer ,allocatable :: LUT(:,:)

  Allocate(Vec(VecSize, VecSize),Eigen(VecSize),LUT(2,Mem_MapSize))
  KpntBands = 0
  Do i=1, Mem_MapSize
     If ((PsiInfo(i)%Kpnt == Kpnt) &
          .and.(PsiInfo(i)%spinup.eqv.Global_spinup)) then
        KpntBands = KpntBands + 1
     End If
  End Do
 
  If (KpntBands > VecSize ) then
     Write(Error_unit,*) 'MiniDiag: error in MiniDiag', Kpnt,KpntBands,VecSize
     stop
  Endif
 
  Call PerformGRRP_ALL( Basis_toProcess, KpntBands, &
       Vec, Eigen, LUT, NewSize, Kpnt)
  Call MakeNewPsi( KpntBands, NewSize, Vec, Eigen, LUT, Kpnt)

  DeAllocate(Vec,Eigen,LUT)
write(Log_Unit,*) 'completed updatekpnt'
  Return
End Subroutine

End Module

