!******************************************************************************
!
! File : hamvxc.f90
!   originally in hamiltonian.f90
!   by : Alan Tackett
!   on : 02/07/97
!  for : PAW
!
! Module for calculating the Hamiltonian and Cohesive Energy of the PAW
! system.  Also contains routines to do H*Psi and O*Psi.
!
! Constains terms pertaining to vxc
!
! Updated for spin by Ping Tang and N. A. W. Holzwarth
!      Last change 5/25/05
!******************************************************************************

Module hamvxc

Use atom_data
Use crystal_data
Use exchange_corr
Use gpoints 
Use mathlib
Use options_data
Use paw_inout
Use projectors
Use spherical_harmonic

Implicit NONE!!!!

!******************************************************************************
Contains
!******************************************************************************

!******************************************************************************
!                                                          ~a   a
!  AccumVxc_LDA - Accumulates the Exchange correlation pot for N + N  to Dij
!    for LDA in Perdew_Wang form only
!
!   R=0 is included!!!!!!!!!!
!
!******************************************************************************

Subroutine AccumVxc_LDA( Energy)
   Real,    Intent(INOUT) :: Energy

   Integer :: i,j,k, nili, njlj, mi, mj, Li, Lj, Rk, RS, count
   Integer ::nklk, nlll, mk, ml, Lk, Ll, alpha, Basis_Size, Rad_Size
   Integer :: ibase,jbase,kbase,lbase, Skip_Size, Rad_Index, Mesh_Size
   Integer, pointer :: nl_base(:), nlm_lut(:,:)
   Complex, Pointer :: Dij(:,:), Wij(:,:),  Ylm(:,:,:)
   Real,    Pointer :: R2(:), Phi_ij_R2(:), TPhi_ij_R2(:)
   Real,    Pointer :: CoreXC(:), CoreDen(:),FnRad(:,:)
   Complex          :: Ylm_l(13), Ylm_k(13), ylm1, ylm2
   Complex, Pointer :: Ylm_i(:), Ylm_j(:)
   Real             ::  Vxc1, Exc1, Vxc2, Exc2, fnint, Esum, PhiRk, TPhiRk
   Complex          :: den_phi, den_Tphi, den
   Real             :: Efn,fn,Fnsum
   Type (Specific_Atom), Pointer :: A
   Type (Atom_Info_Fixed), Pointer :: AT
   Real             :: DR
   Real, Pointer    :: Cij(:,:)
   Character*100    :: Msg

   if(spindependence) then 
        write(error_unit, *) 'AccumVxc_LDA: error -- spindependence should be false'
        stop
   endif 
   Call Start_Timer(Timer(AtomVxc_Timer))
!   Write(msg,*) 'AccumVxc: Error allocating Cij. Size=',Basis_Size
!   Call Check_Error(j, msg, Error_Unit, .TRUE.,paw_wc, "AccumVxc:")

   write(log_unit,*) 'AccumVxc_LDA: start'
   call flush(log_unit)
   
   Do i=1, Specific_Atoms
     A => Atom_List(i)
     AT => AtomType_Info(A%TypeIndex)
     nl_base => AT%nl_Base
     nlm_LUT => AT%nlm_LUT
     Wij => A%Wij
     Dij => A%Dij
     CoreDen => AT%Core_Density
     CoreXC => AT%Core_Xchange
     R2 => AT%R2
     Ylm => AT%Ylm
     Cij => AT%Cij
     FnRad => AT%FnRad
     Mesh_size = AT%Rad_Size


      DR = AT%Mesh_Step
      !DR = AT%Mesh_Step*AT%Rad_Skip
      if (AT%Rad_Skip/=1) then
         write(Log_Unit,*) 'Error in VXC -- Rad_Skip', AT%Rad_Skip
         stop
      endif

     Basis_Size = AT%Basis_Size

     FnSum = 0; ESum = 0
     Do alpha = 1, Num_AngPoints
        Cij = 0
        Do nili=1, Basis_Size     !*** Calulate Cij for n^a and n~^a
           Li = AT%L_Value(nili)
           Ylm_i => Ylm(:, Li+1, alpha)
           ibase = nl_Base(nili);  ibase = ibase + nlm_lut(2,ibase);      
           Do njlj=1, Basis_Size
              Lj = AT%L_Value(njlj)
              Ylm_j => Ylm(:, Lj+1, alpha)
              jbase = nl_Base(njlj);  jbase = jbase + nlm_lut(2,jbase);      
           
              den = 0
              Do mi = -Li, Li
                 ylm1 = CONJG(Ylm_i(mi+Li+1))
              
                 Do mj=-Lj, Lj
                    ylm2 = Ylm_j(mj+Lj+1)
                    den = den + Wij(ibase+mi,jbase+mj) * Ylm1 * Ylm2
                 End Do
              End Do

              Cij(nili, njlj) = Den
           End do
        End Do

        !**** Calculate the Radial Densities and store them *****
        FnRad = 0

        Count = 0
        Do nili=1, Basis_Size
           Do njlj=1, Basis_Size
              count = count + 1
              Phi_ij_R2 => AT%Phi_ij_R2(:,count)
              TPhi_ij_R2 => AT%TPhi_ij_R2(:,count)

              FnRad(1:Mesh_Size,1)=FnRad(1:Mesh_Size,1) &
                   + Cij(nili,njlj)*Phi_ij_R2(1:Mesh_Size)

              FnRad(1:Mesh_Size,2)=FnRad(1:Mesh_Size,2) &
                   + Cij(nili,njlj)*TPhi_ij_R2(1:Mesh_Size)
           End Do
        End Do

        !*** Add CoreDen  ***
        FnRad(1:Mesh_Size,1) = FnRad(1:Mesh_Size,1) + &
             CoreDen(1:Mesh_Size)
        FnRad(1:Mesh_Size,2) = FnRad(1:Mesh_Size,2) + &
             AT%CoreTail_Density(1:Mesh_Size)

             !**** Calculate Vxc and Exc for Ra*****
        Do Rk=1, Mesh_Size
          
           Call pwldafunc((FnRad(Rk,1)),Exc1,Vxc1)
           Call pwldafunc((FnRad(Rk,2)),Exc2,Vxc2)
           
           FnRad(Rk, 3:6) = (/Vxc1, Vxc2, Exc1, Exc2/)
        ! write(6,'(i5,1p7e15.7)') Rk,Vxc1,Vxc2,Exc1,Exc2
        End Do

        FnRad(1:Mesh_Size,7) =   R2(1:Mesh_Size) * &
             (FnRad(1:Mesh_Size,5)*FnRad(1:Mesh_Size,1) &
             - FnRad(1:Mesh_Size,6)*FnRad(1:Mesh_Size,2))

        !FnRad(1:Mesh_Size,7) = FnRad(1:Mesh_Size,7) - &
        !   R2(1:Mesh_Size) *  &
        !     CoreXC(1:Mesh_Size) *  CoreDen(1:Mesh_Size)

        Efn= IntSimpson(Mesh_Size, DR, FnRad(1:Mesh_Size,7),&
            AT%RadGrid) * &
              AngPoints(4,alpha)*4*Pi
        Energy = Energy + Efn 
       !write(6,*) 'alpha efn energy',alpha,efn,energy

        Count = 0
        Do nili=1, Basis_Size
           Li = AT%L_Value(nili)
           Ylm_i => Ylm(:, Li+1, Alpha)
           ibase = nl_Base(nili); ibase = ibase + nlm_lut(2,ibase);


           Do njlj=1, Basis_Size
              Lj = AT%L_Value(njlj)
              Ylm_j => Ylm(:, Lj+1, Alpha)
              jbase = nl_Base(njlj); jbase = jbase + nlm_lut(2,jbase);      
              count = count + 1

              Phi_ij_R2 => AT%Phi_ij_R2(:,count)
              TPhi_ij_R2 => AT%TPhi_ij_R2(:,count)

              !**** Radial Integral Portion ******
              FnRad(1:Mesh_Size, 7) =  &
                 (FnRad(1:Mesh_Size,3) * Phi_ij_R2(1:Mesh_Size) - &
                    FnRad(1:Mesh_Size,4) * TPhi_ij_R2(1:Mesh_Size)) * &
                     R2(1:Mesh_Size) 

              FnRad(1,7) = 0
              FnRad(Mesh_Size,7) = 0
              
              fn = IntSimpson(Mesh_Size, DR, FnRad(1:Mesh_Size,7),&
                     AT%RadGrid) * &
                     AngPoints(4,alpha)*Four_Pi

              Do mi=-Li, Li
                 Do mj=-Lj, Lj
                    Dij(ibase+mi,jbase+mj) = Dij(ibase+mi,jbase+mj) + &
                         fn * CONJG(Ylm_i(Li+Mi+1))*Ylm_j(Lj+mj+1)
                 End Do
              End Do  
           end Do !njlj
        End do !nili
     End Do

  End Do

  Call Stop_Timer(Timer(AtomVxc_Timer))

   write(log_unit,*) 'AccumVxc_LDA: finish'
   call flush(log_unit)
  Return
End Subroutine

Subroutine AccumVxc_LSDA( Energy)
   Real,    Intent(INOUT) :: Energy

   Integer :: i,j,k, nili, njlj, mi, mj, Li, Lj, Rk, RS, count
   Integer ::nklk, nlll, mk, ml, Lk, Ll, alpha, Basis_Size, Rad_Size
   Integer :: ibase,jbase,kbase,lbase, Skip_Size, Rad_Index, Mesh_Size
   Integer, pointer :: nl_base(:), nlm_lut(:,:)
   Complex, Pointer :: Dijup(:,:),Dijdn(:,:), Wijup(:,:), Wijdn(:,:), Ylm(:,:,:)
   Real,    Pointer :: R2(:), Phi_ij_R2(:), TPhi_ij_R2(:)
   Real,    Pointer :: CoreXC(:), CoreDen(:),FnRadup(:,:),FnRaddn(:,:)
   Complex          :: Ylm_l(13), Ylm_k(13), ylm1, ylm2
   Complex, Pointer :: Ylm_i(:), Ylm_j(:)
   Real             ::  Vxc1even,Vxc1odd, Exc1, Vxc2even,Vxc2odd, Exc2, fnint, Esum, PhiRk, TPhiRk
   Complex          :: den_phi, den_Tphi, denup, dendn
   Real             :: Efn,fnup,fndn,Fnsum
   Type (Specific_Atom), Pointer :: A
   Type (Atom_Info_Fixed), Pointer :: AT
   Real             :: DR
   Real, Pointer    :: Cijup(:,:), Cijdn(:,:)
   Character*100    :: Msg

   if(.NOT.spindependence) then 
       write(error_unit, *) 'AccumVxc_LSDA---error: spindependence should be true'
       stop 
   end if
   Call Start_Timer(Timer(AtomVxc_Timer))
!   Write(msg,*) 'AccumVxc: Error allocating Cij. Size=',Basis_Size
!   Call Check_Error(j, msg, Error_Unit, .TRUE.,paw_wc, "AccumVxc:")
   write(log_unit,*) 'AccumVxc_LSDA: start'
   call flush(log_unit)
   
   Do i=1, Specific_Atoms
     A => Atom_List(i)
     AT => AtomType_Info(A%TypeIndex)
     nl_base => AT%nl_Base
     nlm_LUT => AT%nlm_LUT
     Wijup => A%Wij
     Wijdn => A%Wijspin
     Dijup => A%Dij
     Dijdn => A%Dijspin
     CoreDen => AT%Core_Density
     CoreXC => AT%Core_Xchange
     R2 => AT%R2
     Ylm => AT%Ylm
     Cijup => AT%Cij
     Cijdn => AT%Cijspin
     FnRadup => AT%FnRad
     FnRaddn => AT%FnRadspin
     Mesh_size = AT%Rad_Size


      DR = AT%Mesh_Step
      !DR = AT%Mesh_Step*AT%Rad_Skip
      if (AT%Rad_Skip/=1) then
         write(Log_Unit,*) 'Error in VXC -- Rad_Skip', AT%Rad_Skip
         stop
      endif

     Basis_Size = AT%Basis_Size

     FnSum = 0; ESum = 0
     Do alpha = 1, Num_AngPoints
        Cijup = 0; Cijdn = 0
        Do nili=1, Basis_Size     !*** Calulate Cij for n^a and n~^a
           Li = AT%L_Value(nili)
           Ylm_i => Ylm(:, Li+1, alpha)
           ibase = nl_Base(nili);  ibase = ibase + nlm_lut(2,ibase);      
           Do njlj=1, Basis_Size
              Lj = AT%L_Value(njlj)
              Ylm_j => Ylm(:, Lj+1, alpha)
              jbase = nl_Base(njlj);  jbase = jbase + nlm_lut(2,jbase);      
           
              denup = 0; dendn = 0
              Do mi = -Li, Li
                 ylm1 = CONJG(Ylm_i(mi+Li+1))
              
                 Do mj=-Lj, Lj
                    ylm2 = Ylm_j(mj+Lj+1)
                    denup = denup + Wijup(ibase+mi,jbase+mj) * Ylm1 * Ylm2
                    dendn = dendn + Wijdn(ibase+mi,jbase+mj) * Ylm1 * Ylm2
                 End Do
              End Do

              Cijup(nili, njlj) = denup
              Cijdn(nili, njlj) = dendn
           End do
        End Do

        !**** Calculate the Radial Densities and store them *****
        FnRadup = 0; FnRaddn = 0

        Count = 0
        Do nili=1, Basis_Size
           Do njlj=1, Basis_Size
              count = count + 1
              Phi_ij_R2 => AT%Phi_ij_R2(:,count)
              TPhi_ij_R2 => AT%TPhi_ij_R2(:,count)

              FnRadup(1:Mesh_Size,1)=FnRadup(1:Mesh_Size,1) &
                   + Cijup(nili,njlj)*Phi_ij_R2(1:Mesh_Size)

              FnRaddn(1:Mesh_Size,1)=FnRaddn(1:Mesh_Size,1) &
                   + Cijdn(nili,njlj)*Phi_ij_R2(1:Mesh_Size)

              FnRadup(1:Mesh_Size,2)=FnRadup(1:Mesh_Size,2) &
                   + Cijup(nili,njlj)*TPhi_ij_R2(1:Mesh_Size)

              FnRaddn(1:Mesh_Size,2)=FnRaddn(1:Mesh_Size,2) &
                   + Cijdn(nili,njlj)*TPhi_ij_R2(1:Mesh_Size)
           End Do
        End Do

        !*** Add CoreDen  ***
        FnRadup(1:Mesh_Size,1) = FnRadup(1:Mesh_Size,1) + &
             CoreDen(1:Mesh_Size)/2
        FnRaddn(1:Mesh_Size,1) = FnRaddn(1:Mesh_Size,1) + &
             CoreDen(1:Mesh_Size)/2
        FnRadup(1:Mesh_Size,2) = FnRadup(1:Mesh_Size,2) + &
             AT%CoreTail_Density(1:Mesh_Size)/2
        FnRaddn(1:Mesh_Size,2) = FnRaddn(1:Mesh_Size,2) + &
             AT%CoreTail_Density(1:Mesh_Size)/2

             !**** Calculate Vxc and Exc for Ra*****
        Do Rk=1, Mesh_Size
          
           Call pwlsdafunc((FnRadup(Rk,1)+FnRaddn(Rk,1)),(FnRadup(Rk,1)-FnRaddn(Rk,1)),Exc1,Vxc1even, Vxc1odd)
           Call pwlsdafunc((FnRadup(Rk,2)+FnRaddn(Rk,2)),(FnRadup(Rk,2)-FnRaddn(Rk,2)),Exc2,Vxc2even, Vxc2odd)
           
           FnRadup(Rk, 3:6) = (/Vxc1even, Vxc2even, Exc1, Exc2/)
           FnRaddn(Rk, 3:6) = (/Vxc1odd, Vxc2odd, Exc1, Exc2/)
        ! write(6,'(i5,1p7e15.7)') Rk,Vxc1,Vxc2,Exc1,Exc2
        End Do

        FnRadup(1:Mesh_Size,7) =   R2(1:Mesh_Size) * &
             (FnRadup(1:Mesh_Size,5)*(FnRadup(1:Mesh_Size,1)+FnRaddn(1:Mesh_Size,1)) &
             - FnRadup(1:Mesh_Size,6)*(FnRadup(1:Mesh_Size,2)+FnRaddn(1:Mesh_Size,2)))

        !FnRad(1:Mesh_Size,7) = FnRad(1:Mesh_Size,7) - &
        !   R2(1:Mesh_Size) *  &
        !     CoreXC(1:Mesh_Size) *  CoreDen(1:Mesh_Size)

        Efn= IntSimpson(Mesh_Size, DR, FnRadup(1:Mesh_Size,7),&
              AT%RadGrid) * &
              AngPoints(4,alpha)*4*Pi
        Energy = Energy + Efn 
       !write(6,*) 'alpha efn energy',alpha,efn,energy

        Count = 0
        Do nili=1, Basis_Size
           Li = AT%L_Value(nili)
           Ylm_i => Ylm(:, Li+1, Alpha)
           ibase = nl_Base(nili); ibase = ibase + nlm_lut(2,ibase);


           Do njlj=1, Basis_Size
              Lj = AT%L_Value(njlj)
              Ylm_j => Ylm(:, Lj+1, Alpha)
              jbase = nl_Base(njlj); jbase = jbase + nlm_lut(2,jbase);      
              count = count + 1

              Phi_ij_R2 => AT%Phi_ij_R2(:,count)
              TPhi_ij_R2 => AT%TPhi_ij_R2(:,count)

              !**** Radial Integral Portion ******
              FnRadup(1:Mesh_Size, 7) =  &
                 ((FnRadup(1:Mesh_Size,3)+FnRaddn(1:Mesh_Size,3)) * Phi_ij_R2(1:Mesh_Size) - &
                    (FnRadup(1:Mesh_Size,4)+FnRaddn(1:Mesh_Size,4)) * TPhi_ij_R2(1:Mesh_Size)) * &
                     R2(1:Mesh_Size) 

              FnRaddn(1:Mesh_Size, 7) =  &
                 ((FnRadup(1:Mesh_Size,3)-FnRaddn(1:Mesh_Size,3)) * Phi_ij_R2(1:Mesh_Size) - &
                    (FnRadup(1:Mesh_Size,4)-FnRaddn(1:Mesh_Size,4)) * TPhi_ij_R2(1:Mesh_Size)) * &
                     R2(1:Mesh_Size)
 
              FnRadup(1,7) = 0; FnRaddn(1,7) = 0
              FnRadup(Mesh_Size,7) = 0; FnRaddn(Mesh_Size,7) = 0
              
              fnup = IntSimpson(Mesh_Size, DR, FnRadup(1:Mesh_Size,7),&
                     AT%RadGrid) * &
                     AngPoints(4,alpha)*Four_Pi

              fndn = IntSimpson(Mesh_Size, DR, FnRaddn(1:Mesh_Size,7),&
                     AT%RadGrid) * &
                     AngPoints(4,alpha)*Four_Pi

              Do mi=-Li, Li
                 Do mj=-Lj, Lj
                    Dijup(ibase+mi,jbase+mj) = Dijup(ibase+mi,jbase+mj) + &
                         fnup * CONJG(Ylm_i(Li+Mi+1))*Ylm_j(Lj+mj+1)
                    Dijdn(ibase+mi,jbase+mj) = Dijdn(ibase+mi,jbase+mj) + &
                         fndn * CONJG(Ylm_i(Li+Mi+1))*Ylm_j(Lj+mj+1)
                 End Do
              End Do  
           end Do !njlj
        End do !nili
     End Do

  End Do

  Call Stop_Timer(Timer(AtomVxc_Timer))

   write(log_unit,*) 'AccumVxc_LSDA: finish'
   call flush(log_unit)
  Return
End Subroutine
!******************************************************************************
!                                                          ~a   a
!  AccumVxc_GGA - Accumulates the Exchange correlation pot for N + N  to Dij
!    for GGA in Perdew-Burke-Ezerholf form only
!
!******************************************************************************

Subroutine AccumVxc_GGA( Energy)
   Real,    Intent(INOUT) :: Energy

   Integer :: i,j,k, nili, njlj, mi, mj, Li, Lj, Rk, RS, count
   Integer ::nklk, nlll, mk, ml, Lk, Ll, alpha, Basis_Size, Rad_Size
   Integer :: ibase,jbase,kbase,lbase, Skip_Size, Rad_Index, Mesh_Size
   Integer, pointer :: nl_base(:), nlm_lut(:,:)
   Complex, Pointer :: Dij(:,:), Wij(:,:),  Ylm(:,:,:)
   Complex, Pointer :: dYlmdtheta(:,:,:), dYlmdphi(:,:,:)
   Real,    Pointer ::  R2(:), Phi_ij_R2(:), TPhi_ij_R2(:)
   Real,    Pointer :: dPhijdr(:),dTPhijdr(:)
   Real,    Pointer :: CoreXC(:), CoreDen(:),FnRad(:,:),CoreGrad(:)
   Complex          :: ylm1, ylm2,ylm1t,ylm1p,ylm2t,ylm2p
   Complex, Pointer :: Ylm_i(:), Ylm_j(:)
   Complex, Pointer :: dYlmdt_i(:), dYlmdt_j(:)
   Complex, Pointer :: dYlmdp_i(:), dYlmdp_j(:)
   Real             ::  Vxc1, Exc1, Vxc2, Exc2, fnint, Esum, PhiRk, TPhiRk
   Real             ::  fxc,gxc,dfxcdn,dfxcdg
   Complex          :: den_phi, den_Tphi, den, dent, denp
   Real             :: Efn,fn,fnt,fnp,Fnsum
   Type (Specific_Atom), Pointer :: A
   Type (Atom_Info_Fixed), Pointer :: AT
   Real             :: DR,RR,grad
   Real,    Pointer :: Cij(:,:),dCijdtheta(:,:),dCijdphi(:,:)
   Character*100    :: Msg

   if(spindependence) then 
        write(error_unit, *) 'AccumVxc_GGA: error -- spindependence should be false'
        stop
   endif 

   Call Start_Timer(Timer(AtomVxc_Timer))
!   Write(msg,*) 'AccumVxc: Error allocating Cij. Size=',Basis_Size
!   Call Check_Error(j, msg, Error_Unit, .TRUE.,paw_wc, "AccumVxc:")
   write(log_unit,*) 'AccumVxc_GGA: start'
   call flush(log_unit)
   
   Do i=1, Specific_Atoms
     A => Atom_List(i)
     AT => AtomType_Info(A%TypeIndex)
     nl_base => AT%nl_Base
     nlm_LUT => AT%nlm_LUT
     Wij => A%Wij
     Dij => A%Dij
     CoreDen => AT%Core_Density
     CoreGrad => AT%Core_dDendr
     CoreXC => AT%Core_Xchange
     R2 => AT%R2
     DR = AT%Mesh_Step   ! assume that no steps are skipped!
     Ylm => AT%Ylm
     dYlmdtheta => AT%dYlmdtheta
     dYlmdphi => AT%dYlmdphi
     Cij => AT%Cij
     dCijdtheta => AT%dCijdtheta
     dCijdphi => AT%dCijdphi
     FnRad => AT%FnRad
     Mesh_size = AT%Rad_Size


      DR = AT%Mesh_Step
      !DR = AT%Mesh_Step*AT%Rad_Skip
      if (AT%Rad_Skip/=1) then
         write(Log_Unit,*) 'Error in VXC -- Rad_Skip', AT%Rad_Skip
         stop
      endif

     Basis_Size = AT%Basis_Size

     FnSum = 0; ESum = 0

    Do alpha = 1, Num_AngPoints
        Cij = 0
        dCijdtheta=0  ; dCijdphi=0
        Do nili=1, Basis_Size     !*** Calulate Cij for n^a and n~^a
           Li = AT%L_Value(nili)
           Ylm_i => Ylm(:, Li+1, alpha)
           dYlmdt_i => dYlmdtheta(:, Li+1, alpha)
           dYlmdp_i => dYlmdphi(:, Li+1, alpha)
           ibase = nl_Base(nili);  ibase = ibase + nlm_lut(2,ibase);      
           Do njlj=1, Basis_Size
              Lj = AT%L_Value(njlj)
              Ylm_j => Ylm(:, Lj+1, alpha)
              dYlmdt_j => dYlmdtheta(:, Lj+1, alpha)
              dYlmdp_j => dYlmdphi(:, Lj+1, alpha)
              jbase = nl_Base(njlj);  jbase = jbase + nlm_lut(2,jbase);      
           
              den = 0 ; dent = 0 ;   denp = 0
              Do mi = -Li, Li
                 ylm1 = CONJG(Ylm_i(mi+Li+1))
                 ylm1t = CONJG(dYlmdt_i(mi+Li+1))
                 ylm1p = CONJG(dYlmdp_i(mi+Li+1))
              
                 Do mj=-Lj, Lj
                    ylm2 = Ylm_j(mj+Lj+1)
                    ylm2t = dYlmdt_j(mj+Lj+1)
                    ylm2p = dYlmdp_j(mj+Lj+1)
                    den = den + Wij(ibase+mi,jbase+mj) * Ylm1 * Ylm2
                    dent = dent + Wij(ibase+mi,jbase+mj) * &
                         (ylm1t * Ylm2 + ylm1 * ylm2t)
                    denp = denp + Wij(ibase+mi,jbase+mj) * &
                         (ylm1p * Ylm2 + ylm1 * ylm2p)
                 End Do
              End Do

              Cij(nili, njlj) = Den
              dCijdtheta(nili, njlj) = Dent
              dCijdphi(nili, njlj) = Denp
           End Do
        End Do

        !**** Calculate the Radial Densities and store them *****
        FnRad = 0

        Count = 0
        Do nili=1, Basis_Size
           Do njlj=1, Basis_Size
              count = count + 1
              Phi_ij_R2 => AT%Phi_ij_R2(:,count)
              TPhi_ij_R2 => AT%TPhi_ij_R2(:,count)
              dPhijdr => AT%dPhi_ijdr(:,count)
              dTPhijdr => AT%dTPhi_ijdr(:,count)

              FnRad(1:Mesh_Size,1)=FnRad(1:Mesh_Size,1) &          !n
                   + Cij(nili,njlj)*Phi_ij_R2(1:Mesh_Size)

              FnRad(1:Mesh_Size,2)=FnRad(1:Mesh_Size,2) &          !~n 
                   + Cij(nili,njlj)*TPhi_ij_R2(1:Mesh_Size)

              FnRad(1:Mesh_Size,3)=FnRad(1:Mesh_Size,3) &          !dndr
                   + Cij(nili,njlj)*dPhijdr(1:Mesh_Size)

              FnRad(1:Mesh_Size,4)=FnRad(1:Mesh_Size,4) &          !d~ndr 
                   + Cij(nili,njlj)*dTPhijdr(1:Mesh_Size)

              FnRad(1:Mesh_Size,5)=FnRad(1:Mesh_Size,5) &          !dndtheta
                   + dCijdtheta(nili,njlj)*Phi_ij_R2(1:Mesh_Size)

              FnRad(1:Mesh_Size,6)=FnRad(1:Mesh_Size,6) &          !d~ndtheta
                   + dCijdtheta(nili,njlj)*TPhi_ij_R2(1:Mesh_Size)

              FnRad(1:Mesh_Size,7)=FnRad(1:Mesh_Size,7) &          !dndphi
                   + dCijdphi(nili,njlj)*Phi_ij_R2(1:Mesh_Size)

              FnRad(1:Mesh_Size,8)=FnRad(1:Mesh_Size,8) &          !d~ndphi
                   + dCijdphi(nili,njlj)*TPhi_ij_R2(1:Mesh_Size)

           End Do
        End Do

        !*** Add CoreDen  ***
        FnRad(1:Mesh_Size,1) = FnRad(1:Mesh_Size,1) + &
             CoreDen(1:Mesh_Size)
        FnRad(1:Mesh_Size,3) = FnRad(1:Mesh_Size,3) + &
             CoreGrad(1:Mesh_Size)
        FnRad(1:Mesh_Size,2) = FnRad(1:Mesh_Size,2) + &
             AT%CoreTail_Density(1:Mesh_Size)
        FnRad(1:Mesh_Size,4) = FnRad(1:Mesh_Size,4) + &
             AT%GradCoreTail(1:Mesh_Size)

             !**** Calculate Vxc, Gxc,  and Fxc *****
        Do Rk=2, Mesh_Size
          If (AT%RadGrid%islog) then
           RR=AT%RadGrid%drdu(Rk)-AT%RadGrid%drdu(1)
          else
           RR=DR*(Rk-1)
          endif
           grad=SQRT(FnRad(Rk,3)**2+(FnRad(Rk,5)/RR)**2+(FnRad(Rk,7)/RR)**2)
           Call pbefunc((FnRad(Rk,1)),grad,FnRad(Rk,9),&
                         FnRad(Rk,11),FnRad(Rk,13))
            !!!If (grad>machine_zero) then   ! no longer term has /grad          
            !!!      FnRad(Rk,13)=FnRad(Rk,13)/grad
            !!!Else
            !!!      FnRad(Rk,13)=0.d0
            !!!EndIf
           grad=SQRT(FnRad(Rk,4)**2+(FnRad(Rk,6)/RR)**2+(FnRad(Rk,8)/RR)**2)
           Call pbefunc((FnRad(Rk,2)),grad,FnRad(Rk,10),&
                         FnRad(Rk,12),FnRad(Rk,14))
            !!!If (grad>machine_zero) then
            !!!     FnRad(Rk,14)=FnRad(Rk,14)/grad
            !!!Else
            !!!     FnRad(Rk,14)=0.d0
            !!!EndIf
        End Do

        !FnRad(2:Mesh_Size,15) =   R2(2:Mesh_Size) * &
        !     (FnRad(2:Mesh_Size,9) - CoreXC(2:Mesh_Size) &
        !     - FnRad(2:Mesh_Size,10))
        FnRad(2:Mesh_Size,15) =   R2(2:Mesh_Size) * &
             (FnRad(2:Mesh_Size,9) - FnRad(2:Mesh_Size,10))
        FnRad(1,15)=0.d0

        Efn= IntSimpson(Mesh_Size, DR, FnRad(1:Mesh_Size,15),&
              AT%RadGrid) * &
              AngPoints(4,alpha)*4*Pi
        Energy = Energy + Efn 

        Count = 0
        Do nili=1, Basis_Size
           Li = AT%L_Value(nili)
           Ylm_i => Ylm(:, Li+1, Alpha)
           dYlmdt_i => dYlmdtheta(:, Li+1, alpha)
           dYlmdp_i => dYlmdphi(:, Li+1, alpha)
           ibase = nl_Base(nili); ibase = ibase + nlm_lut(2,ibase);


           Do njlj=1, Basis_Size
              Lj = AT%L_Value(njlj)
              Ylm_j => Ylm(:, Lj+1, Alpha)
              dYlmdt_j => dYlmdtheta(:, Lj+1, alpha)
              dYlmdp_j => dYlmdphi(:, Lj+1, alpha)
              jbase = nl_Base(njlj); jbase = jbase + nlm_lut(2,jbase);      
              count = count + 1

              Phi_ij_R2 => AT%Phi_ij_R2(:,count)
              TPhi_ij_R2 => AT%TPhi_ij_R2(:,count)
              dPhijdr => AT%dPhi_ijdr(:,count)
              dTPhijdr => AT%dTPhi_ijdr(:,count)

              !**** Radial Integral Portion ******
              FnRad(1,1) = 0; FnRad(Mesh_Size,1) = 0
              FnRad(1,2) = 0; FnRad(Mesh_Size,2) = 0
              FnRad(1,15) = 0; FnRad(Mesh_Size,15) = 0

              Rk=Mesh_Size-1
              
              FnRad(2:Rk,15) =  R2(2:Rk)* &
                   (FnRad(2:Rk,11) * Phi_ij_R2(2:Rk) - &
                    FnRad(2:Rk,12) * TPhi_ij_R2(2:Rk) + &
                    FnRad(2:Rk,13) * FnRad(2:Rk,3) * dPhijdr(2:Rk) - &
                    FnRad(2:Rk,14) * FnRad(2:Rk,4) * dTphijdr(2:Rk))

              FnRad(2:Rk,1) =  &
                   (FnRad(2:Rk,13) * FnRad(2:Rk,5) * Phi_ij_R2(2:Rk) - &
                    FnRad(2:Rk,14) * FnRad(2:Rk,6) * Tphi_ij_R2(2:Rk))

              FnRad(2:Rk,2) =  &
                   (FnRad(2:Rk,13) * FnRad(2:Rk,7) * Phi_ij_R2(2:Rk) - &
                    FnRad(2:Rk,14) * FnRad(2:Rk,8) * Tphi_ij_R2(2:Rk))


              fn = IntSimpson(Mesh_Size, DR, FnRad(1:Mesh_Size,15),&
                     AT%RadGrid) * &
                     AngPoints(4,alpha)*Four_Pi

              fnt= IntSimpson(Mesh_Size, DR, FnRad(1:Mesh_Size,1),&
                     AT%RadGrid) * &
                     AngPoints(4,alpha)*Four_Pi

              fnp = IntSimpson(Mesh_Size, DR, FnRad(1:Mesh_Size,2),&
                     AT%RadGrid) * &
                     AngPoints(4,alpha)*Four_Pi

              Do mi=-Li, Li
                Do mj=-Lj, Lj
                    Dij(ibase+mi,jbase+mj) = Dij(ibase+mi,jbase+mj) + &
                         fn * CONJG(Ylm_i(Li+Mi+1))*Ylm_j(Lj+mj+1) + &
                         fnt*(CONJG(dYlmdt_i(Li+mi+1))*Ylm_j(Lj+mj+1) + &
                             CONJG(Ylm_i(Li+mi+1))*dYlmdt_j(Lj+mj+1)) + &
                         fnp*(CONJG(dYlmdp_i(Li+mi+1))*Ylm_j(Lj+mj+1) + &
                             CONJG(Ylm_i(Li+mi+1))*dYlmdp_j(Lj+mj+1))
                 End Do
              End Do  
           end Do !njlj
        End do !nili
     End Do

  End Do

  Call Stop_Timer(Timer(AtomVxc_Timer))

   write(log_unit,*) 'AccumVxc_GGA: finish'
   call flush(log_unit)
  Return
End Subroutine

Subroutine AccumVxc_SGGA( Energy)
   Real,    Intent(INOUT) :: Energy

   Integer :: i,j,k, nili, njlj, mi, mj, Li, Lj, Rk, RS, count
   Integer ::nklk, nlll, mk, ml, Lk, Ll, alpha, Basis_Size, Rad_Size
   Integer :: ibase,jbase,kbase,lbase, Skip_Size, Rad_Index, Mesh_Size
   Integer, pointer :: nl_base(:), nlm_lut(:,:)
   Complex, Pointer :: Dijup(:,:), Dijdn(:,:), Wijup(:,:), Wijdn(:,:), Ylm(:,:,:)
   Complex, Pointer :: dYlmdtheta(:,:,:), dYlmdphi(:,:,:)
   Real,    Pointer ::  R2(:), Phi_ij_R2(:), TPhi_ij_R2(:)
   Real,    Pointer :: dPhijdr(:),dTPhijdr(:)
   Real,    Pointer :: CoreXC(:), CoreDen(:),FnRadup(:,:),FnRaddn(:,:),CoreGrad(:)
   Complex          :: ylm1, ylm2,ylm1t,ylm1p,ylm2t,ylm2p
   Complex, Pointer :: Ylm_i(:), Ylm_j(:)
   Complex, Pointer :: dYlmdt_i(:), dYlmdt_j(:)
   Complex, Pointer :: dYlmdp_i(:), dYlmdp_j(:)
   Real             ::  Vxc1even,Vxc1odd, Exc1, Vxc2even,Vxc2odd, &
                        Exc2, fnint, Esum, PhiRk, TPhiRk
   Real             ::  fxc,gxc,dfxcdn,dfxcdg
   Complex          :: den_phi, den_Tphi, denup, dendn, &
                        dentup, dentdn, denpup, denpdn
   Real             :: Efn,fnup,fndn,fntup,fntdn,fnpup,fnpdn,Fnsum
   Type (Specific_Atom), Pointer :: A
   Type (Atom_Info_Fixed), Pointer :: AT
   Real             :: DR,RR,gradup,graddn,grad
   Real,    Pointer :: Cijup(:,:),Cijdn(:,:),dCijdthetaup(:,:),dCijdthetadn(:,:),dCijdphiup(:,:),dCijdphidn(:,:)
   Character*100    :: Msg


   Call Start_Timer(Timer(AtomVxc_Timer))
!   Write(msg,*) 'AccumVxc: Error allocating Cij. Size=',Basis_Size
!   Call Check_Error(j, msg, Error_Unit, .TRUE.,paw_wc, "AccumVxc:")
   if(.not.spindependence) then 
        write(error_unit, *) 'AccumVxc_SGGA: error -- spindependence should be true'
        stop
   endif 

   write(log_unit,*) 'AccumVxc_SGGA: start'
   call flush(log_unit)
   
   Do i=1, Specific_Atoms
     A => Atom_List(i)
     AT => AtomType_Info(A%TypeIndex)
     nl_base => AT%nl_Base
     nlm_LUT => AT%nlm_LUT
     Wijup => A%Wij
     Wijdn => A%Wijspin
     Dijup => A%Dij
     Dijdn => A%Dijspin
     CoreDen => AT%Core_Density
     CoreGrad => AT%Core_dDendr
     CoreXC => AT%Core_Xchange
     R2 => AT%R2
     DR = AT%Mesh_Step   ! assume that no steps are skipped!
     Ylm => AT%Ylm
     dYlmdtheta => AT%dYlmdtheta
     dYlmdphi => AT%dYlmdphi
     Cijup => AT%Cij
     Cijdn => AT%Cijspin
     dCijdthetaup => AT%dCijdtheta
     dCijdthetadn => AT%dCijdthetaspin
     dCijdphiup => AT%dCijdphi
     dCijdphidn => AT%dCijdphispin
     FnRadup => AT%FnRad
     FnRaddn => AT%FnRadspin
     Mesh_size = AT%Rad_Size


      DR = AT%Mesh_Step
      !DR = AT%Mesh_Step*AT%Rad_Skip
      if (AT%Rad_Skip/=1) then
         write(Log_Unit,*) 'Error in VXC -- Rad_Skip', AT%Rad_Skip
         stop
      endif

     Basis_Size = AT%Basis_Size

     FnSum = 0; ESum = 0

    Do alpha = 1, Num_AngPoints
        Cijup = 0; Cijdn = 0
        dCijdthetaup=0  ; dCijdthetadn=0; dCijdphiup=0; dCijdphidn=0
        Do nili=1, Basis_Size     !*** Calulate Cij for n^a and n~^a
           Li = AT%L_Value(nili)
           Ylm_i => Ylm(:, Li+1, alpha)
           dYlmdt_i => dYlmdtheta(:, Li+1, alpha)
           dYlmdp_i => dYlmdphi(:, Li+1, alpha)
           ibase = nl_Base(nili);  ibase = ibase + nlm_lut(2,ibase);      
           Do njlj=1, Basis_Size
              Lj = AT%L_Value(njlj)
              Ylm_j => Ylm(:, Lj+1, alpha)
              dYlmdt_j => dYlmdtheta(:, Lj+1, alpha)
              dYlmdp_j => dYlmdphi(:, Lj+1, alpha)
              jbase = nl_Base(njlj);  jbase = jbase + nlm_lut(2,jbase);      
           
              denup = 0 ; dendn = 0; dentup = 0 ; dentdn = 0;  denpup = 0; denpdn = 0
              Do mi = -Li, Li
                 ylm1 = CONJG(Ylm_i(mi+Li+1))
                 ylm1t = CONJG(dYlmdt_i(mi+Li+1))
                 ylm1p = CONJG(dYlmdp_i(mi+Li+1))
              
                 Do mj=-Lj, Lj
                    ylm2 = Ylm_j(mj+Lj+1)
                    ylm2t = dYlmdt_j(mj+Lj+1)
                    ylm2p = dYlmdp_j(mj+Lj+1)
                    denup = denup + Wijup(ibase+mi,jbase+mj) * Ylm1 * Ylm2
                    dendn = dendn + Wijdn(ibase+mi,jbase+mj) * Ylm1 * Ylm2
                    dentup = dentup + Wijup(ibase+mi,jbase+mj) * &
                         (ylm1t * Ylm2 + ylm1 * ylm2t)
                    dentdn = dentdn + Wijdn(ibase+mi,jbase+mj) * &
                         (ylm1t * Ylm2 + ylm1 * ylm2t)
                    denpup = denpup + Wijup(ibase+mi,jbase+mj) * &
                         (ylm1p * Ylm2 + ylm1 * ylm2p)
                    denpdn = denpdn + Wijdn(ibase+mi,jbase+mj) * &
                         (ylm1p * Ylm2 + ylm1 * ylm2p)
                 End Do
              End Do

              Cijup(nili, njlj) = Denup
              Cijdn(nili, njlj) = Dendn
              dCijdthetaup(nili, njlj) = Dentup
              dCijdthetadn(nili, njlj) = Dentdn
              dCijdphiup(nili, njlj) = Denpup
              dCijdphidn(nili, njlj) = Denpdn
           End Do
        End Do

        !**** Calculate the Radial Densities and store them *****
        FnRadup = 0; FnRaddn = 0

        Count = 0
        Do nili=1, Basis_Size
           Do njlj=1, Basis_Size
              count = count + 1
              Phi_ij_R2 => AT%Phi_ij_R2(:,count)
              TPhi_ij_R2 => AT%TPhi_ij_R2(:,count)
              dPhijdr => AT%dPhi_ijdr(:,count)
              dTPhijdr => AT%dTPhi_ijdr(:,count)

              FnRadup(1:Mesh_Size,1)=FnRadup(1:Mesh_Size,1) &          !n
                   + Cijup(nili,njlj)*Phi_ij_R2(1:Mesh_Size)
              FnRaddn(1:Mesh_Size,1)=FnRaddn(1:Mesh_Size,1) &          !n
                   + Cijdn(nili,njlj)*Phi_ij_R2(1:Mesh_Size)

              FnRadup(1:Mesh_Size,2)=FnRadup(1:Mesh_Size,2) &          !~n 
                   + Cijup(nili,njlj)*TPhi_ij_R2(1:Mesh_Size)
              FnRaddn(1:Mesh_Size,2)=FnRaddn(1:Mesh_Size,2) &          !~n 
                   + Cijdn(nili,njlj)*TPhi_ij_R2(1:Mesh_Size)

              FnRadup(1:Mesh_Size,3)=FnRadup(1:Mesh_Size,3) &          !dndr
                   + Cijup(nili,njlj)*dPhijdr(1:Mesh_Size)
              FnRaddn(1:Mesh_Size,3)=FnRaddn(1:Mesh_Size,3) &          !dndr
                   + Cijdn(nili,njlj)*dPhijdr(1:Mesh_Size)

              FnRadup(1:Mesh_Size,4)=FnRadup(1:Mesh_Size,4) &          !d~ndr 
                   + Cijup(nili,njlj)*dTPhijdr(1:Mesh_Size)
              FnRaddn(1:Mesh_Size,4)=FnRaddn(1:Mesh_Size,4) &          !d~ndr 
                   + Cijdn(nili,njlj)*dTPhijdr(1:Mesh_Size)

              FnRadup(1:Mesh_Size,5)=FnRadup(1:Mesh_Size,5) &          !dndtheta
                   + dCijdthetaup(nili,njlj)*Phi_ij_R2(1:Mesh_Size)
              FnRaddn(1:Mesh_Size,5)=FnRaddn(1:Mesh_Size,5) &          !dndtheta
                   + dCijdthetadn(nili,njlj)*Phi_ij_R2(1:Mesh_Size)

              FnRadup(1:Mesh_Size,6)=FnRadup(1:Mesh_Size,6) &          !d~ndtheta
                   + dCijdthetaup(nili,njlj)*TPhi_ij_R2(1:Mesh_Size)
              FnRaddn(1:Mesh_Size,6)=FnRaddn(1:Mesh_Size,6) &          !d~ndtheta
                   + dCijdthetadn(nili,njlj)*TPhi_ij_R2(1:Mesh_Size)

              FnRadup(1:Mesh_Size,7)=FnRadup(1:Mesh_Size,7) &          !dndphi
                   + dCijdphiup(nili,njlj)*Phi_ij_R2(1:Mesh_Size)
              FnRaddn(1:Mesh_Size,7)=FnRaddn(1:Mesh_Size,7) &          !dndphi
                   + dCijdphidn(nili,njlj)*Phi_ij_R2(1:Mesh_Size)

              FnRadup(1:Mesh_Size,8)=FnRadup(1:Mesh_Size,8) &          !d~ndphi
                   + dCijdphiup(nili,njlj)*TPhi_ij_R2(1:Mesh_Size)
              FnRaddn(1:Mesh_Size,8)=FnRaddn(1:Mesh_Size,8) &          !d~ndphi
                   + dCijdphidn(nili,njlj)*TPhi_ij_R2(1:Mesh_Size)

           End Do
        End Do

        !*** Add CoreDen  ***
        FnRadup(1:Mesh_Size,1) = FnRadup(1:Mesh_Size,1) + &
             CoreDen(1:Mesh_Size)/2
        FnRaddn(1:Mesh_Size,1) = FnRaddn(1:Mesh_Size,1) + &
             CoreDen(1:Mesh_Size)/2

        FnRadup(1:Mesh_Size,3) = FnRadup(1:Mesh_Size,3) + &
             CoreGrad(1:Mesh_Size)/2
        FnRaddn(1:Mesh_Size,3) = FnRaddn(1:Mesh_Size,3) + &
             CoreGrad(1:Mesh_Size)/2

        FnRadup(1:Mesh_Size,2) = FnRadup(1:Mesh_Size,2) + &
             AT%CoreTail_Density(1:Mesh_Size)/2
        FnRaddn(1:Mesh_Size,2) = FnRaddn(1:Mesh_Size,2) + &
             AT%CoreTail_Density(1:Mesh_Size)/2

        FnRadup(1:Mesh_Size,4) = FnRadup(1:Mesh_Size,4) + &
             AT%GradCoreTail(1:Mesh_Size)/2
        FnRaddn(1:Mesh_Size,4) = FnRaddn(1:Mesh_Size,4) + &
             AT%GradCoreTail(1:Mesh_Size)/2

             !**** Calculate Vxc, Gxc,  and Fxc *****
        Do Rk=2, Mesh_Size
          If (AT%RadGrid%islog) then
           RR=AT%RadGrid%drdu(Rk)-AT%RadGrid%drdu(1)
          else
           RR=DR*(Rk-1)
          endif
           gradup=SQRT(FnRadup(Rk,3)**2+(FnRadup(Rk,5)/RR)**2+&
              (FnRadup(Rk,7)/RR)**2)
           graddn=SQRT(FnRaddn(Rk,3)**2+(FnRaddn(Rk,5)/RR)**2+&
              (FnRaddn(Rk,7)/RR)**2)
           grad=SQRT((FnRadup(Rk,3)+FnRaddn(Rk,3))**2&
                     +((FnRadup(Rk,5)+FnRaddn(Rk,5))/RR)**2&
                     +((FnRadup(Rk,7)+FnRaddn(Rk,7))/RR)**2)

            !!!FnRadup(Rk,11) stores dfxcdneven, FnRaddn(Rk,11) stores dfxcdnodd
            !!!FnRadup(Rk,16) stores dcdgbg, other are corresponding to upspin and dnspin
           Call Sggapbefunc(FnRadup(Rk,1),gradup,FnRaddn(Rk,1),graddn,&
                            grad,FnRadup(Rk,9),FnRadup(Rk,11),FnRaddn(Rk,11),&
                            FnRadup(Rk,13),FnRaddn(Rk,13),FnRadup(Rk,16))
            !!!If (grad>machine_zero) then   ! no longer term has /grad          
            !!!      FnRad(Rk,13)=FnRad(Rk,13)/grad
            !!!Else
            !!!      FnRad(Rk,13)=0.d0
            !!!EndIf
           gradup=SQRT(FnRadup(Rk,4)**2+(FnRadup(Rk,6)/RR)**2+&
                   (FnRadup(Rk,8)/RR)**2)
           graddn=SQRT(FnRaddn(Rk,4)**2+(FnRaddn(Rk,6)/RR)**2+&
                   (FnRaddn(Rk,8)/RR)**2)
           grad=SQRT((FnRadup(Rk,4)+FnRaddn(Rk,4))**2&
                     +((FnRadup(Rk,6)+FnRaddn(Rk,6))/RR)**2&
                     +((FnRadup(Rk,8)+FnRaddn(Rk,8))/RR)**2)

            !!!FnRadup(Rk,12) stores dfxcdneven, FnRaddn(Rk,12) stores dfxcdnodd
            !!!FnRaddn(Rk,16) stores dcdgbg, other are correspondent to upspin and dnspin
           Call Sggapbefunc(FnRadup(Rk,2),gradup,FnRaddn(Rk,2),graddn,&
                            grad,FnRadup(Rk,10),FnRadup(Rk,12),FnRaddn(Rk,12),&
                            FnRadup(Rk,14),FnRaddn(Rk,14),FnRaddn(Rk,16))
          
            !!!If (grad>machine_zero) then
            !!!     FnRad(Rk,14)=FnRad(Rk,14)/grad
            !!!Else
            !!!     FnRad(Rk,14)=0.d0
            !!!EndIf
        End Do

        !FnRad(2:Mesh_Size,15) =   R2(2:Mesh_Size) * &
        !     (FnRad(2:Mesh_Size,9) - CoreXC(2:Mesh_Size) &
        !     - FnRad(2:Mesh_Size,10))
        FnRadup(2:Mesh_Size,15) =   R2(2:Mesh_Size) * &
             (FnRadup(2:Mesh_Size,9) - FnRadup(2:Mesh_Size,10))
        FnRadup(1,15)=0.d0; FnRaddn(1,15)=0.d0

        Efn= IntSimpson(Mesh_Size, DR, FnRadup(1:Mesh_Size,15),&
              AT%RadGrid) * &
              AngPoints(4,alpha)*4*Pi

        Energy = Energy + Efn 

        Count = 0
        Do nili=1, Basis_Size
           Li = AT%L_Value(nili)
           Ylm_i => Ylm(:, Li+1, Alpha)
           dYlmdt_i => dYlmdtheta(:, Li+1, alpha)
           dYlmdp_i => dYlmdphi(:, Li+1, alpha)
           ibase = nl_Base(nili); ibase = ibase + nlm_lut(2,ibase);


           Do njlj=1, Basis_Size
              Lj = AT%L_Value(njlj)
              Ylm_j => Ylm(:, Lj+1, Alpha)
              dYlmdt_j => dYlmdtheta(:, Lj+1, alpha)
              dYlmdp_j => dYlmdphi(:, Lj+1, alpha)
              jbase = nl_Base(njlj); jbase = jbase + nlm_lut(2,jbase);      
              count = count + 1

              Phi_ij_R2 => AT%Phi_ij_R2(:,count)
              TPhi_ij_R2 => AT%TPhi_ij_R2(:,count)
              dPhijdr => AT%dPhi_ijdr(:,count)
              dTPhijdr => AT%dTPhi_ijdr(:,count)

              !**** Radial Integral Portion ******
              FnRadup(1,1) = 0; FnRadup(Mesh_Size,1) = 0
              FnRadup(1,2) = 0; FnRadup(Mesh_Size,2) = 0
              FnRadup(1,15) = 0; FnRadup(Mesh_Size,15) = 0
              FnRaddn(1,1) = 0; FnRaddn(Mesh_Size,1) = 0
              FnRaddn(1,2) = 0; FnRaddn(Mesh_Size,2) = 0
              FnRaddn(1,15) = 0; FnRaddn(Mesh_Size,15) = 0

              Rk=Mesh_Size-1

             ! vxcup=(vxceven+vxcodd)/2-(dgxdrup+dgcdr/2) 
             ! vxcdn=(vxceven-vxcodd)/2-(dgxdrup+dgcdr/2)
             ! so next FnRadup(2:Rk,15) actually stores vxceven
             ! and FnRaddn(2:Rk,15) stores vxcodd

!              FnRadup(2:Rk,15) =  R2(2:Rk)* &
!                    ((FnRadup(2:Rk,11)+FnRaddn(2:Rk,11)) * Phi_ij_R2(2:Rk) - &
!                    (FnRadup(2:Rk,12)+FnRaddn(2:Rk,12)) * TPhi_ij_R2(2:Rk) + &
!                    (FnRadup(2:Rk,13) * FnRadup(2:Rk,3) + &
!                    FnRadup(2:Rk,16)*(FnRadup(2:Rk,3)+FnRaddn(2:Rk,3))/2)* dPhijdr(2:Rk) - & 
!                    (FnRadup(2:Rk,14) * FnRadup(2:Rk,4) + &
!                    FnRaddn(2:Rk,16)*(FnRadup(2:Rk,4)+FnRaddn(2:Rk,4))/2)* dTphijdr(2:Rk))

              FnRadup(2:Rk,15) =  R2(2:Rk)* &
                    ((FnRadup(2:Rk,11)+FnRaddn(2:Rk,11)) * Phi_ij_R2(2:Rk) - &
                    (FnRadup(2:Rk,12)+FnRaddn(2:Rk,12)) * TPhi_ij_R2(2:Rk) + &
                    (FnRadup(2:Rk,13) * FnRadup(2:Rk,3) + &
                    FnRadup(2:Rk,16)*(FnRadup(2:Rk,3)+FnRaddn(2:Rk,3)))* dPhijdr(2:Rk) - & 
                    (FnRadup(2:Rk,14) * FnRadup(2:Rk,4) + &
                    FnRaddn(2:Rk,16)*(FnRadup(2:Rk,4)+FnRaddn(2:Rk,4)))* dTphijdr(2:Rk))

!              FnRaddn(2:Rk,15) =  R2(2:Rk)* &
!                   ((FnRadup(2:Rk,11)-FnRaddn(2:Rk,11)) * Phi_ij_R2(2:Rk) - &
!                    (FnRadup(2:Rk,12)-FnRaddn(2:Rk,12)) * TPhi_ij_R2(2:Rk) + &
!                    (FnRaddn(2:Rk,13) * FnRaddn(2:Rk,3) + &
!                    FnRadup(2:Rk,16)*(FnRadup(2:Rk,3)+FnRaddn(2:Rk,3))/2) * dPhijdr(2:Rk) - &
!                    (FnRaddn(2:Rk,14) * FnRaddn(2:Rk,4) + &
!                    FnRaddn(2:Rk,16)*(FnRadup(2:Rk,4)+FnRaddn(2:Rk,4))/2) * dTphijdr(2:Rk))

              FnRaddn(2:Rk,15) =  R2(2:Rk)* &
                   ((FnRadup(2:Rk,11)-FnRaddn(2:Rk,11)) * Phi_ij_R2(2:Rk) - &
                    (FnRadup(2:Rk,12)-FnRaddn(2:Rk,12)) * TPhi_ij_R2(2:Rk) + &
                    (FnRaddn(2:Rk,13) * FnRaddn(2:Rk,3) + &
                    FnRadup(2:Rk,16)*(FnRadup(2:Rk,3)+FnRaddn(2:Rk,3))) * dPhijdr(2:Rk) - &
                    (FnRaddn(2:Rk,14) * FnRaddn(2:Rk,4) + &
                    FnRaddn(2:Rk,16)*(FnRadup(2:Rk,4)+FnRaddn(2:Rk,4))) * dTphijdr(2:Rk))

!              FnRadup(2:Rk,1) =  &
!                   (FnRadup(2:Rk,13) * FnRadup(2:Rk,5) + &
!                    FnRadup(2:Rk,16) * (FnRadup(2:Rk,5)+FnRaddn(2:Rk,5))/2) * Phi_ij_R2(2:Rk) - &
!                    (FnRadup(2:Rk,14) * FnRadup(2:Rk,6) + &
!                    FnRaddn(2:Rk,16) * (FnRadup(2:Rk,6)+FnRaddn(2:Rk,6))/2) * Tphi_ij_R2(2:Rk)

              FnRadup(2:Rk,1) =  &
                   (FnRadup(2:Rk,13) * FnRadup(2:Rk,5) + &
                    FnRadup(2:Rk,16) * (FnRadup(2:Rk,5)+FnRaddn(2:Rk,5))) * Phi_ij_R2(2:Rk) - &
                    (FnRadup(2:Rk,14) * FnRadup(2:Rk,6) + &
                    FnRaddn(2:Rk,16) * (FnRadup(2:Rk,6)+FnRaddn(2:Rk,6))) * Tphi_ij_R2(2:Rk)

!              FnRaddn(2:Rk,1) =  &
!                   (FnRaddn(2:Rk,13) * FnRaddn(2:Rk,5) + &
!                    FnRadup(2:Rk,16) * (FnRadup(2:Rk,5)+FnRaddn(2:Rk,5))/2) * Phi_ij_R2(2:Rk) - &
!                    (FnRaddn(2:Rk,14) * FnRaddn(2:Rk,6) + &
!                    FnRaddn(2:Rk,16) * (FnRadup(2:Rk,6)+FnRaddn(2:Rk,6))/2) * Tphi_ij_R2(2:Rk)

              FnRaddn(2:Rk,1) =  &
                   (FnRaddn(2:Rk,13) * FnRaddn(2:Rk,5) + &
                    FnRadup(2:Rk,16) * (FnRadup(2:Rk,5)+FnRaddn(2:Rk,5))) * Phi_ij_R2(2:Rk) - &
                    (FnRaddn(2:Rk,14) * FnRaddn(2:Rk,6) + &
                    FnRaddn(2:Rk,16) * (FnRadup(2:Rk,6)+FnRaddn(2:Rk,6))) * Tphi_ij_R2(2:Rk)

!              FnRadup(2:Rk,2) =  &
!                   (FnRadup(2:Rk,13) * FnRadup(2:Rk,7) + &
!                    FnRadup(2:Rk,16) * (FnRadup(2:Rk,7)+FnRaddn(2:Rk,7))/2) * Phi_ij_R2(2:Rk) - &
!                    (FnRadup(2:Rk,14) * FnRadup(2:Rk,8) + &
!                    FnRaddn(2:Rk,16) * (FnRadup(2:Rk,7)+FnRaddn(2:Rk,7))/2) * Tphi_ij_R2(2:Rk)

              FnRadup(2:Rk,2) =  &
                   (FnRadup(2:Rk,13) * FnRadup(2:Rk,7) + &
                    FnRadup(2:Rk,16) * (FnRadup(2:Rk,7)+FnRaddn(2:Rk,7))) * Phi_ij_R2(2:Rk) - &
                    (FnRadup(2:Rk,14) * FnRadup(2:Rk,8) + &
                    FnRaddn(2:Rk,16) * (FnRadup(2:Rk,8)+FnRaddn(2:Rk,8))) * Tphi_ij_R2(2:Rk)

!              FnRaddn(2:Rk,2) =  &
!                   (FnRaddn(2:Rk,13) * FnRaddn(2:Rk,7) + &
!                    FnRadup(2:Rk,16) * (FnRadup(2:Rk,7)+FnRaddn(2:Rk,7))/2) * Phi_ij_R2(2:Rk) - &
!                    (FnRaddn(2:Rk,14) * FnRaddn(2:Rk,8) + &
!                    FnRaddn(2:Rk,16) * (FnRadup(2:Rk,7)+FnRaddn(2:Rk,7))/2) * Tphi_ij_R2(2:Rk)

              FnRaddn(2:Rk,2) =  &
                   (FnRaddn(2:Rk,13) * FnRaddn(2:Rk,7) + &
                    FnRadup(2:Rk,16) * (FnRadup(2:Rk,7)+FnRaddn(2:Rk,7))) * Phi_ij_R2(2:Rk) - &
                    (FnRaddn(2:Rk,14) * FnRaddn(2:Rk,8) + &
                    FnRaddn(2:Rk,16) * (FnRadup(2:Rk,8)+FnRaddn(2:Rk,8))) * Tphi_ij_R2(2:Rk)

              fnup = IntSimpson(Mesh_Size, DR, FnRadup(1:Mesh_Size,15),&
                     AT%RadGrid) * &
                     AngPoints(4,alpha)*Four_Pi
              fndn = IntSimpson(Mesh_Size, DR, FnRaddn(1:Mesh_Size,15),&
                     AT%RadGrid) * &
                     AngPoints(4,alpha)*Four_Pi

              fntup= IntSimpson(Mesh_Size, DR, FnRadup(1:Mesh_Size,1),&
                     AT%RadGrid) * &
                     AngPoints(4,alpha)*Four_Pi
              fntdn= IntSimpson(Mesh_Size, DR, FnRaddn(1:Mesh_Size,1),&
                     AT%RadGrid) * &
                     AngPoints(4,alpha)*Four_Pi

              fnpup = IntSimpson(Mesh_Size, DR, FnRadup(1:Mesh_Size,2),&
                     AT%RadGrid) * &
                     AngPoints(4,alpha)*Four_Pi
              fnpdn = IntSimpson(Mesh_Size, DR, FnRaddn(1:Mesh_Size,2),&
                     AT%RadGrid) * &
                     AngPoints(4,alpha)*Four_Pi

              Do mi=-Li, Li
                Do mj=-Lj, Lj
                    Dijup(ibase+mi,jbase+mj) = Dijup(ibase+mi,jbase+mj) + &
                         fnup * CONJG(Ylm_i(Li+Mi+1))*Ylm_j(Lj+mj+1) + &
                         fntup*(CONJG(dYlmdt_i(Li+mi+1))*Ylm_j(Lj+mj+1) + &
                             CONJG(Ylm_i(Li+mi+1))*dYlmdt_j(Lj+mj+1)) + &
                         fnpup*(CONJG(dYlmdp_i(Li+mi+1))*Ylm_j(Lj+mj+1) + &
                             CONJG(Ylm_i(Li+mi+1))*dYlmdp_j(Lj+mj+1))
                    Dijdn(ibase+mi,jbase+mj) = Dijdn(ibase+mi,jbase+mj) + &
                         fndn * CONJG(Ylm_i(Li+Mi+1))*Ylm_j(Lj+mj+1) + &
                         fntdn*(CONJG(dYlmdt_i(Li+mi+1))*Ylm_j(Lj+mj+1) + &
                             CONJG(Ylm_i(Li+mi+1))*dYlmdt_j(Lj+mj+1)) + &
                         fnpdn*(CONJG(dYlmdp_i(Li+mi+1))*Ylm_j(Lj+mj+1) + &
                             CONJG(Ylm_i(Li+mi+1))*dYlmdp_j(Lj+mj+1))
                 End Do
              End Do  
           end Do !njlj
        End do !nili
     End Do

  End Do

  Call Stop_Timer(Timer(AtomVxc_Timer))
   write(log_unit,*) 'AccumVxc_SGGA: finish'
   call flush(log_unit)

  Return
End Subroutine

!******************************************************************************
!
!   CalcVxc_LDA - Tabulates the Vxc potential and also accumulates the Exc
!             energy
!
!        Version for Perdew-Wang LDA form only
!
!     Den   - Density in real space
!     Vxc   - Returned tabulated Vxc in FFT space
!
!    NOTE:  Both Den and Vxc use the large grid(G_HIGH)
!
!******************************************************************************

Subroutine CalcVxc_LDA( Den, SmoothExc)
  Complex, Intent(IN)    :: Den(:)
  Complex, Pointer :: Vxc(:)
  Real, INTENT(INOUT) :: SmoothExc

  Complex, Pointer :: Fn(:)
  Real :: ExcSum, Potxc, Exc, rho, DV
  Integer :: i, N

  write(log_unit,*) 'CalcVxc_LDA: start'
  call flush(log_unit)

  ExcSum = 0
  
  !! construct core tail contribution
  Vxc => SCFvalues%Ve
  Vxc = 0
  Vxc(FFTMap_High(1:Gpnt_Size(G_High)))=SCFvalues%CoreTail(1:Gpnt_Size(G_High))
  Vxc(FFTMap_High(Gpnt_Size(G_High)+1:Gall_Size(G_High))) = &
             Conjg(SCFvalues%CoreTail(2:Gpnt_Size(G_High)))
  Call PerformFFT(FFT_TO_R, G_HIGH, Vxc)

  Do i=1, FFT_Grid(4,G_HIGH)
     rho = Real(Den(i)+Vxc(i))/xtal%volume 
     !!!! -- old form -- Call ExchangeCorr(rho, Potxc, Exc, .FALSE. ,XC_Type)
     Call pwldafunc(rho,Exc,Potxc)
     
     ExcSum = ExcSum + rho*Exc
     Vxc(i) = Potxc 
  End Do

  !TildeXC_Energy = TildeXC_Energy + ExcSum*xtal%volume/FFT_Grid(4,G_HIGH)
  SmoothExc = SmoothExc + ExcSum*xtal%volume/FFT_Grid(4,G_HIGH)

Write(Log_Unit,*) 'CalcVxc: Vxc_energy=',SmoothExc
call flush(log_unit)

  !*** Now convert Vxc to FFT space ***
  write(log_unit,*) 'mag of vxc**2-R',dot_product(vxc,vxc)
  Call PerformFFT(FFT_TO_G, G_HIGH, Vxc)
  write(log_unit,*) 'mag of vxc**2-G',dot_product(vxc,vxc)
  call flush(log_unit)
  
  SCFValues%VXC(1)=Vxc(FFTMap_High(1))
  SCFValues%VXC(2:Gpnt_Size(G_High))=&
   (Vxc(FFTMap_High(2:Gpnt_Size(G_High)))+&
    Conjg(Vxc(FFTMap_High(Gpnt_Size(G_High)+1:Gall_Size(G_High)))))/2

  write(log_unit,*) 'CalcVxc_LDA: finish'
  call flush(log_unit)

  Return
End Subroutine

Subroutine CalcVxc_LSDA( Den, Denspin,SmoothExc)
  Complex, Intent(IN)    :: Den(:), Denspin(:)
  Complex, Pointer :: Vxcup(:), Vxcdn(:)
  Real, INTENT(INOUT) :: SmoothExc

  Complex, Pointer :: Fn(:)
  Real :: ExcSum, Potxceven,Potxcodd, Exc, rho, rhomu, DV
  Integer :: i, N

  ExcSum = 0
  
  !! construct core tail contribution
  Vxcup => SCFvalues%Ve
  Vxcdn => SCFvalues%Vespin
  Vxcup = 0
  Vxcup(FFTMap_High(1:Gpnt_Size(G_High)))=SCFvalues%CoreTail(1:Gpnt_Size(G_High))
  Vxcup(FFTMap_High(Gpnt_Size(G_High)+1:Gall_Size(G_High))) = &
             Conjg(SCFvalues%CoreTail(2:Gpnt_Size(G_High)))
  
  Call PerformFFT(FFT_TO_R, G_HIGH, Vxcup)

  Do i=1, FFT_Grid(4,G_HIGH)
     rho = Real(Den(i)+Denspin(i)+Vxcup(i))/xtal%volume
     rhomu = Real(Den(i)-Denspin(i))/xtal%volume 
     !!!! -- old form -- Call ExchangeCorr(rho, Potxc, Exc, .FALSE. ,XC_Type)
     Call pwlsdafunc(rho,rhomu,Exc,Potxceven, Potxcodd)
     
     ExcSum = ExcSum + rho*Exc
     Vxcup(i) = Potxceven+Potxcodd 
     Vxcdn(i) = Potxceven-Potxcodd
  End Do

  !TildeXC_Energy = TildeXC_Energy + ExcSum*xtal%volume/FFT_Grid(4,G_HIGH)
  SmoothExc = SmoothExc + ExcSum*xtal%volume/FFT_Grid(4,G_HIGH)

Write(Log_Unit,*) 'CalcVxc: Vxc_energy=',SmoothExc

  !*** Now convert Vxc to FFT space ***
  write(log_unit,*) 'mag of vxcup**2-R',dot_product(vxcup,vxcup)
  write(log_unit,*) 'mag of vxcdn**2-R',dot_product(vxcdn,vxcdn)

  Call PerformFFT(FFT_TO_G, G_HIGH, Vxcup)
  Call PerformFFT(FFT_TO_G, G_HIGH, Vxcdn)

  write(log_unit,*) 'mag of vxceven**2-G',dot_product(vxcup,vxcup)
  write(log_unit,*) 'mag of vxceven**2-G',dot_product(vxcdn,vxcdn)
  
  SCFValues%VXC(1)=Vxcup(FFTMap_High(1))
  SCFValues%VXC(2:Gpnt_Size(G_High))=&
   (Vxcup(FFTMap_High(2:Gpnt_Size(G_High)))+&
    Conjg(Vxcup(FFTMap_High(Gpnt_Size(G_High)+1:Gall_Size(G_High)))))/2

  SCFValues%VXCSPIN(1)=Vxcdn(FFTMap_High(1))
  SCFValues%VXCSPIN(2:Gpnt_Size(G_High))=&
   (Vxcdn(FFTMap_High(2:Gpnt_Size(G_High)))+&
    Conjg(Vxcdn(FFTMap_High(Gpnt_Size(G_High)+1:Gall_Size(G_High)))))/2
  Return
End Subroutine

!******************************************************************************
!
!   CalcVxc_GGA - Tabulates the Vxc potential and also accumulates the Exc
!             energy
!
!        Version for PBE GGA form only
!
!     DG    - Density in G space -- tabulated 1..Gpnt_Size(G_High)
!     W1    - on input: Density in real space, on output: Vxc in FFT space
!     W2    - Work array
!
!    NOTE:  Both W1 and W2 use the large grid(G_HIGH)
!
!******************************************************************************

Subroutine CalcVxc_GGA( W1, SmoothExc)   
  Complex, Intent(INOUT) :: W1(:)
  Real, INTENT(INOUT) :: SmoothExc
  Complex, Pointer :: W2(:)
  Complex, Allocatable :: W3(:),DG(:)
  Real :: ExcSum, Potxc, Exc, rho, DV, grad,fxc,dfxcdn, dfxcdgbg
  Integer :: i, j, N, ier

 
  Allocate(W3(FFT_Grid(4,G_HIGH)),DG(Gpnt_Size(G_HIGH)),stat=ier)
    If (ier /= 0) then
       write(Error_Unit,*) 'CalcVxc_GGA:  Cannot allocate work array ', FFT_Grid(4,G_HIGH)
       stop
    EndIf

  DG = SCFValues%RhoSmooth + SCFValues%CoreTail
  ExcSum = 0
  W2 => SCFvalues%Ve
  W2 = 0
  N = Gpnt_Size(G_HIGH)-1
  Do i=1,3
     W3=0
        Do j=2,Gpnt_Size(G_HIGH)
             W3(FFTMap_High(j))=Gpnt(i,j)*DG(j)          
             W3(FFTMap_High(N+j))=-Gpnt(i,j)*CONJG(DG(j))         
        Enddo
     Call PerformFFT(FFT_TO_R, G_HIGH, W3)
     W2 = W2 + W3*CONJG(W3)
  EndDo
           
 W1 = 0
  W1(FFTMap_High(1))=DG(1)
  Do j=2,Gpnt_Size(G_HIGH)
     W1(FFTMap_High(j))=DG(j)
     W1(FFTMap_High(N+j))=CONJG(DG(j))
  Enddo
  Call PerformFFT(FFT_TO_R, G_HIGH, W1)

  Do i=1, FFT_Grid(4,G_HIGH)
     rho = W1(i)/xtal%volume
     grad = SQRT(REAL(W2(i)))/xtal%volume
     call  pbefunc(rho,grad,fxc,dfxcdn,dfxcdgbg)
     
     ExcSum = ExcSum + fxc
     W1(i) = CMPLX(dfxcdn,0)
     W2(i) = CMPLX(dfxcdgbg,0)
  End Do
     W2=W2/xtal%volume

  !TildeXC_Energy = TildeXC_Energy + ExcSum*xtal%volume/FFT_Grid(4,G_HIGH)
  SmoothExc = SmoothExc + ExcSum*xtal%volume/FFT_Grid(4,G_HIGH)

Write(Log_Unit,*) 'CalcVxc: Vxc_energy=',SmoothExc

  Call PerformFFT(FFT_TO_G, G_HIGH, W1)
    W3=0
    W3(1)=W1(FFTMap_High(1))
        Do j=2,Gpnt_Size(G_HIGH)
           W3(j) = ( W1(FFTMap_High(j)) + CONJG(W1(FFTMap_High(N+j))))/2
           !write(log_unit,'("V1: ",i5,1p4e15.7)')j,w3(j)
        Enddo
  Do i=1,3
     W1=0
        Do j=2,Gpnt_Size(G_HIGH)
             W1(FFTMap_High(j))=Gpnt(i,j)*DG(j)         
             W1(FFTMap_High(N+j))=-Gpnt(i,j)*CONJG(DG(j))        
        Enddo
     Call PerformFFT(FFT_TO_R, G_HIGH, W1)
     W1 = W1*W2
     Call PerformFFT(FFT_TO_G, G_HIGH, W1)
        Do j=2,Gpnt_Size(G_HIGH)
             W3(j)=W3(j)+Gpnt(i,j)*(W1(FFTMap_High(j)) - &
                                    CONJG(W1(FFTMap_High(N+j))))/2
           !write(log_unit,'("V1: ",i5,1p4e15.7)')j,w3(j)
        EndDo
  EndDo
  ! Load resultant W3 into FFT grid as W1
    W1=0; W2 =0
      W2(FFTmap_High(1:Gpnt_Size(G_HIGH))) =  W3(1:Gpnt_Size(G_High))
      W2(FFTmap_High(N+2:Gall_Size(G_HIGH))) =  CONJG(W3(2:Gpnt_Size(G_High)))
      W1(FFTmap_High(1:Gpnt_Size(G_HIGH))) = &
             SCFValues%RhoSmooth( 1:Gpnt_Size(G_High))
      W1(FFTmap_High(N+2:Gall_Size(G_HIGH))) =  &
            CONJG(SCFValues%RhoSmooth(2:Gpnt_Size(G_High)))

  write(log_unit,*) 'mag of vxc**2-G',dot_product(W1,W1)

  SCFValues%VXC = W3(1:Gpnt_Size(G_High))
  
  DeAllocate(W3,DG)
  Return
End Subroutine


Subroutine CalcVxc_SGGA( W1up, W1dn,SmoothExc)   
  Complex, Intent(INOUT) :: W1up(:), W1dn(:)
  Real, INTENT(INOUT) :: SmoothExc
  Complex, Pointer :: W2up(:), W2dn(:)
  Complex, Allocatable :: W3up(:),W3dn(:),DGup(:),DGdn(:),W(:)
  Real :: ExcSum, Potxceven,Potxcodd, Exc, rhoup,rhodn, DV, grad,&
         gradup,graddn,fxc,dfxcdneven,dfxcdnodd, dfxdgbgup, dfxdgbgdn, dfcdgbg
  COMPLEX :: term
  Integer :: i, j, N, ier

 
  Allocate(W3up(FFT_Grid(4,G_HIGH)),W3dn(FFT_Grid(4,G_HIGH)),&
    W(FFT_Grid(4,G_HIGH)),&
    DGup(Gpnt_Size(G_HIGH)),DGdn(Gpnt_Size(G_HIGH)),stat=ier)
    If (ier /= 0) then
       write(Error_Unit,*) 'CalcVxc_GGA:  Cannot allocate work array ', FFT_Grid(4,G_HIGH)
       stop
    EndIf

  DGup = SCFValues%RhoSmooth + SCFValues%CoreTail/2
  DGdn = SCFValues%RhoSmoothspin + SCFValues%CoreTail/2
  ExcSum = 0
  W2up => SCFvalues%Ve
  W2dn => SCFvalues%Vespin
  W2up = 0
  W2dn = 0
  W = 0

  N = Gpnt_Size(G_HIGH)-1
  Do i=1,3
     W3up=0
     W3dn=0
        Do j=2,Gpnt_Size(G_HIGH)
             W3up(FFTMap_High(j))=Gpnt(i,j)*DGup(j)          
             W3up(FFTMap_High(N+j))=-Gpnt(i,j)*CONJG(DGup(j))
             W3dn(FFTMap_High(j))=Gpnt(i,j)*DGdn(j)
             W3dn(FFTMap_High(N+j))=-Gpnt(i,j)*CONJG(DGdn(j))         
        Enddo
     Call PerformFFT(FFT_TO_R, G_HIGH, W3up)
     Call PerformFFT(FFT_TO_R, G_HIGH, W3dn)
     W2up = W2up + W3up*CONJG(W3up)
     W2dn = W2dn + W3dn*CONJG(W3dn)
     W    = W   + (W3up+W3dn)*CONJG(W3up+W3dn)
  EndDo
           
  W1up = 0
  W1dn = 0
  w1up(FFTMap_High(1))=DGup(1)
  W1dn(FFTMap_High(1))=DGdn(1)
  Do j=2, Gpnt_Size(G_HIGH)
     W1up(FFTMap_High(j))=DGup(j)
     W1up(FFTMap_High(N+j))=CONJG(DGup(j))
     W1dn(FFTMap_High(j))=DGdn(j)
     W1dn(FFTMap_High(N+j))=CONJG(DGdn(j))
  Enddo
     Call PerformFFT(FFT_TO_R, G_HIGH, W1up)
     Call PerformFFT(FFT_TO_R, G_HIGH, W1dn)
   
  Do i=1, FFT_Grid(4,G_HIGH)
     rhoup = W1up(i)/xtal%volume       !!/2
     rhodn = W1dn(i)/xtal%volume       !!/2
     gradup = SQRT(REAL(W2up(i)))/xtal%volume
     graddn = SQRT(REAL(W2dn(i)))/xtal%volume
     grad   = SQRT(REAL(W(i)))/xtal%volume
     
     call  Sggapbefunc(rhoup,gradup,rhodn,graddn, grad, &
        fxc,dfxcdneven,dfxcdnodd,dfxdgbgup, dfxdgbgdn, dfcdgbg)
     
     ExcSum = ExcSum + fxc
     W1up(i) = CMPLX((dfxcdneven+dfxcdnodd),0)
     W1dn(i) = CMPLX((dfxcdneven-dfxcdnodd),0)
     W2up(i) = CMPLX(dfxdgbgup,0)
     W2dn(i) = CMPLX(dfxdgbgdn,0)
     W(i) = CMPLX(dfcdgbg,0)
  End Do
     W2up=W2up/xtal%volume
     W2dn=W2dn/xtal%volume
     W = W/xtal%volume

  !TildeXC_Energy = TildeXC_Energy + ExcSum*xtal%volume/FFT_Grid(4,G_HIGH)
  SmoothExc = SmoothExc + ExcSum*xtal%volume/FFT_Grid(4,G_HIGH)

Write(Log_Unit,*) 'CalcVxc: Vxc_energy=',SmoothExc

  Call PerformFFT(FFT_TO_G, G_HIGH, W1up)
  Call PerformFFT(FFT_TO_G, G_HIGH, W1dn)
    W3up=0
    W3dn=0
    W3up(1)=W1up(FFTMap_High(1))
    W3dn(1)=W1dn(FFTMap_High(1))
        Do j=2,Gpnt_Size(G_HIGH)
           W3up(j) = ( W1up(FFTMap_High(j)) + CONJG(W1up(FFTMap_High(N+j))))/2
           W3dn(j) = ( W1dn(FFTMap_High(j)) + CONJG(W1dn(FFTMap_High(N+j))))/2
           !write(log_unit,'("V1: ",i5,1p4e15.7)')j,w3up(j),w3dn(j)
        Enddo
  Do i=1,3
     W1up=0
     W1dn=0
        Do j=2,Gpnt_Size(G_HIGH)
             W1up(FFTMap_High(j))=Gpnt(i,j)*DGup(j)         
             W1up(FFTMap_High(N+j))=-Gpnt(i,j)*CONJG(DGup(j)) 
             W1dn(FFTMap_High(j))=Gpnt(i,j)*DGdn(j)         
             W1dn(FFTMap_High(N+j))=-Gpnt(i,j)*CONJG(DGdn(j)) 
        Enddo
     Call PerformFFT(FFT_TO_R, G_HIGH, W1up)
     Call PerformFFT(FFT_TO_R, G_HIGH, W1dn)
     Do j=1, FFT_Grid(4,G_HIGH)
        gradup=-AIMAG(W1up(j))
        graddn=-AIMAG(W1dn(j))
        grad=gradup+graddn
        W1up(j)=gradup*W2up(j)+grad*W(j)          !!/2
        W1dn(j)=graddn*W2dn(j)+grad*W(j)          !!/2
     Enddo                                            
     Call PerformFFT(FFT_TO_G, G_HIGH, W1up)
     Call PerformFFT(FFT_TO_G, G_HIGH, W1dn)
        Do j=2,Gpnt_Size(G_HIGH)
             term=CMPLX(0,Gpnt(i,j))
             W3up(j)=W3up(j)-term*(W1up(FFTMap_High(j)) + &
                                    CONJG(W1up(FFTMap_High(N+j))))/2
             W3dn(j)=W3dn(j)-term*(W1dn(FFTMap_High(j)) + &
                                    CONJG(W1dn(FFTMap_High(N+j))))/2
           !write(log_unit,'("V1: ",i5,1p4e15.7)')j,w3up(j),w3dn(j)
        EndDo
  EndDo
  ! Load resultant W3 into FFT grid as W1
    W1up=0; W2up=0
    W1dn=0; W2dn=0
      W2up(FFTmap_High(1:Gpnt_Size(G_HIGH))) =  W3up(1:Gpnt_Size(G_High))
      W2up(FFTmap_High(N+2:Gall_Size(G_HIGH))) =  CONJG(W3up(2:Gpnt_Size(G_High)))
      W2dn(FFTmap_High(1:Gpnt_Size(G_HIGH))) =  W3dn(1:Gpnt_Size(G_High))
      W2dn(FFTmap_High(N+2:Gall_Size(G_HIGH))) =  CONJG(W3dn(2:Gpnt_Size(G_High)))

      W1up(FFTmap_High(1:Gpnt_Size(G_HIGH))) =  SCFValues%RhoSmooth(1:Gpnt_Size(G_High))
      W1up(FFTmap_High(N+2:Gall_Size(G_HIGH))) = CONJG(SCFValues%RhoSmooth(2:Gpnt_Size(G_High)))
      W1dn(FFTmap_High(1:Gpnt_Size(G_HIGH))) =  SCFValues%RhoSmoothspin(1:Gpnt_Size(G_High))
      W1dn(FFTmap_High(N+2:Gall_Size(G_HIGH))) =  CONJG(SCFValues%RhoSmoothspin(2:Gpnt_Size(G_High)))
        
      
  write(log_unit,*) 'mag of vxc**2-G',dot_product((W2up+W2dn),(W2up+W2dn))

  SCFValues%VXC = W3up(1:Gpnt_Size(G_High))
  SCFValues%VXCSPIN = W3dn(1:Gpnt_Size(G_High))
  
  DeAllocate(W3up,W,DGup)
  DeAllocate(W3dn,DGdn)
  Return
End Subroutine

!******************************************************************************
!
!  PrepareVxc - Prepares the data structures for the caclulation of Vxc
!
!  AT - Pointer to the Atom_Info_Fixed structure
!
!******************************************************************************

Subroutine PrepareVxc(AT)
  Type (Atom_Info_Fixed), Intent(INOUT) :: AT
  
  Real, Pointer :: R2(:), Phi_ij_R2(:,:), TPhi_ij_R2(:,:)
  Real, Pointer :: dPhi_ijdr(:,:),dTPhi_ijdr(:,:)
  Real, Pointer :: Phi_I(:), Phi_j(:), TPhi_i(:), TPhi_j(:)
  Complex, Pointer :: Ylm(:,:,:),dYlmdtheta(:,:,:),dYlmdphi(:,:,:)
  Complex :: Ylm_i(13), dYlmdt(13),dYlmdp(13)
  Real      :: DR,  Ra(3),den,Exc,Vxc,grad,dfxcdn,dfxcdg
  Integer   :: RadSize, Rk, alpha, count, nili, njlj, mi, mj, Li, Lj, Stride


  RadSize = AT%Rad_Size

  !******* First determine how many ij indices there are *******
  count = 0
  Do nili=1, AT%Basis_Size 
     Li = AT%L_Value(nili)
     Do njlj=1, AT%Basis_Size
        Lj = AT%L_Value(njlj)

        count = count + 1
     End Do
  End Do


  Stride = AT%Rad_Skip
  if (stride /=1) then
        write(Log_unit,*) 'Stride must 1'
        stop
  endif

  Li = MAXVal(AT%L_Value)
  mi = 2*Li+1
  Allocate(AT%R2(RadSize), &
       AT%Ylm(mi, Li+1, Num_AngPoints), AT%Phi_ij_R2(RadSize, Count), &
       AT%TPhi_ij_R2(RadSize, Count), AT%Cij(AT%Basis_Size, AT%Basis_Size), &
       STAT=Rk)
  if(spindependence) Allocate(AT%Cijspin(AT%Basis_Size, AT%Basis_Size), STAT=Rk)
  If (XC_Type == XC_LDA_PW) Allocate(AT%FnRad(RadSize,7))
  If (XC_Type == XC_LSDA_PW) Allocate(AT%FnRad(RadSize,7), AT%FnRadspin(RadSize,7),STAT=Rk)
  If (XC_Type == XC_GGA_PBE) then
    Allocate(AT%FnRad(RadSize,15),AT%dYlmdtheta(mi, Li+1, Num_AngPoints),&
         AT%dYlmdphi(mi, Li+1, Num_AngPoints), &
         AT%dCijdtheta(AT%Basis_Size, AT%Basis_Size),&
         AT%dCijdphi(AT%Basis_Size, AT%Basis_Size),&
         AT%dPhi_ijdr(RadSize, Count),&
         AT%dTPhi_ijdr(RadSize, Count),&
         AT%Core_dDendr(RadSize),AT%GradCoreTail(RadSize),STAT=Rk)
  EndIf
  If (XC_Type == XC_SGGA_PBE) then
    Allocate(AT%FnRad(RadSize,16),AT%FnRadspin(RadSize,16),&
         AT%dYlmdtheta(mi, Li+1, Num_AngPoints),&
         AT%dYlmdphi(mi, Li+1, Num_AngPoints), &
         AT%dCijdtheta(AT%Basis_Size, AT%Basis_Size),AT%dCijdthetaspin(AT%Basis_Size, AT%Basis_Size),&
         AT%dCijdphi(AT%Basis_Size, AT%Basis_Size),AT%dCijdphispin(AT%Basis_Size, AT%Basis_Size),&
         AT%dPhi_ijdr(RadSize, Count),&
         AT%dTPhi_ijdr(RadSize, Count),&
         AT%Core_dDendr(RadSize),AT%GradCoreTail(RadSize),STAT=Rk)
  EndIf

 !**** fix core part -- this was previously in InitSystem
        If (XC_Type == XC_LDA_PW .OR. XC_Type == XC_LSDA_PW) then
          ! Do Rk=2, AT%Mesh_Size
          !    den = AT%Core_Density(Rk)
          !    If (XC_Type == XC_LDA_PW) Call pwldafunc(den,Exc,Vxc)
          !    AT%Core_Xchange(Rk) = Exc
          ! End Do
          ! !AT%Core_Xchange(1) = 2*AT%Core_Xchange(2)-AT%Core_Xchange(3)
          ! AT%Core_Xchange(1) = 3*(AT%Core_Xchange(2)-AT%Core_Xchange(3))&
          !             +  AT%Core_Xchange(4)
        Else If (XC_Type == XC_GGA_PBE .OR. XC_Type == XC_SGGA_PBE) then
           RadSize=AT%Mesh_Size
           Call nderiv(AT%Mesh_Step,AT%Core_Density(:),&
                 AT%Core_dDendr(:),RadSize,AT%RadGrid)
           Call nderiv(AT%Mesh_Step,AT%CoreTail_Density(:),&
                 AT%GradCoreTail(:),RadSize,AT%RadGrid)
          ! Do Rk=1,RadSize
          !  grad=ABS(AT%Core_dDendr(Rk))     ! stores gradient
          !  Call  pbefunc(AT%Core_Density(Rk),grad,AT%Core_Xchange(Rk),dfxcdn,dfxcdg)
          !            !! Remember: for GGA Core_Xchange is fxc
          ! Enddo
        Else
          Write(Error_Unit, * ) 'Initsystem:  Error in XC_Type', XC_Type
          Stop
        EndIf
 !**** end core part

  R2 => AT%R2
  Ylm => AT%Ylm
   If (XC_Type == XC_GGA_PBE .OR. XC_Type == XC_SGGA_PBE) then
         dYlmdtheta => AT%dYlmdtheta
         dYlmdphi => AT%dYlmdphi
         dPhi_ijdr => AT%dPhi_ijdr
         dTPhi_ijdr => AT%dTPhi_ijdr
   endif

  !********** Make R^2 ************
  !DR = AT%Mesh_Step*Stride !** Calc Radial weighting and R
  DR = AT%Mesh_Step
  Do Rk=1, RadSize
     if (AT%RadGrid%islog) then
      R2(Rk) = (AT%RadGrid%drdu(Rk)-AT%RadGrid%drdu(1))**2
     else
      R2(Rk) = (DR*(Rk-1))**2
     endif
  End Do

  count = 0
  Do nili=1, AT%Basis_Size
     Li = AT%L_Value(nili)
     Phi_i =>AT%Phi(1:AT%Mesh_Size:Stride,nili)
     TPhi_i =>AT%TPhi(1:AT%Mesh_Size:Stride,nili)

     Do Rk=1, Num_AngPoints  !** Calculate the Ylm's **

        Ra = AngPoints(1:3, Rk)
        Ylm_i = Spharm(Ra(1), Ra(2), Ra(3), Li, .TRUE.)
        Ylm(1:2*Li+1, Li+1, Rk) =  Ylm_i(1:2*Li+1)
      If (XC_Type == XC_GGA_PBE .OR. XC_Type == XC_SGGA_PBE) then
        Call GradYlm(Ra(1),Ra(2),Ra(3),Li,dYlmdt,dYlmdp)
        dYlmdtheta(1:2*Li+1, Li+1, Rk) =  dYlmdt(1:2*Li+1)
        dYlmdphi(1:2*Li+1, Li+1, Rk) =  dYlmdp(1:2*Li+1)
      EndIf
     End Do

     Do njlj=1, AT%Basis_Size
        count = count + 1

        Phi_j =>AT%Phi(1:AT%Mesh_Size:Stride,njlj)
        TPhi_j =>AT%TPhi(1:AT%Mesh_Size:Stride,njlj)

        Phi_ij_R2 => AT%Phi_ij_R2
        TPhi_ij_R2 => AT%TPhi_ij_R2

        Phi_ij_R2(2:RadSize,count) = &
                 Phi_i(2:RadSize)*Phi_j(2:RadSize)/R2(2:RadSize)
        !Phi_ij_R2(1,count)=2*Phi_ij_R2(2,count)-Phi_ij_R2(3,count)
        Phi_ij_R2(1,count)=3*(Phi_ij_R2(2,count)-Phi_ij_R2(3,count)) &
                             + Phi_ij_R2(4,count)
        TPhi_ij_R2(2:RadSize,count) = &
                 TPhi_i(2:RadSize)*TPhi_j(2:RadSize)/R2(2:RadSize)
        !TPhi_ij_R2(1,count)=2*TPhi_ij_R2(2,count)-TPhi_ij_R2(3,count)
        TPhi_ij_R2(1,count)=3*(TPhi_ij_R2(2,count)-TPhi_ij_R2(3,count)) &
                             + TPhi_ij_R2(4,count)
       If (XC_Type == XC_GGA_PBE .or. XC_Type == XC_SGGA_PBE) then
        call nderiv(DR,Phi_ij_R2(:,count),dPhi_ijdr(:,count),RadSize,&
            AT%RadGrid)
        call nderiv(DR,TPhi_ij_R2(:,count),dTPhi_ijdr(:,count),RadSize,&
            AT%RadGrid)
       EndIf

     End Do
  End Do

  Return
End Subroutine

End Module
