!******************************************************************************
!
! File : laplacian.f90
!   by : Alan Tackett
!   on : 08/19/98
!  for : PAW Proagram
!
!
!
!******************************************************************************


Module laplacian

Use atom_data
Use search_sort
Use misc
Use gpoints

Implicit NONE!!!!!!!!!!!!!!

!******************************************************************************
Contains
!******************************************************************************

!******************************************************************************
!
!  CalcDEL2 - Calculates the Laplacian of the given grid
!
!  Vin    - Input Vector                     2
!  Vout   - Output vector containing Vout=DEL  Vin
!
!     NOTE:   the Vout array is ZEROED!!!!
!
!******************************************************************************

Subroutine CalcDEL2( Vin, Vout)
  Complex, Intent(IN)  :: Vin(:)  
  Complex, TARGET, Intent(OUT) :: Vout(:)  

  Real :: c1
  Integer :: gi, N

!Write(*,*) 'CalcDEL2: Start!!!!!!!!!!'
  Vout = 0

  c1 = -1

  N = Gpnt_Size(G_LOW) - 1

  Do gi=1, Gpnt_Size(G_LOW)
     Vout(gi) = c1*Vin(gi)*Gpnt(4,gi)**2
  End Do
  Do gi=2, Gpnt_Size(G_LOW)
     Vout(gi+N) = c1*Vin(gi+N)*Gpnt(4,gi)**2
  End Do

!Write(*,*) 'CalcDEL2: End!!!!!!!!!!'
  Return
End Subroutine

!******************************************************************************
!
!  CalcDEL2_PHASE - Calculates the Laplacian of the given grid
!
!  Kvec   - Kpoint Vector
!  BLochK - Corresponding BLock Phase
!  Vin    - Input Vector                           2      
!  Vout   - Output vector containing Vout=      DEL  (Vin ) in Fourier space
!
!     NOTE:  the Vout array is ZEROED!!!!
!
!******************************************************************************

Subroutine CalcDEL2_PHASE( Kvec, Vin, Vout)
  Real,    Intent(IN)  :: Kvec(:)
  Complex, Intent(IN)  :: Vin(:)  
  Complex, TARGET, Intent(OUT) :: Vout(:)  

  Real :: c1, G(3), G2
  Integer :: gi, N


  c1 = DOT_PRODUCT(Kvec,Kvec)

  If (c1<1E-20) then
     Call CalcDEL2( Vin, Vout)
  else
     c1 = -1

     N = Gpnt_Size(G_LOW)-1
     
     Do gi=1, Gpnt_Size(G_LOW)
        G = Kvec + Gpnt(1:3,gi)
        G2 = DOT_PRODUCT(G,G)
        Vout(gi) = c1*Vin(gi)*G2
     End Do
     Do gi=2, Gpnt_Size(G_LOW)
        G = Kvec - Gpnt(1:3,gi)
        G2 = DOT_PRODUCT(G,G)
        Vout(gi+N) = c1*Vin(gi+N)*G2
     End Do
  End If

  Return
End Subroutine


!******************************************************************************
!
!  CalcDEL2_TIME - Applies the KE and vector potential operator
!           to the given vector.
!
!  Kvec   - Kpoint Vector
!  BLochK - Corresponding BLock Phase
!  Vin    - Input Vector                      ikr           2         -ikr
!  Vout   - Output vector containing Vout=   e    (-i*DEL-A)  (Vin * e    )
!
!     NOTE: The Vout array is ZEROED!!!!
!           Also notice the (+) sign on the entire expression.  This
!           is the OPPOSITE than the way the KE is calc'ed above so Hpsi must
!           be changed accordingly.
!
!******************************************************************************

Subroutine CalcDEL2_TIME( Kvec, Vin, Vout)
  Real,    Intent(IN)  :: Kvec(:)
  Complex, Intent(IN)  :: Vin(:)  
  Complex, TARGET, Intent(OUT) :: Vout(:)  

  Real :: c1, G(3), G2, KA(3)
  Integer :: gi, N


  KA = Kvec - Vector_Pot

  N = Gpnt_Size(G_LOW)-1
  
  Do gi=1, Gpnt_Size(G_LOW)
     G = KA + Gpnt(1:3,gi)
     G2 = DOT_PRODUCT(G,G)
     Vout(gi) = Vin(gi)*G2
  End Do
  Do gi=2, Gpnt_Size(G_LOW)
     G = KA - Gpnt(1:3,gi)
     G2 = DOT_PRODUCT(G,G)
     Vout(gi+N) = Vin(gi+N)*G2
  End Do

  Return
End Subroutine

End Module
