!******************************************************************************
!
! File : memmgr.f90
!   by : Alan Tackett
!   on : 04/26/97
!  for : PAW project
!
! This module contains routines for controlling the memory allocation and
! distribution between the projectors, psi's, and FAS vectors  
!
! Updated for spin by Ping Tang and N. A. W. Holzwarth
!    Last change 5/23/05
!******************************************************************************


Module memmgr

Use mem_data
Use paw_inout
Use projectors
Use psilib
Use options_data
Use anderson_mixing

Implicit NONE!!!!!!!!!!!!!


!******************************************************************************
Contains
!******************************************************************************

!******************************************************************************
!
! GetNextFreeIndex - Returns the next free Psi index 
!
! Index - Free record returned
!
!******************************************************************************

Subroutine GetNextFreeIndex(Index)
  Integer, Intent(OUT) :: Index
 
  Real :: mag
  Index = 1
  Do While ((Index <= Mem_MapSize) .AND. &
             (PsiInfo(Index)%Available==Mem_Used))
     Index = Index + 1
  End Do

  If (Index > Mem_MapSize) then
     Write(Error_Unit,*) 'GetNextFreeIndex: Mem_MapSize=',Mem_MapSize, &
          ' * New Index=',Index
     Write(Log_Unit,*) 'GetNextFreeIndex: Mem_MapSize=', &
          Mem_MapSize, ' * New Index=',Index
     Call Flush(Error_Unit)
     Call Flush(Log_Unit)
     stop
     mag = 0
     mag = 1.0/mag
  End If

  Return
End Subroutine

!******************************************************************************
!
!  MemUsed - Calculates the amount of memory 
!
!  TotalUsed - Total Memory Used ,ie, ProjUsed+PsiUsed
!  ProjUsed  - Projector memory used
!  PsiUsed   - Psi memory used
!
!******************************************************************************

Subroutine MemUsed( TotalUsed)
  Integer, intent(OUT) :: TotalUsed

  Integer :: LSize
  Type (Global_handle), Pointer :: LH
  
  LSize = PsiArraySize
  LH => Globalmap
  
  TotalUsed = LH%NumPsi*LSize
  
  Return
End Subroutine


!******************************************************************************
!
! DisplayBufferInfo - Displays the Buffer Information to the specified device
!
!   Output - Output unit
!
!******************************************************************************

Subroutine DisplayBufferInfo(Output)
  Integer, Intent(IN) :: OutPut

  Integer :: i, j, LSize
  Integer ::  Mem_Proj, Mem_Psi,  Calc_Proj,Calc_Psi
  Integer :: TotalMem
  Type (Global_handle), Pointer :: LH
  Type (Mem_handle), Pointer   :: MH(:), MH2(:)
  Integer :: CacheTotal, CacheHits, CacheMisses
  Real :: Hits, miss, total
  

  Write(output, *) 'BUFFER_INFO: Printing Psi buffer info ********************'
  Write(output, *) 'Max Records :', Mem_MapSize
  TotalMem = 0
     LH => Globalmap    
     LSize = PsiArraySize     
     Write(Output, *) 'Buffer info  * Base Size:',LSize
     Write(Output, *) '   Allocated Psi:',LH%NumPsi
     Write(Output, *) '   Loaded    Psi:',LH%PsiLoaded
     Call MemUsed(TotalMem)
     Write(Output, *) '   Memory  Total:',TotalMem 

     Write(Output,*) ' Cache  Statistics: '

     CacheTotal = MemCache%Psi_Read%Hits + MemCache%Psi_Read%Misses
     Write(Output,*) '    *Psi Read*  Total:',CacheTotal, &
          ' * Hits:',MemCache%Psi_Read%Hits, &
          ' * Misses:',MemCache%Psi_Read%Misses
     CacheTotal = MemCache%Psi_Write%Hits + MemCache%Psi_Write%Misses
     Write(Output,*) '    *Psi Write* Total:',CacheTotal, &
          ' * Hits:',MemCache%Psi_Write%Hits, &
          ' * Misses:',MemCache%Psi_Write%Misses
     CacheTotal = MemCache%Psi_Read%Hits + MemCache%Psi_Read%Misses &
            + MemCache%Psi_Write%Hits + MemCache%Psi_Write%Misses
     CacheHits = MemCache%Psi_Read%Hits + MemCache%Psi_Write%Hits
     CacheMisses = MemCache%Psi_Read%Misses + MemCache%Psi_Write%Misses

     Write(Output,*) '    *Psi Both* Total:',CacheTotal, &
          ' * Hits:',CacheHits, ' * Misses:',CacheMisses

        Total = MemCache%Psi_Read%Hits + MemCache%Psi_Read%Misses
        if (Total > 0) then
           hits = (1.0*MemCache%Psi_Read%Hits)/total
           miss = (1.0*MemCache%Psi_Read%Misses)/total
           Write(Output,*) '    *Psi Read*  %Hits:',Hits, ' * %Misses:',Miss
        End If

        Total = MemCache%Psi_Write%Hits + MemCache%Psi_Write%Misses
        if (Total > 0) then
           hits = (1.0*MemCache%Psi_Write%Hits)/total
           miss = (1.0*MemCache%Psi_Write%Misses)/total
           Write(Output,*) '    *Psi Write* %Hits:',Hits, ' * %Misses:',Miss
        End If

        Total = MemCache%Psi_Read%Hits + MemCache%Psi_Read%Misses &
             + MemCache%Psi_Write%Hits + MemCache%Psi_Write%Misses
        if (Total > 0) then        
           Hits = MemCache%Psi_Read%Hits + MemCache%Psi_Write%Hits
           Miss = MemCache%Psi_Read%Misses+ MemCache%Psi_Write%Misses
           hits = Hits/total
           miss = Miss/total
           Write(Output,*) '    *Psi Both* %Hits:',Hits, ' * %Misses:',Miss
        End If

     Write(Output, *) '   Psi Data : Buffer, Disk, Store'
     MH => LH%Psi_handle
!!     Do j=1, LH%NumPsi
!!        If (j<=LH%PsiLoaded) then
!!           Write(Output, *) '       PSI:', j, ' * ',MH(j)%Index,' * ', Size(MH(j)%Ptr)
!!        else
!!           Write(Output, *) '     # PSI:', j, ' * ',MH(j)%Index,' * ', Size(MH(j)%Ptr)
!!        End If
!!
!!!**           Write(Output, *) '        Psi=',MH(j)%Ptr
!!     End Do


  Write(Output,*) 'Total Memory-->  Max:', MemWords, ' * BaseFree:',BaseFree, ' * Used:', TotalMem, ' * Free:', MemFree

  Write(Output,*) ' Overall Cache Statistics (Unweighted): '

  CacheHits = MemCache%Psi_Read%Hits
  CacheMisses = MemCache%Psi_Read%Misses
  CacheTotal = CacheHits + CacheMisses
  Write(Output,*) '    *Psi Read*  Total:',CacheTotal, &
       ' * Hits:',CacheHits, &
       ' * Misses:',CacheMisses

  CacheHits = MemCache%Psi_Write%Hits
  CacheMisses = MemCache%Psi_Write%Misses
  CacheTotal = CacheHits + CacheMisses
  Write(Output,*) '    *Psi Write* Total:',CacheTotal, &
       ' * Hits:',CacheHits, &
       ' * Misses:',CacheMisses

  CacheHits = MemCache%Psi_Write%Hits + MemCache%Psi_Read%Hits
  CacheMisses = MemCache%Psi_Write%Misses+ MemCache%Psi_Read%Misses
  CacheTotal = CacheHits + CacheMisses
  Write(Output,*) '    *Psi Both* Total:',CacheTotal, &
       ' * Hits:',CacheHits, &
       ' * Misses:',CacheMisses

    !**** %Psi hits/miss *****
  Hits = MemCache%Psi_Read%Hits
  Miss = MemCache%Psi_Read%Misses
  Total = Hits + Miss
  If (Total > 0) then
     Hits = Hits/Total
     Miss = Miss/Total
     Write(Output,*) '    *Psi Read*  %Hits:',Hits, ' * %Misses:',Miss
  End IF
  Hits = MemCache%Psi_Write%Hits
  Miss = MemCache%Psi_Write%Misses
  Total = Hits + Miss
  if (Total > 0) then
     Hits = Hits/Total
     Miss = Miss/Total
     Write(Output,*) '    *Psi Write* %Hits:',Hits, ' * %Misses:',Miss
  End If

  Hits = MemCache%Psi_Read%Hits + MemCache%Psi_Write%Hits
  Miss = MemCache%Psi_Read%Misses + MemCache%Psi_Write%Misses
  Total = Hits + Miss
  if (Total > 0) then
     Hits = Hits/Total
     Miss = Miss/Total
     Write(Output,*) '    *Psi Both*  %Hits:',Hits, ' * %Misses:',Miss
  End If

  !-----------------------------------------

  Write(Output,*) ' Overall Cache Statistics (Weighted): '


  CacheHits = MemCache%Psi_Read%Hits 
  CacheMisses = MemCache%Psi_Read%Misses 
  CacheTotal = CacheHits + CacheMisses
  Write(Output,*) '    *Psi Read*  Total:',CacheTotal, &
       ' * Hits:',CacheHits, &
       ' * Misses:',CacheMisses

  CacheHits = MemCache%Psi_Write%Hits
  CacheMisses = MemCache%Psi_Write%Misses
  CacheTotal = CacheHits + CacheMisses
  Write(Output,*) '    *Psi Write* Total:',CacheTotal, &
       ' * Hits:',CacheHits, &
       ' * Misses:',CacheMisses

  CacheHits = (MemCache%Psi_Write%Hits + MemCache%Psi_Read%Hits )
  CacheMisses = (MemCache%Psi_Write%Misses+MemCache%Psi_Read%Misses)
  CacheTotal = CacheHits + CacheMisses
  Write(Output,*) '    *Psi Both*  Total:',CacheTotal, &
       ' * Hits:',CacheHits, &
       ' * Misses:',CacheMisses

    !**** %Psi hits/miss *****
  Hits = MemCache%Psi_Read%Hits
  Miss = MemCache%Psi_Read%Misses 
  Total = Hits + Miss
  if (Total > 0) then
     Hits = Hits/Total
     Miss = Miss/Total
     Write(Output,*) '    *Psi Read*  %Hits:',Hits, ' * %Misses:',Miss
  End If
  Hits = MemCache%Psi_Write%Hits 
  Miss = MemCache%Psi_Write%Misses 
  Total = Hits + Miss
  if (Total > 0) then
     Hits = Hits/Total
     Miss = Miss/Total
     Write(Output,*) '    *Psi Write*  %Hits:',Hits, ' * %Misses:',Miss
  End If
  Hits = (MemCache%Psi_Read%Hits + MemCache%Psi_Write%Hits) 
  Miss = (MemCache%Psi_Read%Misses + MemCache%Psi_Write%Misses) 
  Total = Hits + Miss
  if (Total > 0) then
     Hits = Hits/Total
     Miss = Miss/Total
     Write(Output,*) '    *Psi Both*   %Hits:',Hits, ' * %Misses:',Miss
  End IF

  If (MemFree < 0) then
     Write(Error_Unit, *) 'DisplayBufferInfo: Memory Error!!! Memory Free:',MemFree
  End If

  Write(Output,*) '*********************************************************'
  Call Flush(OutPut)

  Return
End Subroutine

!******************************************************************************
!
!  Mem_AllocBuffers - Ensures that the required
!         number of buffers are available.
!
!  MH    - Mem_handle array
!  CurrSize - Current number of buffers allocated(RETURNED)
!  NewSize  - Number of buffers wanted
!
!******************************************************************************

Subroutine Mem_AllocBuffers( MH, CurrSize, NewSize)
  Type (Mem_Handle), Pointer :: MH(:)
  Integer, Intent(INOUT)     :: CurrSize
  Integer, intent(IN)        :: NewSize

  Integer :: i,  k
  Integer :: LSize
  Character*100 :: msg

  LSize = PsiArraySize          

!write(*,*) 'mem_allocbuffer: Size(MH) = ',size(MH), ' * CurrSize=',CurrSize, ' * NewSize=',NewSize
  Do i = CurrSize+1, NewSize
     Allocate(MH(i)%Ptr(LSize), STAT=k)
     Write(msg,*) 'Mem_AllocBuffers: Could Not allocate buffer!',  &
           ' * Size:',LSize
     Call Check_Error(k, msg, Error_Unit, .TRUE.,paw_wc, "Mem_AllocBuffers:")

     MH(i)%Index = 0

     MemFree = MemFree - LSize
  End Do

  If (NewSize > CurrSize)  CurrSize = NewSize

  Return
End Subroutine

!******************************************************************************
!
!  CalcBestMemConfig - Calculates the best memory configuration for the given
!            Case
!
!     Proj_Size - Size of each projector 
!     Psi_Size  - Size of each Wave function
!
!     Proj_toLoad - Number of projectors to load 
!     Psi_toLoad  - Number of Psi' buffers to have
!
!******************************************************************************

Subroutine CalcBestMemConfig(MaxProj, MaxPsi,Proj_Size, Psi_Size,Proj_toLoad,Psi_toLoad, ProjMem, PsiMem)
  Integer, Intent(IN)  :: MaxProj
  Integer, Intent(IN)  :: MaxPsi
  Integer, Intent(IN)  :: Proj_size
  Integer, Intent(IN)  :: Psi_Size
  Integer, Intent(OUT) :: Proj_toLoad
  Integer, Intent(OUT) :: Psi_toLoad
  Integer, Intent(OUT) :: ProjMem
  Integer, Intent(OUT) :: PsiMem

  Integer :: i,j,k


Write(Log_Unit,*) 'CalcBestMemConfig: BaseFree=',BaseFree
  Proj_toLoad = (BaseFree - Psi_Size)/Proj_Size
  Proj_ToLoad = Min(Proj_toLoad, MaxProj)
  ProjMem = Proj_toLoad*Proj_Size
  
  Psi_toLoad = (BaseFree - ProjMem)/Psi_Size
Write(Log_Unit,*) 'CalcBestMemConfig: Psi_toLoad=',Psi_toLoad, ' * MaxPsi=',MaxPsi, ' * PRojMem=',ProjMem, ' * Psi_Size=',Psi_Size
  Psi_toLoad = Min(Psi_toLoad, MaxPsi)
  PsiMem = Psi_toLoad*Psi_Size

  Return
End Subroutine


!******************************************************************************
!
!  Phase_Generic - Configures the system to contain only the current
!            Psi's and projectors.
!
!  Psi_toUse - List of Psi's to use. 1=Use
!  Kpnt - Kpnt index for projectors
!
!******************************************************************************

Subroutine Phase_Generic( Psi_toUse,  Kpnt)
  Integer, Intent(IN) :: Psi_toUse(:)
  Integer, Intent(IN) :: Kpnt
  
  Integer :: i, j, NumPsi, NumProj, MaxPsi, CSize, NumAtoms
  !Integer :: Sets_toLoad(Mem_MapSize)
  Integer :: LSize, PSize, ProjMem, PsiMem, MemWanted, mem
  Type (Global_handle), Pointer :: LH, PH
  Character*100 :: msg

   PsiBand_toProcess = Psi_toUse

   Mem_Kpnt = Kpnt

   Psi_toProcess = MH_Skip
   Where (PsiBand_toProcess == MH_toProcess) Psi_toProcess = MH_toProcess
   Call PreparePsi( Psi_toProcess)

   Return
End Subroutine

!*****************************************************************************
!
! Phase_Cluster
!
!*****************************************************************************

Subroutine Phase_Cluster( ClusterIndex)
   Integer, Intent(IN) :: CLusterIndex

   Integer :: i

   Psi_toProcess = MH_SKIP
   Where (ClusterMap == CLusterIndex) Psi_ToProcess = MH_toProcess

   i=1
   Do While ((ClusterMap(i)/=ClusterIndex) .AND. (i<=Mem_MapSize))
      i = i + 1
   End Do

   If (i>Mem_MapSize) then
      write(Error_Unit,*) "Phase_Cluster: Error in index",ClusterIndex,&
           clustermap,i
      call flush(Error_Unit)
      stop
   ENdif
   Call Phase_Generic( Psi_toProcess, PsiInfo(i)%Kpnt)

   Return
End Subroutine


!******************************************************************************
!
!  AllocBestMemConfig - Allocs the data structures for the memory management
!
!******************************************************************************

Subroutine AllocBestMemConfig
  
  Integer :: i, j, NumPsi, NumPRoj
  Integer :: LSize, ProjMem, PsiMem
  Type (Global_handle), Pointer :: LH
  Real :: ProjMem_MEG

   LSize = PsiArraySize   ;    LH => Globalmap     

Proj_Max = 0
Write(*,*) 'AllocBestMemConfig: Proj_Max=',Proj_Max, ' * TotalPsi=',TotalPsi
   Call CalcBestMemConfig(Proj_Max, TotalPsi, 1, LSize, &
        NumProj, NumPSi, ProjMem, PsiMem)



   BaseFree = MemFree
Write(Log_Unit,*) 'AllocBestMemConfig: NumProj=',NumProj, ' * NumPsi=',NumPsi

   Call Mem_AllocBuffers( LH%Psi_Handle, LH%NumPsi, NumPsi)
  
   Return
End Subroutine



!******************************************************************************
!
! AllocatePotential - Allocate the memory for V 
!          
!   NewSize - Number of grid points 
!
!******************************************************************************

Subroutine AllocatePotential( NewSize, SmallSize, Ve_F_size)
  Integer, Intent(IN) :: NewSize
  Integer, Intent(IN) :: SmallSize
  Integer, Intent(IN) :: Ve_F_Size
  Integer :: Used
  Integer       :: i, j, atom
  Character*200 :: msg

  Used = NewSize

  Allocate(SCFvalues%Ve(Used), SCFvalues%Work(Used), &
       SCFvalues%V_local(SmallSize), &
       SCFvalues%RhoSmooth(SmallSize), &
       SCFvalues%CoreTail(SmallSize), &
       SCFvalues%VXC(SmallSize), &
       SCFvalues%RhoHat(SmallSize), &
       SCFvalues%Ve_Fourier(Ve_F_Size), &
       SCFvalues%cVe_F(Ve_F_Size), &
       STAT=j)
  if(spindependence) THEN 
       Allocate(SCFvalues%Vespin(Used),& 
          SCFvalues%RhoSmoothspin(SmallSize), &
          SCFvalues%VXCSPIN(SmallSize), &
          SCFvalues%Ve_Fourierspin(Ve_F_Size), &
          SCFvalues%cVe_Fspin(Ve_F_Size), &
       STAT=j)
  endif

  Write(msg,*) 'AllocatePotential: Could Not allocate  buffer!',  &
        ' * Size:',Used
  Call Check_Error(j, msg, Error_Unit, .TRUE.,paw_wc, "AllocatePotential:")
  
  SCFvalues%Ve = 0
  SCFvalues%V_Local = 0
  SCFvalues%RhoSmooth = 0
  SCFvalues%RhoHat = 0
  SCFvalues%Ve_Fourier = 0
  SCFvalues%cVe_F = 0
  SCFvalues%VXC = 0
  SCFvalues%CoreTail = 0

  if(spindependence) then
     SCFvalues%Vespin = 0
     SCFvalues%RhoSmoothspin = 0
     SCFvalues%Ve_Fourierspin = 0
     SCFvalues%cVe_Fspin = 0
     SCFvalues%VXCSPIN = 0
  endif

  MemFree = MemFree - 2*Used - 3*SmallSize - Ve_F_Size

  Call InitAnderson(SCFvalues%AC, Error_Unit, Mix_Size, &
       Ve_F_Size, 1E-5, V_NewMix, AndersonConditionNo)
  if(spindependence) then
      Call InitAnderson(SCFvalues%ACspin, Error_Unit, Mix_Size, &
       Ve_F_Size, 1E-5, V_NewMix, AndersonConditionNo)
   endif
  Return
End Subroutine


!******************************************************************************
!
!  AllocateGlobal - Allocates the memory management data structures and 
!                   initializes them for use.
!
!******************************************************************************

Subroutine AllocateGlobal
  Integer :: i, j,  PSize, LSize
  Character*100 :: msg


  Allocate(Globalmap,STAT=i)
  msg = 'AllocateGlogal: Could Not allocate Globalmap!'
  Call Check_Error(i, msg, Error_Unit, .TRUE.,paw_wc, "Allocatemap:")

Write(*,*) 'AllocateGlobal: Proj_Max=',Proj_max, ' * Mem_MapSize=',Mem_MapSize
  Psize = Proj_Max
  LSize = Mem_MapSize

       Allocate(Globalmap%Psi_handle(LSize), STAT=j)
       Write(msg,*) 'AllocateGlobal: Could Not allocate Globalmap BUFFER!'
       Call Check_Error(j, msg, Error_Unit, .TRUE.,paw_wc, "AllocateGlobal:")  

       Globalmap%NumPSI = 0
       Globalmap%PsiLoaded = 0
       Globalmap%Psi_Handle(:)%Index = 0

  Return
End Subroutine

!******************************************************************************
!
!  InitMemMgr - Initializes the memory manager routines
!
!  Mem_Size  - The amount of memory to use in Mbytes
!
!
!  Note : This routine should be called AFTER the  projectors have
!         been initialized.
!
!******************************************************************************

Subroutine InitMemMgr(Mem_Size)
  Real, Intent(IN) :: Mem_Size

  Complex :: C
  Integer :: i,j,k,  CalcMinBands
  Real    :: t1
  Character*100 :: msg

  MinBands = TotalElectrons

  ExtraPsi = 0.25*MinBands + 1
  Write(Log_Unit, *) 'InitMemMgr: Extra Bands=',ExtraPsi, ' * Min Bands=',MinBands
  MinBands = MinBands + ExtraPsi

  CalcMinbands = 2*MinBands


  TotalPsi = 2*MinBands !** * NumKpnts
  TotalPsi = MAX(TotalPsi, NumBands)


  If (User_MaxTotalPsi > 0) then
     Write(Log_Unit,*) 'InitMemMgr: Overiding Total Psi! Calculated =', &
          TotalPsi, ' * User Specified =',User_MaxTotalPsi

     TotalPsi = User_MaxTotalPsi
  End If

  If (User_MinPsi > 0) then
     Write(Log_Unit,*) 'InitMemMgr: Overiding Min Psi! Calculated =', &
          MinBands, ' * User Specified =',User_MinPsi

     MinBands = User_MinPsi
  End If


  MemSize = Mem_Size
  write(Log_Unit,*) 'InitMemMgr: TotalPsi=',TotalPsi, ' NumBands=',NumBands, ' MinBands=',MinBands, ' * NumKpnts=',NumKpnts

  MemWords = Mem_Size*1024*1024 / SizeOf_C !** 1 Word = 1 Complex(Double)
  MemFree = MemWords

          !** Check to make sure we have the min. memory required **
  i = 4*Gpnt_Size(G_High) + 2*FFT_Grid(4,G_HIGH) + 1*Gall_Size(G_LOW)
  If (i>MemWords) then
     t1 = i*SizeOf_Complex/(1024.0*1024.0)
     Write(Error_Unit,*) 'InitMemMgr: Not enough memory requested!'
     Write(Error_Unit,*) 'InitMemMgr:    Memory Requested :', MemSize
     Write(Error_Unit,*) 'InitMemMgr:       Memory Needed :', t1
     Write(Error_Unit,*) 'InitMemMgr:      Total Elements :', i
     Write(Error_Unit,*) 'InitMemMgr:   Element BreakDown : '
     Write(Error_Unit,*) 'InitMemMgr:        3x', Gpnt_Size(G_High)
     Write(Error_Unit,*) 'InitMemMgr:        2x', FFT_Grid(4,G_Low)
     Write(Error_Unit,*) 'InitMemMgr:        1x', Gall_Size(G_Low)
     STOP
  End IF

  Mem_MaxPsi = 2*MinBands

  Mem_MapSize = 4*MAX(CalcMinBands,TotalPsi)*NumKpnts +  1 + 1 + 1 + 1 + 1

          !**** Allocate the Kpoint LUT ******
  Allocate(PsiInfo(Mem_MapSize), STAT=i)
  msg = 'InitMemMgr: Could Not allocate WhichKpnt!'
  Call Check_Error(i, msg, Error_Unit, .TRUE.,paw_wc, "InitMemMgr:")

  PsiInfo(:)%Kpnt = 0
  PsiInfo(:)%spinup = .true.
  PsiInfo(:)%DoSave = 0
  PsiInfo(:)%Available = Mem_Available
  PsiInfo(:)%CorrIndex = 0
  PsiInfo(:)%OxIndex = 0
  PsiInfo(:)%MemBufIndex = 0
  PsiInfo(:)%Energy = 0
  PsiInfo(:)%KE = 0
  PsiInfo(:)%Occupancy = 0
  PsiInfo(:)%Error = -1
  PsiInfo(:)%PDOT_Stored = .FALSE.
  PsiInfo(:)%KE_Stored = .FALSE.
  
  Call AllocateGlobal   !** Allocate Data structure's 

  Allocate(SCFvalues, STAT=i)
  msg = 'InitMemMgr: Could Not allocate SCFvalues!'
  Call Check_Error(i, msg, Error_Unit, .TRUE.,paw_wc, "InitMemMgr:")

 ! If ((V_MixType == MIX_V) .OR. (V_MixType == MIX_VEFF)) then
 !    Call AllocatePotential(FFT_Grid(4,G_PROJ), Gpnt_Size(G_HIGH), &
 !         Gpnt_Size(G_PROJ))
 ! else
 !    Call AllocatePotential(FFT_Grid(4,G_PROJ), Gpnt_Size(G_HIGH), &
 !         Gpnt_Size(G_High))
 ! End If

  Call AllocatePotential(FFT_Grid(4,G_HIGH), Gpnt_Size(G_HIGH), &
        Gpnt_Size(G_HIGH))

  Allocate(MemCache, STAT=i)
  msg = 'InitMemMgr: Could Not allocate Work Buffers!'
  Call Check_Error(i, msg, Error_Unit, .TRUE.,paw_wc, "InitMemMgr:")

  MemCache%Psi_Read%Hits = 0
  MemCache%Psi_Read%Misses = 0
  MemCache%Psi_Write%Hits = 0
  MemCache%Psi_Write%Misses = 0

         !*** Allocate Cluster Buffers ****
  Allocate(ClusterMap(Mem_MapSize), ClusterSize(Mem_MapSize), &
           Psi_toProcess(Mem_MapSize), Psi_Processed(Mem_MapSize), &
           PsiBand_toProcess(Mem_MapSize), STAT=i)
  msg = 'InitMemMgr: Could Not allocate Cluster Buffers!'
  Call Check_Error(i, msg, Error_Unit, .TRUE.,paw_wc, "InitMemMgr:")


  ClusterMap = 0
  ClusterSize = 0
  LastCluster = 0

  BaseFree = MemFree

  Psi_Restart = .TRUE.

  Write(Log_Unit, *) 'InitMemMgr: Mem_MapSize = ', Mem_MapSize
  Write(Log_Unit, *) 'InitMemMgr: MemWords = ', MemWords, &
       ' * FreeWords=', MemFree


  Call AllocBestMemConfig

  Write(Log_Unit, *) 'InitMemMgr: MemWords = ', MemWords, &
       ' * FreeWords=', MemFree

  Return
End Subroutine

End Module
