!******************************************************************************
!
!  File : misc.f90
!    by : Alan Tackett
!    on : 10/17/95
!   for : Misc general purpose functions
!
!  This module contains misc. general purpose routines that don't fit
!  in another library. Below is a list of routines contained :
!
!
!  Check_Error(Alloc_error, Error_String, Unit, Die, WC, IDText)
!      Routine for processing an error message return from an intrinsic
!      function, ie, ALLOCATE, OPEN, READ, WRITE, etc. A descriptive
!      message is printed if one occured and dies(default, can be changed).
!
!
!  Swap(a, b)
!      Swaps the values A and B
!
!  PrintDate(Unit, Text)
!      Prints the date and time to the specified unit along with the TEXT 
!
!*****************************************************************************

Module misc

Use word
Use timing

Implicit NONE!!!!

Interface Swap
  Module Procedure R4_Swap
  Module Procedure R8_Swap
  Module Procedure C8_Swap
  Module Procedure C16_Swap
  Module Procedure I4_Swap
  Module Procedure I8_Swap
End Interface

Contains

!****************************************************************************
!
!  Check_Error
!      Routine for processing an error message return from an intrinsic
!      function, ie, ALLOCATE, OPEN, READ, WRITE, etc. A descriptive
!      message is printed if one occured and dies(default, can be changed).
!
!   Error_Number - Error value returned from intrinsic
!   Error_String - Text string to print if an error occurs(OPTIONAL)
!   Unit         - Output Unit(OPTIONAL)
!   Die          - If true then the progam aborts if an error occurs
!                  otherwise only the error message is printed and
!                  control is returned to the calling program.
!                  If this parameter is not present then the default
!                  is to die if an error occurs.(OPTIONAL)
!   WC           - Word Context(OPTIONAL)
!   IDText       - If present then Calls Word_GetAndPrint(OPTIONAL)
!
!****************************************************************************

Subroutine Check_Error(Error_Number, Error_String, Unit, Die, WC, IDText)
  Integer,                 Intent(IN) :: Error_Number
  character*(*), Optional, Intent(IN) :: Error_String
  Integer,       Optional, Intent(IN) :: Unit
  Logical,       Optional, Intent(IN) :: Die
  Type (Word_Context),  OPTIONAL, Intent(INOUT) :: WC     
  Character*(*), Optional, Intent(IN) :: IDText

  Integer :: u
  Real :: tmp
  Character*100 :: errmsg, token


  If (Error_Number /= 0) then
    If (Present(Unit)) then
       u = Unit
    else
       u = 6
    End If

    If (Present(Error_String)) Write(u, *) Error_String
    Write(u,*) TRim(IDText) // ' Error Value : ', Error_Number
    Call Flush(u)


    Write(token,*) Error_Number
    !!Call GError(errmsg)  !** Not available on the SP2 or CRAY!!!!
    Write(u,*) TRim(IDText),'(',TRIM(Token),') ', Trim(errmsg)
!**    Write(u,*) TRim(IDText),'(',TRIM(Token),') '  !** USe this line if cray

    if (Present(IDText) .AND. Present(WC)) Call Word_GetAndPrint(WC, u, IDText)

    if (Present(Die)) then
       if (Die) then
          tmp = 0
          tmp = 1.0/tmp  !** Generate a core dump
       End If
       tmp = 0
       tmp = 1.0/tmp  !** Generate a core dump
    else
          tmp = 0
          tmp = 1.0/tmp  !** Generate a core dump
    End IF
tmp=0
tmp=1.0/tmp
write(u,*) 'Hello!!!!!!!!'
  End If

  Return
End Subroutine  

!****************************************************************************
!
! Swap - Swaps two elements
! 
! A, B - Elements to swap
!
!****************************************************************************

!************************** R4_Swap *******************************************

Subroutine R4_Swap(A, B)
  Real*4, Intent(INOUT) :: A, B

  Real*4  :: T

  T = A
  A = B
  B = T

  Return
End Subroutine

!************************** R8_Swap *******************************************

Subroutine R8_Swap(A, B)
  Real*8, Intent(INOUT) :: A, B

  Real*8  :: T

  T = A
  A = B
  B = T

  Return
End Subroutine

!************************** C8_Swap *****************************************

Subroutine C8_Swap(A, B)
  Complex*8, Intent(INOUT) :: A, B

  Complex*8  :: T

  T = A
  A = B
  B = T

  Return
End Subroutine

!************************** C16_Swap *****************************************

Subroutine C16_Swap(A, B)
  Complex*16, Intent(INOUT) :: A, B

  Complex*16  :: T

  T = A
  A = B
  B = T

  Return
End Subroutine

!************************** I4_Swap *****************************************

Subroutine I4_Swap(A, B)
  Integer*4, Intent(INOUT) :: A, B

  Integer*4  :: T

  T = A
  A = B
  B = T

  Return
End Subroutine

!************************** C8_Swap *****************************************

Subroutine I8_Swap(A, B)
  Integer*8, Intent(INOUT) :: A, B

  Integer*8  :: T

  T = A
  A = B
  B = T

  Return
End Subroutine

!******************************************************************************
!
!  PrintDate - Prints the date to the specified unit with TEXT prepended.
!
!    Unit - Output unit
!    Text - Text for prepending. All trailing blanks are removed. 
!
!******************************************************************************

Subroutine PrintDate(Unit, Text)
  Integer,       Intent(IN) :: Unit
  Character*(*), Intent(IN) :: Text

  Character*10 :: DateStr, TimeStr
  Character*50 :: FmtStr

  Call Date_And_Time(DateStr, TimeStr)

  FmtStr=' ' // DateStr(5:6) // '/' // DateStr(7:8) // '/' // DateStr(1:4) // &
        ', ' // TimeStr(1:2) // ':' // TimeStr(3:4) // ':' // &
          TimeStr(5:10)

  Write(Unit, '(a,a)')  Trim(Text), Trim(FmtStr)

  Return
End Subroutine

End Module
