!******************************************************************************
!
! File : openfile.f90
!   by : Alan Tackett
!   on : 10/31/95
!  for : PAW Project
!
!  Routine to open an output file for subsequent writing.  If a file is 
!  already open then it is closed before the new one is opened.
!
!  Below is the Format for the command:
!
!     OPEN  type filname
!
!  Where: type     - is one of the following - LOG, ERROR, or OUTPUT.
!                    This determines which file is affected.
!         filename - The name of the file to open.  If the name is
!                    SCREEN then ouput is redirected to the screen.
!
!******************************************************************************

Subroutine OpenIt(OldUnit, OldName, NewUnit, NewName)
   Use Word
   Use paw_inout

   implicit none

   Integer,       Intent(IN)    :: OldUnit
   Character*(*), Intent(IN)    :: OldName
   Integer,       Intent(INOUT) :: NewUnit
   Character*(*), Intent(INOUT) :: NewName
   
   Character*200 :: Token

   Token = NewName
   CAll UpperCase(Token)

   If (OldUnit /= 6) Close(OldUnit)

   if (Trim(Token) == "SCREEN") then
      NewUnit = 6
   else
      Open(NewUnit, FILE=NewName, RECL=100000)
   End If

   Write(NewUnit, *) '    '
   
   Return
End Subroutine

Subroutine OpenFile(Input_WC)
  Use paw_inout
  Use word

  Interface
     Subroutine OpenIt(OldUnit, OldName, NewUnit, NewName)
       Use Word
       Use paw_inout
       
       Integer,       Intent(IN)    :: OldUnit
       Character*(*), Intent(IN)    :: OldName
       Integer,       Intent(INOUT) :: NewUnit
       Character*(*), Intent(INOUT) :: NewName
     End Subroutine OpenIt
  End Interface

  Type (Word_context) :: Input_WC

  Character*200 :: token, filename, oldfile
  Integer       :: ierr, tlen, oldfd
  
  Call GetNextWord(Input_WC, Token, tlen)
  if (W_Error == W_EOF) Call EOF_Error(Input_WC, 'OpenFile:')
  Call GetNextWord(Input_WC, Filename, tlen)
  if (W_Error == W_EOF) Call EOF_Error(Input_WC, 'OpenFile:')

  Call UpperCase(token)

  If (Trim(token) == "LOG") then
     oldfd = Log_Unit;           oldFile = Log_Name;
     Log_Unit = LOG_BASE_UNIT;   Log_Name = filename; 
     Call OpenIt(oldfd, oldfile, Log_Unit, Log_Name)
  else If (Trim(token) == "OUTPUT") then
     oldfd = Output_Unit;             oldFile = Output_Name;
     Output_Unit = OUTPUT_BASE_UNIT;  Output_Name = filename; 
     Call OpenIt(oldfd, oldfile, Output_Unit, Output_Name)
  else If (Trim(token) == "ERROR") then
     oldfd = Error_Unit;            oldFile = Error_Name;
     Error_Unit = ERROR_BASE_UNIT;  Error_Name = filename; 
     Call OpenIt(oldfd, oldfile, Error_Unit, Error_Name)  
  else
     Call Word_GetAndPrint(Input_WC, Error_Unit, "OpenFile:")
     Write(Error_Unit, *) 'OpenFile: Unknown Filetype :',Trim(Token)
     Write(Error_Unit, *) 'OpenFile: Command Ignored.'
  End If

  Return
End Subroutine

  
