!******************************************************************************
!
! File : options_data.f90
!   by : Alan Tackett
!   on : 12/07/95
!  for : PAW Project
!
! This file contains the data structures holding run-time options for the  PAW
! program.
!
! Updated for spin by Ping Tang and N. A. W. Holzwarth
!     Last changed 5/23/05
!******************************************************************************

Module options_data


Integer, PARAMETER :: SIM_AE  = 0          !** All-electron calculation
Integer, PARAMETER :: SIM_PAW = 1          !** PAW calculation
Integer, PARAMETER :: SIM_SEPM = 2         !** Semi-empirical PP calculation
Integer, PARAMETER :: SIM_TIME = 3         !** Time-dependent calculation

            !*** Type of potential mixing to perform **
Integer, PARAMETER :: MIX_V       = 1     !** Mix V(n~+n^)
Integer, PARAMETER :: MIX_VEFF    = 2     !** Mix Veff=V(n~+n^)+Vxc+Vlocal
Integer, PARAMETER :: MIX_DENSITY = 3     !** Mix the smooth density

Integer, PARAMETER :: MIN_MEMORY = 1      !** Conserve memory
Integer, PARAMETER :: MIN_TIME   = 2      !** Minimize time (default)
Integer, PARAMETER :: PROJ_RS    = 3      !** Minimize time (default)

Integer, PARAMETER :: MD_PREVIOUS = 1     !** Use previous initial guess for MD
Integer, PARAMETER :: MD_LCAO     = 2     !** Use LCAO for MD initial guess

Integer, PARAMETER :: GUESS_LCAO   = 1
Integer, PARAMETER :: GUESS_RANDOM = 2

Integer, PARAMETER :: GEOMETRY_BROYDEN = 1
Integer, PARAMETER :: GEOMETRY_LINEAR  = 2

Integer, PARAMETER :: EIGEN_DAVIDSON = 1  !** Jacobi-Davidson eigenvalue solver
Integer, PARAMETER :: EIGEN_ENERGY   = 2  !** Energy partition scheme
Integer, PARAMETER :: EIGEN_JDQZ     = 3  !** JDQZ scheme
Integer, PARAMETER :: EIGEN_BLOCK_DAVIDSON = 4  !** Block-Davidson solver

Real    :: Vector_Pot(3)  !** Current vector potential value

Integer :: TIME_MaxN  !** Max applications of Hpsi to be used
Real    :: TIME_tol   !** Correction vector tolerance controlling accuracy 

!****************Spin parameter***********************
Logical           :: Spindependence
Logical           :: Global_Spinup
!****************************************************
logical           :: Nodiagwithload
!logical           :: Estimate_selfenergy  !** disabled
!logical           :: Estimate_scselfenergy !** if true, use atom-centered
                    !also disabled         ! all-electron basis for estimate

Integer           :: Initial_guess
Integer           :: Proj_mode
Integer           :: Run_Mode             !** Run mode
Integer           :: Eigen_Mode
Integer           :: Angular_Points
Integer           :: Grid(3)              !** Curvilinear grid dimensions
Integer           :: LCAO_Cells(3)        !** LCAO summation cells
Integer           :: DoLCAO               !** Gen LCAO wfns=1
Real              :: LCAO_Gcut            !** Gcut for LCAO guess
Logical           :: SCF_DoGramSchmidt    !** Do explicit GS orthog in SCF
Integer           :: AtomicVxc_Step       !** Step size for Atomic Vxc calc
Integer           :: MD_guess             !** MD initial guess type
Logical           :: forces_Always_Calc_H
Real              :: Force_Zero           !** force min non-0 value
Real              :: Eigen_Max            !** Used for Band Structure mode
Real              :: AndersonConditionNo  !** Anderson Mix SVD parameter   
Logical           :: Calc_O_Eigenvalues   !** Controls eigenvalue solver
Logical           :: Filter_Potential     !** Controls truncation of V
Real              :: Overlap_Tol          !** Min overlap eigenvalue kept
                                          !     for generalized eigenvalues
Logical           :: HamLoaded            !** True if CalcHam or LoadHam called
Real              :: ClusterTol           !** Tolerance for eigenvalue clusters
Integer           :: Geometry_Move        !** type of move for geometry update
Integer           :: Mix_Size             !** # of vects to mix for V/den
Logical           :: SaveUnmixed          !** Option for storedata output

Character*100     :: SolnFilename         !** File to read initial guess from

Integer           :: Vxc_MinSize          !** Min # of points for Vxc calc

Real      :: Psi_Memory           !** Mem for Psi's
Real      :: Proj_Memory          !** MEmory to use in Mbytes for Radial Projs
Real      :: Bloch_Memory         !** MEmory to use in Mbytes for Phase
Real      :: Ylm_Memory           !** Mem for Ylm's

Integer           :: User_MaxTotalPsi !** Max Psi's set by the user(optional)
Integer           :: User_MinPsi      !** Min Psi's set by the user(optional)
Integer           :: GMRES_Size       !** Number of basis vectors for GMRES



Real      :: TimeStep         !** Time step for MD runs

Real      :: Geometry_NewMix  !** Mixing factor for Gemotry relaxations
Real      :: V_Weight         !** Scale factor for V
Real      :: V_NewMix         !** Mixing factor for V
Real      :: V_DampNewMix
Real      :: Dij_NewMix       !** Mixing factor for Dij's
Real      :: Dij_DampNewMix
Real      :: Mix_DampRelative
Integer   :: V_MixType        !** Type of potential mixing(V or Rho)
Real      :: V_Smooth_Width   !** Smoothing of V in H*Psi to control conv.


Integer           :: V_CorrIter(3)   !** V control(1=Loops,2=DoGMRES,3=JacIter)
Integer           :: Psi_CorrIter(3) !** Psi control

Real      :: Debug_Fraction_Slice(3)
Integer, Pointer  :: Debug_Slice(:,:)
Integer           :: Debug_Dir                           
Integer           :: Debug_Grid_Dir

Integer           :: Atomic_Mode       !** Type of calculation AE or PAW
Real              :: PW_Gcut(3)           !** PW G-cut
Real, Pointer     :: User_Kpoints(:,:)  !** List of k-points from the user 
                                        !** in fractions of recip lattice

!******** Boundary condition data *******
Integer, PARAMETER :: BC_PLANE = 0 !** Plane boundary condition
Integer, PARAMETER :: BC_VALUE = 0 !** Fill condition with value

Type Boundary_Data
   Integer :: Condition_Type  !** Type of condition
   Integer :: Plane_Dir       !** Which Plane ->x,y,z
   Integer :: Fill_type       !** Fill type
   Integer :: Plane_Start     !** Start Plane index
   Integer :: Plane_End       !** End plane index
   Complex :: Value   !** Fill Value
End Type

Type (Boundary_Data), Pointer :: Boundary_List(:)
Integer                       :: Boundary_Size

Integer, PARAMETER :: BC_PER    = 0   !** Periodic boundary conditions
Integer, PARAMETER :: BC_NONPER = 1   !** Non-periodic boundary cond 

Integer :: BC_Mode(3)


Type Electric_Field_Type  !** Define the electric field data structure
  Integer :: Etype
  Real    :: E0(3)
  Real    :: T0
  Real    :: Phase
  Real    :: Frequency
End Type

Type (Electric_Field_type) :: E_field

Integer, PARAMETER :: E_SINE = 0  !** Sine wave E-field


!*** Timing variables **
Real :: Parallel_Time      !** Parallel time
Real :: Serial_Time        !** Serial or setup time
Real :: Global_Start_Time  !** Starting time AFTER init for SCF solve
Real :: Start_Time       !** Local start time for accumulating Par/Serial total

!*** Load Balancing variables ***
Real :: Scale_MaxJobPerCPU
Real :: Scale_MinJobPerCPU


Integer :: Matrix_Range(5)

Integer, PARAMETER :: RANGE_DO = 1
Integer, PARAMETER :: RANGE_SKIP = 2

Logical :: BandStructure_Mode  !** Do band struct calcs with fixed Ham
Logical :: ReCalculate_Occupancies  !** false for band structure plotting   

Integer  :: Dummy_Integer   !** Dummy variables for intermediate testing
Real     :: Dummy_Real   

Integer, PARAMETER :: JDQZ_PRECOND_ONCE    = 0 !** Make 1 prec/iter
Integer, PARAMETER :: JDQZ_PRECOND_CLUSTER = 1 !** Prec each cluster/iter

Real     :: JDQZ_Energy     !** Shift energy for JDQZ method 
Integer  :: JDQZ_MaxMV      !** Max number of matrix-vector prod's for JDQZ 
Integer  :: JDQZ_WaitIter   !** Wait iterations before JDQZ method is used
Integer  :: JDQZ_Precond_Mode !** Preconditioner mode
Integer  :: JDQZ_Qmax        
Integer  :: JDQZ_Qmin
Integer  :: JDQZ_Jmax
Integer  :: JDQZ_Jmin
Integer  :: JDQZ_MaxPerBand
Real     :: JDQZ_eps
Real     :: JDQZ_TargetShift
Logical  :: JDQZ_Spatial_Selection

End Module
