!***************************************************************************
!
! File : paw_init.f90
!   by : Alan Tackett
!   on : 7/27/98
!  for : PAW Project
!
!
!  PAW_Init - Performs Initialization for the PAW program
!
!***************************************************************************

Subroutine PAW_Init
  Use crystal_data
  Use paw_inout
  Use exchange_corr
  Use atom_data
  Use options_data
  Use spherical_harmonic
  Use mem_data
  Use local_criteria_lib
  Implicit NONE!!!!!

  Print_Level = PRINT_NORMAL           !** Output Level ***

!***  Adjoint_PARALLEL = .TRUE.         !** Default adjoint mode to parallel

  Log_Name    = "SCREEN"               !** Init output filenames
  Output_Name = "SCREEN"
  Error_Name  = "SCREEN"
  Band_Name = "paw.bands" 
  Charge_Name = "paw.charges"
  Force_Name = "paw.forces"
  DOS_Name   = "paw.dos"
  PLOT_Name  = "plot"
  Sym_Name   = 'RUNTIME'
  Pos_Name   = "paw.pos"
  Best_Name  = ""

  Log_Unit    = 6                      !*** Setup Output Units ****
  Output_Unit = 6
  Error_Unit  = 6
  
  Length_Output_Unit = LENGTH_UNIT     !*** Define Output Units ***
  Energy_Output_Unit = ENERGY_UNIT 

  XC_Type = XC_LDA_PW                  !*** Set default Exchange Correlation type ***
  XC_Type_Set = .false.                !*** Exchange-correlation type not yet explicitly set
  Spindependence = .false.             !*** Default spin unrestricted calculation
  Global_spinup = .true.               !*** Default to Dij, etc.
                                       !*** false uses Dijspin, etc.

  Nodiagwithload = .false.             !** used in LoadHamandPsi
                                       ! .true. used to by-pass diagonalization
                                       ! of Hamiltonian

  !Estimate_selfenergy = .false.        !** no longer used
  !Estimate_scselfenergy = .false.      ! If true, use atom-centered all-
      !disabled                        !   electron basis <=Rc for estimate
                                       
  Angular_Points = 12   !** Set angular integration mesh
  Run_Mode = MIN_TIME   !** Minimize execution time
  Proj_mode = 0
  SCF_DoGramSchmidt = .FALSE.  !** Don't do explicit GS orthog 
  Eigen_Mode = EIGEN_DAVIDSON
  Calc_O_Eigenvalues = .true. !Diagonalize O matrix for gen. eigen. problem
  Filter_potential = .false. !Do not truncate potential at G_PROJ
  Overlap_Tol = 1E-11 ! minimum overlap eigenvalue for generalized eigenvalues
  HamLoaded = .FALSE.   ! set to true with CalcHam or LoadHam
  SaveUnmixed = .FALSE.  ! Storedata output from Anderson mix

  Atom_Types = 0                       !*** Init different Atom Types
  Max_Atom_types = 0
  UseShells = .false.        !** Default is not to use shell parameterization
  Specific_Atoms = 0         !** Initialize atom count

  Atomic_Mode = SIM_PAW           !** Default to PAW simulation

  Vector_Pot = 0              !** Zero vector potential 
  TIME_MaxN = 20
  Time_Tol  = 1E-15

  Xtal_Defined = .FALSE.               !*** No Xtal is defined ***
!**  BZ_Defined   = .FALSE.               !*** No BZ currently set ***

  BC_Mode = BC_PER

  LCAO_Cells = 1                      !** Sum over nn for initial LCAO psi's
  DoLCAO     = 1   !** Use LCAO initial wave functions
  Initial_guess = DoLCAO

  ClusterTol = 0.07

  Psi_Memory  = 40
  Proj_Memory = 40
  Bloch_Memory = 5
  Ylm_Memory = 5

  PAW_FirstTime = .TRUE.

  !CAll InitSpharm

  User_MaxTotalPsi  = -1    !** By default disable this option
  User_MinPsi  = -1         !** By default disable this option


  Boundary_Size = -1     !** Default is no special boundary conditions
  BC_Mode = BC_PER

  TimeStep = 1      !** Default DT for MD run

  V_Weight = 1.0
  V_NewMix = 0.5
  V_DampNewMix = 0.25
  Dij_NewMix = 1.0
  Dij_DampNewMix = 1.0
  V_MixType = MIX_VEFF
  Geometry_NewMix = 0.5
  Mix_DampRelative = 0.2

  LCAO_Gcut = 10000      !** Init LCAO Gcut to use ALL Psi G's
  
  MD_Guess = MD_PREVIOUS
  Forces_Always_Calc_H = .FALSE.
  Force_Zero = 1E-10

  Eigen_Max = 1         !** Init max eigenvalue for CalcBandMerit
  AndersonConditionNo = 10000     !** Initial value for Anderson SVD

  
  AtomicVxc_Step = 1     !** Don't skip any is the default

  V_CorrIter = (/10, 1, 10/)     !** GMRES solver Loops, DoGMRES, JacobiIter
  Psi_CorrIter = (/10, 1, 10/)

  Debug_Fraction_SLice = 0.0
  Debug_Dir = 1

  NuclearEnergy = 0
  

  BandStructure_Mode = .FALSE.
  ReCalculate_Occupancies = .TRUE.

  Scale_MaxJobPerCPU = 0.5
  Scale_MinJobPerCPU = 0.1  !** Currently not used


  Geometry_Move = GEOMETRY_BROYDEN

  Dummy_Integer = 0
  Dummy_Real = 0

  Anchor = .false.
  AnchorIndex = 1

  Mix_Size = 5    !** Default V/density mixing size

  NULLIFY(Move_AC)

  JDQZ_Energy = -5
  JDQZ_MaxMV  = 5
!  JDQZ_WaitIter = 5
  JDQZ_WaitIter = 500   !** Never turn it on
  JDQZ_Precond_Mode = JDQZ_PRECOND_ONCE
  JDQZ_Qmax = 10
  JDQZ_Qmin = 5
  JDQZ_Jmax = 2*JDQZ_Qmax
  JDQZ_Jmin = JDQZ_Qmax
  JDQZ_MaxPerBand = 2
  JDQZ_eps = 1E-10
  JDQZ_TargetShift = -0.1
  JDQZ_Spatial_Selection = .FALSE.

  V_Smooth_Width = 0   !** Default to no High G smoothing of the potential

  Return
End Subroutine
