!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Program prepareballandstick
!
!   Stand-alone program to determine atomic positions and bonds
!    Calculate atomic positions and bonds for Ball and Stick model
!         (Data Explorer output and XCrysden output)
!
!   This version is based on Atomplot3d but takes interactive input
!      input and output is bohr units for dx, anstroms for XCrysden
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
Implicit None

 Real(8), allocatable :: ballpoints(:,:)
 Real(8) :: resx, resy, resz ,plotmat(3,3),solvmat(3,3),v(3),vin(3),vout(3)
 Real(8) :: chk,rtol,tx,ty,tz,XA,YA,ZA
 Real(8), parameter :: atol=1.e-3
 Integer, allocatable :: noballs(:)
 Integer :: ncount,ncountmax,many,ierr,npair,i,j,k,n,specific_atoms,types,iostat
 Character(128) :: atompositionfile,PLOT_NAME
 Character(2) :: sym

 Real(8) :: X(3),Y(3),Z(3),volume,maxbond,bondtol,basis(3,3),origin(3)
 Real(8), allocatable :: atompos(:,:)
 Integer, allocatable :: typeindex(:)
 Character(2), allocatable :: typemap(:),asym(:)
 integer, parameter :: CHARGEOUT_UNIT = 15


 Read(5,*) basis
 write(6,*) 'basis', basis
 Read(5,*) X,Y,Z
 write(6,*) 'X,Y,Z',X,Y,Z
 Read(5,*) origin
 write(6,*) 'origin', origin
 Read(5,*) maxbond,bondtol
 write(6,*) 'maxbond,bondtol',maxbond,bondtol
 Read(5,*) types
 write(6,*) 'types',types

 volume = ABS(basis(1,1)*(basis(2,2)*basis(3,3)-basis(3,2)*basis(2,3)) +&
              basis(2,1)*(basis(3,2)*basis(1,3)-basis(1,2)*basis(3,3)) +&
              basis(3,1)*(basis(1,2)*basis(2,3)-basis(2,2)*basis(1,3)))

 Allocate(typemap(types))
 Read(5,*) typemap
 write(6,*) 'typemap', typemap

 read(5,*) atompositionfile, PLOT_NAME
 write(6,*) 'files',atompositionfile, PLOT_NAME

 open(7,file=TRIM(atompositionfile),form='formatted')
 specific_atoms=0
    do
        read(7,*,iostat=i) sym,XA,YA,ZA
        write(6,*) sym,XA,YA,ZA
        if (i/=0) exit
        specific_atoms=specific_atoms+1
        write(6,*) sym,XA,YA,ZA
    enddo

 write(6,*) 'specific_atoms = ', specific_atoms

 allocate(atompos(3,specific_atoms),typeindex(specific_atoms))

 rewind(7)
    do i=1,specific_atoms
       read(7,*) sym,atompos(1,i),atompos(2,i),atompos(3,i)
       typeindex(i)=0
       do j=1,types
          if (TRIM(typemap(j))==TRIM(sym)) typeindex(i)=j
       enddo
       if (typeindex(i)<1) then
          write(6,*) 'Type not found ', &
              sym,atompos(1,i),atompos(2,i),atompos(3,i)
          exit
       endif
       write(6,*) i,' sym ', typeindex(i)
     enddo

  rtol=bondtol  !Tolerance of including atoms

  rtol=MAX(atol,rtol)

! Determine length of plotting cell boundaries
    resx=SQRT(DOT_Product(X,X))
    resy=SQRT(DOT_Product(Y,Y))
    resz=SQRT(DOT_Product(Z,Z))
    XA=resx
    YA=resy
    ZA=resz


  Open(unit=CHARGEOUT_UNIT,&
   file=Trim(PLOT_Name)//".axes.dx",form="formatted")

   write(CHARGEOUT_UNIT, &
     '("object 1 class array type float rank 0 items 8 data follows")')
   write(CHARGEOUT_UNIT,'("  1  1  1  1  1  1  1  1")')
   write(CHARGEOUT_UNIT,'("attribute ""dep"" string ""positions""")')
   write(CHARGEOUT_UNIT,'("#")')
   write(CHARGEOUT_UNIT,&
    '("object 2 class array type float rank 1 shape 3 items 8 data follows")')

   v=0
   write(CHARGEOUT_UNIT,'(1p3e15.7)') v(1:3)
   write(CHARGEOUT_UNIT,'(1p3e15.7)') XA,v(2),v(3)
   write(CHARGEOUT_UNIT,'(1p3e15.7)') v(1),YA,v(3)
   write(CHARGEOUT_UNIT,'(1p3e15.7)') XA,YA,v(3)
   write(CHARGEOUT_UNIT,'(1p3e15.7)') v(1),v(2),ZA
   write(CHARGEOUT_UNIT,'(1p3e15.7)') XA,V(2),ZA
   write(CHARGEOUT_UNIT,'(1p3e15.7)') v(1),YA,ZA
   write(CHARGEOUT_UNIT,'(1p3e15.7)') XA,YA,ZA

   write(CHARGEOUT_UNIT,'("attribute ""dep"" string ""positions""")')
   write(CHARGEOUT_UNIT,'("#")')

   write(CHARGEOUT_UNIT, &
    '("object 3 class array type int rank 1 shape 2 items 12 data follows")')
   write(CHARGEOUT_UNIT, '("  0  1")')
   write(CHARGEOUT_UNIT, '("  0  2")')
   write(CHARGEOUT_UNIT, '("  1  3")')
   write(CHARGEOUT_UNIT, '("  2  3")')
   write(CHARGEOUT_UNIT, '("  4  5")')
   write(CHARGEOUT_UNIT, '("  4  6")')
   write(CHARGEOUT_UNIT, '("  5  7")')
   write(CHARGEOUT_UNIT, '("  6  7")')
   write(CHARGEOUT_UNIT, '("  0  4")')
   write(CHARGEOUT_UNIT, '("  1  5")')
   write(CHARGEOUT_UNIT, '("  2  6")')
   write(CHARGEOUT_UNIT, '("  3  7")')

   write(CHARGEOUT_UNIT,'("attribute ""element type"" string ""lines""")')
   write(CHARGEOUT_UNIT,'("attribute ""ref"" string ""positions""")')
   write(CHARGEOUT_UNIT,'("#")')
   write(CHARGEOUT_UNIT,'("object ""molecule"" class field")')
   write(CHARGEOUT_UNIT,'("component ""data"" value 1")')
   write(CHARGEOUT_UNIT,'("component ""positions"" value 2")')
   write(CHARGEOUT_UNIT,'("component ""connections"" value 3")')
   write(CHARGEOUT_UNIT,'("attribute ""name"" string ""molecule""")')
   write(CHARGEOUT_UNIT,'("#")')
   write(CHARGEOUT_UNIT,'("end")')

   close (CHARGEOUT_UNIT)

! file for atomic positions
  Open(unit=CHARGEOUT_UNIT,&
    file=Trim(PLOT_Name)//".atom.dx",form="formatted")


! Rough estimate of number atoms in plot

    many=resx*resy*resz/volume+1
    ncountmax=8*(many**3)*Specific_Atoms 

    write(6,*)   ' many = ', many

  Allocate(noballs(ncountmax),ballpoints(3,ncountmax),asym(ncountmax))

  noballs=0
  ballpoints=0


   ncount=0

   If (maxbond.gt.0.) then
     tx=rtol*maxbond/resx
     ty=rtol*maxbond/resy
     tz=rtol*maxbond/resz
   Else
     chk=volume**(0.33333)
     tx=rtol*chk/resx
     ty=rtol*chk/resy
     tz=rtol*chk/resz
   EndIf


   do n=1,Specific_Atoms      ! Some assumptions here about shape of cell

     do i = -many, many
        do j = -many, many
           do k = -many, many

           v = atompos(:,n) - origin + (/i,j,k/) 

           v= MATMUL(basis,v)

           vout(1) = DOT_Product(X,v)/resx
           vout(2) = DOT_Product(Y,v)/resy
           vout(3) = DOT_Product(Z,v)/resz

           vin(1)=vout(1)/resx
           vin(2)=vout(2)/resy
           vin(3)=vout(3)/resz

!  Check if atom is in plotting cell
           if (vin(1).gt.-tx.and.vin(1).lt.1+tx &
               .and.vin(2).gt.-ty.and.vin(2).lt.1+ty &
               .and.vin(3).gt.-tz.and.vin(3).lt.1+tz) then
               
              ncount=ncount+1 
              noballs(ncount)=typeindex(n)
              asym(ncount)=typemap(typeindex(n))
              ballpoints(:,ncount)=vout

           Endif

           Enddo
         Enddo     
       Enddo
    Enddo


    ballpoints(:,1:ncount) = ballpoints(:,1:ncount)

!  output atomic coordinates for Data Explorer
 
    write(CHARGEOUT_UNIT, &
     '("object 1 class array type float rank 0 items ",i5," data follows")') &
      ncount
    write(CHARGEOUT_UNIT,*) (noballs(i),i=1,ncount)   

    write(CHARGEOUT_UNIT,'("attribute ""dep"" string ""positions""")')
    write(CHARGEOUT_UNIT,'("#")')
    write(CHARGEOUT_UNIT,&
     '("object 2 class array type float rank 1 shape 3 items ",i5," data follows")') &
      ncount

         do i=1,ncount
          write(CHARGEOUT_UNIT,*) ballpoints(1:3,i)
          enddo

        write(CHARGEOUT_UNIT,'("attribute ""dep"" string ""positions""")')
        write(CHARGEOUT_UNIT,'("#")')


         chk=(maxbond)**2
         npair=0  
          do i=1,ncount-1
           do j=i+1,ncount
            v=ballpoints(:,i)-ballpoints(:,j)
            if(DOT_Product(v,v).le.chk) then
            npair=npair+1
            ! write(CHARGEOUT_UNIT,*) i,j
            endif
           enddo
          enddo

   write(CHARGEOUT_UNIT, &
   '("object 3 class array type int rank 1 shape 2 items ",i5," data follows")')&
   npair 
          do i=1,ncount-1
           do j=i+1,ncount
            v=ballpoints(:,i)-ballpoints(:,j)
            if(DOT_Product(v,v).le.chk) then
             write(CHARGEOUT_UNIT,*) i-1,j-1
            endif
           enddo
          enddo

         write(CHARGEOUT_UNIT,'("attribute ""element type"" string ""lines""")')
         write(CHARGEOUT_UNIT,'("attribute ""ref"" string ""positions""")')
         write(CHARGEOUT_UNIT,'("#")')
         write(CHARGEOUT_UNIT,'("object ""molecule"" class field")')
         write(CHARGEOUT_UNIT,'("component ""data"" value 1")')
         write(CHARGEOUT_UNIT,'("component ""positions"" value 2")')
         write(CHARGEOUT_UNIT,'("component ""connections"" value 3")')
         write(CHARGEOUT_UNIT,'("attribute ""name"" string ""molecule""")')
         write(CHARGEOUT_UNIT,'("#")')
         write(CHARGEOUT_UNIT,'("end")')

      Close(CHARGEOUT_UNIT)



!  XCrysDen output
  Open(unit=CHARGEOUT_UNIT,&
    file=Trim(PLOT_Name)//".xsf",form="formatted")
  Write(CHARGEOUT_UNIT,'("CRYSTAL")')
  Write(CHARGEOUT_UNIT,'("PRIMVEC   1")')
  write(CHARGEOUT_UNIT,'(1p3e15.7)') X(1:3)*0.5291772108
  write(CHARGEOUT_UNIT,'(1p3e15.7)') Y(1:3)*0.5291772108
  write(CHARGEOUT_UNIT,'(1p3e15.7)') Z(1:3)*0.5291772108
  Write(CHARGEOUT_UNIT,'("PRIMCOORD   1")')
  Write(CHARGEOUT_UNIT,'(i10  "   1")')  ncount

     do i=1,ncount
        
        write(CHARGEOUT_UNIT,'(a2,2x,1p3e15.7)') asym(i),  &
                      ballpoints(1:3,i)*0.5291772108
     enddo



End program
