!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Program prepareballandstickplus
!
!   Stand-alone program to determine atomic positions and bonds
!    Calculate atomic positions and bonds for Ball and Stick model
!         (Data Explorer output and XCrysden output)
!
!   This version is based on Atomplot3d but takes interactive input
!      input and output is bohr units for dx, anstroms for XCrysden
!   This version has some embellishments for Li3PO4  --
!      assume we want supercell in integer multiples of original cell
!      Scheme will only work with orthogonal cell
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
Implicit None

 Real(8), allocatable :: ballpoints(:,:),nballpoints(:,:)
 Real(8) :: resx, resy, resz ,plotmat(3,3),solvmat(3,3),v(3),vin(3),vout(3)
 Real(8) :: chk,rtol,tx,ty,tz,XA,YA,ZA,ua,ub,uc,ttx,tty,ttz
 Real(8), parameter :: atol=1.e-3
 Integer, allocatable :: noballs(:),nnoballs(:)
 Integer :: ncount,ncountmax,many,ierr,npair,i,j,k,n,specific_atoms,types
 Integer :: na,nb,nc,idum(500)
 Character(128) :: atompositionfile,PLOT_NAME
 Character(2) :: sym

 Real(8) :: X(3),Y(3),Z(3),volume,maxbond,bondtol,basis(3,3),origin(3)
 Real(8), allocatable :: atompos(:,:)
 Integer, allocatable :: typeindex(:),inside(:),inlist(:),pair1(:),pair2(:),it(:)
 Character(2), allocatable :: typemap(:),asym(:),nasym(:)
 integer, parameter :: CHARGEOUT_UNIT = 15


 Read(5,*) basis     ! crystal basis functions in bohr
 write(6,*) 'basis', basis
 Read(5,*) na,nb,nc    ! integer multiples of lattice vectors to make 
                       !   supercell
 X=na*basis(:,1); Y=nb*basis(:,2);     Z=nc*basis(:,3)
 write(6,*) 'X,Y,Z',X,Y,Z
 Read(5,*) origin       !  origin of plot in fractional coordinates
 write(6,*) 'origin', origin
 Read(5,*) maxbond,bondtol     ! in bohr
 write(6,*) 'maxbond,bondtol',maxbond,bondtol
 Read(5,*) types
 write(6,*) 'types',types

 volume = ABS(basis(1,1)*(basis(2,2)*basis(3,3)-basis(3,2)*basis(2,3)) +&
              basis(2,1)*(basis(3,2)*basis(1,3)-basis(1,2)*basis(3,3)) +&
              basis(3,1)*(basis(1,2)*basis(2,3)-basis(2,2)*basis(1,3)))

 Allocate(typemap(types),inlist(types))
 Read(5,*) typemap,inlist
 write(6,*) 'typemap', typemap
 write(6,*) 'inlist', inlist
 

 read(5,*) atompositionfile, PLOT_NAME
 write(6,*) 'files',atompositionfile, PLOT_NAME

 open(7,file=TRIM(atompositionfile),form='formatted')
 specific_atoms=0
    do
        read(7,*,iostat=i) sym,XA,YA,ZA
        write(6,*) sym,XA,YA,ZA
        if (i/=0) exit
        specific_atoms=specific_atoms+1
        write(6,*) sym,XA,YA,ZA
    enddo

 write(6,*) 'specific_atoms = ', specific_atoms

 allocate(atompos(3,specific_atoms),typeindex(specific_atoms),&
              inside(specific_atoms))

 rewind(7)
    do i=1,specific_atoms
       read(7,*) sym,atompos(1,i),atompos(2,i),atompos(3,i)
       typeindex(i)=0
       do j=1,types
          if (TRIM(typemap(j))==TRIM(sym)) typeindex(i)=j
       enddo
       if (typeindex(i)<1) then
          write(6,*) 'Type not found ', &
              sym,atompos(1,i),atompos(2,i),atompos(3,i)
          exit
       endif
       write(6,*) i,' sym ', typeindex(i)
       inside(i)=inlist(typeindex(i))
     enddo

  rtol=bondtol  !Tolerance of including atoms

  rtol=MAX(atol,rtol)

! Determine length of plotting cell boundaries
    resx=SQRT(DOT_Product(X,X))
    resy=SQRT(DOT_Product(Y,Y))
    resz=SQRT(DOT_Product(Z,Z))
    XA=resx
    YA=resy
    ZA=resz
    ua=sqrt(Dot_Product(Basis(:,1),Basis(:,1)))
    ub=sqrt(Dot_Product(Basis(:,2),Basis(:,2)))
    uc=sqrt(Dot_Product(Basis(:,3),Basis(:,3)))

    n=(na+1)*(nb+1)*(nc+1)

  Open(unit=CHARGEOUT_UNIT,&
   file=Trim(PLOT_Name)//".axes.dx",form="formatted")

   idum=1
   write(CHARGEOUT_UNIT, &
     '("object 1 class array type float rank 0 items ",i4," data follows")') n
   write(CHARGEOUT_UNIT,'(80i4)') (idum(i),i=1,n)
   write(CHARGEOUT_UNIT,'("attribute ""dep"" string ""positions""")')
   write(CHARGEOUT_UNIT,'("#")')
   write(CHARGEOUT_UNIT,&
    '("object 2 class array type float rank 1 shape 3 items ",i4," data follows")') n

   do i=1,na+1
      do j=1,nb+1
         do k=1,nc+1
            v=(i-1)*basis(:,1)+(j-1)*basis(:,2)+(k-1)*basis(:,3)
            write(CHARGEOUT_UNIT,'(1p3e15.7)') v(1:3)
         enddo
       enddo
   enddo

   write(CHARGEOUT_UNIT,'("attribute ""dep"" string ""positions""")')
   write(CHARGEOUT_UNIT,'("#")')

   n=na*(nb+1)*(nc+1)+(na+1)*(nb*(nc+1)+(nb+1)*nc)
   write(CHARGEOUT_UNIT, &
    '("object 3 class array type int rank 1 shape 2 items",i4," data follows")' )   n
   n=0
   do i=1,na+1
      do j=1,nb+1
         do k=1,nc+1
            n=n+1
            if (k>1) write(CHARGEOUT_UNIT, '(2i4)') n-2,n-1
            if (j>1) write(CHARGEOUT_UNIT, '(2i4)') n-nb-1-1,n-1
            if (i>1) write(CHARGEOUT_UNIT, '(2i4)') n-(nb+1)*(nc+1)-1,n-1
         enddo
      enddo
    enddo

   write(CHARGEOUT_UNIT,'("attribute ""element type"" string ""lines""")')
   write(CHARGEOUT_UNIT,'("attribute ""ref"" string ""positions""")')
   write(CHARGEOUT_UNIT,'("#")')
   write(CHARGEOUT_UNIT,'("object ""molecule"" class field")')
   write(CHARGEOUT_UNIT,'("component ""data"" value 1")')
   write(CHARGEOUT_UNIT,'("component ""positions"" value 2")')
   write(CHARGEOUT_UNIT,'("component ""connections"" value 3")')
   write(CHARGEOUT_UNIT,'("attribute ""name"" string ""molecule""")')
   write(CHARGEOUT_UNIT,'("#")')
   write(CHARGEOUT_UNIT,'("end")')

   close (CHARGEOUT_UNIT)

! file for atomic positions
  Open(unit=CHARGEOUT_UNIT,&
    file=Trim(PLOT_Name)//".atom.dx",form="formatted")


! Rough estimate of number atoms in plot

    many=resx*resy*resz/volume+1
    ncountmax=8*(many**3)*Specific_Atoms 

    write(6,*)   ' many = ', many

  Allocate(noballs(ncountmax),ballpoints(3,ncountmax),asym(ncountmax),&
   nnoballs(ncountmax),nballpoints(3,ncountmax),nasym(ncountmax),&
          pair1(ncountmax),pair2(ncountmax),it(ncountmax))

  noballs=0;nnoballs=0
  ballpoints=0;nballpoints=0


   ncount=0

   If (maxbond.gt.0.) then
     tx=rtol*maxbond/resx
     ty=rtol*maxbond/resy
     tz=rtol*maxbond/resz
   Else
     chk=volume**(0.33333)
     tx=rtol*chk/resx
     ty=rtol*chk/resy
     tz=rtol*chk/resz
   EndIf


   do n=1,Specific_Atoms      ! Some assumptions here about shape of cell

     if (inside(n)==0) then
        ttx=0; tty=0; ttz=0
     else 
        ttx=tx;  tty=ty;   ttz=tz
     endif

     do i = -many, many
        do j = -many, many
           do k = -many, many

           v = atompos(:,n) - origin + (/i,j,k/) 

           v= MATMUL(basis,v)

           vout(1) = DOT_Product(X,v)/resx
           vout(2) = DOT_Product(Y,v)/resy
           vout(3) = DOT_Product(Z,v)/resz

           vin(1)=vout(1)/resx
           vin(2)=vout(2)/resy
           vin(3)=vout(3)/resz

!  Check if atom is in plotting cell
           if (vin(1).gt.-ttx.and.vin(1).lt.1+ttx &
               .and.vin(2).gt.-tty.and.vin(2).lt.1+tty &
               .and.vin(3).gt.-ttz.and.vin(3).lt.1+ttz) then
               
              ncount=ncount+1 
              noballs(ncount)=typeindex(n)
              asym(ncount)=typemap(typeindex(n))
              ballpoints(:,ncount)=vout
              it(ncount)=inside(n)

           Endif

           Enddo
         Enddo     
       Enddo
    Enddo

   write(6,*) 'ncount = ', ncount

         chk=(maxbond)**2
         npair=0  
          do i=1,ncount-1
           do j=i+1,ncount
            v=ballpoints(:,i)-ballpoints(:,j)
            if(DOT_Product(v,v).le.chk) then
             npair=npair+1
             pair1(npair)=i;pair2(npair)=j
            endif
           enddo
          enddo

    write(6,*) 'pairs = ', npair

        !  Eliminate unbounded atoms with inside=1

        i=0
          do j=1,ncount
             if(it(j)>0) then
               do k=1,npair
                  if (pair1(k)==j.or.pair2(k)==j) then
                     i=i+1
                     nballpoints(:,i)=ballpoints(:,j)
                     nasym(i)=asym(j)
                     nnoballs(i)=noballs(j)
                     exit
                  endif
               enddo
              else
                i=i+1
                nballpoints(:,i)=ballpoints(:,j)
                nasym(i)=asym(j)
                nnoballs(i)=noballs(j)
              endif
          enddo

     ncount=i
     write(6,*) 'new number ', ncount

!  output atomic coordinates for Data Explorer
 
    write(CHARGEOUT_UNIT, &
     '("object 1 class array type float rank 0 items ",i5," data follows")') &
      ncount
    write(CHARGEOUT_UNIT,*) (nnoballs(i),i=1,ncount)   

    write(CHARGEOUT_UNIT,'("attribute ""dep"" string ""positions""")')
    write(CHARGEOUT_UNIT,'("#")')
    write(CHARGEOUT_UNIT,&
     '("object 2 class array type float rank 1 shape 3 items ",i5," data follows")') &
      ncount

         do i=1,ncount
          write(CHARGEOUT_UNIT,*) nballpoints(1:3,i)
          enddo

        write(CHARGEOUT_UNIT,'("attribute ""dep"" string ""positions""")')
        write(CHARGEOUT_UNIT,'("#")')


         chk=(maxbond)**2
         npair=0  
          do i=1,ncount-1
           do j=i+1,ncount
            v=nballpoints(:,i)-nballpoints(:,j)
            if(DOT_Product(v,v).le.chk) then
            npair=npair+1
            ! write(CHARGEOUT_UNIT,*) i,j
            endif
           enddo
          enddo

   write(CHARGEOUT_UNIT, &
   '("object 3 class array type int rank 1 shape 2 items ",i5," data follows")')&
   npair 
          do i=1,ncount-1
           do j=i+1,ncount
            v=nballpoints(:,i)-nballpoints(:,j)
            if(DOT_Product(v,v).le.chk) then
             write(CHARGEOUT_UNIT,*) i-1,j-1
            endif
           enddo
          enddo

         write(CHARGEOUT_UNIT,'("attribute ""element type"" string ""lines""")')
         write(CHARGEOUT_UNIT,'("attribute ""ref"" string ""positions""")')
         write(CHARGEOUT_UNIT,'("#")')
         write(CHARGEOUT_UNIT,'("object ""molecule"" class field")')
         write(CHARGEOUT_UNIT,'("component ""data"" value 1")')
         write(CHARGEOUT_UNIT,'("component ""positions"" value 2")')
         write(CHARGEOUT_UNIT,'("component ""connections"" value 3")')
         write(CHARGEOUT_UNIT,'("attribute ""name"" string ""molecule""")')
         write(CHARGEOUT_UNIT,'("#")')
         write(CHARGEOUT_UNIT,'("end")')

      Close(CHARGEOUT_UNIT)



!  XCrysDen output
  Open(unit=CHARGEOUT_UNIT,&
    file=Trim(PLOT_Name)//".xsf",form="formatted")
  Write(CHARGEOUT_UNIT,'("CRYSTAL")')
  Write(CHARGEOUT_UNIT,'("PRIMVEC   1")')
  write(CHARGEOUT_UNIT,'(1p3e15.7)') X(1:3)*0.5291772108
  write(CHARGEOUT_UNIT,'(1p3e15.7)') Y(1:3)*0.5291772108
  write(CHARGEOUT_UNIT,'(1p3e15.7)') Z(1:3)*0.5291772108
  Write(CHARGEOUT_UNIT,'("PRIMCOORD   1")')
  Write(CHARGEOUT_UNIT,'(i10  "   1")')  ncount

     do i=1,ncount
        
        write(CHARGEOUT_UNIT,'(a2,2x,1p3e15.7)') nasym(i),  &
                      nballpoints(1:3,i)*0.5291772108
     enddo



End program
