!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Program preparespindos
! 9/4/99   Natalie Holzwarth
!
!   Program to read dos or pdos output from pwpaw pgm and generate
!      plotting output
!
!  calling argument :  filename for pwpaw output
!
!  For pdos output, the result is averaged over the specified spheres        
!
!  Modified 5/23/05 for spin density plots
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!

implicit none

Real*8            :: derf
Real, parameter   :: eunit=13.6056981  ! Rydberg to eV conversion
Real, allocatable :: charges(:),e(:),dos(:),pdos(:,:),tdos(:)
Real, allocatable :: dosdn(:),pdosdn(:,:),mu(:),tdosup(:),tdosdn(:)
Integer, allocatable :: NMap(:),Map(:,:)
Real :: Pi,sigma,e0,de,x,enk,fnk,term,fac,shift
Character*2 :: eval
Character*80 :: filenm,description
Character*4 :: spin
Integer :: Nsphere,Ntot
Integer :: ne,npdos,nk,ierr,i,j,k,ie,iunit

External GetArg

      Call GetArg(1,filenm)

      iunit=7

      open(unit=iunit, file=trim(filenm), form='formatted')
      open(unit=iunit+1, file=trim(filenm)//'.dosout', form='formatted')
      write(iunit+1,*) ' # HEADER LINE -- dos results for ',trim(filenm)

      read(iunit,*) Nsphere    !# of pdos spheres
      read(iunit,*) Ntot       !# of k and eigenvalue entries

      If (Nsphere.gt.0) then
        Write(6,*) 'PDOS information available for',Nsphere,' sites'
        Do i=1,Nsphere
           read(iunit,'(a80)') description
           write(6,'(a80)') description
        End Do

        Allocate(charges(Nsphere))

        Write(6,*) 'enter # of pdos results to calculate'
        read(5,*) npdos

        If (npdos.gt.0) then

         Allocate(NMap(npdos),Map(npdos,Nsphere))
         open(unit=iunit+2, file=trim(filenm)//'.pdosout', form='formatted')
         write(iunit+2,*) ' # HEADER LINE -- pdos results for ',trim(filenm)

         Do i=1,npdos        
            write(6,*) 'For pdos #',i,' enter no. of spheres and list them'
            read(5,*) j,(Map(i,k),k=1,j)
            NMap(i)=j
         End Do
 
       Endif   !npdos >0

     Endif   !Nsphere >0

     Write(6,*) 'Enter # energies for dos and pdos arrays'
     Read(5,*) ne

     Write(6,*) 'Enter e0, de in eV for dos and pdos arrays'
     Read(5,*) e0,de

     Write(6,*) 'Enter energy shift in Ry for energy 0'
     Read(5,*) shift

     Write(6,*) 'Enter sigma for gaussian smearing'
     Read(5,*) sigma
     Pi=ACOS(-1.0)
     fac=1.0/(SQRT(Pi)*sigma)    !Each spin has weight 1

     Allocate(e(ne),dos(ne),tdos(ne),dosdn(ne),tdosdn(ne),mu(ne),tdosup(ne))
     If (npdos.gt.0) Allocate(pdos(ne,npdos),pdosdn(ne,npdos))

     dos=0;  tdos=0 ; tdosup=0; mu=0; tdosdn=0

     do i=1,ne
       e(i)=e0+de*(i-1)
     enddo

     if (npdos.gt.0) pdos=0
     if (npdos.gt.0) pdosdn=0

     do k=1,Ntot

      read(iunit,'(1p5e15.7,a4)') x,x,x,fnk,enk,spin
      If (Nsphere.gt.0) read(iunit,*) (charges(i),i=1,Nsphere)

       enk=(enk-shift)*eunit    ! shift 0 of energy and change to eV units

       do ie=1,ne

         term=fac*fnk*EXP(-((e(ie)-enk)/sigma)**2)
         if (spin=='  up') then
            dos(ie)=dos(ie)+term
            tdos(ie)=tdos(ie)+fnk*(1+derf((e(ie)-enk)/sigma))/2
         else if (spin=='  dn') then
            dosdn(ie)=dosdn(ie)+term
            tdosdn(ie)=tdosdn(ie)+fnk*(1+derf((e(ie)-enk)/sigma))/2
         else
            write(6,*) 'Error in file -- spin = ',spin
            stop
         endif


         If (npdos.gt.0) then
            Do j=1,npdos
               x=0
               Do i=1,NMap(j)
                  x=x+charges(Map(j,i))
               End Do
               if (spin=='  up') then
                 pdos(ie,j)=pdos(ie,j)+x*term
               else if (spin=='  dn') then
                 pdosdn(ie,j)=pdosdn(ie,j)+x*term
               endif
            End Do
         EndIf
       
       Enddo   ! ie (ne)

    Enddo  ! k (Ntot)

    If (npdos.gt.0) then
      do j=1,npdos
         pdos(:,j)=pdos(:,j)/NMap(j)
         pdosdn(:,j)=pdosdn(:,j)/NMap(j)
      Enddo
    EndIf 

    write(iunit+1,'(" # HEADER LINE: ne shift sigma -- ",i6,2x,1p2e15.7)')& 
                        ne,shift,sigma
    if (npdos.gt.0) then
       write(iunit+2,&
         '(" # HEADER LINE: ne npdos shift sigma -- ",2i6,2x,1p2e15.7)')& 
                        ne,npdos,shift,sigma
       Do j=1,npdos
          write(iunit+2,'(" # Map for #",i5,90i3)')j,(Map(j,i),i=1,NMap(j))
       Enddo
    EndIf

    
    Call Filter(dos,ne)
    Call Filter(dosdn,ne)
    Call Filter(tdos,ne)
    Call Filter(tdosdn,ne)
    If (npdos.gt.0) Call Filter(pdos,ne*npdos)
    If (npdos.gt.0) Call Filter(pdosdn,ne*npdos)
     
    mu=dos-dosdn
    tdosup=tdos
    tdos=tdos+tdosdn
    DO ie=1,ne
       write(iunit+1,'(1p7e15.7)') e(ie),dos(ie),dosdn(ie),tdosup(ie),&
             tdosdn(ie),tdos(ie),e(ie)/eunit+shift
       If (npdos.gt.0) then
         write(iunit+2,'(1p25e15.7)') e(ie),(pdos(ie,j),j=1,npdos),&
             (pdosdn(ie,j),j=1,npdos)
       Endif
    EndDo

  Stop
  End  
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! subroutine filter
!
!  zeros elements of an array with values smaller then tol
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine filter(x,n)
Real,  Intent(INOUT):: x(n)
Integer , Intent(IN) :: n
Real , parameter :: tol=1.e-13
Integer    :: i

do i=1,n

   if (ABS(x(i)).lt.tol) x(i)=0

Enddo

Return
End

