!******************************************************************************
!
! File : psilib.f90
!   by : Alan Tackett
!   on : 7/29/98
!  for : PAW
!
! Module containing routines for moving the wave functions to and from
! memory.
!
!******************************************************************************

Module psilib

Use mem_data
Use word
Use paw_inout
Use misc
Use fileio

Implicit NONE!!!!


Logical           :: Psi_Restart
Integer           :: Psi_Pos        !** Current Psi buffer position
!Integer, PRIVATE :: PSI_Loaded     !** How many psi's are loaded in the buffer

!******************************************************************************
Contains
!******************************************************************************

!******************************************************************************
!
! ReadPSI - Loads a particular PSI record from disk
!
! Rec    - Record to load
! Psi    - Psi Handle
!
!******************************************************************************

Subroutine ReadPSI( Rec, Psi)
  Integer,           Intent(IN)    :: Rec
  Type (Mem_Handle), Intent(INOUT) :: Psi

  Type (Mem_Handle)   , Pointer   :: MH1
  Character*100 :: token
  Integer :: i

  If (Rec == 0) RETURN !**** Exit if dummy record requested *****

  If ((Psi%Index>0)) then
     If(PsiInfo(Psi%Index)%DoSave==1) then    ! need to save psi
         If (CanAlloc(PsiArraySize)) then
            Call AllocMemHandle(PsiArraySize, i, MH1)
            Call Swap_Mem_Handles(MH1,Psi)
            PsiInfo(MH1%Index)%MemBufIndex=i
         Else
           Call WritePsi_NOW( Psi)
         EndIf
     End If
  End If


  Write(token,*) 'ReadPsi: Error reading Wave functions! :', &
       ' * Rec : ',Rec

    Call ReadFile_LOW(FD_Psi,  Rec, Psi%Ptr, token)
    Psi%Index = Rec;
  
    MemCache%Psi_Read%Misses=MemCache%Psi_Read%Misses + 1


  Return
End Subroutine

!******************************************************************************
!
! WritePSI_NOW - Writes a particular PSI to disk
!
! Psi    - Psi Handle   
!
!******************************************************************************

Subroutine WritePSI_NOW(  Psi)
  Type (Mem_Handle), Intent(INOUT) :: Psi

  Character*100 :: token

  IF ( (Psi%Index /= 0)) then
     Write(token,*) 'Error Writing Wave functions! :', &
          ' * Rec : ',Psi%Index
     Call WriteFile_LOW(FD_Psi,  Psi%Index, Psi%Ptr, token)

     PsiInfo(Psi%Index)%MemBufIndex = 0
     Psi%Index = 0

     MemCache%Psi_Write%Misses=MemCache%Psi_Write%Misses+1
  End If

  Return
End Subroutine

!******************************************************************************
!
! SavePsi_toBuffer - Saves the Psi Buffer if possible 
!                    otherwise  PSi is saved ta disk
!
!  PsiIndex - Psi's disk Index
!
!  Psi      - Psi to save
!
!******************************************************************************

Subroutine SavePsi_toBuffer( PsiIndex, Psi)
  Integer,           Intent(IN) :: PsiIndex
  Complex,   Intent(IN) :: Psi(:)

  Type(Mem_handle), Pointer :: Psi1(:)
  Type(Mem_handle), Pointer :: MH1
  Integer :: i, NumPsi, j
  Character*100 :: msg

  If (PsiIndex <= 0 ) then

    write(Error_Unit,*) 'Entered SavePsi_toBuffer with PsiIndex=',PsiIndex
    return
  EndIf

  ! Assume this vector is to be kept
  PsiInfo(PsiIndex)%DoSave=1
  PsiInfo(PsiIndex)%Available=Mem_Used

  NumPsi = Globalmap%NumPsi
  Psi1 => Globalmap%Psi_Handle
  !** Check for buffer match ** 
  i = PsiInfo(PsiIndex)%MemBufIndex
  If (i > 0) then
     Psi1(i)%Ptr = Psi;  
     Psi1(i)%Index=PsiIndex
     MemCache%Psi_Write%Hits = MemCache%Psi_Write%Hits + 1
  else         !*** Not in buffer, so save to blank buffer or disk ***
     i = 1
     Do While ((Psi1(i)%Index /= 0) .AND. (i < NumPsi))
        i = i + 1
     End Do

     !*** Check if there's free memory for a buffer or a blank buffer ***
     If (Psi1(i)%Index == 0) then   !** Blank Buffer
        Psi1(i)%Ptr = Psi
        MemCache%Psi_Write%Hits = MemCache%Psi_Write%Hits + 1

        Psi1(i)%Index = PsiIndex;    
        PsiInfo(PsiIndex)%MemBufIndex = i
     else if (CanAlloc( PsiArraySize)) then

        CAll AllocMEmHandle( PsiArraySize, i, MH1)
        MH1%Ptr = Psi
        MemCache%Psi_Write%Hits = MemCache%Psi_Write%Hits + 1

        MH1%Index = PsiIndex;    
        PsiInfo(PsiIndex)%MemBufIndex = i
     else     !**** Must store it to disk ******
        Write(msg, *) 'SavePsi_toBuffer: Error writing Psi index:',&
            PsiIndex,  ' * RecSize=',Size(Psi)
        Call WriteFile_LOW(FD_Psi,  PsiIndex, Psi, Msg)
        MemCache%Psi_Write%Misses=MemCache%Psi_Write%Misses+1
        PsiInfo(PsiIndex)%MemBufIndex = 0
     End If
  End If

  Return
End Subroutine


!******************************************************************************
!
! GetPsi_fromBuffer - Gets the Psi from the
!         current Buffer if possible otherwise the PSi is read from
!         disk
!
!  PsiIndex - Psi's disk Index
!  Psi      - Psi to read
!
!******************************************************************************

Subroutine GetPsi_fromBuffer( PsiIndex, Psi)
  Integer,           Intent(IN) :: PsiIndex
  Complex,   Intent(OUT) :: Psi(:)

  Type(Mem_handle), Pointer :: Psi1(:)
  Type(Mem_handle), Pointer :: MH1
  Integer :: i, NumPsi
  Character*100 :: msg

  If(PsiIndex<=0) then
   write(Error_Unit,*) 'Error in GetPsi_FromBuffer - PsiIndex=',PsiIndex 
   call flush(Error_Unit)
   Stop  
  EndIf

  NumPsi = GlobalMap%NumPsi
  Psi1 => GlobalMap%Psi_Handle


  !** Check for buffer match ** 
  If (PsiInfo(PsiIndex)%MemBufIndex > 0) then
     i = PsiInfo(PsiIndex)%MemBufIndex
     Psi = Psi1(i)%Ptr
     MemCache%Psi_Read%Hits = MemCache%Psi_Read%Hits + 1
  else          !*** Not in buffer, so load from disk ***
     Write(msg, *) 'GetPsi_fromBuffer: Error reading Psi index:',PsiIndex
!write(Log_Unit,*) 'in getpsi reading index',psiindex,psiinfo(psiindex)%membufindex
     Call ReadFile_LOW(FD_Psi,  PsiIndex, Psi, Msg)
     MemCache%Psi_Read%Misses = MemCache%Psi_Read%Misses + 1

       !*** Check if there's free memory for a buffer
     If (CanAlloc( PsiArraySize)) then
        Call AllocMEmHandle( PsiArraySize, i, MH1)
        MH1%Ptr = Psi

        MH1%Index = PsiIndex;    
        PsiInfo(PsiIndex)%MemBufIndex = i
!write(Log_Unit,*) 'ingetpsi -- saving to allocated slot', psiindex,i
     End If
  End If

  Return
End Subroutine


!******************************************************************************
!
!  PreparePsi - Loads the Psi's  into the data buffer and
!               initializes Psi_Processed 
!
!  Process_List - List of Psi's for loading. A Psi will be marked for 
!                 loading if its Process_List index == MH_toProcess.
!                 Otherwise it will be ignored.
!
!
!******************************************************************************

Subroutine PreparePsi( Process_List)
  Integer, Intent(IN) :: Process_List(:)

  Integer :: i, j, k, PsiNeeded, PsiLoaded, Used,Psi1Index,Psi2Index
  Integer :: LSize
  Type (Global_handle), Pointer :: LH
  Type (Mem_handle), Pointer   :: MH(:)
  Type (Mem_handle), Pointer   :: MH1
  Character*100 :: msg
  Complex, Pointer :: Ptr(:)

  Psi_Processed = Process_List


  LSize = PsiArraySize
  LH => Globalmap
  MH => LH%PSi_handle
  Used = 0

!write(Log_Unit,*) 'inpreparepsi',SUM(Psi_Processed)

  PsiNeeded = SUM(Psi_Processed)
  If ((LH%NumPsi>0).AND.(PsiNeeded>0)) Then
     Do i=1, LH%NumPsi    !** Move the Good Psi's to the front
        Psi1Index=MH(i)%Index
        If(Psi1Index > 0) then
          IF (PSi_Processed(Psi1Index) == MH_toProcess) then
           Used = Used + 1
           If (i /= Used) then
           Psi2Index=MH(Used)%Index
           If (Psi2Index > 0) then
              Call Swap_Mem_Handles(MH(i), MH(Used))
              PsiInfo(Psi1Index)%MemBufIndex = used
              PsiInfo(Psi2Index)%MemBufIndex = i
           Else
              Call Swap_Mem_Handles(MH(i),MH(Used))
              PsiInfo(Psi1Index)%MemBufIndex = used
           EndIf  ! Psi2Index>0
           EndIf  !i/= Used
           
          Psi_Processed(MH(Used)%Index) = MH_Processed  !** Skip in next phase
          End If !PsiProcessed
         EndIf ! PsiIndex > 0
     End Do
   EndIf

  PsiNeeded = PsiNeeded - Used   
  PsiLoaded = Used
  i=Used

!write(Log_Unit,*) 'inpreparepsi -- used loaded needed' , Used,Psiloaded,Psineeded
  If (PsiNeeded > 0) then
    Do
       i=i+1
       If ((SUM(Psi_Processed)<=0).OR.(i>LH%NumPsi)) Exit
       j = 1
       Call GetNextIndex_toProcess(j, Psi_Processed)
       If (j <= 0 ) then
          write(Error_Unit,*) 'Error in PreparePsi -- j=',j
          Stop
       EndIf
       Psi1Index=MH(i)%Index
!write(Log_Unit,*) 'inpreparepsi -- i,j',i,j,Psi1Index
       If (Psi1Index > 0) then  ! need to save psi1Index
         Call ReadPsi( j, MH(i))
         PsiInfo(j)%MemBufIndex=i
         Psi_Processed(j)=MH_Processed
!write(Log_Unit,*) 'inpreparepsi -- Called ReadPsi'     
       Else
         Used=PsiInfo(j)%MemBufIndex
!write(Log_Unit,*) 'inpreparepsi -- Did not call readpsi', used,i,j
         If (Used > 0 ) then
            Call Swap_Mem_Handles(MH(Used),MH(i))
            PsiInfo(j)%MemBufIndex=i
!write(Log_Unit,*) 'preparepsi--strange happenings',Used,i,j,MH(i)%Index
         Else
            write(msg,*)'Error in PreparePsi - j = ',j
            Call ReadFile_LOW(FD_Psi, j ,MH(i)%Ptr,msg)
            PsiInfo(j)%MemBufIndex = i
            MH(i)%Index=j
!write(Log_Unit,*) 'preparepsi--Called ReadFile',i,j,MH(i)%Index
         EndIf
         Psi_Processed(j)=MH_Processed
       EndIf
 
           PsiLoaded = PsiLoaded + 1
        
     End Do
   EndIf
   Psi_Restart = .TRUE.
   Psi_Processed = Process_List
   LH%PsiLoaded = PsiLoaded


  Return
End Subroutine


!******************************************************************************
!
!  GetNextPSI - Returns the Next Psi or Set of Psi's
!
!  Flag       - Index=0 if no more Psi's, >0 otherwise
!  Psi1       - Current  Psi 
!
!  NOTE: No error checking is done. 
!
!******************************************************************************

Subroutine GetNextPsi(Flag, Psi1)
  Integer,                     Intent(OUT) :: Flag
  Type (Mem_Handle),           Pointer     :: Psi1

  Integer :: i, j, Left, Psi_Loaded, List(Mem_MapSize), A(1)

  Flag = 0
  PSI_Loaded = Globalmap%PsiLoaded
 

  If (PSI_Restart) then   !** Restart
     Psi_Pos = 1
     Psi_restart = .FALSE.
     Psi_Processed = Psi_toProcess
  End IF

  If (Psi_Pos > Psi_Loaded) then  !** Try to load the next set
     PSI_Loaded = Globalmap%NumPsi
     Left=SUM(Psi_Processed)

     If (Left <= 0) then   !** No Psi's left to process so restart
        Psi_Restart = .TRUE.
        Psi_Processed = Psi_toProcess
     End If

     List=Psi_Processed
     Call PreparePsi( List)

     if (Left > 0) Psi_Restart = .FALSE.

    Psi_Pos = 1
  End If

  If (.NOT. Psi_Restart) then
     Psi1 => Globalmap%PSI_Handle(Psi_Pos)

     MemCache%Psi_Read%Hits = &
          MemCache%Psi_Read%Hits + 1

     Flag = Psi1%Index
     If(Flag>0)  then
         Psi_Processed(Flag) = MH_PROCESSED   !** Mark as processed
         Flag=PsiInfo(Flag)%Kpnt
     Else
         write(Error_Unit,*) ' Error in GetNextPsi -- flag=',&
                  flag,globalmap%Psiloaded
     EndIf
     Psi_Pos = PSi_Pos + 1
  else                  !** Finished Processing
!Write(*,*) 'GetNExtPsi: BZ_Index=0!!!!!!!!!!!!!!!!!!!!'
     Flag     = 0
!     Nullify(Psi1)
  End If

  Return
End Subroutine


!*****************************************************************************
!
!  GetNextPsi_UnProcessed - Gets the next UNPROCESSED Psi
!       and also initialize sequential processing
!
!  Flag       - Flag=0 if no more Psi's need to be processed,
!                   >0 otherwise
!  VecProcessed - Array determining which vectors have been processed.
!           A processed Psi contains a 0 in the appropriate array slot.
!  PsiIndex - Psi and FAS disk Index returned
!  PsiPtr   - Pointer to the next Psi
!
!****************************************************************

Subroutine GetNextPsi_UnProcessed(Flag,VecProcessed,PsiIndex,PsiPtr)
  Integer,         Intent(OUT)   :: Flag
  Integer,         Intent(INOUT) :: VecProcessed(:)
  Integer,         Intent(OUT)   :: PsiIndex
  Complex, Intent(OUT)   :: PsiPtr(:)

 !Integer :: i, j, Used, Psiinbuffer,Left, Psi_list(Mem_MapSize)
  Integer :: i, j, Used, Psiinbuffer,Left
  Integer ,allocatable :: Psi_list(:)
  Type(Mem_handle), Pointer :: MH1(:)
  Type(Mem_handle) :: temp   
  Complex, Pointer  :: Ptr(:)
  Character*100  :: msg

  Allocate(Psi_list(Mem_MapSize))
  Flag = 0
  If (SUM(VecProcessed) <= 0) return   ! no more vectors left

 
  MH1 => Globalmap%Psi_Handle

  i=1
  Do While ((i<Mem_MapSize) .AND. ((VecProcessed(i) == MH_Processed) .OR. &
     (Psi_toProcess(i) == MH_Skip)))
        i = i + 1
     End Do

     If ((VecProcessed(i) == MH_Processed) .OR. &
          (Psi_toProcess(i) == MH_Skip)) then
        Flag  = 0
        DeAllocate(Psi_list)
        RETURN      !*** None left to process so exit early
     End If

     j=PsiInfo(i)%MemBufIndex
     If (j > 0) then

        PsiIndex = i
        Flag = PsiInfo(PsiIndex)%Kpnt
        PsiPtr = MH1(j)%Ptr

        MemCache%Psi_Read%Hits = &
          MemCache%Psi_Read%Hits + 1
     
     Else
        Call GetPsi_fromBuffer( i , PsiPtr)

        PsiIndex = i
        Flag = PsiInfo(PsiIndex)%Kpnt

     End if


  !*** Now we need to move all the unprocessed Psi's to the front of  ***
  !*** the buffer and fill the remaining slots with unprocessed Psi's ***

  Psi_List = MH_Skip
  Where ((Psi_toProcess == MH_toProcess) .AND. &
       (VecProcessed == MH_Processed))     Psi_List = MH_toProcess
  VecProcessed(PsiIndex) = MH_Processed  !** Mark returned psi as processed
  Psi_List(PsiIndex)=MH_Processed

  Call PreparePsi( PSi_List)

  Psi_Restart = .FALSE.  !*** Don't want GetNextPsi using Psi_toPRocess
  Psi_Pos = 1
  DeAllocate(Psi_list)

  Return
End Subroutine


!******************************************************************************
!
!  InitPsiLib - Initializes the PSIlib routines for use.
!               Currently it only opens the Psi  data files.
!
!******************************************************************************

Subroutine InitPsiLib

  implicit none

  Integer :: RecSize, err
  Character*100 :: msg

  Psi_Restart = .TRUE.


  RecSize = SizeOf_Complex * PsiArraySize
  Write(Log_Unit,*) 'InitPsiLib: Rec Size=',RecSize
  call flush(Log_unit)

  Open(FD_Psi, Form="UNFORMATTED", status="SCRATCH", &
       Access="DIRECT", RECL=REcSize, IOSTAT=err)
  Write(msg,*) ' Error opening file. Rec Size=',RecSize
  Call Check_Error(err, msg, Error_Unit, .TRUE., PAW_wc, &
        "InitPsiLib:")

  Return
End Subroutine

End Module
