!***************************************************************************
!
! File : read_input.f90
!   by : Alan Tackett
!   on : 7/27/98
!  for : PAW Project
!
!
!  Read_Input - Reads and Parses the Input to the PAW program
!               This is the Main execution loop
! 
! Updated for spin by Ping Tang and N. A. W. Holzwarth
!    Last change 5/23/05
!***************************************************************************


Subroutine Read_Input(WC)
  Use paw_inout
  Use atom_data
  Use crystal_data
  Use options_data
  Use relaxsys
  Use debug
  Use hamiltonian
  Use mem_data
  Use memmgr
  Use forces
  Use ldatom_info
  Use storedata
  Use charges
  Use solver
  Use local_criteria_lib

  Implicit NONE!!!!!!!

  Type (Word_Context) :: WC  

  Character(132) :: Token, token2
  Integer       :: Tlen, i, j, PM(3),  itmp
  Logical       :: IsNew,keep
  Real          :: rtmp

  Call GetNextWord(WC, Token, tlen)
!Write(*,*) 'Read_Input : Token: !',trim(token),'! * Error=',error

  Do While (W_Error /= W_EOF)
    Call UpperCase(token)

!Write(*,*) 'Read_Input : Token: !',trim(token),'!'
    If (Trim(Token) == "ALL_ELECTRON") then
       Atomic_Mode = SIM_AE
       Write(Log_Unit,*) 'Atomic_Mode = ALL ELECTRON'
    else if (Trim(Token) == "PAW") then
       Atomic_Mode = SIM_PAW
       Write(Log_Unit,*) 'Atomic_Mode = PAW'
    else if (Trim(Token) == "SEPM") then
       Atomic_Mode = SIM_SEPM
       BandStructure_Mode = .TRUE.
       DoLCAO = GUESS_RANDOM
       Initial_guess = DoLCAO
       Write(Log_Unit,*) 'Atomic_Mode = SEPM'
       write(log_unit,*) 'Semi-empirical mode no longer works'
       stop
    else if (Trim(token) == "DUMMY_INTEGER") then
       Call GetNumber(WC, Dummy_Integer)
       Write(Log_Unit,*) 'Read_Input: Dummy_Integer:',Dummy_Integer
    else if (Trim(token) == "DUMMY_REAL") then
       Call GetNumber(WC, Dummy_REAL)
       Write(Log_Unit,*) 'Read_Input: Dummy_REAL:',Dummy_REAL
    else if (Trim(Token) == "MAX_ATOMTYPES") then !** Allocate space for atoms
       Call InitAtomTypes(WC)
    else if (Trim(token) == "ATOMTYPE") then    !** Load Atom Type
       Call LdAtomType(WC)
    !else if (Trim(token) == "V_SCSELF") then !** Load scself terms
    !        --disabled--                     !  must be called after ldatomtype
    !  Call LoadSCself(WC)
    else if (Trim(token) == "ATOMTYPE_SEPM") then   !** Load an SEPM Atom Type
       Call LdAtomType_SEPM(WC)
    else if (Trim(token) == "ATOMTYPE_OCCUPANCY") then  !** Load Type Occ
       Call LdAtomTypeOcc(WC)
    else if (Trim(token) == "MAX_SPECIFIC_ATOMS") then !** Allocate specfic
       Call InitSpecificAtoms(WC)                      !** atoms space
    else if (Trim(token) == "ATOM") then        !** Load an atom
       Call LdAtom(WC)
    else if (Trim(token) == "ATOM_LIST") then        !** Load a list of atoms
       Call LdAtomList(WC)
    else if (Trim(token) == "USESHELLS") then        !** Shell Parameters    
       UseShells=.true.
       Call LdShellInfo(WC,.false.,.true.)
    else if (Trim(token) == "USESHELLS_WITHORIGIN") then!** Shell Parameters    
       UseShells=.true.
       Call LdShellInfo(WC,.true.,.true.)
    else if (Trim(token) == "USEFRACSHELLS_WITHORIGIN") then!**Shell Parameters
       UseShells=.true.
       Call LdShellInfo(WC,.true.,.false.)
    else if (Trim(token) == "SPIN") then        !** Spin restricted calculation
       Spindependence=.true.
       Write(Log_Unit,*) 'Read_Input: Spin dependent calculation'
    !else if (Trim(token) == "WITH_SELFENERGY") then   !** Estimate self-energy
    !   Estimate_selfenergy=.true.
    !   Write(Log_Unit,*) 'Read_Input: Coulomb self energy will be estimated'
    !else if (Trim(token) == "WITH_SCSELFENERGY") then  !** Estimate self-energy
    !  Estimate_scselfenergy=.true.
    !   Write(Log_Unit,*) 'Read_Input: Coulomb self energy will be estimated'
    !   Write(Log_Unit,*) '   from all-electron atomic basis functions'
    else if (Trim(Token) == "PRINT") then       !** User text output
       Call DoPrint(WC)
    else if (Trim(token) == "ANGULAR_POINTS") then
       Call GetNumber(WC, Angular_Points)
       Write(Log_Unit,*) 'Read_Input: Angular_Points:',Angular_points
    else if (Trim(token) == "MIX_SIZE") then
       Call GetNumber(WC, Mix_Size)
       Write(Log_Unit,*) 'Read_Input: Mix_Size:',Mix_Size
    else if (Trim(token) == "MINIMIZE_TIME") then
       Run_Mode = MIN_TIME
       Write(Log_Unit,*) 'Read_Input: Run_Mode = MIN_TIME'
    else if (Trim(token) == "MINIMIZE_MEMORY") then
       Run_Mode = MIN_MEMORY
       Write(Log_Unit,*) 'Read_Input: Run_Mode = MIN_MEMORY'
    else if (Trim(token) == "GEOMETRY_BROYDEN") then
       Geometry_Move = GEOMETRY_BROYDEN
       Write(Log_Unit,*) 'Read_Input: Geometry_Move = GEOMETRY_BROYDEN'
    else if (Trim(token) == "GEOMETRY_LINEAR") then
       Geometry_Move = GEOMETRY_LINEAR
       Write(Log_Unit,*) 'Read_Input: Geometry_Move = GEOMETRY_LINEAR'
    else if (Trim(token) == "EIGENVALUE_DAVIDSON") then
       Eigen_Mode = EIGEN_DAVIDSON
       Write(Log_Unit,*) 'Read_Input: Eigen_Mode = EIGEN_DAVIDSON'
    else if (Trim(token) == "EIGENVALUE_ENERGY") then
       Eigen_Mode = EIGEN_ENERGY
       Write(Log_Unit,*) 'Read_Input: Eigen_Mode = EIGEN_ENERGY'
    else if (Trim(token) == "EIGENVALUE_JDQZ") then
       Eigen_Mode = EIGEN_JDQZ
       Write(Log_Unit,*) 'Read_Input: Eigen_Mode = EIGEN_JDQZ'
    else if (Trim(token) == "EIGENVALUE_BLOCK_DAVIDSON") then
       Eigen_Mode = EIGEN_BLOCK_DAVIDSON
       Write(Log_Unit,*) 'Read_Input: Eigen_Mode = EIGEN_BLOCK_DAVIDSON'
    else if (Trim(token) == "JDQZ_ENERGY") then
       Call GetNumber(WC, JDQZ_energy)
       Write(Log_Unit,*) 'Read_Input: JDQZ_Energy = ', JDQZ_Energy
    else if (Trim(token) == "JDQZ_QMAX") then
       Call GetNumber(WC, JDQZ_QMAX)
       Write(Log_Unit,*) 'Read_Input: JDQZ_Qmax = ', JDQZ_Qmax
    else if (Trim(token) == "JDQZ_QMIN") then
       Call GetNumber(WC, JDQZ_QMin)
       Write(Log_Unit,*) 'Read_Input: JDQZ_Qmin = ', JDQZ_Qmin
    else if (Trim(token) == "JDQZ_JMAX") then
       Call GetNumber(WC, JDQZ_JMAX)
       Write(Log_Unit,*) 'Read_Input: JDQZ_Jmax = ', JDQZ_Jmax
    else if (Trim(token) == "JDQZ_JMIN") then
       Call GetNumber(WC, JDQZ_JMin)
       Write(Log_Unit,*) 'Read_Input: JDQZ_Jmin = ', JDQZ_Jmin
    else if (Trim(token) == "JDQZ_MAXPERBAND") then
       Call GetNumber(WC, JDQZ_MaxPerBand)
       Write(Log_Unit,*) 'Read_Input: JDQZ_MaxPerBand = ', JDQZ_MaxPerBand
    else if (Trim(token) == "JDQZ_EPS") then
       Call GetNumber(WC, JDQZ_eps)
       Write(Log_Unit,*) 'Read_Input: JDQZ_eps = ', JDQZ_eps
    else if (Trim(token) == "JDQZ_TARGETSHIFT") then
       Call GetNumber(WC, JDQZ_TargetShift)
       Write(Log_Unit,*) 'Read_Input: JDQZ_TargetShift = ', JDQZ_TargetShift
    else if (Trim(token) == "JDQZ_MAXMV") then
       Call GetNumber(WC, JDQZ_MaxMV)
       Write(Log_Unit,*) 'Read_Input: JDQZ_MaxMV = ', JDQZ_MaxMV
    else if (Trim(token) == "JDQZ_WAITITER") then
       Call GetNumber(WC, JDQZ_WaitIter)
       Write(Log_Unit,*) 'Read_Input: JDQZ_WaitIter = ', JDQZ_WaitIter
    else if (Trim(token) == "JDQZ_PRECONDITION_ONCE") then
       JDQZ_Precond_Mode = JDQZ_PRECOND_ONCE
       Write(Log_Unit,*) 'Read_Input: JDQZ_Precond_Mode = JDQZ_PRECOND_ONCE'
    else if (Trim(token) == "JDQZ_PRECONDITION_CLUSTER") then
       JDQZ_Precond_Mode = JDQZ_PRECOND_CLUSTER
       Write(Log_Unit,*) 'Read_Input: JDQZ_Precond_Mode = JDQZ_PRECOND_CLUSTER'
    else if (Trim(token) == "JDQZ_SPATIAL_SELECTION") then
       JDQZ_Spatial_Selection = .TRUE.
       Write(Log_Unit,*) 'Read_Input: JDQZ_Spatial_Selection = .TRUE.'
    else if (Trim(token) == "V_SMOOTH_WIDTH") then
       Call GetNumber(WC, V_Smooth_Width)
       Write(Log_Unit,*) 'Read_Input: V_Smooth_Width =', V_Smooth_Width
    else if (Trim(token) == "RANDOM_GUESS") then
       DoLCAO = GUESS_RANDOM
       Initial_guess = DoLCAO
       Write(Log_Unit,*) 'Read_Input: Random Initial Guess'
    else if (Trim(token) == "REAL_SPACE_PROJECTORS") then
       Proj_Mode = PROJ_RS
       Write(Log_Unit,*) 'Read_Input: Proj_Mode = PROJ_RS'
       write(log_unit,*) 'Real space projector option no longer works'
       stop
    else if (Trim(token) == "ANDERSON_CONDITIONNO") then
       call GetNumber(WC, AndersonConditionNo)
       Write(Log_Unit,*) &
          'Read_Input: AndersonConditionNo = ', AndersonConditionNo
    else if (Trim(token) == "SCF_DO_GRAM-SCHMIDT") then
       SCF_DoGramSchmidt = .TRUE.
       Write(Log_Unit,*) 'Read_Input: Performing explicit Gram-Schmidt', &
            ' orthogonalization in SCF_Solver.'
    else if (Trim(token) == "NO_O_EIGENVALUES") then
       Calc_O_Eigenvalues= .FALSE.
       Write(Log_Unit,*) 'Read_Input: Direct solutions of generalized', &
            ' eigenvalue problem (O not first diagonalized)'
    else if (Trim(token) == "Filter_Potential") then
       Filter_Potential= .TRUE.
       Write(Log_Unit,*) 'Read_Input: Potential functions filtered'
    else if (Trim(token) == "OVERLAP_TOL") then
       Call GetNumber(WC, Overlap_Tol)
       Write(Log_Unit,*) 'Read_Input: Overlap_Tol =', Overlap_Tol
    else if (Trim(token) == "SET_NAME") then
       Call GetNextWord(WC,token, tlen)
       Call UpperCase(Token)
       If (Trim(Token) == "FORCE_NAME") then
          Call GetNextWord(WC,token,tlen)
          If (tlen >  0) Force_Name=Trim(token)
          If (tlen >  0) Write(Log_Unit,*) 'Read_Input: Force output = ',&
                  Force_Name
       else if (Trim(Token) == "POSITION_ARCHIVE_NAME") then
          Call GetNextWord(WC,token,tlen)
          If (tlen >  0) Pos_Name=Trim(token)
          If (tlen >  0) Write(Log_Unit,*) 'Read_Input: Position output = ',&
                  Pos_Name
       else if (Trim(Token) == "CURRENT_NAME")  then
          Call GetNextWord(WC, TotalCurrent_Name, tlen)
       else if (TRIM(Token) == "ELECTRIC_FIELD_NAME") then
          Call GetNextWord(WC, E_Name, tlen)
       else
          Write(Error_Unit,*) 'Read_input: SET_NAME failed ',Trim(Token)
       endif            
    else if (Trim(Token) == "STORE_LOWEST_ENERGY") then
       Call GetNextWord(WC,token,tlen)
       If (tlen >  0) then
           Best_Name=Trim(token)
       Else
           Best_Name="Best"
       EndIf
       Write(Log_Unit,*) 'Read_Input: Store lowest energy results to ', &
               Best_Name
       BestCohesiveEnergy=-1.E30
    else if (Trim(token) == "SET_EIGENMAX") then
       Call GetNumber(WC, Eigen_Max)
       Write(Log_Unit,*) 'Read_Input: Set Eigen_Max = ', Eigen_Max
    else if (Trim(Token) == "PRINT_LEVEL") then !** Set Print level
       Call GetNExtWord(WC,token, tlen)
       Call UpperCase(Token)
       IF (Trim(Token) == "TERSE") then
          Print_Level = PRINT_TERSE
       else if (trim(Token) == "COMMANDS") then
          Print_Level = PRINT_COMMANDS
       else if (trim(Token) == "NORMAL") then
          Print_Level = PRINT_NORMAL
       else if (trim(Token) == "VERBOSE") then
          Print_Level = PRINT_VERBOSE
       else
          Write(Error_Unit,*) 'Read_Input: Unknown Print_Level:',TRim(Token)
          Call Word_GetAndPrint(WC, Error_Unit, 'Read_Input: ')
       End If
    else if (Trim(Token) == "OPEN") then        !** Change an output file
       Call OpenFile(WC)
    !!!!  XC type now set in atomic data files ---
    !!!! else if (Trim(Token) == "XC_TYPE") then        !** Read XC type
    !!!!    Call GetNextWord(WC, Token, tlen)
    !!!!    Call UpperCase(token)
    !!!!    If (Trim(Token) == "PERDEW-WANG") then
    !!!!       XC_Type = XC_PERDEW
    !!!!        Write(Log_Unit,*) '>XC_Type ',TRIM(Token)
    !!!!   else if (Trim(Token) == "VWN") then
    !!!!      XC_Type = XC_VWN
    !!!!      Write(Log_Unit,*) '>XC_Type ',TRIM(Token)
    !!!!   else if (Trim(Token) == "WIGNER") then
    !!!!      XC_Type = XC_WIGNER
    !!!!      Write(Log_Unit,*) '>XC_Type ',TRIM(Token)
    !!!!   else 
    !!!!      Write(Error_Unit,*) 'Read_Input: Unknown XC_Type:',Trim(Token)
    !!!!      Write(Log_Unit,*) 'Read_Input: Unknown XC_Type:',Trim(Token)
    !!!!   End If
    else if (Trim(Token) == "SUPERCELL") then   !** Load the supercell info
       Call LdSuperCell(WC)
    else if (Trim(Token) == "PLANEWAVE_CUTOFFS") then  !** Load the metric info
       Call LdPlaneWave(WC)
    !else if (Trim(Token) == "LCAO_CELLS") then  !** Determine nn sum for LCAO
    !   Call GetNumbers(WC, LCAO_Cells, 3)
    else if (Trim(Token) == "LOAD_SOLUTION") then  !** Load a guess from disk 
       Call GetNextWord(WC, SolnFileName, tlen)
       Call LoadGuess(Trim(SolnFileName))
       DoLCAO = 0    !** Disable LCAO wfn generation
    else if (Trim(Token) == "FORCES_ALWAYS_CALC_H") then  
       Forces_Always_calc_H = .TRUE.
       Write(Log_Unit,*) 'Read_Input: Always calculating H for the forces.'
    else if (Trim(Token) == "MD_GUESS")  then !** MD initial guess mode **
       Call GetNextWord(WC, token, tlen)

       Call UpperCase(token)
       If (trim(token) == "PREVIOUS") then
          MD_Guess = MD_PREVIOUS
          Write(log_Unit,*) 'Read_Input: MD_GUESS = PREVIOUS'
       else if (trim(token) == "LCAO") then
          MD_Guess = MD_LCAO
          Write(log_Unit,*) 'Read_Input: MG_GUESS = LCAO'
       else
          Write(Log_Unit,*) 'Read_Input: MG_GUESS Unknown mode :',trim(token)
          Write(Log_Unit,*) 'Read_Input: Should be PREVIOUS or LCAO'
       End If
    else if (Trim(Token) == "CLUSTER_TOLERANCE") then !** Cluster tolerance
       Call GetNumber(WC, ClusterTol)
       Write(Log_UNit,*) 'Read_Input: Cluster tol',ClusterTol
    else if (Trim(Token) == "STORE_SOLUTION") then !** Store Soln to Disk
       Call GetNextWord(WC, Token, tlen)
       Token2 = Token
       Call UpperCase(Token2)
       If (Trim(Token2) == "BINARY") then
          Call GetNextWord(WC, Token, tlen)
          Call StoreSolution_Binary(TRIM(token))
       else If (Trim(Token2) == "TEXT") then
          Call GetNextWord(WC, Token, tlen)
          Call StoreSolution_Text(TRIM(token))
       Else
          Write(Error_Unit,*) 'Read_Input: unknown store_solution type', &
            Token2
          stop
       End If
    else if (Trim(Token) == "STORE_HAMILTONIAN") then !** Store H to Disk
       Call GetNextWord(WC, Token, tlen)
       Call StoreHam_Binary(TRIM(token))
    else if (Trim(Token) == "LOAD_HAMILTONIAN") then !** Load H from Disk
       Call GetNextWord(WC, Token, tlen)
       Call LoadHam_Binary(TRIM(token))
       BandStructure_Mode = .TRUE.
    else if (Trim(Token) == "STORE_HAM_AND_PSI") then !** Store H & Psi to Disk
       Call GetNextWord(WC, Token, tlen)
       Call StoreHamandPsi_Binary(TRIM(token))
    else if (Trim(Token) == "STOREUNMIXED") then !** Store unmixed H to Disk
                  ! Default stores H from Andersonmix
       SaveUnmixed=.TRUE.
    else if (Trim(Token) == "LOAD_HAM_AND_PSI") then !** Load H & Psi from Disk
       Call GetNextWord(WC, Token, tlen)
       Call LoadHamandPsi_Binary(TRIM(token))
       DoLCAO = 0    !** Disable LCAO wfn generation
    else if (Trim(Token) == "LOAD_HAM_AND_PSI_NODIAG") then !** Load H & Psi from Disk
       Call GetNextWord(WC, Token, tlen)
       Nodiagwithload=.true.
       Call LoadHamandPsi_Binary(TRIM(token))
       DoLCAO = 0    !** Disable LCAO wfn generation
    else if (Trim(Token) == "STORE_VXC_FOURIER") then !** Load H from Disk
       Call GetNextWord(WC, Token, tlen)
       Call Store_VXC_Fourier(TRIM(token))
    else if (Trim(Token) == "STOREPS_XSF") then !** Store PS V,n~,psi in XSF
       Call GetNextWord(WC, Token, tlen)
       Call StorePS_XSF(TRIM(token))
    else if (Trim(Token) == "STORE_XYZ") then !** Store RasMol/Chime file
       Call GetNextWord(WC, Token, tlen)
       Call GetNumber(WC, rtmp)
       Call GetNextWord(WC, token2, tlen)
       i = SearchList(Token2, Atom_LUT)

!       if ((i<=0)) then     !** Name not used
!          Write(Error_Unit,*) "Read_Input:(Store_XYZ) Name NOT used! Name =", &
!               Trim(Token2)
!          Call Word_GetAndPrint(WC, Error_Unit, 'Read_Input:')
!          STOP
!       End If

       Write(Log_Unit,*) 'Read_Input: Store_XYZ ', TRIM(token), ' ',rtmp, &
            ' ', TRIM(token2)
       Call StoreXYZ(TRIM(token), rtmp, i)
    else if (Trim(Token) == "PSI_MEMORY") then  !** Get psi memory size
       Call GetNumber(WC, Psi_Memory)
    else if (Trim(Token) == "PROJ_MEMORY") then  !** Get radial proj mem size
       Call GetNumber(WC, Proj_Memory)
    else if (Trim(Token) == "BLOCH_MEMORY") then  !** Get memory size for phase
       Call GetNumber(WC, Bloch_Memory)
    else if (Trim(Token) == "YLM_MEMORY") then  !** Get memory size for Ylm's
       Call GetNumber(WC, Ylm_Memory)
    else if (Trim(Token) == "GMRES_SIZE") then    !** Number of GMRES vec's
       Call GetNumber(WC, GMRES_Size)
       Write(Log_Unit,*) 'Read_Input:  GMRES_Size =',GMRES_Size
    else if (Trim(Token) == "V_CORRITER") then    !** V control
       Call GetNumbers(WC, V_corriter)
       Write(Log_Unit,*) 'Read_Input:  V_corriter =',V_corriter
    else if (Trim(Token) == "DEBUG_SLICE") then    !** V control
       Call GetNumbers(WC, Debug_Fraction_Slice)
       Call GetNumbers(WC, Debug_Dir)
       Write(Log_Unit,*) 'Read_Input:  Debug_Fraction_slice =', &
            Debug_Fraction_Slice
       Write(Log_Unit,*) 'Read_Input:  Debug_Dir =',Debug_Dir
    else if (Trim(Token) == "PSI_CORRITER") then    !** Psi control
       Call GetNumbers(WC, Psi_corriter)
       Write(Log_Unit,*) 'Read_Input:  Psi_corriter =',Psi_corriter
    else if (Trim(Token) == "LCAO_GCUT") then    !** LCAO Gcut
       Call GetNumbers(WC, LCAO_Gcut)
       Write(Log_Unit,*) 'Read_Input:  LCAO_Gcut =',LCAO_Gcut
    !!else if (Trim(Token) == "ATOMICVXC_STEP") then !** Core Vxc Step-taken out
    !!   Call GetNumbers(WC, AtomicVxc_Step)
    !!   Write(Log_Unit,*) 'Read_Input:  Psi_corriter =',Psi_corriter
    else if (Trim(Token) == "INITIALIZE_SYSTEM") then  !** Init calculation
       Call InitSystem
    else if (Trim(Token) == "RELAX") then            !** run the program
       Call ldRelax(WC)
    else if (Trim(Token) == "MAX_TOTALPSI") then            !** Max Total Psi's
       Call GetNumber(WC, User_MaxTotalPsi)
    else if (Trim(Token) == "MINPSI") then            !** Min Psi's
       Call GetNumber(WC, User_MinPsi)
    else if (Trim(Token) == "QUIT") then     !** Return to driver program
        Return            !Return to driving pgm
    else if (trim(Token) == "BOUNDARY_CONDITIONS") then !** Load boundary cond.
       Call GetNumbers(WC, BC_Mode)
       Write(Log_Unit,*) 'Read_Input: Boundary Conditions :',BC_Mode
    else if (Trim(Token) == "TIME_STEP") then  !** Read the MD DT
       Call GetNumber(WC, TimeStep)
       Write(Log_Unit,*) 'Read_Input: Time Step :',TimeStep
    else if (Trim(Token) == "FORCE_ZERO") then  !** Read the 0 for force
       Call GetNumber(WC, Force_Zero)
       Write(Log_Unit,*) 'Read_Input: Force Zero value :',Force_Zero
    !TDelse if (Trim(Token) == "ELECTRIC_FIELD") then  !** Read the E-field
       !TDCall ldElectricField(WC)
    else if ((Trim(Token) == "FREEZE") .OR. (Trim(Token) == "UNFREEZE")) then
       Call GetNextWord(WC, Token2, tlen)

       If (Trim(Token2) == "ALL") then
          If (Trim(Token) == "FREEZE") then
             Atom_List(:)%Freeze = .TRUE.
             Write(Log_Unit,*)'Read_Input: Freezing ALL atoms.'
          else
             Atom_List(:)%Freeze = .FALSE.
             Write(Log_Unit,*)'Read_Input: Un-Freezing ALL atoms.'
          end If

             !** Reset Verlet data **
          Do i=1, Specific_Atoms
             Atom_List(i)%Force%Predictor_Phase = .TRUE.
             Atom_List(i)%Force%NumMoves = 1
             Atom_List(i)%Force%Velocity = 0
             Atom_List(i)%Force%Acceleration = 0
             Atom_List(i)%Force%Pos = 0
             Atom_List(i)%Force%Pos(:,1) = Atom_List(i)%Pos
             Atom_List(i)%Force%Velocity = 0
          End do
       else
          i = SearchList(Token2, Atom_LUT)
          
          if (i<=0) then     !** Name not used
             Write(Error_Unit,*) "Read_Input: (FREEZE) Name NOT used! Name: ",&
                  Trim(Token2)
             Call Word_GetAndPrint(WC, Error_Unit, 'Read_Input:')
             STOP
          End If
          
          If (trim(Token) == "FREEZE") then
             Atom_List(i)%Freeze = .TRUE.
             Write(Log_Unit,*)'Read_Input: Freezing Atom ',TRim(Atom_List(i)%Name)
          else
             Atom_List(i)%Freeze = .FALSE.
             Write(Log_Unit,*) 'Read_Input: Un-Freezing Atom ', &
                  TRim(Atom_List(i)%Name)
          End If

          !**** Need to reset the Verlet info ******
          Atom_List(i)%Force%Predictor_Phase = .TRUE.
          Atom_List(i)%Force%NumMoves = 1
          Atom_List(i)%Force%Velocity = 0
          Atom_List(i)%Force%Acceleration = 0
          Atom_List(i)%Force%Pos = 0
          Atom_List(i)%Force%Pos(:,1) = Atom_List(i)%Pos
          Atom_List(i)%Force%Velocity = 0
       End if

    else if (Trim(Token) == "ANCHOR" ) then
       Anchor = .true.
       Call GetNextWord(WC, Token2, tlen)
       i = SearchAddList(Token2, Atom_LUT, IsNew)
       if ((IsNew) .AND. (i>0)) then     !** Name not used
           Write(Error_Unit, *) "Read_Input: (ANCHOR) Name NOT used! Name=",&
                Trim(Token2)
           Call Word_GetAndPrint(WC, Error_Unit, 'Read_Input:')
           STOP
       End If
       AnchorIndex=i

    else if (Trim(Token) == "CALCULATE") then 
       Call GetNextWord(WC, token, tlen)
       Call UpperCase(Token)
       If (Trim(Token) == "ENERGY") then
          Call CalcHam( rtmp)
          Call Output_Energy(rtmp)
       else if (Trim(Token) == "SYMMETRY") then
          Call GetNextWord(WC, Token, tlen)
          Call StoreSymmetry(TRIM(token))
       else if (Trim(Token) == "ERROR") then
          Call CheckPotError( Log_Unit)
          Do i=1, LastCLuster
             Call CheckClusterError(Log_Unit, i, .FALSE.)
          End Do
       else if (Trim(Token) == "BAND_STRUCTURE") then  !** run the band struct
          Call ldBandStruct(WC)
       else if (Trim(Token) == "SURFACE_BANDS") then  !** run surface bands  
          Call ldSurfBands(WC)
       else If (Trim(Token) == "FORCES") then
          Call GetNextWord(WC, Token, tlen)
          If (tlen > 0) Force_Name=trim(Token)
          Call CalcForces
          If (UseShells) Call ShellForce
          Do i=1, Specific_Atoms
             Write(Log_Unit,*) 'Read_Input: ',Trim(Atom_List(i)%Name), ' F=', &
                  Atom_List(i)%Force%CurrForce
             Write(Log_Unit,*) 'Read_Input: ',Trim(Atom_List(i)%Name), &
                  ' Fxtal=', MatMul(xtal%InverseBasis, &
                  Atom_List(i)%Force%CurrForce)
          End Do

          Call Store_ForcesAndPositions(TRIM(Force_Name), &
                   'Force calculation from Read_Input')
       else If (Trim(Token) == "DOS") then
          Call GetNextWord(WC, Token, tlen)
          If (tlen > 0) DOS_Name=trim(Token)
          Open(unit=CHARGEOUT_UNIT,file=Trim(DOS_Name),form="formatted")
          i=0
          write(CHARGEOUT_UNIT,*) i
          call bandcharge(i,WC)
          Close(CHARGEOUT_UNIT)
       else If (Trim(Token) == "PDOS") then
          write(Log_unit,*) 'Read_Input: PDOS calculation'
          Call GetNextWord(WC, Token, tlen)
          write(Log_unit,*) 'Read_Input: Opening file ',TRIM(Token)
          If (tlen > 0) DOS_Name=trim(Token)
          Open(unit=CHARGEOUT_UNIT,file=Trim(DOS_Name),form="formatted")
          Call GetNumber(WC, i)
          If (i.le.0) then
            write(Log_Unit,*) "warning pdos input with Nsphere=0"
            i=0
          Endif
          write(Log_unit,*) 'Read_Input: No. of PDOS''s = ',i
          write(CHARGEOUT_UNIT,*) i
          call bandcharge(i,WC)
          Close(CHARGEOUT_UNIT)
       else if (Trim(Token) == "PARTIAL_DENSITIES") then
          Call GetNumber(WC,i)
          If (i.le.0) then
            write(error_unit,*) "error in read_input -- "  
            write(error_unit,*) "number following PARTIAL_DENSITIES <1"
            stop
          Endif
          call partialden(i,WC)
       else
          Write(Error_Unit, *) "Read_Input: Don't know how to CALCULATE ", &
               Trim(Token)
       End If
    else if (Trim(Token) == "PLOT") then
       Call GetNextWord(WC, token, tlen)
       Call UpperCase(Token)
       If (Trim(token) == "3DSETUP") then
         call Ld3ddata(WC)
       Else if (Trim(Token) == "3DATOM") then
         Call Atomplot3d
       Else if (Trim(Token) == "3D-DENSITY") then
         Call GetNextWord(WC, Token2, tlen)
         If (tlen > 0) PLOT_Name=trim(Token2)
          Call Atomplot3d
          Call Volplot
       Else if (Trim(Token) == "3D-ESTATIC") then
         Call GetNextWord(WC, Token2, tlen)
         call UpperCase(Token2)
             If (TRIM(Token2)=="KEEP_SPHERE") then
                keep=.true.
             Else if (TRIM(Token2)=="VOID_SPHERE") then
                keep=.false.
             Else 
                Write(Error_Unit,*) 'Error in input -- for 3D-STATIC'
                Write(Error_Unit,*) ' need KEEP_SPHERE or VOID_SPHERE'
                stop
             endif
         Call GetNextWord(WC, Token2, tlen)
         If (tlen > 0) PLOT_Name=trim(Token2)
          Call Atomplot3d
          Call Estatic(keep)
       Else if (Trim(token) == "2DSETUP") then
         call Ld2ddata(WC)
       Else if (Trim(Token) == "2D-DENSITY") then
         Call GetNextWord(WC, Token2, tlen)
         If (tlen > 0) PLOT_Name=trim(Token2)
          Call Planegrid
       else
          Write(Error_Unit, *) "Read_Input: Don't know how to PLOT ", &
               Trim(Token)
       End If
    else if (Trim(Token) == "FRICTION") then
       Call GetNextWord(WC, Token2, tlen)
       i = SearchAddList(Token2, Atom_LUT, IsNew)

       if (IsNew) then     !** Name not used
          Write(Error_Unit,*) "Read_Input: (FRICTION) Name NOT used! Name =", &
               Trim(Token2)
          Call Word_GetAndPrint(WC, Error_Unit, 'Read_Input:')
          STOP
       End If

       Call GetNumber(WC, Atom_List(i)%Friction)
    else If (Trim(Token) == "V_WEIGHT") then  !** Change the V weight factor
       Call GetNumber(WC, V_Weight)
       Write(Log_Unit,*) 'Read_Input: Adjusting V_Weight:',V_Weight
    else If (Trim(Token) == "V_NEWMIX") then  !** Change the V mix factor
       Call GetNumber(WC, V_NewMix)
       Call GetNumber(WC, V_DampNewMix)
       Write(Log_Unit,*) 'Read_Input: Adjusting V_NewMix:',V_NewMix, &
            ' *', V_DampNewMix
    else If (Trim(Token) == "DIJ_NEWMIX") then  !** Change the Dij mix factor
       Call GetNumber(WC, Dij_NewMix)
       Call GetNumber(WC, Dij_DampNewMix)
       Write(Log_Unit,*) 'Read_Input: Adjusting Dij_NewMix:',Dij_NewMix, &
            ' *', Dij_DampNewMix
    else If (Trim(Token) == "MIX_SECONDVALUE") then  !** Damp RelE factor
       Call GetNumber(WC, Mix_DampRelative)
       Write(Log_Unit,*) 'Read_Input: Adjusting Mix_DampRelative:', &
            Mix_DampRelative
    else If (Trim(Token) == "GEOMETRY_NEWMIX") then  !** Geometry mix factor
       Call GetNumber(WC, Geometry_NewMix)
       Write(Log_Unit,*) 'Read_Input: Adjusting Geometry_NewMix:', &
            Geometry_NewMix
    else If (Trim(Token) == "MIX_V") then  !** Mix V
       V_MixType = MIX_V
       Write(Log_Unit,*) 'Read_Input: Mixing V'
    else If (Trim(Token) == "MIX_VEFF") then  !** Mix Veff
       V_MixType = MIX_VEFF
       Write(Log_Unit,*) 'Read_Input: Mixing Veff'
    else If (Trim(Token) == "MIX_DENSITY") then  !** Mix the density
       V_MixType = MIX_DENSITY
       Write(Log_Unit,*) 'Read_Input: Mixing the Density'
    else If (Trim(Token) == "WRITEPSISLICE") then
       Call GetNextWord(WC, Token2, tlen)
       Call GetNumber(WC, i)
       Call GetNumbers(WC, PM)

       Write(Log_Unit,*) 'Read_Input: WritePsiSlice:  * EXT=',TRIM(token2), ' * PM=',PM, ' * Dir=',i
       Call WritePsiSlice( PM, i, Token2)
    else If (Trim(Token) == "MOVE") then  !*** Move an atom
       Call GetNextWord(WC, Token2, tlen)
       i = SearchAddList(Token2, Atom_LUT, IsNew)

       if (IsNew) then     !** Name not used
          Write(Error_Unit, *) "Read_Input: (MOVE) Name NOT used! Name =", &
               Trim(Token2)
          Call Word_GetAndPrint(WC, Error_Unit, 'Read_Input:')
          STOP
       End If

       Call GetNumbers(WC, Atom_List(i)%Pos)
    write(6,*) 'Read_Input -- note this code is not working --'

       Write(Log_Unit,*) 'Read_Input: Moving atom ',Trim(Atom_List(i)%Name), &
            ' to ',Atom_List(i)%Pos
       Atom_List(i)%Pos = MATMUL(Xtal%Recip, Atom_List(i)%Pos)

       Atom_List(i)%Force%Predictor_Phase = .TRUE.
       Atom_List(i)%Force%NumMoves = 1
       Atom_List(i)%Force%Velocity = 0
       Atom_List(i)%Force%Acceleration = 0
       Atom_List(i)%Force%Pos = 0
       Atom_List(i)%Force%Pos(:,1) = Atom_List(i)%Pos
    else If (Trim(Token) == "MOVE") then  !*** Move an atom
       Call GetNextWord(WC, Token2, tlen)
       i = SearchAddList(Token2, Atom_LUT, IsNew)

       if (IsNew) then     !** Name not used
          Write(Error_Unit, *) "Read_Input: (MOVE) Name NOT used! Name =", &
               Trim(Token2)
          Call Word_GetAndPrint(WC, Error_Unit, 'Read_Input:')
          STOP
       End If

       Atom_List(i)%Force%Predictor_Phase = .TRUE.
       Atom_List(i)%Force%NumMoves = 1
       Atom_List(i)%Force%Velocity = 0
       Atom_List(i)%Force%Acceleration = 0
       Atom_List(i)%Force%Pos = 0
       Atom_List(i)%Force%Velocity = 0

       Call GetNumbers(WC, Atom_List(i)%Force%Velocity(:,1))
       Write(Log_Unit,*) 'Read_Input: Firing atom ',Trim(Atom_List(i)%Name), &
            ' with velocity ',Atom_List(i)%Force%Velocity(:,1)
    !else if (Trim(token) == "FIND_NEARBY_VECTORS") then
    !   Call GetNumbers(WC, rtmp)
    !   Call GetNumbers(WC, itmp)
    !   Write(log_Unit,*) 'FIND_NEARBY_VECTORS:  Tol=',rtmp, ' * nVects=',itmp
    !   Call Find_Nearby_Vectors(rtmp, itmp)
    else if (TRIM(token) == 'LOCAL_CRITERIA') then
       Call LdLocalCriteria(WC)
    else
       Write(Error_Unit, *) "Unknown Command : ", Trim(Token)
    End If    

    Call GetNextWord(WC, Token, tlen)
!Write(*,*) 'Read_Input : Loop End Token: !',trim(token),'!'
  End Do
End Subroutine

