!******************************************************************************
!
! File : storedata.f90
!   by : Alan Tackett
!   on : 05/13/99
!  for : PWPAW
!
!  This module contains routines for reading and writing the wave functions 
!  associated information.
!
! Updated for spin by Ping Tang and N. A. W. Holzwarth except for
!  the following routines that are normally not called:
!    Store_Vxc_FOURIER,    StoreSolution_Text,  StorePS_XSF,
!    LoadTextGuess
!  Last change 5/23/05
!******************************************************************************


Module storedata

Use paw_inout
!Use metric
Use mem_data
Use memmgr
Use psilib
Use fileio
Use paw_inout
Use search_sort
Use hamiltonian
Use options_data
Use word
Use basis_lib
Use gpoints
Use strings
Use grrp
Use crystal_symmetry

Implicit None!!!!!!!!!!!!!!

!******************************************************************************
Contains
!******************************************************************************

!******************************************************************************
!
!  StoreXYZ - Stores a file in Xmol XYZ format
!
!    Filename - Output filename
!    Rmax     - Max radius of atoms to include.  IF Rmax <= 0 then
!               just the unit cell is stored.
!    Ao       - Center around given atomindex
!
!******************************************************************************

Subroutine StoreXYZ(Filename, Rmax, Ao)
  Character*(*), Intent(IN) :: FileName
  Real,          Intent(IN) :: Rmax
  Integer,       Intent(IN) :: Ao
  
  Integer :: i, x,y,z, Lo(3), Hi(3), loop, n
  Real    :: Ra(3), Rxtal(3), R(3), r2, Radius2, Ro(3)
  Character*4 :: Element


  If (Ao>0) then
     Ro = MatMul(xtal%Basis, Atom_List(Ao)%Frac_Pos)
  else
     Ro = 0
  End if

Write(Log_Unit,*) 'StoreXYZ: Ro=',Ro

  Open(SCRATCH_UNIT, FILE=TRIM(Filename))
  
  If (Rmax <= 0) then
     Radius2 = 1E10
     Lo = 0
     Hi = 0
  else
     Radius2 = Rmax*Rmax

     R = Rmax
     Hi = MATMUL(xtal%InverseBasis, R+Ro)
     i = MAxVal(Hi)+1
     Hi = i

     Lo = MATMUL(xtal%InverseBasis, -R-Ro)
     i = MinVal(Lo)-1
     Lo = i
  End If

!Radius2= 1E10
Write(Log_Unit,*) 'StoreXYZ: Bound=',Lo, ' * ', Hi

  Do loop=1, 2
     If ( loop==2) then
        Write(Scratch_Unit,*) n
        Write(Scratch_Unit,*) ' '
     End if

     n = 0

     Do z=Lo(3), Hi(3)
        Do y=Lo(2), Hi(2)
           Do x=Lo(1), Hi(1)
              
              Rxtal = MATMUL(Xtal%Basis, (/x,y,z/))
              Do i=1, Specific_Atoms
                 element = AtomType_Info(Atom_List(i)%TypeIndex)%Atom_Name(1:2)
                 Ra = MATMUL(Xtal%Basis, Atom_List(i)%Frac_Pos)
                 
                 R = Ra+Rxtal-Ro
                 r2 = DOT_PRODUCT(R,R)
                 R = 0.529*R
                 If (r2 <= Radius2) then
                    n = n + 1
                    if (Loop==2) &
                      Write(Scratch_Unit,'(a5,3(2x,f10.6))') element, R
                 End if
              End Do
           End Do
        End Do
     End Do
  End Do

  Write(Scratch_Unit,*) ' '
  Close(Scratch_Unit)

  Return
End Subroutine

!******************************************************************************
!
!  StoreSymmetry - Calculates and stores the symmetry operations to a file
!
!   Filename - Output filename
!
!******************************************************************************

Subroutine StoreSymmetry(Filename)
  character*(*), Intent(IN) :: FileName
 
  Include "atoms.i"
  Include "symmetry.h"

  Real :: A(3,3), B(3,3), Pos(Atom_Types, Specific_Atoms, 3)
  Integer :: i,j, k, Count(Atom_Types)
  Logical, External :: rest_inv
  Character*2 :: name(Max_Atom_Types)
  Logical :: RUNTIME

  RunTime = .FALSE.
  If (TRIM(Filename) == 'RUNTIME') RunTime = .TRUE.

  Write(Log_Unit, *) 'StoreSymmetry: Output File:',TRIM(FileName)
  A = xtal%Basis
  B = xtal%InverseBasis

  ntyp = Atom_Types
  natp = Specific_Atoms

  name(:) = "  "
  Do i=1, Atom_Types
     name(i) = Atomtype_Info(i)%Atom_Name(1:2)
  End Do

  Count = 0
  Pos = 0

  Do i=1, Specific_Atoms
     j = Atom_List(i)%TypeIndex
     Count(j) = Count(j) + 1
     k = Count(j)
     Pos(j, k, :) = Atom_List(i)%Frac_Pos
!Write(Log_Unit,*) 'i=',i, ' * R=',Pos(j,k,:)
  End Do

  Call SymGen(Log_Unit, a(1,1),b(1,1), Pos, Count, Atom_Types, i)

!  if (rest_inv(Log_Unit, i, Atom_types, Count, pos, name)) then
!     Call SymGen(Log_Unit, a(1,1),b(1,1), pos, Count, Atom_Types, i)
!  End If

  Where (ABS(tnp) < 1E-14) tnp = 0  !** Filter underflow

  !*** Now store the rot/trans matrices to disk ***
  If (.NOT. RunTime) then
     Open(Scratch_Unit, FILE=TRIM(Filename),form='formatted',&
              position='append')
     
     Write(Scratch_Unit, *) 'Rot_Size', ntrans
     Write(Scratch_Unit, *) ' '
     
     Do i=1, ntrans
        Write(Scratch_Unit, "('Matrix', I4)") i
        Do j=1, 3
           Write(Scratch_Unit, "(3I4)") cmtrx(i,j,:)
        End Do
        Write(Scratch_Unit,*) 'End'
        Write(Scratch_Unit, "('Translation', I4, 3e20.12)") i, tnp(i,:)
        Write(Scratch_Unit,*) ' '
     End Do
     
     Write(Scratch_Unit,*) ' '
     
     Close(Scratch_Unit)
  else
     xtal%Rot_Size = ntrans
!**     Allocate(xtal%rotMatrix(3,3,ntrans), xtal%trans(3,ntrans))

     Do i=1, ntrans
        xtal%RotMatrix(:,:,i) = cmtrx(i,:,:)
        xtal%Trans(:,i) = tnp(i,:)
     End Do
  End If

  Return
End subroutine
!******************************************************************************
!
! Output_Energy -- writes output energy terms to output_unit
!
!******************************************************************************
Subroutine Output_Energy(Energy)
  Real, Intent(IN) :: Energy

  Integer :: i,j

  If (bandstructure_mode) return

  Call Printdate(Output_Unit,'#  Energy results generated on date ')
  Write(Output_Unit,*) ' '
  Write(Output_Unit,*) '# Current Fractional Positions '
  Write(Output_Unit,*) '#Atom_List FRAC_POSITION'
  Do i=1, Specific_Atoms                                 
     j=Atom_List(i)%TypeIndex                
     If (Atom_List(i)%Freeze) then
        write(Output_Unit,'(" # ", A8,"  ",A8, 1p3e13.5, " # FROZEN")') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Frac_Pos
     Else If (Anchor.and.i==AnchorIndex) then
        write(Output_Unit,'(" # ", A8,"  ",A8, 1p3e13.5, " # ANCHORED")') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Frac_Pos
     else
        write(Output_Unit,'(" # ", A8,"  ",A8, 1p3e13.5)') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Frac_Pos
             
     End If
  End Do

     Write(Output_Unit,*) ' '
     Write(Output_Unit,*) ' --  Cohesive Energy = ',Energy
     Write(Output_Unit,*) ' --          CalcHam = ',Energy-AtomicEnergy
     Write(Output_Unit,*) ' --           Atomic = ',AtomicEnergy
     Write(Output_Unit,*) ' --             Diag = ',Diag_Energy
     Write(Output_Unit,*) ' --          Hartree = ',Hartree_Energy
     Write(Output_Unit,*) ' --              Hat = ',Hat_Energy
     If (XC_Type == XC_LDA_PW) &
      Write(Output_Unit,*) ' --           OneExc = ',OneExc_Energy,' (LDA-PW)'
     If (XC_Type == XC_GGA_PBE) &
      Write(Output_Unit,*) ' --           OneExc = ',OneExc_Energy,' (GGA-PBE)'
     Write(Output_Unit,*) ' --              KE~ = ',TildeKE_Energy
     Write(Output_Unit,*) ' --             Pot~ = ',TildePot_Energy
     If (XC_Type == XC_LDA_PW) &
      Write(Output_Unit,*) ' --              XC~ = ',TildeXC_Energy,' (LDA-PW)'
     If (XC_Type == XC_GGA_PBE) &
      Write(Output_Unit,*) ' --              XC~ = ',TildeXC_Energy,' (GGA-PBE)'
     Write(Output_Unit,*) ' --          Hat-Hat = ',Hat_Hat_Energy

  Return
 End Subroutine Output_Energy

!******************************************************************************
!
!  Store_Best_Frac_Pos - Store positions corresponding to lowest energy
!
!     filename - Output Filename
!
!******************************************************************************

Subroutine Store_Best_Frac_Pos(FileName,Energy)
  Character*(*), Intent(IN) :: Filename
  Real         , Intent(IN) :: Energy  

  Integer     :: i,j,k

  open(unit=Scratch_Unit,file=Trim(FileName),form='formatted')

  Call Printdate(Scratch_Unit,'#  Best results generated on date ')
  Write(Scratch_Unit,*) '#  Cohesive Energy  :',Energy

  Write(Scratch_Unit,*) ' '
  Write(Scratch_Unit,*) '# Current Fractional Positions'
  Write(Scratch_Unit,*) ' Atom_List FRAC_POSITION'
  Do i=1, Specific_Atoms                                 
     j=Atom_List(i)%TypeIndex                
     If (Atom_List(i)%Freeze) then
        write(Scratch_Unit,'("   ", A8,"  ",A8, 1p3e15.7, " # FROZEN")') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Frac_Pos
     Else If (Anchor.and.i==AnchorIndex) then
        write(Scratch_Unit,'("   ", A8,"  ",A8, 1p3e15.7, " # ANCHORED")') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Frac_Pos
     else
        write(Scratch_Unit,'("   ", A8,"  ",A8, 1p3e15.7)') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Frac_Pos
             
     End If
  End Do

  Write(Scratch_Unit,*) ' End'
  Close (Scratch_Unit)
  Return
  End Subroutine

!******************************************************************************
!
!  Store_BestPositions - Store positions corresponding to lowest energy
!
!     filename - Output Filename
!
!******************************************************************************

Subroutine Store_BestPositions(FileName,Energy)
  Character*(*), Intent(IN) :: Filename
  Real         , Intent(IN) :: Energy  

  Integer     :: i,j,k

  open(unit=Scratch_Unit,file=Trim(FileName),form='formatted')

  Call Printdate(Scratch_Unit,'#  Best results generated on date ')
  Write(Scratch_Unit,*) '#  Cohesive Energy  :',Energy

  Write(Scratch_Unit,*) ' '
  Write(Scratch_Unit,*) '# Current Cartesian Positions in Bohr Units'
  Write(Scratch_Unit,*) ' Atom_List CART_POSITION'
  Do i=1, Specific_Atoms                                 
     j=Atom_List(i)%TypeIndex                
     If (Atom_List(i)%Freeze) then
        write(Scratch_Unit,'("   ", A8,"  ",A8, 1p3e15.7, " # FROZEN")') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Pos
     Else If (Anchor.and.i==AnchorIndex) then
        write(Scratch_Unit,'("   ", A8,"  ",A8, 1p3e15.7, " # ANCHORED")') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Pos
     else
        write(Scratch_Unit,'("   ", A8,"  ",A8, 1p3e15.7)') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Pos
             
     End If
  End Do

  Write(Scratch_Unit,*) ' End'
  Close (Scratch_Unit)
  Return
  End Subroutine

!******************************************************************************
!
!  Store_ForcesAndPositions - Stores the forces and the new fractional coords
!        for the atoms
!
!     filename - Output Filename
!
!******************************************************************************

Subroutine Store_ForcesAndPositions(FileName,message)
  Character*(*), Intent(IN) :: Filename
  Character*(*), Intent(IN) :: message 

  Integer :: i,j,k
  Real :: F(3),Inpos(3,Specific_Atoms),Outpos(3,Specific_Atoms)

  open(unit=Force_Unit,file=Trim(Force_Name),form='formatted', &
       position='append')

  Write(Force_Unit,*)
  Call Printdate(Force_Unit,'#  Force results generated on date ')
  Write(Force_Unit,*) '#  Low,High PW_Gcut :',PW_Gcut(G_LOW),PW_Gcut(G_HIGH)
  Write(Force_Unit,*) '#  Cohesive Energy  :',CohesiveEnergy
  Write(Force_Unit,*) message

!  F = 0
!  Do i=1, Specific_Atoms 
!     F = F + atom_list(i)%force%currforce
!  Enddo
!  Write(Force_Unit,'(" #         Net Force :", 1p3e15.7)') NetForce

  Write(Force_Unit,*) ' '
  Write(Force_Unit,*) '# Current Cartesian Positions in Bohr Units'
  Write(Force_Unit,*) '#Atom_List CART_POSITION'
  Do i=1, Specific_Atoms                                 
     j=Atom_List(i)%TypeIndex                
     If (Atom_List(i)%Freeze) then
        write(Force_Unit,'(" # ", A8,"  ",A8, 1p3e15.7, " # FROZEN")') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Pos
     Else If (Anchor.and.i==AnchorIndex) then
        write(Force_Unit,'(" # ", A8,"  ",A8, 1p3e15.7, " # ANCHORED")') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Pos
     else
        write(Force_Unit,'(" # ", A8,"  ",A8, 1p3e15.7)') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Pos
             
     End If
  End Do

  Write(Force_Unit,*) ' '
  Write(Force_Unit,*) '#  Cartesian Forces (Ry/bohr) '      
  Do i=1, Specific_Atoms 
     write(Force_Unit,'(" # ", A8,1p3e15.7)')&
          Atom_List(i)%Name, atom_list(i)%force%currforce
  Enddo
  
  Write(Force_Unit,*) ' '
  Write(Force_Unit,*) '#  Fractional Forces (Ry) '
  Do i=1, Specific_Atoms                                    
     write(Force_Unit,'(" # ", A8,1p3e15.7)') &
          Atom_List(i)%Name, MatMul(xtal%InverseBasis, &
          atom_list(i)%force%currforce)
  Enddo

  If (UseShells) then
    Write(Force_Unit,*) ' '
    Write(Force_Unit,*) '#  Generalized Parameters and Forces'
    Do i=1, Shell%NPARAMS                                     
     write(Force_Unit,'(" # ", i5,5x,1p2e15.7)') &
          i, Shell%C(i), Shell%ShellForce(i)              
    Enddo
  EndIf
  
  !Write(Force_Unit,*) ' '
  !Write(Force_Unit,*) '# New Fractional Positions '
  !Write(Force_Unit,*) 'Atom_List FRAC_POSITION'
!
!    Do i=1,Specific_Atoms
!        F = atom_list(i)%force%currforce
!        Where (ABS(F) < Force_Zero) F = 0
!        Inpos(:,i)=Atom_List(i)%Frac_Pos + MatMul(xtal%InverseBasis, F)
!    Enddo
!    Call Symmetrize_Pos(Inpos,Outpos)
!    
!  Do i=1, Specific_Atoms                                 
!     j=Atom_List(i)%TypeIndex                
!     If (Atom_List(i)%Freeze) then
!        write(Force_Unit,'("   ", A8,"  ",A8, 1p3e15.7, " # FROZEN")') &
!             Atom_List(i)%Name, &
!             AtomType_Info(j)%Atom_Name, &
!             Atom_List(i)%Frac_Pos
!     Else If (Anchor.and.i==AnchorIndex) then
!!        write(Force_Unit,'("   ", A8,"  ",A8, 1p3e15.7, " # ANCHORED")') &
!             Atom_List(i)%Name, &
!             AtomType_Info(j)%Atom_Name, &
!             Atom_List(i)%Frac_Pos
!     else
!        F = atom_list(i)%force%currforce
!        Where (ABS(F) < Force_Zero) F = 0
!        write(Force_Unit,'("   ", A8,"  ",A8, 1p3e15.7)') &
!             Atom_List(i)%Name, &
!             AtomType_Info(j)%Atom_Name, Outpos(:,i)
!                                 ! Atom_List(i)%Frac_Pos +   &
!                                 ! MatMul(xtal%InverseBasis, F)
!             
!     End If
!  Enddo
!  
!  Write(Force_Unit,*) 'End'

  Call Flush(Force_Unit)

  close(Force_Unit) 

  Return
end subroutine

!******************************************************************************
!
!  Position_Archive  - File for storing positions and forces in Cartesian form
!
!
!******************************************************************************

Subroutine Position_Archive(Cohesive_Diff)
  Real, intent(IN) :: Cohesive_Diff

  Integer :: i,j,k,index,ier
  Character*200 :: label
  Real  :: Energy,R(3),F(3)

  open(unit=Positions_Unit,file=Trim(Pos_Name),form='formatted',&
           position='append')

  Write(Log_Unit,*) 'Position_Archive: updating with STEP'
  Call Printdate(Positions_Unit,'STEP        on ')
  Write(Positions_Unit,&
  '("  Cohesive_Energy and Error   ",1p2e20.10)')CohesiveEnergy,Cohesive_Diff

  Do i=1, Specific_Atoms                                 
     j=Atom_List(i)%TypeIndex                
     If (Atom_List(i)%Freeze) then
      write(Positions_Unit,'(A6,"  ",A6,1p3e18.10," FROZEN  "/14x,1p3e18.10)') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Pos, atom_list(i)%force%currforce
     Else If (Anchor.and.i==AnchorIndex) then
      write(Positions_Unit,'(A6,"  ",A6,1p3e18.10," ANCHORED"/14x,1p3e18.10)') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Pos, atom_list(i)%force%currforce
     else
      write(Positions_Unit,'(A6,"  ",A6,1p3e18.10,"         "/14x,1p3e18.10)') &
             Atom_List(i)%Name, &
             AtomType_Info(j)%Atom_Name, &
             Atom_List(i)%Pos, atom_list(i)%force%currforce
     End If
  End Do

  Call Flush(Positions_Unit)
  Close(Positions_Unit)

 Return
End Subroutine

!******************************************************************************
!
!  Store_Vxc_FOURIER - Stores the Vxc potential to disk in fourier space
!
!    Filename - File to store data
!
! 05/11/05 not updated for spin
!******************************************************************************

Subroutine Store_Vxc_Fourier(FileName)
  Use exchange_corr
  Character*(*), Intent(IN) :: FileName

  Integer :: i, j
  Complex, Pointer :: Den(:)
 !Complex :: ProjDotPsi(PLM_MAX, NumBands,NumKpnts), d, v
  Complex ::  d, v
  Complex ,allocatable :: ProjDotPsi(:,:,:)
  character*200 :: msg
  Real    :: Energy

  Allocate(ProjDotPsi(PLM_MAX, NumBands,NumKpnts))

  write(Error_Unit,*) 'Entered Store_VXC_Fourier which has not been properly debugged'
       stop

!**  Call CalcHam( Energy)
  !*** Allocate the Large FFT grids ***
  i = FFT_Grid(4,G_HIGH)
  Allocate(Den(i), STAT=j)
  msg = 'CalcHam: Could Not allocate LARGE buffers!'
  Call Check_Error(j, msg, Error_Unit, .TRUE.,paw_wc, "CalcHam:")
  !****************


  !*** Store the density in the 3d FFT grid ***
  Den = 0   
  Den(FFTmap_High(1:Gpnt_Size(G_High))) = SCFvalues%RhoSmooth
  Den(FFTmap_High(Gpnt_Size(G_High)+1:))=CONJG(SCFvalues%RhoSmooth(1:))

  !*** Convert it to real space ***
  Call PerformFFT(FFT_TO_R, G_HIGH, Den)

 !NAWH 5/9/05 -- not clearly correct:
  If (XC_type == XC_LDA_PW) Call CalcVxc_LDA( Den,Energy)  !** Calculate Vxc, Answer is in FFT space **

  !*** Store the results ***
  Open (Scratch_Unit, file=TRIM(filename), RECL=10000)
  Write(Scratch_Unit, *) '1.Gx 2.Gy 3.Gz 4.|G|  5.Vxc(R) 6.Vxc(I) 7.|Vxc|   8.Den(R) 9.Den(I) 10.|Den|'

  i = 1  !** G=0 is handled separately
  v = Den(FFTmap_High(i))
  d = SCFvalues%RhoSmooth(i)
  write(Scratch_Unit,*) Gpnt(:,i), & 
       Real(v), Imag(v), SQRT(REAL(CONJG(v)*v)), &
       Real(d), Imag(d), SQRT(REAL(CONJG(d)*d))
  Do i=2, Gpnt_Size(G_High)
     v = Den(FFTmap_High(i))
     d = SCFvalues%RhoSmooth(i)
     write(Scratch_Unit,*) Gpnt(:,i), & 
          Real(v), Imag(v), SQRT(REAL(CONJG(v)*v)), &
          Real(d), Imag(d), SQRT(REAL(CONJG(d)*d))

     v = CONJG(v)
     d = CONJG(d)
     write(Scratch_Unit,*) -Gpnt(1:3,i), Gpnt(4,i), & 
          Real(v), Imag(v), SQRT(REAL(CONJG(v)*v)), &
          Real(d), Imag(d), SQRT(REAL(CONJG(d)*d))
  End Do
 
  Close(Scratch_Unit)

  DeAllocate(Den)  !** Free the space
  DeAllocate(ProjDotPsi)

  Return
End Subroutine

!******************************************************************************
!
! StoreSolution_Binary - Stores the Wave functions and potentials to disk
!            in raw binary format.
!
! Filename - Filename
!
!******************************************************************************

Subroutine StoreSolution_Binary( FileName)
  Character*(*), Intent(IN) :: Filename

  Integer :: atype,natoms,i,j,Kpnt,Bands,DI,spin,s
  Real :: Kvec(3)
  Complex, Pointer :: Psi(:)
  Integer*2, pointer :: i1(:,:)

  Write(Log_Unit,*) 'StoreSolution_Binary: Storing Results in :',TRIM(FileNAme)

  Open (Scratch_Unit, file=TRIM(filename), form='unformatted')


  
  Write(Scratch_Unit) 'PWPAW1'    !agreed flag

 !*** Write basis vectors
  write(Scratch_Unit) xtal%basis(:,1), xtal%basis(:,2), xtal%basis(:,3)

 !*** Write atom information
  write(Scratch_Unit) Atom_types

  Do atype=1,Atom_Types

     i = AtomType_Range(1,atype)
     natoms=AtomType_Range(2,atype) - i +1
     write(Scratch_Unit) natoms,AtomType_Info(atype)%Atom_Name(1:2)
 
     Do j = AtomType_Range(1,atype), AtomType_Range(2,atype)
        write(Scratch_Unit) Atom_List(j)%Frac_Pos
    end do
  end do

  Write(Scratch_Unit) Gall_Size(G_Low)

  Allocate(i1(3,Gall_Size(G_Low)), STAT=i)
  If (i/=0) then
     Write(*,*) 'StoreSolution: ERROR in allocation', 'Gall=',Gall_Size(G_Low)
     STOP
  End If


  i1(1:3,1)=Npnt(1:3,1)
   do i =2,Gpnt_Size(G_Low)
    i1(1:3,i)=Npnt(1:3,i)
    i1(1:3,i+Gpnt_Size(G_Low)-1)=-Npnt(1:3,i)
   enddo

  Write(Scratch_Unit) (i1(1,i),i1(2,i),i1(3,i),i=1,Gall_Size(G_Low))
 
  if (.not.spindependence) spin=1
  if (spindependence) spin=2
  
  Write(Scratch_Unit) NumKpnts,spin

  ! Prepare to write out wave functions
  Call GetBuffer(psi)

   Do Kpnt = 1, NumKpnts
    do s=1,spin
     if (s==1) Global_spinup=.true.
     if (s==2) Global_spinup=.false.

     Bands=0

      Do i=1,Mem_MapSize
        If(PsiInfo(i)%Kpnt==Kpnt &
         .and. (PsiInfo(i)%spinup.eqv.Global_spinup)) Bands=Bands+1
       enddo

    Write(Scratch_Unit) User_Kpoints(:,Kpnt), s,Bands

      Do i=1,Mem_MapSize
        If(PsiInfo(i)%Kpnt==Kpnt &
         .and. (PsiInfo(i)%spinup.eqv.Global_spinup)) then
         Call GetPsi_fromBuffer(i ,Psi)
         Write(Scratch_Unit) PsiInfo(i)%Energy , PsiInfo(i)%Occupancy, &
            (Psi(j),j=1,Gall_Size(G_Low))
        endif
      enddo
         
      enddo
    Enddo

  Call FreeBuffer(Psi)
  Close(Scratch_Unit)
  Deallocate(i1)

   write(6,*) ' completed storesolution_binary '
  Return
End Subroutine



!******************************************************************************
!
! StoreSolution_Text - Stores the Wave functions and potentials to disk
!            in a human readable format.
!
! Filename - Filename
!
!  05/11/05 not updated for spin
!******************************************************************************

Subroutine StoreSolution_Text( FileName)

  Character*(*), Intent(IN) :: Filename
 
  Integer :: atype,natoms,i,j,Kpnt,Bands,DI, atom
  Real :: Kvec(3)
  Complex, Pointer :: Psi(:)
  Integer*2, Pointer :: i1(:,:)
 
  Write(Log_Unit,*) 'StoreSolution_Text: Storing Results in :',TRIM(FileNAme)
 
  Open (Scratch_Unit, file=TRIM(filename), form='formatted')
 
 
 
  Write(Scratch_Unit,'(a6)') 'PWPAW1'    !agreed flag
 
 !*** Write basis vectors
  write(Scratch_Unit,'(1p3e15.7)') xtal%basis(:,1), xtal%basis(:,2), xtal%basis(:,3)
 
 !*** Write atom information
  write(Scratch_Unit,'(i5)') Atom_types
 
  Do atype=1,Atom_Types
 
     i = AtomType_Range(1,atype)
     natoms=AtomType_Range(2,atype) - i +1
     write(Scratch_Unit,'(i5,3x,a2)') natoms,AtomType_Info(i)%Atom_Name(1:2)
 
    Do j = AtomType_Range(1,atype), AtomType_Range(2,atype)
       Atom = AtomType_Map(j) 
       write(Scratch_Unit,'(1p3e15.7)') Atom_List(atom)%Frac_Pos
    enddo
  enddo
 
  Write(Scratch_Unit,'(i10)') Gall_Size(G_Low)
 
  Allocate(i1(3,Gall_Size(G_Low)), STAT=i)
  If (i/=0) then
     Write(*,*) 'StoreSolution: ERROR in allocation', 'Gall=',Gall_Size(G_Low)
     STOP
  End If
 
  i1(1:3,1)=Npnt(1:3,1)
   do i =2,Gpnt_Size(G_Low)
    i1(1:3,i)=Npnt(1:3,i)
    i1(1:3,i+Gpnt_Size(G_Low)-1)=-Npnt(1:3,i)
   enddo

  Write(Scratch_Unit,'(18i4))')(i1(1,i),i1(2,i),i1(3,i),i=1,Gall_Size(G_Low))
 
  Write(Scratch_Unit,'(i5)') NumKpnts
 
  ! Prepare to write out wave functions
  Call GetBuffer(psi)
 
   Do Kpnt = 1, NumKpnts
 
     Bands=0
 
      Do i=1,Mem_MapSize
        If(PsiInfo(i)%Kpnt==Kpnt) Bands=Bands+1
       enddo
 
    Write(Scratch_Unit,'(1p3e15.7,i5)') User_Kpoints(:,Kpnt), Bands
 
      Do i=1,Mem_MapSize
        If(PsiInfo(i)%Kpnt==Kpnt) then
         Call GetPsi_fromBuffer( i,Psi)
         Write(Scratch_Unit,'(1p2e15.7)') PsiInfo(i)%Energy , PsiInfo(i)%Occupancy 
         Write(Scratch_Unit,'(2("(",1pe15.7,",",1pe15.7,") "))') &
               (Psi(j),j=1,Gall_Size(G_Low))
        endif
 
      enddo
    Enddo
 
  Call FreeBuffer(Psi)
  Close(Scratch_Unit)
  Deallocate(i1)
  Return
End Subroutine

!******************************************************************************
!
!  StorePS_XSF - Stores the PS potential and density in XSF format
!
!    filename - Output filename
!
! 05/11/05 - not updated for spin
!******************************************************************************
Subroutine StorePS_XSF( FileName)

  Character*(*), Intent(IN) :: Filename
  
  Real, PARAMETER :: to_Ang = 0.529177249 

  
  Integer :: atype,natoms,i,j,k,Kpnt,Bands,DI
  Real :: Kvec(3), Ro(3)
  Complex, Pointer :: Work(:), Den(:), Psi(:)
  Character*200 :: dataname, token

  write(Log_Unit,*) 'StorePS_XSF called, but no longer works -- stopping'
  stop
  Write(Log_Unit,*) 'StorePS_XSF: Storing Results in :',TRIM(FileNAme)
 
  Open (Scratch_Unit, file=TRIM(filename), form='formatted')

  Write(Scratch_Unit,*) 'DIM-GROUP'
  Write(Scratch_Unit,*) 3, 1
  Write(Scratch_Unit,*) 'PRIMVEC'
  Write(Scratch_Unit,*) xtal%Basis(:,1)*to_Ang
  Write(Scratch_Unit,*) xtal%Basis(:,2)*to_Ang
  Write(Scratch_Unit,*) xtal%Basis(:,3)*to_Ang
 
  Do j=1, 2
     Write(Scratch_Unit,*) ' '
     if (j==1) then
        Write(Scratch_Unit,*) 'PRIMCOORD'
        Write(Scratch_Unit,*) Specific_atoms, 1
     else
        Write(Scratch_Unit,*) 'ATOMS'
     end If

     
     Do i=1, Specific_Atoms
        Ro = MatMul(xtal%InverseBasis, Atom_List(i)%Pos)
        Where (Ro < 0) Ro = Ro + 1   !** Wrap atom to unit cell
        Ro = MatMul(xtal%Basis, Ro)
        Write(Scratch_Unit,*) &
             AtomType_Info(Atom_List(i)%TypeIndex)%Atomic_Charge, &
             Ro*to_Ang
     End Do
  End do

  !*** Generic header to start 3d grid data ****

  Write(Scratch_Unit,*) ' '
  Write(Scratch_Unit,*) 'BEGIN_BLOCK_DATAGRID3D'
  Write(Scratch_Unit,*) '3d_data'

  !*** Now store the potential(Veff) ***
  Write(Scratch_Unit,*) ' '
  Write(Scratch_Unit,*) 'DATAGRID_3D_Veff'
  Write(Scratch_Unit,*) FFT_Grid(1:3,G_PROJ)
  !Call CalcRealSpace( (/2,2,2/), Ro)
  Ro = -0.275*Ro     !** Goofy offset to get the ball+stick to match Veff
  Write(Scratch_Unit,*) Ro
  Write(Scratch_Unit,*) xtal%Basis(:,1)*to_Ang
  Write(Scratch_Unit,*) xtal%Basis(:,2)*to_Ang
  Write(Scratch_Unit,*) xtal%Basis(:,3)*to_Ang

  Write(Scratch_Unit,*) (SCFvalues%Ve(i), i=1, FFT_Grid(4,G_PROJ))
  Write(Scratch_Unit,*) 'END_DATAGRID_3D'
  Write(Scratch_Unit,*) ' '

  !*** Now store the |psi|^2 for each wave function ***
  Allocate(Den(FFT_Grid(4,G_PROJ)), Work(FFT_Grid(4,G_PROJ)))
  Call GetBuffer( Psi)
  Den = 0
  Do k=1, NumKpnts
     Do j=1, Mem_MapSize
        If (PsiInfo(j)%Kpnt == k) then 
           Call GetPsi_FromBuffer( j, Psi)

           Call RS_toR(Psi, Work)
           Work = Work*CONJG(Work)
           Den = Den + Work*PsiInfo(j)%Occupancy

           !** Create the dataset name
           dataname = 'DATAGRID_3D_pden_Kpnt=('
           !** Add the frac kpoint **
           Write(token,'(F7.3)') User_Kpoints(1,k)
           dataname = TRIM(dataname) // TRIM(ADJUSTL(token)) // ','
           Write(token,'(F7.3)') User_Kpoints(2,k)
           dataname = TRIM(dataname) // TRIM(ADJUSTL(token)) // ','
           Write(token,'(F7.3)') User_Kpoints(3,k)
           dataname = TRIM(dataname) // TRIM(ADJUSTL(token)) // ')_Energy='

           !** Add the Energy **
           Write(token, '(F7.3)') PsiInfo(j)%Energy
           dataname = TRIM(dataname) // TRIM(ADJUSTL(token)) // '_Occ='
           !** Lastly add the occupancy **
           Write(token, '(F7.3)') PsiInfo(j)%Occupancy
           dataname = TRIM(dataname) // TRIM(ADJUSTL(token)) 

           Write(Scratch_Unit,*) TRIM(dataname)
           Write(Scratch_Unit,*) FFT_Grid(1:3,G_PROJ)
           Write(Scratch_Unit,*) Ro
           Write(Scratch_Unit,*) xtal%Basis(:,1)*to_Ang
           Write(Scratch_Unit,*) xtal%Basis(:,2)*to_Ang
           Write(Scratch_Unit,*) xtal%Basis(:,3)*to_Ang
           
           Write(Scratch_Unit,*) (REAL(Work(i)), i=1, FFT_Grid(4,G_PROJ))
           Write(Scratch_Unit,*) 'END_DATAGRID_3D'
           Write(Scratch_Unit,*) ' '           
        End If
     End Do
  End Do

  !** Now store the total density **
  Write(Scratch_Unit,*) 'DATAGRID_3D_SmoothDensity'
  Write(Scratch_Unit,*) FFT_Grid(1:3,G_PROJ)
  Write(Scratch_Unit,*) Ro
  Write(Scratch_Unit,*) xtal%Basis(:,1)*to_Ang
  Write(Scratch_Unit,*) xtal%Basis(:,2)*to_Ang
  Write(Scratch_Unit,*) xtal%Basis(:,3)*to_Ang
  
  Write(Scratch_Unit,*) (REAL(Den(i)), i=1, FFT_Grid(4,G_PROJ))
  Write(Scratch_Unit,*) 'END_DATAGRID_3D'
  Write(Scratch_Unit,*) ' '           

  Call FreeBuffer(Psi)
  DeAllocate(Den, Work)

  !** Close the 3d grid data section ***
  Write(Scratch_Unit,*) 'END_BLOCK_DATAGRID3D'
  Write(Scratch_Unit,*) ' '

  Close(Scratch_Unit)
  Return
End Subroutine
 
!******************************************************************************
!
!  LoadGuess - Loads a previous set of Psi's and Potential from disk
!
!  Filename - Filename to load
!
!******************************************************************************

Subroutine LoadGuess( Filename)
  Character*(*), Intent(IN) :: FileName


  Character*(6)  :: FileType
  Character*(100) :: token
  Integer :: err

  FileType = ' '
Write(*,*) 'LoadGuess: Before OPEN'
  Open(Scratch_Unit, File=FileName, FORM='UNFORMATTED')
Write(*,*) 'LoadGuess: Before Read'
  Read(Scratch_Unit, IOSTAT=err) FileType
Write(*,*) 'LoadGuess: After Read FT=',FileType
  Close(Scratch_Unit)

  If (err==0.AND.FileType(1:6)=='PWPAW1') then
Write(*,*) 'LoadGuess: binary input file'

     Call LoadBinaryGuess( FileName)
  Else
     Open(Scratch_Unit, File=FileName, FORM='FORMATTED')
Write(*,*) 'LoadGuess: Before Read'
     Read(Scratch_Unit, *,IOSTAT=err) FileType
Write(*,*) 'LoadGuess: After Read FT=',FileType
     Close(Scratch_Unit)
     If (err==0.AND.FileType(1:6)=='PWPAW1') then
Write(*,*) 'LoadGuess: text input file'
Write(*,*) '  This option is not up to date -- need to',&
             ' do some more programing work first'
            stop

        Call LoadTextGuess( FileName)
     Else
     Write(Error_unit,*)'LoadGuess: Error reading file:',Filename
     STOP

     EndIf

  ENdIf

  Return
End Subroutine

!******************************************************************************
!
!  LoadBinaryGuess - Loads a previous set of Psi's  from disk
!
!  Filename - Filename to load
!
!******************************************************************************

Subroutine LoadBinaryGuess( Filename)
  Character*(*), Intent(IN)  :: FileName

  Complex, Pointer :: OldPsi(:), NewPsi(:)
  Integer :: Gdim(3), PM(3), Bands, Band, NumK, Kpnt, i, j, k, LSize, DI, dir
 !Integer :: ioerr, nkind, natoms, t, s, ng,  Psi_toUse(Mem_MapSize)
  Integer :: ioerr, nkind, natoms, t, s, ng,spin
  Integer,allocatable ::  Psi_toUse(:)
  Integer*2, Pointer :: i1(:,:)
  Real :: Kvec(3), mag, occ, energy, DV, R(3)
  Character*100 :: token, token2
  Character*200 :: longname
  Real :: Basis(3,3), tau(3), curr(2), err
  Logical :: ok
  Integer, Pointer :: Gfile_Map(:)
  Type (Mem_Handle), Pointer :: Psi
 !Complex :: PDOT(PLM_Max), dot
  Complex ::  dot
  Complex ,allocatable :: PDOT(:)
  Complex, Pointer  :: Vec(:,:), Work(:)
 !Real     :: Eigen(TotalPsi)
  Real ,allocatable :: Eigen(:)
  Integer  :: NewSize,VecSize
 !Integer  :: LUT(2,Mem_MapSize), VecSize
  Integer ,allocatable :: LUT(:,:)


  Allocate(Psi_toUse(Mem_MapSize), PDOT(PLM_Max), &
      Eigen(TotalPsi), LUT(2,Mem_MapSize))
  !*** Make sure ALL wave functions are free ***
  PsiInfo(:)%Kpnt = 0
  PsiInfo(:)%spinup = .true.
  PsiInfo(:)%Occupancy = 0
  PsiInfo(:)%DoSave=1
  PsiInfo(:)%Available=Mem_Available
  PsiInfo(:)%PDOT_Stored = .FALSE.
  PsiInfo(:)%KE_Stored = .FALSE.  
  
  curr = (/V_NewMix, Dij_NewMix/)
  V_NewMix = 1;  Dij_NewMix = 1;


  longname = 'LoadBinaryGuess: Loading file:' // Trim(FileName)

  Call PrintDate(Log_Unit,TRIM(longname)) 
  call flush(Log_unit)

  Open (Scratch_Unit, File=Trim(FileName), FORM='UNFORMATTED')

  Read(Scratch_Unit,iostat=ioerr) token(1:6)   !*** Read in keyword   
  Write(longname,*)'LoadBinaryGuess: Error reading file:',Filename
  Call Check_Error(ioerr,longname,Error_Unit,.true.,PAW_WC,"LoadBinaryGuess")
  call flush(Error_unit)
  if (token(1:6).ne.'PWPAW1') then
     Write(Error_Unit,*)'LoadBinaryGuess: Wrong File Type:',Trim(Filename), &
          ' * Magic Number:',token(1:6), ' * Should be PWPAW1'
     STOP
  endif

  !*** Read in the Basis vectors ***
  Read(Scratch_Unit) Basis(:,1), Basis(:,2), Basis(:,3)
  Write(Log_Unit,*) 'LoadBinaryGuess: A=',Basis(:,1)
  Write(Log_Unit,*) 'LoadBinaryGuess: B=',Basis(:,2)
  Write(Log_Unit,*) 'LoadBinaryGuess: C=',Basis(:,3)
  call flush(log_unit)

  mag = SUM((Basis - xtal%basis)**2)
  If (mag > 1E-9) then 
     Write(Log_Unit,*) 'LoadBinaryGuess: WARNING! Basis Vectors Differ!'
  end if 

  Read(Scratch_Unit) nkind
  !write(log_unit,*)  'LoadBinaryGuess: ',nkind
  !call flush(log_unit)

  Do i=1, nkind
     token = ' '
     Read(scratch_Unit) natoms, token(1:2)
      !write(log_unit,*)  'LoadBinaryGuess: ',natoms, token(1:2)
      !call flush(log_unit)

     k = SearchList_Struct(token, AtomType_LUT)
     !Write(Log_Unit,*) 'LoadBinaryGuess: Atom Kind:',Trim(Token), &
     !     ' * Natoms=',Natoms, ' * LUT=',TRIM(AtomType_Info(k)%Atom_Name)
     !call flush(Log_unit)


     if (k <= 0) then     !** Name not found 
        Write(Log_Unit, *) "LoadBinaryGuess: WARNING Atom Type not Found! Name=", &
             Trim(token)
        call flush(Log_unit)
     End If

     !*** Read individual atoms ***
     Do j=1, natoms
        Read(scratch_Unit) Tau
        !write(log_unit,*)  'LoadBinaryGuess: ',Tau
        !call flush(log_unit)

        s = -1
        Do t=1, Specific_Atoms
           !write(log_unit,*) 'in loadbindaryguess',t,Specific_atoms
           !write(log_unit,*) 'tau',tau
           !write(log_unit,*) 'Atom_List(t)%Frac_pos',Atom_List(t)%Frac_pos
           err = SUM((Tau - Atom_List(t)%Frac_Pos)**2)
           !call flush(log_unit)
           If (err < 1E-9) s = t
        End Do

        If (s > 0) then
           Write(Log_Unit,*) 'LoadBinaryGuess: atom=',j, ' * Maps to atom ',s
        else
           Write(Log_Unit,*) 'LoadBinaryGuess: WARNING atom=',j, &
                " Doesn't map to ANY atom!!!!!"
        End If
     End Do
  End Do


  !**** Read in old G-points ******
  Read(Scratch_Unit) ng  
  write(Log_Unit,*) 'LoadBinaryGuess: ng = ',ng

  !**** Read G-points and create Gfile_Map ****
  Allocate(OldPsi(ng), GFile_Map(ng),i1(3,ng), STAT=i)
  If (i/=0) then
     Write(*,*) 'LoadBinaryGuess: ERROR=',i, ' * NG=',ng
     STOP
  End If


  Read(Scratch_Unit)(i1(1,i),i1(2,i),i1(3,i),i=1,ng)
!  Do i=1, Ng
!     Write(*,*) 'LBG: i=',i, ' * i1(:,i)=',i1(:,i)
!  End Do


  GFile_Map = -1
  do i=1,ng
     do j=1,Gpnt_Size(G_LOW)
        if (SUM((i1(1:3,i)-Npnt(1:3,j))**2).eq.0) then
           GFile_Map(i)=j
           exit
        else if (SUM((i1(1:3,i)+Npnt(1:3,j))**2).eq.0) then
           GFile_Map(i)=j+Gpnt_Size(G_LOW)-1
           exit
        endif
     end do

     If (GFile_Map(i) == -1) then
        Write(Log_Unit,*) 'LoadBinaryGuess: Gn=',i1(1:3,i), ' has no Match!!!'
     End If
  end do

  Call GetBuffer( NewPsi)
  Call GetBuffer( Work)

  Read(Scratch_Unit) NumK,spin   !*** Determine the Number of Kpnts to read
  Write(Log_Unit,*) 'LoadBinaryGuess: Total Number of Kpoints :',NumK
  Call Flush(Log_Unit)
  if (spin==1) then
     spindependence=.false.
     Write(Log_Unit,*) 'LoadBinaryGuess: No spin dependence in calculation'
  else if (spin==2) then
     spindependence=.true.
     Write(Log_Unit,*) 'LoadBinaryGuess: Spin dependent calculation'
  else
     Write(Log_Unit,*) 'LoadBinaryGuess: Spin dependence not set correctly'
     Write(Log_Unit,*) '                 Program stopping for correction'   
     stop
  endif

  call flush(Log_unit)

  ClusterMap=0; ClusterSize=0
  Do Kpnt=1, NumK   !**** Process each Kpnt
   do s=1,spin
     Read(Scratch_Unit) Kvec,t,Bands  !** Get the K-point vector and # of bands
     if (t==1) Global_spinup=.true.
     if (t==2) Global_spinup=.false.

     !** Search for matching K-point ****
     k = -1
     Do j=1, BZ%TotalUniq
        err = SUM((Kvec - User_Kpoints(:,j))**2)
        If (err < 1E-9) k = j
     End Do

     Write(Log_Unit, *) 'LoadBinaryGuess: K-Index=',Kpnt, ' * Kpnt=', Kvec, &
          ' * Bands=',Bands

     If (k > 0) then
        Write(Log_Unit,*) 'LoadBinaryGuess: K-point maps to ',k
     else
        Write(Error_Unit,*) 'LoadBinaryGuess: No matching K-point found!', &
             " Doesn't map to ANY atom!!!!!"
        STOP
     End If

     Psi_toUSe = MH_Skip
     Call Phase_Generic( Psi_toUse,  k)

     Do i=1, Bands      !*** Now load all the bands for the Kvec

       Call GetNextFreeindex(DI)
        
       Read(Scratch_Unit) Energy, Occ, (OldPsi(j), j=1, ng)
       
!       Write(Log_Unit, *) 'LoadBinaryGuess: Reading Band:',DI, &
!             ' * Energy=',Energy, ' * Occ=',Occ
!       Call Flush(Log_Unit)

       !*** Map OldPsi to NewPsi ***
       NewPsi = 0
       Do j=1, ng
         If (Gfile_Map(j)>0) NewPsi(Gfile_Map(j)) = OldPsi(j) !*REMOVE COMMENT
       End Do

       PsiInfo(DI)%Kpnt = k
       PsiInfo(DI)%spinup=Global_spinup
       PsiInfo(DI)%MemBufIndex=0       
       PsiInfo(DI)%Energy = Energy
       PsiInfo(DI)%Occupancy = Occ
       PsiInfo(DI)%CorrIndex = 0
       j=(Kpnt-1)*spin+t
       ClusterMap(DI) = j
       ClusterSize(j) = ClusterSize(j) + 1
       LastCluster = Kpnt

!       Write(Log_Unit,*) 'LoadBinaryGuess: DOT=',DOT_PRODUCT(NewPsi,NewPsi)
       PsiInfo(DI)%PDOT_Stored = .FALSE.
       PsiInfo(DI)%KE_Stored = .FALSE.
       Call Opsi( NewPsi, Work, PDot, k, &
            PSiInfo(DI)%PDOT_Stored)
 
       dot = Basis_DotProd( NewPsi, Work, .FALSE.)
       dot = SQRT(Real(Dot))
       NewPsi = NewPsi / Dot   !** Normalize so Qt+Qh=0
       PsiInfo(DI)%PDOT = PDOT/dot

       Call SavePsi_toBuffer( DI, NewPsi) !*REMOVE COMMENT
     End Do
   End Do
  End Do

  Close(Scratch_Unit)

  occ = SUM(PsiInfo(:)%Occupancy)

  Write(Log_Unit,*) 'LoadBinaryGuess:      Total Occupancy Read =',occ  
  Write(Log_Unit,*) 'LoadBinaryGuess: Total Number of Electrons =',TotalElectrons
  occ = occ - TotalElectrons
  If (ABS(occ) > 1E-5) then
     Write(Log_Unit,*) 'LoadBinaryGuess: ERROR Difference in Occ:',occ
     Write(Error_Unit,*) 'LoadBinaryGuess: ERROR Difference in Occ:',occ
  End If

  !*** Make the Band map for CalcHam ***

  Call MakeBandMap
  Occupancy = 0.0
  Do i=1, Mem_MapSize
     Band = BandMap(1,i)
     Kpnt = BandMap(2,i)
     
     If (Kpnt /= 0) then
        Occupancy(Band, Kpnt) = PsiInfo(i)%Occupancy
     End If
  End Do
  ReCalculate_Occupancies=.false.   !Keep same occupancies
  Call CalcNewOccAndFermi    !Calculate Fermi energy     
  ReCalculate_Occupancies=.true.

  !** Orthog the wave functions for each K-point ***
!Write(Log_Unit,*) 'LoadBinaryGuess: Before OrthogVec CalcHam.'
!Write(*,*) 'LoadBinaryGuess: Before OrthogVec CalcHam.'

  Call CalcMeritFunction(  Energy)

!-4-30-01-Call DisplayBufferInfo(Log_Unit)
!Call LRU_DisplayBufferInfo(LRU_Proj, Log_Unit)
!Call LRU_DisplayBufferInfo(LRU_RadPRoj, Log_Unit)
!-4-30-01-Call LRU_DisplayBufferInfo(LRU_Phase, Log_Unit)
!-4-30-01-Call LRU_DisplayBufferInfo(LRU_Ylm, Log_Unit)

!-4-30-01-  Do i=1, NumKpnts
!-4-30-01-     Psi_toUSe = MH_Skip
!-4-30-01-     Where (PsiInfo(:)%Kpnt == i) Psi_toUse = MH_toProcess
!-4-30-01-     VecSize = SUM(Psi_toUse)
!-4-30-01-     Allocate(Vec(VecSize, VecSize))


!FIXME**
!-4-30-01-     Call PerformGRRP_ALL( Psi_toUse, VecSize, &
!-4-30-01-       Vec, Eigen, LUT, NewSize, i)
!-4-30-01-     Call MakeNewPsi( VecSize, NewSize, Vec, Eigen, LUT, i)
!***
!-4-30-01-     DeAllocate(Vec)
!-4-30-01-  End Do
!-4-30-01-Write(Log_Unit,*) 'LoadBinaryGuess: After Orthog'

!Make Calculate New Occupation numbers

!-4-30-01-  Call MakeBandMap
!-4-30-01-  Call CalcNewOccAndFermi
!-4-30-01-  Call ReCluster_All

!-4-30-01-  PsiInfo(:)%PDOT_Stored = .FALSE.  !** Clear PDOT's
!-4-30-01-  PsiInfo(:)%KE_Stored = .FALSE.    !** and KE's

!-4-30-01-  Call CalcMeritFunction(Energy)

!-4-30-01-  Write(Log_Unit,*) 'LoadBinaryGuess: Initial Wave functions'
!-4-30-01-  Do j=1, NumKpnts
!-4-30-01-     Psi_toUSe = MH_Skip
!-4-30-01-     Where (PsiInfo(:)%Kpnt == j) Psi_toUse = MH_toProcess
!-4-30-01-     Call Phase_Generic( Psi_toUse,  j)
!-4-30-01-     Do i=1, Mem_MapSize
!-4-30-01-        If (PsiInfo(i)%Kpnt == j) Call CheckError( i, Log_Unit)
!-4-30-01-     End Do
!-4-30-01-  End Do


!  Write(Log_Unit, *) 'LoadBinaryGuess: LastCluster=',LastCluster
!  Write(Log_Unit, *) 'LoadBinaryGuess: ClusterSize=',CLusterSize
!  Write(Log_Unit, *) 'LoadBinaryGuess:      ClusterMap=',ClusterMap
!  Write(Log_Unit, *) 'LoadBinaryGuess: PsiInfo(:)%Kpnt=',PsiInfo(:)%Kpnt

  Call PrintDate(Log_Unit, 'LoadBinaryGuess: End of Routine!')

  Call Flush(Log_Unit)

  DeAllocate(Gfile_Map, OldPsi,i1)
  Call FreeBuffer(NewPsi)
  Call FreeBuffer(Work)

  V_NewMix = Curr(1);  Dij_NewMix = Curr(2)
  DeAllocate(Psi_toUse,PDOT,Eigen,LUT)

  Return
End Subroutine

!******************************************************************************
!
!  LoadTextGuess - Loads a previous set of Psi's  from disk -- Not properly updated
!
!  Filename - Filename to load
!
! 05/11/05 not updated for spin
!******************************************************************************

Subroutine LoadTextGuess( Filename)
  Character*(*), Intent(IN)  :: FileName

  Complex, Pointer :: OldPsi(:), NewPsi(:)
  Integer :: Gdim(3), PM(3), Bands, Band, NumK, Kpnt, i, j, k, LSize, DI, dir
 !Integer :: ioerr, nkind, natoms, t, s, ng,  Psi_toUse(Mem_MapSize)
  Integer :: ioerr, nkind, natoms, t, s, ng
  Integer ,allocatable :: Psi_toUse(:)
  Integer*2, Pointer :: i1(:,:)
  Real :: Kvec(3), mag, occ, energy, DV, R(3)
  Character*100 :: token, token2
  Real :: Basis(3,3), tau(3), curr(2), err
  Logical :: ok
  Integer, Pointer :: Gfile_Map(:)
  Type (Mem_Handle), Pointer :: Psi
 !Complex :: PDOT(PLM_Max), dot
  Complex ::  dot
  Complex ,allocatable :: PDOT(:)
  Complex, Pointer  :: Vec(:,:), Work(:)
 !Real     :: Eigen(TotalPsi)
  Real ,allocatable :: Eigen(:)
  Integer  :: NewSize,VecSize
 !Integer  :: LUT(2,Mem_MapSize), VecSize
  Integer ,allocatable :: LUT(:,:)

  Allocate(Psi_toUse(Mem_MapSize), PDOT(PLM_Max), &
      Eigen(TotalPsi), LUT(2,Mem_MapSize))

  !*** Make sure ALL wave functions are free ***
  PsiInfo(:)%Kpnt = 0
  PsiInfo(:)%DoSave=0
  PsiInfo(:)%Available=Mem_Available
  PsiInfo(:)%PDOT_Stored = .FALSE.
  PsiInfo(:)%KE_Stored = .FALSE.  
  
  curr = (/V_NewMix, Dij_NewMix/)
  V_NewMix = 1;  Dij_NewMix = 1;


  token = 'LoadTextGuess: Loading file:' // Trim(FileName)
  Call PrintDate(Log_Unit,TRIM(token)) 

  Open (Scratch_Unit, File=Trim(FileName), FORM='FORMATTED')

  Read(Scratch_Unit,*, iostat=ioerr) token(1:6)  !*** Read in keyword   
  Write(token2,*)'LoadTextGuess: Error reading file:',Filename
  Call Check_Error(ioerr,token2,Error_Unit,.true.,PAW_WC,"LoadTextGuess")

  if (token(1:6).ne.'PWPAW1') then
     Write(Error_Unit,*)'LoadTextGuess: Wrong File Type:',Trim(Filename), &
          ' * Magic Number:',token(1:6), ' * Should be PWPAW1'
     STOP
  endif

  !*** Read in the Basis vectors ***
  Read(Scratch_Unit,*) Basis(:,1), Basis(:,2), Basis(:,3)
  Write(Log_Unit,*) 'LoadTextGuess: A=',Basis(:,1)
  Write(Log_Unit,*) 'LoadTextGuess: B=',Basis(:,2)
  Write(Log_Unit,*) 'LoadTextGuess: C=',Basis(:,3)

  mag = SUM((Basis - xtal%basis)**2)
  If (mag > 1E-9) then 
     Write(Log_Unit,*) 'LoadTextGuess: WARNING! Basis Vectors Differ!'
  end if 

  Read(Scratch_Unit,*) nkind

  Do i=1, nkind
     token = ' '
     Read(scratch_Unit,*) natoms, token(1:2)

     k = SearchList_Struct(token, AtomType_LUT)
     Write(Log_Unit,*) 'LoadTextGuess: Atom Kind:',Trim(Token), &
          ' * Natoms=',Natoms, ' * LUT=',TRIM(AtomType_Info(k)%Atom_Name)


     if (k <= 0) then     !** Name not found 
        Write(Log_Unit, *) "LoadTextGuess: WARNING Atom Type not Found! Name=", &
             Trim(token)
     End If

     !*** Read individual atoms ***
     Do j=1, natoms
        Read(scratch_Unit,*) Tau

        s = -1
        Do t=1, Specific_Atoms
!           write(log_unit,*) 'in loadbindaryguess',t,Specific_atoms
!           write(log_unit,*) 'tau',tau
!           write(log_unit,*) 'Atom_List(t)%Frac_pos',Atom_List(t)%Frac_pos
           err = SUM((Tau - Atom_List(t)%Frac_Pos)**2)
           If (err < 1E-9) s = t
        End Do

        If (s > 0) then
           Write(Log_Unit,*) 'LoadTextGuess: atom=',j, ' * Maps to atom ',s
        else
           Write(Log_Unit,*) 'LoadTextGuess: WARNING atom=',j, &
                " Doesn't map to ANY atom!!!!!"
        End If
     End Do
  End Do


  !**** Read in old G-points ******
  Read(Scratch_Unit,*) ng  
  write(Log_Unit,*) 'LoadTextGuess: ng = ',ng

  !**** Read G-points and create Gfile_Map ****
  Allocate(OldPsi(ng), GFile_Map(ng),i1(3,ng), STAT=i)
  If (i/=0) then
     Write(*,*) 'LoadTextGuess: ERROR=',i, ' * NG=',ng
     STOP
  End If


  Read(Scratch_Unit,*)(i1(1,i),i1(2,i),i1(3,i),i=1,ng)
!  Do i=1, Ng
!     Write(*,*) 'LBG: i=',i, ' * i1(:,i)=',i1(:,i)
!  End Do


  GFile_Map = -1
  do i=1,ng
     do j=1,Gpnt_Size(G_LOW)
        if (SUM((i1(1:3,i)-Npnt(1:3,j))**2).eq.0) then
           GFile_Map(i)=j
           exit
        else if (SUM((i1(1:3,i)+Npnt(1:3,j))**2).eq.0) then
           GFile_Map(i)=j+Gpnt_Size(G_LOW)-1
           exit
        endif
     end do

     If (GFile_Map(i) == -1) then
        Write(Log_Unit,*) 'LoadTextGuess: Gn=',i1(1:3,i), ' has no Match!!!'
     End If
  end do

  Call GetBuffer( NewPsi)
  Call GetBuffer( Work)

  Read(Scratch_Unit,*) NumK   !*** Determine the Number of Kpnts to read
  Write(Log_Unit,*) 'LoadTextGuess: Total Number of Kpoints :',NumK
  Call Flush(Log_Unit)

  Do Kpnt=1, NumK   !**** Process each Kpnt
     Read(Scratch_Unit,*) Kvec, Bands  !** Get the K-point vector and # of bands

     !** Search for matching K-point ****
     s = -1
     Do t=1, BZ%TotalUniq
        err = SUM((Kvec - User_Kpoints(:,t))**2)
        If (err < 1E-9) s = t
     End Do

     Write(Log_Unit, *) 'LoadTextGuess: K-Index=',Kpnt, ' * Kpnt=', Kvec, &
          ' * Bands=',Bands

     If (s > 0) then
        Write(Log_Unit,*) 'LoadTextGuess: K-point maps to ',s
     else
        Write(Error_Unit,*) 'LoadTextGuess: No matching K-point found!', &
             " Doesn't map to ANY atom!!!!!"
        STOP
     End If

     Psi_toUSe = MH_Skip
     Call Phase_Generic(Psi_toUse,  s)

     Do i=1, Bands      !*** Now load all the bands for the Kvec
       Call GetNextFreeindex(DI)
       
       Read(Scratch_Unit,*) Energy, Occ
       Read(Scratch_Unit,*) (OldPsi(j), j=1, ng)

!       Write(Log_Unit, *) 'LoadTextGuess: Reading Band:',DI, &
!             ' * Energy=',Energy, ' * Occ=',Occ
!       Call Flush(Log_Unit)

       !*** Map OldPsi to NewPsi ***
       NewPsi = 0
       Do j=1, ng
         If (Gfile_Map(j)>0) NewPsi(Gfile_Map(j)) = OldPsi(j) !*REMOVE COMMENT
       End Do

       PsiInfo(DI)%Kpnt = s
       PsiInfo(DI)%MemBufIndex=0       
       PsiInfo(DI)%Energy = Energy
       PsiInfo(DI)%Occupancy = Occ
       PsiInfo(DI)%CorrIndex = 0
       ClusterMap(DI) = Kpnt
       ClusterSize(Kpnt) = ClusterSize(Kpnt) + 1
       LastCluster = Kpnt

!       Write(Log_Unit,*) 'LoadTextGuess: DOT=',DOT_PRODUCT(NewPsi,NewPsi)
       PsiInfo(DI)%PDOT_Stored = .FALSE.
       Call Opsi( NewPsi, Work, PDot, s, &
            PSiInfo(DI)%PDOT_Stored)
 
       dot = Basis_DotProd( NewPsi, Work, .FALSE.)
       dot = SQRT(Real(Dot))
       NewPsi = NewPsi / Dot   !** Normalize so Qt+Qh=0
       PsiInfo(DI)%PDOT = PDOT/dot

       Call SavePsi_toBuffer( DI, NewPsi) !*REMOVE COMMENT
     End Do
  End Do

  Close(Scratch_Unit)

  occ = SUM(PsiInfo(:)%Occupancy)

  Write(Log_Unit,*) 'LoadTextGuess:      Total Occupancy Read =',occ  
  Write(Log_Unit,*) 'LoadTextGuess: Total Number of Electrons =',TotalElectrons
  occ = occ - TotalElectrons
  If (ABS(occ) > 1E-5) then
     Write(Log_Unit,*) 'LoadTextGuess: ERROR Difference in Occ:',occ
     Write(Error_Unit,*) 'LoadTextGuess: ERROR Difference in Occ:',occ
  End If

  !*** Make the Band map for CalcHam ***

  Call MakeBandMap
  Occupancy = 0.0
  Do i=1, Mem_MapSize
     Band = BandMap(1,i)
     Kpnt = BandMap(2,i)
     
     If (Kpnt /= 0) then
        Occupancy(Band, Kpnt) = PsiInfo(i)%Occupancy
     End If
  End Do

  !** Orthog the wave functions for each K-point ***
!Write(Log_Unit,*) 'LoadTextGuess: Before OrthogVec CalcHam.'
!Write(*,*) 'LoadTextGuess: Before OrthogVec CalcHam.'

  Call CalcMeritFunction( Energy)

Call DisplayBufferInfo(Log_Unit)
!Call LRU_DisplayBufferInfo(LRU_Proj, Log_Unit)
!Call LRU_DisplayBufferInfo(LRU_RadPRoj, Log_Unit)
Call LRU_DisplayBufferInfo(LRU_Phase, Log_Unit)
Call LRU_DisplayBufferInfo(LRU_Ylm, Log_Unit)

  Do i=1, NumKpnts
     Psi_toUSe = MH_Skip
     Where (PsiInfo(:)%Kpnt == i) Psi_toUse = MH_toProcess
     VecSize = SUM(Psi_toUse)
     Allocate(Vec(VecSize, VecSize))


!FIXME**
     Call PerformGRRP_ALL( Psi_toUse, VecSize, &
       Vec, Eigen, LUT, NewSize, i)
     Call MakeNewPsi( VecSize, NewSize, Vec, Eigen, LUT, i)
!***
     DeAllocate(Vec)
  End Do
Write(Log_Unit,*) 'LoadTextGuess: After Orthog'

!Make Calculate New Occupation numbers
  Call MakeBandMap
  Call CalcNewOccAndFermi
  Call ReCluster_All


  PsiInfo(:)%PDOT_Stored = .FALSE.  !** Clear PDOT's
  PsiInfo(:)%KE_Stored = .FALSE.    !** and KE's

  Call CalcMeritFunction( Energy)

  Write(Log_Unit,*) 'LoadTextGuess: Initial Wave functions'
  Do j=1, NumKpnts
     Psi_toUSe = MH_Skip
     Where (PsiInfo(:)%Kpnt == j) Psi_toUse = MH_toProcess
     Call Phase_Generic( Psi_toUse,  j)
     Do i=1, Mem_MapSize
        If (PsiInfo(i)%Kpnt == j) Call CheckError( i, Log_Unit)
     End Do
  End Do


!  Write(Log_Unit, *) 'LoadTextGuess: LastCluster=',LastCluster
!  Write(Log_Unit, *) 'LoadTextGuess: ClusterSize=',CLusterSize
!  Write(Log_Unit, *) 'LoadTextGuess:      ClusterMap=',ClusterMap
!  Write(Log_Unit, *) 'LoadTextGuess: PsiInfo(:)%Kpnt=',PsiInfo(:)%Kpnt

  Call PrintDate(Log_Unit, 'LoadTextGuess: End of Routine!')

  Call Flush(Log_Unit)

  DeAllocate(Gfile_Map, OldPsi,i1)
  Call FreeBuffer(NewPsi)
  Call FreeBuffer(Work)

  V_NewMix = Curr(1);  Dij_NewMix = Curr(2)
  DeAllocate(Psi_toUse,PDOT,Eigen,LUT)

  Return
End Subroutine

!******************************************************************************
!
! StoreHam_Binary - Stores the Dij and potentials to disk
!            in raw binary format.
!
! Filename - Filename
!
!******************************************************************************

Subroutine StoreHam_Binary( FileName)
  Character*(*), Intent(IN) :: Filename

  Integer :: atype,natoms,i,j,k,s,t,Kpnt,Bands,DI, atom,spin
  Real :: Kvec(3)
  Complex, Pointer :: Vg(:), Work(:)
  Integer*2, pointer :: i1(:,:)
  

  Write(Log_Unit,*) 'StoreHam_Binary: Storing Results in :',TRIM(FileNAme)

  Open (Scratch_Unit, file=TRIM(filename), form='unformatted')

  
  Write(Scratch_Unit) 'HPWPAW1'    !agreed flag

 !*** Write basis vectors
  write(Scratch_Unit) xtal%basis(:,1), xtal%basis(:,2), xtal%basis(:,3)

 !*** Write atom information
 if (.not.spindependence) spin=1
 if (spindependence) spin=2

  write(Scratch_Unit) Atom_types,spin

  Do atype=1,Atom_Types
     t = AtomType_Info(atype)%nlm_Size

     i = AtomType_Range(1,atype)
     natoms=AtomType_Range(2,atype) - i +1
     write(Scratch_Unit) natoms,AtomType_Info(atype)%Atom_Name(1:2)
     
     Do j = AtomType_Range(1,atype), AtomType_Range(2,atype)
        Atom = AtomType_Map(j)
        write(Scratch_Unit) Atom_List(Atom)%Frac_Pos
        write(Scratch_Unit) ((Atom_List(Atom)%Dij(k,s),k=1,t),s=1,t)
        if(spindependence) &
          write(Scratch_Unit) ((Atom_List(Atom)%Dijspin(k,s),k=1,t),s=1,t)
     enddo
  enddo

  Write(Scratch_Unit) Gall_Size(G_HIGH)  !Gall is "ng", the size of
                                          !the potential and density arrays

  Allocate(i1(3,Gall_Size(G_HIGH)), STAT=i)
  If (i/=0) then
     Write(*,*) 'StoreSolution: ERROR in allocation', 'Gall=',Gall_Size(G_HIGH)
     STOP
  End If

  !Make an array i1 of all the integer g values, positive and negative
  i1(1:3,1)=Npnt(1:3,1)
  do i =2,Gpnt_Size(G_HIGH)  !Gall_Size =  2*Gpnt_size -1
     i1(1:3,i)=Npnt(1:3,i)
     i1(1:3,i+Gpnt_Size(G_HIGH)-1)=-Npnt(1:3,i) 
        !Only positive G's stored so create neg's here:
        !  Order is g=0,g1,g2...,gn,-g1,-g2,....,-gn
  enddo

  Write(Scratch_Unit) (i1(1,i),i1(2,i),i1(3,i),i=1,Gall_Size(G_HIGH))

  Work =>SCFvalues%Work
  Work = SCFvalues%Ve  
  Call PerformFFT(FFT_TO_G, G_HIGH, Work)
  Allocate(Vg(Gall_Size(G_HIGH)))
  Vg = Work(FFTMap_HIGH(1:Gall_Size(G_HIGH)))
  
  write(Scratch_Unit) (Vg(i),i=1,Gall_Size(G_HIGH))
  Vg=0
  Vg(1:Gpnt_Size(G_HIGH)) = SCFvalues%RhoSmooth(1:Gpnt_Size(G_HIGH))
  Vg(Gpnt_Size(G_HIGH)+1:Gall_Size(G_HIGH)) = &
          CONJG(SCFvalues%RhoSmooth(2:Gpnt_Size(G_HIGH)))

  write(Scratch_Unit) (Vg(i),i=1,Gall_Size(G_HIGH))

  if (spindependence) then
    Work = SCFvalues%Vespin
    Call PerformFFT(FFT_TO_G, G_HIGH, Work)
    Vg = Work(FFTMap_HIGH(1:Gall_Size(G_HIGH)))
  
    write(Scratch_Unit) (Vg(i),i=1,Gall_Size(G_HIGH))
    Vg=0
    Vg(1:Gpnt_Size(G_HIGH)) = SCFvalues%RhoSmoothspin(1:Gpnt_Size(G_HIGH))
    Vg(Gpnt_Size(G_HIGH)+1:Gall_Size(G_HIGH)) = &
          CONJG(SCFvalues%RhoSmoothspin(2:Gpnt_Size(G_HIGH)))

    write(Scratch_Unit) (Vg(i),i=1,Gall_Size(G_HIGH))
  endif

  Close(Scratch_Unit)
  Deallocate(i1,Vg)

   write(Log_Unit,*) 'StoreHam_Binary: Completed!!!! '
  Return
End Subroutine


!******************************************************************************
!
!  Loadham_Binary - Loads a previous set of Dij's and potential
!
!  Filename - Filename to load
!
!******************************************************************************

Subroutine LoadHam_Binary( Filename)
  Character*(*), Intent(IN)  :: FileName

  Complex, Pointer :: OldPot(:), NewPot(:), Work(:)
  Integer :: Gdim(3), PM(3), Bands, Band, NumK, Kpnt, i, j, k, LSize, DI, dir
 !Integer :: ioerr, nkind, natoms, t, s, ng,  Psi_toUse(Mem_MapSize)
  Integer :: ioerr, nkind, natoms, t, s, ng,spin
  Integer ,allocatable ::  Psi_toUse(:)
  Integer :: a,b,c
  Integer*2, Pointer :: i1(:,:)
  Real :: Kvec(3), mag, occ, energy, DV, R(3)
  Character*100 :: token, token2
  Real :: Basis(3,3), tau(3), err
  Logical :: ok
  Integer, Pointer :: Gfile_Map(:)
  Type (Mem_Handle), Pointer :: Psi   ! for test
 !Complex :: PDOT(PLM_Max)            ! for test 
  Complex ,allocatable:: PDOT(:)            ! for test 


  Allocate(Psi_toUse(Mem_MapSize), PDOT(PLM_Max) )
  Write(Log_Unit,*) 'LoadHam: Loading file:',Trim(FileName)
  call flush(Log_unit)

  Open (Scratch_Unit, File=Trim(FileName), FORM='UNFORMATTED')

  Read(Scratch_Unit,iostat=ioerr) token(1:7)   !*** Read in keyword   
  Write(token2,*)'LoadHam: Error reading file:',Filename
  Call Check_Error(ioerr,token2,Error_Unit,.true.,PAW_WC,"LoadHam")
  call flush(Error_unit)

  if (token(1:7).ne.'HPWPAW1') then
     Write(Error_Unit,*)'LoadHam: Wrong File Type:',Trim(Filename), &
          ' * Magic Number:',token(1:7), ' * Should be HPWPAW1'
     STOP
  endif

  !*** Read in the Basis vectors ***
  Read(Scratch_Unit) Basis(:,1), Basis(:,2), Basis(:,3)
  Write(Log_Unit,*) 'LoadHam: A=',Basis(:,1)
  Write(Log_Unit,*) 'LoadHam: B=',Basis(:,2)
  Write(Log_Unit,*) 'LoadHam: C=',Basis(:,3)
  call flush(log_unit)

  mag = SUM((Basis - xtal%basis)**2)
  If (mag > 1E-9) then 
     Write(Log_Unit,*) 'LoadHam: WARNING! Basis Vectors Differ!'
  end if 

  Read(Scratch_Unit) nkind,spin

  write(Log_unit,*) 'LoadHam:',nkind,spin

  if (spin==1) then
     spindependence=.false.
     Global_spinup=.true.
  else if (spin==2) then
     spindependence=.true.
  else
     write(log_unit,*) 'LoadHam: error reading spin ', spin
     stop
  endif

  Do i=1, nkind
     token = ' '
     Read(scratch_Unit) natoms, token(1:2)

     k = SearchList_Struct(token, AtomType_LUT)
     Write(Log_Unit,*) 'LoadHam: Atom Kind:',Trim(Token), &
          ' * Natoms=',Natoms, ' * LUT=',TRIM(AtomType_Info(k)%Atom_Name)


     if (k <= 0) then     !** Name not found 
        Write(Log_Unit, *) "LoadHam: WARNING Atom Type not Found! Name=", &
             Trim(token)
     End If

     !*** Read individual atoms ***
     Do j=1, natoms
        Read(scratch_Unit) Tau
         write(6,*) 'j,natoms',j,natoms 
        
        s = -1
        Do t=1, Specific_Atoms
!           write(6,*) 'in loadham',t,Specific_atoms
!           write(6,*) 'tau',tau
!           write(6,*) 'Atom_List(t)%Frac_pos',Atom_List(t)%Frac_pos
           err = SUM((Tau - Atom_List(t)%Frac_Pos)**2)
           If (err < 1E-9) s = t
        End Do

        If (s > 0) then
           Write(Log_Unit,*) 'LoadHam: atom=',j, ' * Maps to atom ',s

           t = AtomType_Info(Atom_List(s)%TypeIndex)%nlm_Size
           Read(Scratch_Unit) ((Atom_List(s)%Dij(a,b),a=1,t),b=1,t)
           if (spindependence) &
              Read(Scratch_Unit) ((Atom_List(s)%Dijspin(a,b),a=1,t),b=1,t)
        else
           Write(Log_Unit,*) 'LoadHam: ERROR atom=',j, &
                " Doesn't map to ANY atom!!!!!"
           STOP
        End If
     End Do
  End Do


  !**** Read in old G-points ******
  Read(Scratch_Unit) ng  
  write(Log_Unit,*) 'LoadHam: ng = ',ng

  !**** Read G-points and create Gfile_Map ****
  j = Max(ng,Gall_Size(G_HIGH))
  Allocate(OldPot(j), GFile_Map(ng),i1(3,ng), STAT=i)
  If (i/=0) then
     Write(*,*) 'LoadHam: ERROR=',i, ' * NG=',ng
     STOP
  End If

  Read(Scratch_Unit)(i1(1,i),i1(2,i),i1(3,i),i=1,ng)

  GFile_Map = -1
  do i=1,ng
     do j=1,Gpnt_Size(G_HIGH)
        if (SUM((i1(1:3,i)-Npnt(1:3,j))**2).eq.0) then
           GFile_Map(i)=j
           exit
        else if (SUM((i1(1:3,i)+Npnt(1:3,j))**2).eq.0) then
           GFile_Map(i)=j+Gpnt_Size(G_HIGH)-1
           exit
        endif
     end do

     If (GFile_Map(i) == -1) then
        Write(Log_Unit,*) 'LoadHam: Gn=',i1(1:3,i), ' has no Match!!!'
     End If
  enddo

Write(Log_Unit,*) 'LoadHam: Before LoadPot'
  Read(Scratch_Unit) (OldPot(j), j=1, ng)


  !*** Map OldPsi to NewPsi ***
  Work  => SCFValues%Work
  Work   = 0
  Do j=1, ng
     If (Gfile_Map(j)>0) Work(Gfile_Map(j)) = OldPot(j) 
  End Do

! Make sure that potential is real
  OldPot=0
  Oldpot(1)=Real(Work(1))
  do j=2,Gpnt_Size(G_HIGH)
     i=j-1+Gpnt_Size(G_HIGH)
     OldPot(j)=(Work(j)+CONJG(Work(i)))/2
     OldPot(i)=CONJG(OldPot(j))
  EndDo
  Work   = 0
  Work(FFTMap_HIGH(1:Gall_Size(G_HIGH))) = OldPot(1:Gall_Size(G_HIGH))
  Call PerformFFT(FFT_TO_R, G_HIGH, Work  )

  SCFvalues%Ve = Work

Write(Log_Unit,*) 'LoadHam: Before LoadDensity'
  Read(Scratch_Unit) (OldPot(j), j=1, ng)
  Work   = 0
  Do j=1, ng
     If (Gfile_Map(j)>0) Work(Gfile_Map(j)) = OldPot(j) 
  End Do

! Make sure that density is real
  SCFvalues%RhoSmooth=0
  SCFvalues%RhoSmooth(1)=Real(Work(1))
  do j=2,Gpnt_Size(G_HIGH)
     i=j-1+Gpnt_Size(G_HIGH)
     SCFvalues%RhoSmooth(j)=(Work(j)+CONJG(Work(i)))/2
  EndDo

  if (spindependence) then
    Read(Scratch_Unit) (OldPot(j), j=1, ng)
    Work   = 0
    Do j=1, ng
     If (Gfile_Map(j)>0) Work(Gfile_Map(j)) = OldPot(j) 
    End Do

    ! Make sure that potential is real
      OldPot=0
      Oldpot(1)=Real(Work(1))
      do j=2,Gpnt_Size(G_HIGH)
         i=j-1+Gpnt_Size(G_HIGH)
         OldPot(j)=(Work(j)+CONJG(Work(i)))/2
         OldPot(i)=CONJG(OldPot(j))
      EndDo
      Work   = 0
      Work(FFTMap_HIGH(1:Gall_Size(G_HIGH))) = OldPot(1:Gall_Size(G_HIGH))
      Call PerformFFT(FFT_TO_R, G_HIGH, Work  )
      SCFvalues%Vespin = Work

     Read(Scratch_Unit) (OldPot(j), j=1, ng)
     Work   = 0
     Do j=1, ng
        If (Gfile_Map(j)>0) Work(Gfile_Map(j)) = OldPot(j) 
     End Do

     ! Make sure that density is real
       SCFvalues%RhoSmoothspin=0
       SCFvalues%RhoSmoothspin(1)=Real(Work(1))
       do j=2,Gpnt_Size(G_HIGH)
          i=j-1+Gpnt_Size(G_HIGH)
          SCFvalues%RhoSmoothspin(j)=(Work(j)+CONJG(Work(i)))/2
       EndDo

  endif

  Close(Scratch_Unit)

  Write(Log_Unit, *) 'LoadHam: End of Routine!'

  Close(Scratch_Unit)

  Write(Log_Unit, *) 'LoadHam: End of Routine!'

  Call Flush(Log_Unit)

  DeAllocate(Gfile_Map, OldPot,i1)
  DeAllocate(Psi_toUse, PDOT)

  HamLoaded=.true.

  Return
End Subroutine

!******************************************************************************
!
!  ReCluster - Recluster's the eigenvalues for the given Kpoint
!
!  Kpnt - Kpoint to recluster
!  MaxEnergy  - Max Kpnt energy
!  MaxCLuster - Cluster the MAx Energy corresponds to
!  NumPsi     - Total number of PSi's for kpoint
!
!******************************************************************************

Subroutine ReCluster(Kpnt, MaxEnergy, MaxCluster, NumPsi)
  Integer,      Intent(IN)  :: Kpnt
  Real, Intent(OUT) :: MaxEnergy
  Integer,      Intent(OUT) :: MaxCLuster
  Integer,      Intent(INOUT) :: NumPsi

 !Integer :: LUT(Mem_MapSize), i, ListSize, SortLUT(Mem_MapSize), j, k
  Integer :: i, ListSize, j, k
  Integer ,allocatable :: LUT(:), SortLUT(:)
  Integer :: Startcluster, n
 !Real :: Eigen(Mem_MapSize)
  Real ,allocatable :: Eigen(:)
    
  !write(log_unit,*) 'Entering ReCluster with Kpnt =',Kpnt,Global_spinup

    !write(log_unit,*) 'ReCluster: ', Kpnt,Global_spinup
      !do j=1,Mem_MapSize
      !   write(Log_unit,*) 'PsiInfo:',j,PsiInfo(j)%Kpnt,PsiInfo(j)%spinup
      !enddo

  Allocate(LUT(Mem_MapSize), SortLUT(Mem_MapSize), Eigen(Mem_MapSize))
  ListSize = 0  ; LUT = 0; SortLUT=0       !*** Create LUT and Eigenvalue list
  Eigen = 1.e30
  Do i=1, Mem_MapSize
     If ((PsiInfo(i)%Kpnt == Kpnt) .and. &
         (PsiInfo(i)%spinup.eqv.Global_spinup)) then
        ListSize = ListSize + 1
        LUT(ListSize) = i
        Eigen(ListSize) = PsiInfo(i)%Energy
        !write(log_unit,*) 'ReCluster: ',i,ListSize,Eigen(ListSize),PsiInfo(i)%Kpnt
     End if
  End Do

!Write(Log_Unit,*) 'ReCluster: Kpnt=',Kpnt, ' * ListSize=',ListSize, ' * MinBands=',MinBands, ' * LUT=',LUT(1:ListSize)
!Write(Log_Unit,*) 'ReCluster: PsiInfo(:)%Kpnt=',PsiInfo(:)%Kpnt
!Write(Log_Unit,*) 'ReCluster: ClusterMap=',ClusterMap

           !** Sort the eigenvalues ***
  Call Insertion_Sort(Eigen(1:ListSize), SortLUT(1:ListSize), .TRUE.)  

  LastCLuster = LastCLuster + 1
  If (LastCluster > Mem_MapSize) then
     write(Error_Unit,*) 'Error in recluster -- LastCluster > Mem_MapSize', &
         LastCluster, Mem_MapSize
     stop
  EndIf

  StartCluster = LastCluster

  ClusterMap(LUT(SortLut(1:ListSize))) = 0 !** De-associate Psi's with clusters

  !******* Only want 1 Psi in cluster beyond MINBANDS wave functions *******
  i = 1
  k = LUT(SortLUT(i))
  ClusterMap(k) = LastCLuster
  ClusterSize(LastCluster) = 1
  !write(log_unit,*) 'ReCluster: MinBands',MinBands,TotalPsi
  Do While ((i<ListSize) .AND. ((i-ClusterSize(LastCluster)) < MinBands) &
       .AND. (i<TotalPsi))
     i = i + 1
     j = LUT(SortLUT(i-1))
     k = LUT(SortLUT(i))

     If (ABS(PsiInfo(k)%Energy - PsiInfo(j)%Energy) > ClusterTol) then
        LastCluster = LastCluster + 1
        If (LastCluster > Mem_MapSize) then
           write(Error_Unit,*) &
               'Error in recluster -- LastCluster > Mem_MapSize', &
               LastCluster, Mem_MapSize
           stop
        EndIf
        ClusterSize(LastCluster) = 0
     End If

     ClusterMap(k) = LastCluster
     ClusterSize(LastCluster) = ClusterSize(LastCluster) + 1
     !write(Log_unit,*)'ReCluster: ',i,k,ClusterMap(k),ClusterSize(LastCluster)
  End Do

  MaxCluster = LastCluster
  MaxEnergy = PsiInfo(k)%Energy
  NumPsi = i

  !write(Log_unit,*)'ReCluster: ', Kpnt,MaxCluster,MaxEnergy,NumPsi
!**NEW**
  If (Eigen_Mode == EIGEN_ENERGY) then
     Write(Log_Unit,*) 'ReCluster: Start Map=',ClusterMap
     ClusterSize(LastCluster+1) = 0
     j = StartCluster - 2
     Do i=StartCluster, LastCluster-1, 2
        j = j + 2
        
        k = Min(ClusterSize(i),ClusterSize(i+1))
        If (k <= 1) then
           Where (ClusterMap == i) ClusterMap = j
           Where (ClusterMap == i+1) ClusterMap = j
           ClusterSize(j) = ClusterSize(i) + ClusterSize(i+1)
           ClusterSize(i+1) = 0
           j = j - 1
        End If
     End Do
     
     j = StartCluster-1
     Do i=StartCluster, LastCluster
        j = j + 1
        LUT = 0
        Where (ClusterMap == i) LUT = 1
        k = SUM(LUT)
        If (k == 0) then
           j = j - 1
        else
           Where (ClusterMap == i) ClusterMap = j
        End If
     End Do
     
     LastCluster = j
  End If

!  Where ((ClusterMap == 0) .AND. (PsiInfo(:)%Kpnt == Kpnt)) PsiInfo(:)%Kpnt = 0

!Write(Log_Unit,*) 'ReCluster: After PsiInfo(:)%Kpnt=',PsiInfo(:)%Kpnt
!Write(Log_Unit,*) 'ReCluster: After ClusterMap=',ClusterMap, ' * ClusterSize=',ClusterSize(1:LastCluster)

  DeAllocate(LUT, SortLUT, Eigen)

  Return
End Subroutine

!*****************************************************************************
!
! ReCluster_All - Reclusters all the Wave Functions
!
!*****************************************************************************

Subroutine ReCluster_All

 !Integer :: Kpnt, i, NumPsi, Vec_toProcess(Mem_MapSize), MaxCLuster,temp
  Integer :: Kpnt, i, j,NumPsi, MaxCLuster,temp,spin,s
  Integer ,allocatable :: Vec_toProcess(:)
  Real :: MaxEnergy

  Allocate(Vec_toProcess(Mem_MapSize))
  LastCluster = 0
  CLusterMap = 0
  ClusterSize = 0

  if(.not.spindependence) spin=1
  if(spindependence) spin=2
  
  Do i=1, NumKpnts
   do s=1,spin
     if (s==1) Global_spinup=.true.
     if (s==2) Global_spinup=.false.

    !write(log_unit,*) 'ReCluster_ALL: ', i,s,Global_spinup
      !do j=1,Mem_MapSize
      !   write(Log_unit,*) 'PsiInfo:',j,PsiInfo(j)%Kpnt,PsiInfo(j)%spinup
      !enddo

     Vec_toProcess = MH_SKIP
     Where (PsiInfo(:)%Kpnt == i &
       .and. (PsiInfo(:)%spinup.eqv.Global_spinup)) Vec_toProcess = MH_toProcess
     NumPsi = SUM(Vec_toProcess)
 
     !write(log_unit,*) 'Recluster_ALL: ', i,s,NumPsi

     Call ReCluster(i, MaxEnergy, MaxCluster, NumPsi)
   enddo
  End Do

  Do i=1,Mem_MapSize
     If (ClusterMap(i) > 0) then
        PsiInfo(i)%DoSave=1
        PsiInfo(i)%Available=Mem_Used
     Else
        PsiInfo(i)%Kpnt=0
        PsiInfo(i)%spinup=.true.
        PsiInfo(i)%DoSave=0
        PsiInfo(i)%Available=Mem_Available
        temp=PsiInfo(i)%MemBufIndex
        if (temp>0) then
           Globalmap%Psi_Handle(temp)%Index=0
           PsiInfo(i)%MemBufIndex=0
        EndIf
     EndIf
  EndDo 
  DeAllocate(Vec_toProcess)

  Return
End Subroutine

!******************************************************************************
!
! StoreHamandPsi_Binary - Stores the Dij and potentials to disk
!            in raw binary format -- also stores wfns
!      Useful for subsequent runs with new k-point grid
!
! Filename - Filename
!
!******************************************************************************

Subroutine StoreHamandPsi_Binary( FileName)
  Character*(*), Intent(IN) :: Filename

  Integer :: atype,natoms,i,j,k,s,t,Kpnt,Bands,DI, atom,spin
  Real :: Kvec(3)
  Complex, Pointer :: Vg(:), Work(:),Psi(:)
  Integer*2, pointer :: i1(:,:)
  

  Write(Log_Unit,*) 'StoreHamandPsi_Binary: Storing Results in :',TRIM(FileNAme)

  Open (Scratch_Unit, file=TRIM(filename), form='unformatted')

  
  Write(Scratch_Unit) 'KPWPAW1'    !agreed flag

 !*** Write basis vectors
  write(Scratch_Unit) xtal%basis(:,1), xtal%basis(:,2), xtal%basis(:,3)

 !*** Write atom information
  if (.not.spindependence) spin=1
  if (spindependence) spin=2
  write(Scratch_Unit) Atom_types,spin

  Do atype=1,Atom_Types
     t = AtomType_Info(atype)%nlm_Size

     i = AtomType_Range(1,atype)
     natoms=AtomType_Range(2,atype) - i +1
     write(Scratch_Unit) natoms,AtomType_Info(atype)%Atom_Name(1:2)
     
     Do j = AtomType_Range(1,atype), AtomType_Range(2,atype)
        Atom = AtomType_Map(j)
        write(Scratch_Unit) Atom_List(Atom)%Frac_Pos
        if (SaveUnmixed) then
           write(Scratch_Unit) ((Atom_List(Atom)%cDij(k,s),k=1,t),s=1,t)
           if(spindependence) &
             write(Scratch_Unit) ((Atom_List(Atom)%cDijspin(k,s),k=1,t),s=1,t)
        else
           write(Scratch_Unit) ((Atom_List(Atom)%Dij(k,s),k=1,t),s=1,t)
           if(spindependence) &
             write(Scratch_Unit) ((Atom_List(Atom)%Dijspin(k,s),k=1,t),s=1,t)
        endif
     enddo
  enddo

  Write(Scratch_Unit) Gall_Size(G_HIGH)  !Gall is "ng", the size of
                                          !the potential and density arrays

  Allocate(i1(3,Gall_Size(G_HIGH)), STAT=i)
  If (i/=0) then
     Write(*,*) 'StoreSolution: ERROR in allocation', 'Gall=',Gall_Size(G_HIGH)
     STOP
  End If

  !Make an array i1 of all the integer g values, positive and negative
  i1(1:3,1)=Npnt(1:3,1)
  do i =2,Gpnt_Size(G_HIGH)  !Gall_Size =  2*Gpnt_size -1
     i1(1:3,i)=Npnt(1:3,i)
     i1(1:3,i+Gpnt_Size(G_HIGH)-1)=-Npnt(1:3,i) 
        !Only positive G's stored so create neg's here:
        !  Order is g=0,g1,g2...,gn,-g1,-g2,....,-gn
  enddo

  Write(Scratch_Unit) (i1(1,i),i1(2,i),i1(3,i),i=1,Gall_Size(G_HIGH))

  Allocate(Vg(Gall_Size(G_HIGH)))
  if (.not.SaveUnmixed) then
      Work =>SCFvalues%Work
      Work = SCFvalues%Ve  
      Call PerformFFT(FFT_TO_G, G_HIGH, Work)
      Vg=0
      Vg = Work(FFTMap_HIGH(1:Gall_Size(G_HIGH)))
  else
      Vg=0
      Vg(1:Gpnt_Size(G_HIGH)) = SCFvalues%cVe_F(1:Gpnt_Size(G_HIGH))
      Vg(Gpnt_Size(G_HIGH)+1:Gall_Size(G_HIGH)) = &
          CONJG(SCFvalues%cVe_F(2:Gpnt_Size(G_HIGH)))
  endif
  write(Scratch_Unit) (Vg(i),i=1,Gall_Size(G_HIGH))

  Vg=0
  Vg(1:Gpnt_Size(G_HIGH)) = SCFvalues%RhoSmooth(1:Gpnt_Size(G_HIGH))
  Vg(Gpnt_Size(G_HIGH)+1:Gall_Size(G_HIGH)) = &
          CONJG(SCFvalues%RhoSmooth(2:Gpnt_Size(G_HIGH)))
  write(Scratch_Unit) (Vg(i),i=1,Gall_Size(G_HIGH))

  if (spindependence) then
    if (.not.SaveUnmixed) then
       Work = SCFvalues%Vespin
       Call PerformFFT(FFT_TO_G, G_HIGH, Work)
       Vg=0
       Vg = Work(FFTMap_HIGH(1:Gall_Size(G_HIGH)))
    else
       Vg=0
       Vg(1:Gpnt_Size(G_HIGH)) = SCFvalues%cVe_Fspin(1:Gpnt_Size(G_HIGH))
       Vg(Gpnt_Size(G_HIGH)+1:Gall_Size(G_HIGH)) = &
          CONJG(SCFvalues%cVe_Fspin(2:Gpnt_Size(G_HIGH)))
   endif
   write(Scratch_Unit) (Vg(i),i=1,Gall_Size(G_HIGH))

    Vg=0
    Vg(1:Gpnt_Size(G_HIGH)) = SCFvalues%RhoSmoothspin(1:Gpnt_Size(G_HIGH))
    Vg(Gpnt_Size(G_HIGH)+1:Gall_Size(G_HIGH)) = &
          CONJG(SCFvalues%RhoSmoothspin(2:Gpnt_Size(G_HIGH)))

    write(Scratch_Unit) (Vg(i),i=1,Gall_Size(G_HIGH))
  endif

 !! Now add Psi's

  Write(Scratch_Unit) Gall_Size(G_Low)
                                                                                  
  i1(1:3,1)=Npnt(1:3,1)
   do i =2,Gpnt_Size(G_Low)
    i1(1:3,i)=Npnt(1:3,i)
    i1(1:3,i+Gpnt_Size(G_Low)-1)=-Npnt(1:3,i)
   enddo
                                                                                
  Write(Scratch_Unit) (i1(1,i),i1(2,i),i1(3,i),i=1,Gall_Size(G_Low))
                                                                                
  Write(Scratch_Unit) NumKpnts

  ! Prepare to write out wave functions
  Call GetBuffer(psi)
                                                                                
   Do Kpnt = 1, NumKpnts
    do s=1,spin
     if (s==1) Global_spinup=.true.
     if (s==2) Global_spinup=.false.

                                                                                
     Bands=0
                                                                                
      Do i=1,Mem_MapSize
        If(PsiInfo(i)%Kpnt==Kpnt &
         .and. (PsiInfo(i)%spinup.eqv.Global_spinup)) Bands=Bands+1
       enddo
                                                                                
    Write(Scratch_Unit) User_Kpoints(:,Kpnt), s,Bands
                                                                                
      Do i=1,Mem_MapSize
        If(PsiInfo(i)%Kpnt==Kpnt &
         .and. (PsiInfo(i)%spinup.eqv.Global_spinup)) then
         Call GetPsi_fromBuffer(i ,Psi)
         Write(Scratch_Unit) PsiInfo(i)%Energy , PsiInfo(i)%Occupancy, &
            (Psi(j),j=1,Gall_Size(G_Low))
        endif
      enddo

      enddo
    Enddo

  Call FreeBuffer(Psi)

  Close(Scratch_Unit)
  Deallocate(i1,Vg)

   write(Log_Unit,*) 'StoreHamand_Binary: Completed!!!! '
  Return
End Subroutine


!******************************************************************************
!
!  LoadHamandPsi_Binary - Loads a previous set of Dij's and potential
!         as well as wfn's                                  
!      Useful for run with new k-point grid
!
!  Filename - Filename to load
!
!******************************************************************************

Subroutine LoadHamandPsi_Binary( Filename)
  Character*(*), Intent(IN)  :: FileName

  Complex, Pointer :: OldPot(:),  Work(:),OldPsi(:),NewPsi(:)
  Integer :: Gdim(3),PM(3),Bands,Band,NumK,Kpnt,i,j,k,LSize,DI, dir
  Integer :: ioerr, nkind, natoms, t, s, ng,spin,Flag
  Integer ,allocatable ::  Psi_toUse(:)
  Integer :: a,b,c
  Integer*2, Pointer :: i1(:,:)
  Real :: Kvec(3), mag, occ, energy, DV, R(3)
  Character*100 :: token, token2
  Real :: Basis(3,3), tau(3),  err
  Logical :: ok
  Integer, Pointer :: Gfile_Map(:)
  Type (Mem_Handle), Pointer :: Psi 
  Integer  :: NewSize,VecSize
  Integer ,allocatable :: mapper(:),unmap(:,:),nmap(:)
  Real, allocatable :: skpoints(:,:),difflist(:)

  Allocate(Psi_toUse(Mem_MapSize))
  Write(Log_Unit,*) 'LoadHamandPsi: Loading file:',Trim(FileName)
  call flush(Log_unit)

  Open (Scratch_Unit, File=Trim(FileName), FORM='UNFORMATTED')

  Read(Scratch_Unit,iostat=ioerr) token(1:7)   !*** Read in keyword   
  Write(token2,*)'LoadHam: Error reading file:',Filename
  Call Check_Error(ioerr,token2,Error_Unit,.true.,PAW_WC,"LoadHamandPsi")
  call flush(Error_unit)

  if (token(1:7).ne.'KPWPAW1') then
     Write(Error_Unit,*)'LoadHam: Wrong File Type:',Trim(Filename), &
          ' * Magic Number:',token(1:7), ' * Should be KPWPAW1'
     STOP
  endif

  !*** Read in the Basis vectors ***
  Read(Scratch_Unit) Basis(:,1), Basis(:,2), Basis(:,3)
  Write(Log_Unit,*) 'LoadHam: A=',Basis(:,1)
  Write(Log_Unit,*) 'LoadHam: B=',Basis(:,2)
  Write(Log_Unit,*) 'LoadHam: C=',Basis(:,3)
  call flush(log_unit)

  mag = SUM((Basis - xtal%basis)**2)
  If (mag > 1E-9) then 
     Write(Log_Unit,*) 'LoadHam: WARNING! Basis Vectors Differ!'
  end if 

  Read(Scratch_Unit) nkind,spin

  write(Log_unit,*) 'LoadHam:',nkind,spin

  if (spin==1) then
    If(spindependence) then
     write(error_unit,*)'LoadHamandPsi: Load file error -- ',&
       ' Load file not spindependent but control file is'
     stop
    endif 
     spindependence=.false.
     Global_spinup=.true.
     Write(Log_Unit,*) 'LoadHamandPsi: No spin dependence in calculation'
  else if (spin==2) then
    If(.not.spindependence) then
     write(error_unit,*)'LoadHamandPsi: Load file error -- ',&
       ' Load file spindependent but control file is not'
     stop
    endif
     spindependence=.true.
     Write(Log_Unit,*) 'LoadHamandPsi: Spin dependence in calculation'
  else
     write(log_unit,*) 'LoadHam: error reading spin ', spin
     stop
  endif

  Do i=1, nkind
     token = ' '
     Read(scratch_Unit) natoms, token(1:2)

     k = SearchList_Struct(token, AtomType_LUT)
     Write(Log_Unit,*) 'LoadHam: Atom Kind:',Trim(Token), &
          ' * Natoms=',Natoms, ' * LUT=',TRIM(AtomType_Info(k)%Atom_Name)


     if (k <= 0) then     !** Name not found 
        Write(Log_Unit, *) "LoadHam: WARNING Atom Type not Found! Name=", &
             Trim(token)
     End If

     !*** Read individual atoms ***
     Do j=1, natoms
        Read(scratch_Unit) Tau
         write(6,*) 'j,natoms',j,natoms 
        
        s = -1
        Do t=1, Specific_Atoms
         Do c=-5,5
          Do b=-5,5
           Do a=-5,5
          
!           write(6,*) 'in loadham',t,Specific_atoms
!           write(6,*) 'tau',tau
!           write(6,*) 'Atom_List(t)%Frac_pos',Atom_List(t)%Frac_pos
           err = SUM((Tau +(/a,b,c/)- Atom_List(t)%Frac_Pos)**2)
           If (err < 1E-9) s = t
           Enddo
          Enddo
         Enddo
        End Do

        If (s > 0) then
           Write(Log_Unit,*) 'LoadHam: atom=',j, ' * Maps to atom ',s

           t = AtomType_Info(Atom_List(s)%TypeIndex)%nlm_Size
           Read(Scratch_Unit) ((Atom_List(s)%Dij(a,b),a=1,t),b=1,t)
           if (spindependence) &
              Read(Scratch_Unit) ((Atom_List(s)%Dijspin(a,b),a=1,t),b=1,t)
        else
           Write(Log_Unit,*) 'LoadHam: ERROR atom=',j, &
                " Doesn't map to ANY atom!!!!!"
           STOP
        End If
     End Do
  End Do

 ! store Andersonmix parameters
      Do i=1, Specific_Atoms    !** Mix each atom's Dij separately
         t = 0    !** Copy new Dij's to 1-D mixing array ***            
         Do j=1, AtomType_Info(Atom_list(i)%TypeIndex)%nlm_Size
            Do k=1, j                                       
               t = t + 1                          
               Atom_list(i)%Dij_old(t) = (Atom_List(i)%Dij(j,k) + &
                        CONJG(Atom_List(i)%Dij(k,j)))/2
               if(spindependence) then
                   Atom_list(i)%Dij_oldspin(t) = (Atom_List(i)%Dijspin(j,k) + &
                        CONJG(Atom_List(i)%Dijspin(k,j)))/2
               endif
 
            End Do
         End Do 
      EndDo

  !**** Read in old G-points ******
  Read(Scratch_Unit) ng  
  write(Log_Unit,*) 'LoadHam: ng = ',ng

  !**** Read G-points and create Gfile_Map ****
  j = Max(ng,Gall_Size(G_HIGH))
  Allocate(OldPot(j), GFile_Map(ng),i1(3,ng), STAT=i)
  If (i/=0) then
     Write(*,*) 'LoadHam: ERROR=',i, ' * NG=',ng
     STOP
  End If

  Read(Scratch_Unit)(i1(1,i),i1(2,i),i1(3,i),i=1,ng)

  GFile_Map = -1
  do i=1,ng
     do j=1,Gpnt_Size(G_HIGH)
        if (SUM((i1(1:3,i)-Npnt(1:3,j))**2).eq.0) then
           GFile_Map(i)=j
           exit
        else if (SUM((i1(1:3,i)+Npnt(1:3,j))**2).eq.0) then
           GFile_Map(i)=j+Gpnt_Size(G_HIGH)-1
           exit
        endif
     end do

     If (GFile_Map(i) == -1) then
        Write(Log_Unit,*) 'LoadHamandPsi: Gn=',i1(1:3,i), ' has no Match!!!'
     End If
  enddo

Write(Log_Unit,*) 'LoadHamandPsi: Before LoadPot'
  Read(Scratch_Unit) (OldPot(j), j=1, ng)


  !*** Map OldPsi to NewPsi ***
  Work  => SCFValues%Work
  write(Log_unit,*) 'LoadHamandPsi: size of work', size(work)
  call flush(log_unit)
  Work   = 0
  Do j=1, ng
     If (Gfile_Map(j)>0) Work(Gfile_Map(j)) = OldPot(j) 
  End Do

! Make sure that potential is real
  OldPot=0
  Oldpot(1)=Real(Work(1))
  do j=2,Gpnt_Size(G_HIGH)
     i=j-1+Gpnt_Size(G_HIGH)
     OldPot(j)=(Work(j)+CONJG(Work(i)))/2
     OldPot(i)=CONJG(OldPot(j))
  EndDo
  write(Log_unit,*) 'LoadHamandPsi: before work', FFTMAP_HIGH(1),&
        FFTMAP_HIGH(GALL_SIZE(G_HIGH)),maxval(fftmap_high(1:Gall_Size(G_HIGH)))
  call flush(log_unit)
  Work   = 0
  !!!!!do i=1,Gall_Size(G_HIGH)
  !!!!! write(Log_unit,*) i,FFTMap_High(i),oldpot(i)
  !!!!! call flush(Log_unit)
  !!!!! work(FFTMap_High(i))=oldpot(i)
  !!!!!enddo
  Work(FFTMap_HIGH(1:Gall_Size(G_HIGH))) = OldPot(1:Gall_Size(G_HIGH))
  Call PerformFFT(FFT_TO_R, G_HIGH, Work  )
  SCFvalues%Ve = Work
  
 ! store Andersonmix parameters
    SCFvalues%Ve_Fourier(1:) = OldPot(1)
    SCFvalues%Ve_Fourier(2:Gpnt_Size(G_HIGH)) =  &
      (OldPot(2:Gpnt_Size(G_HIGH)) +  &
       CONJG(OldPot(Gpnt_Size(G_HIGH)+1:Gall_Size(G_HIGH))))/2


Write(Log_Unit,*) 'LoadHam: Before LoadDensity'
  Read(Scratch_Unit) (OldPot(j), j=1, ng)
  Work   = 0
  Do j=1, ng
     If (Gfile_Map(j)>0) Work(Gfile_Map(j)) = OldPot(j) 
  End Do

! Make sure that density is real
  SCFvalues%RhoSmooth=0
  SCFvalues%RhoSmooth(1)=Real(Work(1))
  do j=2,Gpnt_Size(G_HIGH)
     i=j-1+Gpnt_Size(G_HIGH)
     SCFvalues%RhoSmooth(j)=(Work(j)+CONJG(Work(i)))/2
  EndDo

  if (spindependence) then
    Read(Scratch_Unit) (OldPot(j), j=1, ng)
    Work   = 0
    Do j=1, ng
     If (Gfile_Map(j)>0) Work(Gfile_Map(j)) = OldPot(j) 
    End Do

    ! Make sure that potential is real
      OldPot=0
      Oldpot(1)=Real(Work(1))
      do j=2,Gpnt_Size(G_HIGH)
         i=j-1+Gpnt_Size(G_HIGH)
         OldPot(j)=(Work(j)+CONJG(Work(i)))/2
         OldPot(i)=CONJG(OldPot(j))
      EndDo
      Work   = 0
      Work(FFTMap_HIGH(1:Gall_Size(G_HIGH))) = OldPot(1:Gall_Size(G_HIGH))
      Call PerformFFT(FFT_TO_R, G_HIGH, Work  )
      SCFvalues%Vespin = Work

 ! store Andersonmix parameters
    SCFvalues%Ve_Fourierspin(1:) = OldPot(1)
    SCFvalues%Ve_Fourierspin(2:Gpnt_Size(G_HIGH)) =  &
      (OldPot(2:Gpnt_Size(G_HIGH)) +  &
       CONJG(OldPot(Gpnt_Size(G_HIGH)+1:Gall_Size(G_HIGH))))/2

     Read(Scratch_Unit) (OldPot(j), j=1, ng)
     Work   = 0
     Do j=1, ng
        If (Gfile_Map(j)>0) Work(Gfile_Map(j)) = OldPot(j) 
     End Do

     ! Make sure that density is real
       SCFvalues%RhoSmoothspin=0
       SCFvalues%RhoSmoothspin(1)=Real(Work(1))
       do j=2,Gpnt_Size(G_HIGH)
          i=j-1+Gpnt_Size(G_HIGH)
          SCFvalues%RhoSmoothspin(j)=(Work(j)+CONJG(Work(i)))/2
       EndDo

  endif

  deallocate(oldpot)

  !! Now load in old wavefunctions

!Make sure ALL wave functions are free ***
  PsiInfo(:)%Kpnt = 0
  PsiInfo(:)%spinup = .true.
  PsiInfo(:)%Occupancy = 0
  PsiInfo(:)%DoSave=1
  PsiInfo(:)%Available=Mem_Available
  PsiInfo(:)%PDOT_Stored = .FALSE.
  PsiInfo(:)%KE_Stored = .FALSE.
                                       
    
  !**** Read in old G-points  (G-LOW) ******
  Read(Scratch_Unit) ng
  write(Log_Unit,*) 'LoadHamandPsi: ng = ',ng
                                     
  !**** Read G-points and create Gfile_Map ****
  Allocate(OldPsi(ng), STAT=i)
  If (i/=0) then
     Write(*,*) 'LoadHamandPsi: ERROR=',i, ' * NG=',ng
     STOP
  End If
                                
  
  Read(Scratch_Unit)(i1(1,i),i1(2,i),i1(3,i),i=1,ng)
!  Do i=1, Ng
!     Write(*,*) 'LBG: i=',i, ' * i1(:,i)=',i1(:,i)
!  End Do
 

  GFile_Map = -1
  do i=1,ng
     do j=1,Gpnt_Size(G_LOW)
        if (SUM((i1(1:3,i)-Npnt(1:3,j))**2).eq.0) then
           GFile_Map(i)=j
           exit
        else if (SUM((i1(1:3,i)+Npnt(1:3,j))**2).eq.0) then
           GFile_Map(i)=j+Gpnt_Size(G_LOW)-1
           exit
        endif
     end do
                                      
     If (GFile_Map(i) == -1) then
        Write(Log_Unit,*) 'LoadHamandPsi: Gn=',i1(1:3,i), ' has no Match!!!'
     End If
  end do

  If (Associated(Work)) Nullify(Work) 
  Call GetBuffer( NewPsi)
  Call GetBuffer( Work)
  
  Read(Scratch_Unit) NumK   !*** Determine the Number of Kpnts to read
  Write(Log_Unit,*) 'LoadHamandPsi: Total Number of Kpoints :',NumK
  Call Flush(Log_Unit)
  
  Allocate(mapper(NumKpnts),unmap(NumK,NumKpnts),nmap(NumK),&
     difflist(NumK),skpoints(3,NumK),stat=i)
     if (i/=0) then
       write(Error_unit,*) 'LoadHamandPsi:  error in allocation',&
            NumKpnts,NumK,i
       stop
     endif
  call flush(Log_unit)
 
  Do Kpnt=1, NumK   !**** Process each Kpnt
   do s=1,spin
     Read(Scratch_Unit) Kvec,t,Bands  !** Get the K-point vector and # of bands
     if (t==1) Global_spinup=.true.
     if (t==2) Global_spinup=.false.
     skpoints(:,Kpnt)=Kvec
     
     Write(Log_Unit, *) 'LoadHamandPsi: K-Index=',Kpnt, ' * Kpnt=', Kvec, &
          ' * Bands=',Bands
     call flush(Log_Unit)
     Psi_toUSe = MH_Skip
     Call Phase_Generic( Psi_toUse,  kpnt)
     
     Do i=1, Bands      !*** Now load all the bands for the Kvec
    
       Call GetNextFreeindex(DI)
       Read(Scratch_Unit) Energy, Occ, (OldPsi(j), j=1, ng)
       !*** Map OldPsi to NewPsi ***
       NewPsi = 0
       Do j=1, ng
         If (Gfile_Map(j)>0) NewPsi(Gfile_Map(j)) = OldPsi(j)
       End Do
                                                            
       PsiInfo(DI)%Kpnt = kpnt
       PsiInfo(DI)%spinup=Global_spinup
       PsiInfo(DI)%MemBufIndex=0
       PsiInfo(DI)%Energy = Energy
       PsiInfo(DI)%Occupancy = Occ
       PsiInfo(DI)%CorrIndex = 0

       PsiInfo(DI)%PDOT_Stored = .FALSE.
       PsiInfo(DI)%KE_Stored = .FALSE.
 ! Note wfn is unnormalized
       Call SavePsi_toBuffer( DI, NewPsi) !*REMOVE COMMENT
     End Do
   End Do
  End Do

  mapper=0
  do Kpnt=1,NumKpnts
    write(log_unit,'("LoadHamandPsi: Mapping k = ",1p3e15.7)')&
         User_Kpoints(:,Kpnt)
     do i=1,NumK
        Kvec=User_Kpoints(:,Kpnt)-skpoints(:,i)
        Kvec=MatMul(xtal%Recip,Kvec)
       difflist(i)=Dot_Product(Kvec,Kvec)
     enddo
     k=1
     mapper(Kpnt)=k
     if (NumK>1) then
       do j=2,NumK
          if (difflist(j)<difflist(k)) then
             k=j; mapper(Kpnt)=j
          endif
       enddo
     endif
     write(log_unit,'("   to k = ",i5,1p3e15.7,1x,1pe15.7)')&
         mapper(kpnt),skpoints(:,mapper(kpnt)),difflist(mapper(kpnt))
  enddo
                                                                                
  unmap=0;nmap=0
  do i=1,NumK
     do Kpnt=1,Numkpnts
        if (mapper(Kpnt)==i) then
           nmap(i)=nmap(i)+1
           unmap(i,nmap(i))=Kpnt
        endif
     enddo
     if (nmap(i)>0) then
       write(log_unit,*) 'LoadHamandPsi: old k --> new k',&
        i,nmap(i),(unmap(i,j),j=1,nmap(i))
     else
       write(log_unit,*) 'LoadHamandPsi: old k --> new k',&
        i,nmap(i)
     endif
  enddo

  ! update PsiInfo dataset
      do k=1,NumK
         if (nmap(k)==0) then
           !  remove entries for nmap= 0
            do i=1,Mem_MapSize
               if (PsiInfo(i)%Kpnt==k) then
                  PsiInfo(i)%Kpnt = 0
                  PsiInfo(i)%spinup = .true.
                  PsiInfo(i)%Occupancy = 0
                  PsiInfo(i)%DoSave=0
                  PsiInfo(i)%Available=Mem_Available
                  PsiInfo(i)%PDOT_Stored = .FALSE.
                  PsiInfo(i)%KE_Stored = .FALSE.
                  j=PsiInfo(i)%MemBufIndex
                  if (j>0) then
                     GlobalMap%Psi_Handle(j)%Index=0
                  endif
                  PsiInfo(i)%MemBufIndex=0
                endif
            enddo
         endif
      enddo

    Psi_toUSe = MH_Skip
    do k=1,NumK
       if (nmap(k)>=1) then
          do i=1,Mem_MapSize
             if (PsiInfo(i)%Kpnt==k) Psi_toUse(i)=MH_toProcess
          enddo
       endif
    enddo
    k=1

   !do i=1,Mem_MapSize
   !   if( Psi_toUse(i)==MH_toProcess) write(log_Unit,*) 'mem',i,PsiInfo(i)%Kpnt
   !enddo
   call flush(log_unit)

       call Phase_Generic(Psi_ToUSe,k)
       
      !i=1
      !write(log_unit,*) 'before GetNextPsi',i
      !call flush(log_unit)
       call GetNextPsi(Flag,Psi)
       Do While (Flag>0)
          t=Psi%Index
          k=PsiInfo(t)%Kpnt
            !write(log_unit,*) 'GetPsi', t,k
            !call flush(log_unit)

         do s=1,nmap(k)
           Call GetNextFreeindex(DI)
            !write(log_unit,*) 'Nextfreeindex',DI
            !call flush(log_unit)
          PsiInfo(DI)%Kpnt = unmap(k,s)
          PsiInfo(DI)%spinup=PsiInfo(t)%spinup
          PsiInfo(DI)%MemBufIndex=0
          PsiInfo(DI)%Energy = PsiInfo(t)%Energy
          PsiInfo(DI)%Occupancy = 0
          PsiInfo(DI)%CorrIndex = 0
          PsiInfo(DI)%PDOT_Stored = .FALSE.
          Call SavePsi_toBuffer( DI, Psi%Ptr)
         enddo
                                                                                
          !i=i+1
          !write(log_unit,*) 'before GetNextPsi',i
          !call flush(log_unit)
         Call GetNextPsi(Flag,Psi)
       Enddo

 ! Release old kpoint wfn's
   do t=1,Mem_MapSize
     if ( Psi_toUse(t)==MH_toProcess  ) then
        PsiInfo(t)%Kpnt = 0
        PsiInfo(t)%spinup = .true.
        PsiInfo(t)%Occupancy = 0
        PsiInfo(t)%DoSave=0
        PsiInfo(t)%Available=Mem_Available
        PsiInfo(t)%PDOT_Stored = .FALSE.
        PsiInfo(t)%KE_Stored = .FALSE.
        j=PsiInfo(t)%MemBufIndex
        if (j>0) then
          GlobalMap%Psi_Handle(j)%Index=0
        endif
        PsiInfo(t)%MemBufIndex=0
     endif
   enddo

  write(log_unit,*) 'LoadHamandPsi: Completed PsiInfo '
  call flush(log_unit)


  Close(Scratch_Unit)
  Deallocate(OldPsi)
  DeAllocate(Gfile_Map,i1,mapper,unmap,nmap,skpoints,difflist)
  Call FreeBuffer(NewPsi)
  Call FreeBuffer(Work)

 !Using new basis functions, update energies and occupations

  If(.not.Nodiagwithload) then
    Do j=1, NumKpnts
    do s=1,spin
     if(s==1)Global_spinup=.true.
     if(s==2)Global_spinup=.false.
     PSi_toUse = MH_Skip
     Where (PsiInfo(:)%Kpnt == j&
       .and.(PsiInfo(:)%spinup.eqv.Global_spinup)) PSi_toUse = MH_toProcess
                                                                                
     Where (PsiInfo(:)%Kpnt == j&
       .and.(PsiInfo(:)%spinup.eqv.Global_spinup)) PsiInfo(:)%PDOT_STORED = .FALSE.
     Where (PsiInfo(:)%Kpnt == j&
       .and.(PsiInfo(:)%spinup.eqv.Global_spinup)) PsiInfo(:)%KE_STORED = .FALSE.
                                                                                
     Call Phase_Generic( Psi_toUse,  j) 
     k = 0
     Do i=1, Mem_MapSize
        If (PsiInfo(i)%Kpnt == j&
         .and.(PsiInfo(i)%spinup.eqv.Global_spinup)) then
           Call CheckError(i, Log_Unit)
           k = k + 1
        End IF
     End Do
     Call MiniDiag( j, k, Psi_toUse)
   End Do
  End Do
                                                                                
  PsiInfo(:)%PDOT_Stored = .FALSE.   !** Clear the PDOT's
  PsiInfo(:)%KE_Stored = .FALSE.   !** and KE's
                                                                                
  Call MakeBandMap

  Call CalcNewOccAndFermi
                                                                                
     Do i=1, Mem_MapSize
        If (PsiInfo(i)%Kpnt /=0) then
            !write(log_unit,*)'LoadHamandPsi:',i,PsiInfo(i)%Kpnt,& 
            !  PsiInfo(i)%Energy,PsiInfo(i)%Occupancy,PsiInfo(i)%spinup
            !call flush(log_unit)
        endif
     enddo
                                                                                
  Call ReCluster_All

 Endif
                                                                                
  Write(Log_Unit, *) 'LoadHamandPsi: End of Routine!'

  Call Flush(Log_Unit)

  DeAllocate(Psi_toUse)

  HamLoaded=.true.

  Return
End Subroutine

End Module
