!******************************************************************************
!
! File : structfact.f90
!   by : Alan Tackett
!   on : 04/19/1999
!  for : PW-PAW
!
!  Contains the routines for creating and storing the atom structure
!  factors, exp(i*G*Ra).
!
!******************************************************************************

Module structfact

Use lrulib
Use gpoints
Use work_mgr
Use atom_data

Implicit NONE!!!!!!!!!

Type (COMPLEX_LRU_Context) :: LRU_Phase

Integer :: Phase_Dir

!******************************************************************************
Contains
!******************************************************************************

!******************************************************************************
!
!  GetStructFactor - Gets the specificed structure factor from disk
!
!    Atom  - Specific Atom to get
!    Phase - Pointer to the data(Returned)
!
!******************************************************************************

Subroutine GetStructFactor( atom, Phase)
  Integer,         Intent(IN) :: atom
  Complex,         Pointer    :: Phase(:)

  Call LRU_GetRec(LRU_Phase,  atom, Phase)
  
  Return
End Subroutine

!******************************************************************************
!
!  StoreStructFactors - Stores the atomic structure factors to disk
!
!******************************************************************************

Subroutine StoreStructFactors
  Integer       :: i, j, k,  N ,Gi(3)
  Real  :: G(3), R(3), THeta
  Complex, Pointer :: Phase(:)
  Character*100 :: token

Write(Log_Unit,*) 'StoreStructFactors: Start!!!!!!!!!!!!!!!!'
!RETURN !**REMOVE**

   Call GetBuffer( Phase)
   N = Gpnt_Size(G_Low) - 1

   Do i=1, Specific_Atoms
      R = Atom_List(i)%Pos
      Phase = 0
      
      Phase(1) = 1
      Do j=2, Gpnt_Size(G_LOW)  !** Skip G=0 term
         G = Gpnt(1:3,j)

         Theta = DOT_PRODUCT(G, R)
         Phase(j) = CMPLX(cos(Theta), Sin(Theta))
         Phase(j+N) = CONJG(Phase(j))
      End Do

      Write(token,*) 'StoreStructFactors: Error Writing Data! :', &
       ' * Atom=',i

      Call FilterValue(Phase)
      Call LRU_PutRec(LRU_Phase,  i, Phase)

!      Call WriteFile_LOW(LRU_Phase%FD_Base, i,  Phase, token)
   End Do

   Call FreeBuffer(Phase)

Write(Log_Unit,*) 'StoreStructFactors: Completed store! Filtered'
   Return
End Subroutine



!******************************************************************************
!
! InitStructFactors - Initializes theatomic Structure factor routines
!
!   FD_Base - Base File unit
!   MaxMem - Max amout of memory to use for structure factors(Mbytes)
!   Ng     - Total number of G-points, includes both +G and -G
!
!******************************************************************************

Subroutine InitStructFactors(FD_Base, MaxMem)
  Integer,       Intent(IN) :: FD_Base
  Real,          Intent(IN) :: MaxMem

!RETURN !**REMOVE**

  Call LRU_InitContext(LRU_Phase, 'StructFact', FD_Base, MaxMem, &
       Specific_Atoms, Gall_Size(G_LOW) ,  .TRUE.)

  Call StoreStructFactors  !** Create the structure factors

  Return
End Subroutine


End Module
