c
c $Id: symgen.f,v 1.1.1.1 2003/08/06 02:45:29 natalie Exp $
c
c $Log: symgen.f,v $
c Revision 1.1.1.1  2003/08/06 02:45:29  natalie
c PWPAW source code implemented on Osiris
c
c Revision 1.1  1992/04/21  00:43:59  alberto
c New parameter scheme
c
c Revision 1.1  1991/12/16  00:16:05  alberto
c Initial revision
c
      subroutine symgen(nfd, a,b,coorat,natom,ntype,invers_no)
c
      implicit none
c
      include 'atoms.i'
      include 'symmetry.h'
c
c     This is the driver routine to generate the symmetry
c     operations.
c
c     Adapted by J.L. Martins from the program GROUP
c     written in 1974 by Warren and Worlton
c     Computer Physics Communications, vol 8, 71-74 (1974)
c     incorporated by Eckold et al into UNISOFT. 
c
c     Modified by Alberto Garcia (1990)
c
c
c     Let's see if we can get the rotations both in real and
c     in reciprocal space:
c
c     gmtrx : g-space representation
c     rmtrx: r-space representation
c
c     Input:
c
c     a(i,j) is the i-th cartesian component of the j-th primitive
c     translation vector of the direct lattice and thus it is the 
c     transpose of the matrix A defined by Jones in 'The Theory
c     of Brillouin Zones and Electronic States in Crystals'
c
c     coorat(i,j,k) is the k-th component (lattice coordinates) of
c     the position of the j-th atom of type i.
c
c     natom(i) is the number of atoms of type i.
c
c     ntype is the total number of types of atoms.
c
c     internal:
c
c     b contains the reciprocal lattice vectors in the 
c     crystallographic usage, that is, WITHOUT the 2pi factor.
c     This matrix IS Jones'.
c
c     na is the number of atoms.
c
c     ity(i) is an integer distinguishing atoms of different type
c     i.e. different atomic species.
c
c     x(j,i) is the j-th cartesian component of the position vector for
c     the i-th atom in the unit cell.
c
c     output:
c
c     ntrans is the number of point group operations.
c
c     gmtrx is the matrix of rotations in g-lattice coordinates.
c     rmtrx is the matrix of rotations in r-lattice coordinates.
c
c     tnp(oper,ilat) is the fractional translation in latt. coor.
c
c     invers_no is the operation number of the inversion (0 if not
c     present). It is used to restore the inversion symmetry by
c     a simple change of origin when possible
c
C     .. Scalar Arguments ..
      integer nfd, ntype, invers_no
C     ..
C     .. Array Arguments ..
      Real a(3,3), coorat(ntyp,natp,3)
      integer natom(ntyp)
C     ..
C     .. Local Scalars ..
      Real xdum
      integer i, ihg, ind, ipm, j, k, l, li, m, na, nat, ierr
C     ..
C     .. Local Arrays ..
      Real b(3,3), r(49,3,3), r1(3,3), rlat(48,3,3),
     1                 trans(48,3,3), x(3,ntyp*natp)
      integer ib(48), ity(ntyp*natp)
      character id(48)*10
C     ..
C     .. External Subroutines ..
      external atftmt, pgl, symm_ident !**, symchk
C     ..
C     .. Intrinsic Functions ..
      intrinsic int, mod
C     ..
      ipm = 0
c
c     Calculate cartesian coordinates, atom types, and number of atoms.
c
      na = 0
      do 40 i = 1, ntype
         nat = natom(i)
         do 30 j = 1, nat
            ind = na + j
            do 20 k = 1, 3
               x(k,ind) = 0.0d0
               do 10 l = 1, 3
                  x(k,ind) = x(k,ind) + a(k,l)*coorat(i,j,l)
   10          continue
   20       continue
            ity(ind) = i
   30    continue
         na = na + natom(i)
   40 continue
c
c     Determine reciprocal lattice basis vectors.
c     We know that 
c
c               T                             T -1         -1
c              B A  = 2pi (1), so   B  = 2pi(A )   = 2pi(a )
c
c     and we can use the linpack (SCILIB version) routines 
c     sgefa and sgedi to 
c     compute the determinant (celvol) and the inverse.
c     But note that the 2pi factor is NOT needed here.
c
!      do 888 i=1,3
!         do 999 j=1,3
!            b(i,j) = a(i,j)
!  999    continue
!  888 continue
c
!      call invers(b,xdum)
c
      call pgl(a,b,r,ntrans,ib,ihg)
c
c     Subroutine pgl determines the point group of the lattice and the
c     crystal system. The array ib contains the locations of the group
c     operations and ntrans is the order of the group.
c
      call atftmt(nfd,a,b,x,r,tnp,trans,ity,na,ib,ihg,ipm,li,ntrans,
     &            invers_no)
c
c     Subroutine atftmt determines the point group of the crystal, 
c     the atom transformation table f0, the fractional translations 
c     tnp associated with each rotation and the multiplication 
c     table mt for the point group of the crystal. The array ib now 
c     contains operations in the point group of the crystal and ntrans 
c     is the order of this group.
c
c     if(li.gt.0) write(nfd,180)
c 180 format (5x,'the point group of the crystal contains the',
c    1' inversion, therefore,',/,5x,'time reversal invariance will',
c    2' be invoked for all wave vectors.')
c
c     We have the rotations in cartesian coordinates.
c     Transform into lattice coordinates (r-space and g-space)
c
         do 130 l = 1, ntrans
c 
c           In terms of the real-space basis vectors:
c
c              y' = ( B R a ) y     ( y are the r-lattice coord.)
c
c           Trans * a ...
c     
            do 90 j = 1, 3
               do 80 k = 1, 3
                  r1(j,k) = 0.0d0
                  cmtrx(l,j,k) = trans(l,j,k)
                  do 70 m = 1, 3
                     r1(j,k) = r1(j,k) + trans(l,j,m)*a(m,k)
   70             continue
   80          continue
   90       continue
c
c           B * Trans * a
c
            do 120 j = 1, 3
               do 110 k = 1, 3
                  rlat(l,j,k) = 0.0d0
                  do 100 m = 1, 3
                     rlat(l,j,k) = rlat(l,j,k) + b(j,m)*r1(m,k)
  100             continue
                  rmtrx(l,j,k) = nint(rlat(l,j,k))
  110          continue
  120       continue
  130    continue
c
c        Identify the symmetry operations
c
         call symm_ident(ntrans,rmtrx,tnp,id)
c
c
         do 200 l = 1, ntrans
c            
c           In terms of the g-space basis vectors:
c                      T
c              z' = ( a  R b ) z    ( z are the g-lattice coord.)
c
c           a(transpose) * Trans ...
c
            do 160 j = 1, 3
               do 150 k = 1, 3
                  r1(j,k) = 0.0d0
                  do 140 m = 1, 3
                     r1(j,k) = r1(j,k) + a(m,j)*trans(l,m,k)
  140             continue
  150          continue
  160       continue
c                                                    T
c           a(transpose) * Trans * B(transpose)    (B  = b)
c
            do 190 j = 1, 3
               do 180 k = 1, 3
                  rlat(l,j,k) = 0.0d0
                  do 170 m = 1, 3
                     rlat(l,j,k) = rlat(l,j,k) + r1(j,m)*b(k,m)
  170             continue
                  gmtrx(l,j,k) = nint(rlat(l,j,k))
  180          continue
  190       continue
  200    continue
c
c       write the matrices and fractional translations
c
      write(nfd,9000)
 9000 format(///4x,'Rotation matrices (cart-lattice) and fractional',
     1      ' translations (r-lattice)',//)
c
      do 240 i = 1, ntrans
         write(nfd,9010) i, ((cmtrx(i,j,k),k=1,3),j=1,3),
     1     (tnp(i,k),k=1,3), id(i)
c
  240 continue
 9010 format(i5,3(3x,3i3),4x,3f7.4,1x,a10)
c
!     call symchk(ierr,ntrans,rmtrx,tnp,1)
!      call symchk(ierr,ntrans,gmtrx,tnp,0)
c
      return
c
      end
c     
