!*****************************************************************************
!
!  File : word.f90
!    by : Alan Tackett
!    on : 3/10/95
!   for : General purpose input parser
!
! Modified on 11/29/95 to handle multiple Input configurations.
!
!*****************************************************************************

Module Word

Use strings

implicit none


         !***** Define GLOBAL Constants ******
!**Integer      :: W_OK  = 0         ! Next word is Ok
!**Integer      :: W_BAD = 1         ! Bad value
!**Integer      :: W_EOL = -2        ! Reached EOL
!**Integer      :: W_EOF = -1        ! Reached EOF

Character*3, PARAMETER:: KBD_INPUT = 'KBD' ! Read Input from Keyboard


Integer      :: W_OK           ! Next word is Ok
Integer      :: W_EOL         ! Reached EOL
Integer      :: W_EOF         ! Reached EOF

         !***** Define GLOBAL Variables ******
Integer             :: W_ERROR           ! Read Error Status

Type Buffer_Struct               !*** Define Buffer Structure
      Character*(200)   :: FileName
      Character*(200)   :: Text
      Integer           :: Line
      Integer           :: Column
      Integer           :: TextLen
      Integer           :: LastError
End Type

Type Word_Context         !**** MAin structure that holds all file info ****
  Character     :: CommentChar    ! Designates Comment Character
  character     :: LiteralChar    ! Designates Literal character
  Character(20) :: IncludeStr     ! Designates include file
  Character(30) :: DelimiterStr   ! Contains word delimiters

  Character(500):: KbdBuffer      ! KEyboard Input
  Integer       :: KbdPos         ! Position in Keyboard Buffer
  Integer       :: KbdLen         ! Length of Kbd Buffer
  Logical       :: UsingKbd       ! Determines if keyboard is in use

  Integer       :: StartUnit      ! Starting Unit Number
  Integer       :: CurrUnit       ! Current Unit Number
  Integer       :: MaxNest        ! Max # of nested includes Allowed

  Logical       :: DEbugNExtLine  ! Controls skipping to next line
  Type (Buffer_Struct), Pointer :: Buffer(:)    !** Input buffer
End Type


         !***** Define PRIVATE Constants ******
Character, PRIVATE :: SINGLEQUOTE = '"'  ! Define Single Quote Character
Character, PRIVATE :: DOUBLEQUOTE = "'"  ! Define Double Quote Character


Interface GetNumber                   !** Interface for single number
  Module Procedure GetIntSingle
  Module Procedure GetRealSingle
  Module Procedure GetDBLSingle 
  Module Procedure GetCMPLXSingle
  Module Procedure GetZCMPLXSingle
End Interface

Interface GetNumbers                  !** Interface for single or multiple nums
  Module Procedure GetIntSingle
  Module Procedure GetIntArray

  Module Procedure GetRealSingle
  Module Procedure GetRealArray
  Module Procedure GetDBLSingle 
  Module Procedure GetDBLArray

  Module Procedure GetCMPLXSingle
  Module Procedure GetCMPLXArray
  Module Procedure GetZCMPLXSingle
  Module Procedure GetZCMPLXArray
End Interface


Contains

!*****************************************************************************
!
!  DetWordConstants - Determines the word I/O constants
!
!*****************************************************************************

Subroutine DetWordConstants(fd)
  Integer, Intent(IN) :: fd

  Integer, PARAMETER :: N=15
  Character*(N) :: Text1, Text2
  Character :: c
  Integer :: i, err, DWC_Ok, DWC_EOL, DWC_EOF

  text1 = 'This is a test'
  text2 = 'Second line'
  
!  text="SCRATCH"
!  Open(fd, status=text1, IOSTAT=err)
  Open(fd, status="SCRATCH", IOSTAT=err)
  If (err /= 0) then
     Write(*,*) 'DetWordConstants: Error opening file!! Err=',err
     !Call PError
     STOP
  End If
  Write(fd, *) text1
  Write(fd, *) text2
  
  Rewind(fd)

  Read(fd, '(A1)', ADVANCE='NO', IOSTAT=Err) c
  Write(*,*) 'DWC: W_Ok=',err
  DWC_Ok = err
  Do While (Err==DWC_Ok)
     Read(fd, '(A1)', ADVANCE='NO', IOSTAT=Err) c
  End Do

  Write(*,*) 'DWC: W_EOL=',err
  DWC_EOL = err
  Do While ((Err==DWC_Ok) .OR. (err==DWC_EOL))
     Read(fd, '(A1)', ADVANCE='NO', IOSTAT=Err) c
  End Do
  Write(*,*) 'DWC: W_EOF=',err
  DWC_EOF = err

  Close(fd)

  W_Ok = DWC_Ok
  W_EOL = DWC_EOL
  W_EOF = DWC_EOF

  Return
End Subroutine


!*****************************************************************************
!
! LoadBuffer - Loads a Text Buffer with Data
!
! fd   - Input File
! Text - Text retrieved
! TextLen - Length of the text
!
!*****************************************************************************

Subroutine LoadBuffer(fd, Text, TextLen, LastError)
  Integer,       Intent(IN) :: fd
  Character*(*), Intent(OUT) :: Text
  Integer,       Intent(OUT) :: TextLen
  Integer,       Intent(OUT) :: LastError


  Character :: c
  
  TextLen = 0
  
  LastError = W_OK

  Do While (LastError == W_OK)
    if (fd == 6) then 
      Read(*, '(A1)', ADVANCE='NO', IOSTAT=LastError) c
    else
      Read(fd, '(A1)', ADVANCE='NO', IOSTAT=LastError) c
    End If

    TextLen = TextLen + 1
    Text(TextLen:TextLen) = c
  End Do

  Return
End Subroutine
    
!*****************************************************************************
!
!  getc - Returns the next character from fd
!
!  WC   - Word Context to store info in
!  c    - Character Read
!
!  Upon Return it sets W_ERROR.
!
!*****************************************************************************

Subroutine GetC(WC, c)
  Type (Word_Context), Intent(INOUT) :: wc
  Character,           Intent(OUT)   :: c

  Integer :: i, j


  i = WC%CurrUnit - WC%StartUnit + 1
 
  if (WC%UsingKbd) then
    if (WC%CurrUnit == 6) then
       i = 1
    else
       i = i + 1
    End If
  End If

  W_Error = W_Ok

  if (WC%Buffer(i)%Column >= WC%Buffer(i)%TextLen) then
!write(*,*) 'Before Loadbuffer!!!!!!!'
    Call LoadBuffer(WC%CurrUnit, WC%Buffer(i)%Text, WC%Buffer(i)%TextLen,   &
                    WC%Buffer(i)%LastError)
!write(*,*) 'After Loadbuffer!!!!!!!'
    WC%Buffer(i)%Column = 0
    WC%Buffer(i)%Line = WC%Buffer(i)%Line + 1
  End If

  WC%Buffer(i)%Column = WC%Buffer(i)%Column + 1
  j = WC%Buffer(i)%Column
  c = WC%Buffer(i)%Text(j:j)

  if (WC%Buffer(i)%Column >= WC%Buffer(i)%TextLen) then
      W_Error = WC%Buffer(i)%LastError
  end if
 
  Return
End Subroutine


!*****************************************************************************
!
!  InitWord - Initializes the Word getting routines
!
!  WC        - Word Context to store info in
!  FirstUnit - Starting Unit number to use. All unit number from
!              STARTUNIT..(STARTUNIT+MaxInc) can be used for nesting
!  Filename  - Filename to open for input.  If the filename has the
!              special name defined in KBD_INPUT then input is
!              taken from the keyboard.
!  Delim     - List of delimiters
!  Comment   - The Comment character.  Comments are assumed to continue
!              until EOL is reached.
!  Lit       - Literal starting and ending Character, ie, 'Alan Tackett'
!              is returned as ONE token if Lit="'".
!  MaxInc    - MAx Level of includes allowed. IF 0 then IncStr is ignored.
!  IncStr    - Include String. 
!
!
!  Upon return W_ERROR is set to W_OK if the file was opened.
!  Otherwise it is set to the OPEN statement error condition.
!
!*****************************************************************************

Subroutine InitWord(wc, FirstUnit, Filename, Delim,comment,Lit, MaxInc, IncStr)
  Type (Word_Context), Intent(OUT) :: wc
  Integer,      Intent(IN) :: FirstUnit
  Character(*), Intent(IN) :: Filename
  Character(*), Intent(IN) :: Delim
  Character,    Intent(IN) :: Comment
  Character,    Intent(IN) :: Lit
  Integer,      Intent(IN) :: MaxInc
  Character(*), Optional   :: IncStr

  Integer :: i


!**  Call DetWordConstants(FirstUnit)

  WC%DebugNextLine = .FALSE.
 
  WC%StartUnit = FirstUnit
  WC%CurrUnit = FirstUnit
  WC%MaxNest = MaxInc

  WC%DelimiterStr = Delim
  WC%CommentChar = Comment
  WC%LiteralChar = Lit

  if (WC%MaxNest > 0 ) then
     WC%IncludeStr = IncStr
     Call UpperCase(WC%IncludeStr)
  End If

  Allocate(WC%Buffer(WC%MaxNest+1))

  If (FileName == KBD_INPUT) then
     WC%UsingKBD = .TRUE.
     WC%CurrUnit = 6
  else
     WC%UsingKBD = .FALSE.
     write(*,*) 'Filename',Filename
     Open(WC%CurrUnit, FILE=FileName,  IOSTAT=W_ERROR)
  End If

  WC%Buffer(1)%Column = 0
  WC%Buffer(1)%Line   = 0
  WC%Buffer(1)%Text   = " "     
  WC%Buffer(1)%TextLen = 0
  WC%Buffer(1)%FileName = FileName

  Return
End Subroutine


!*****************************************************************************
!
! CloseWord - Close the Open Input Files and free the buffer memory associated
!             with the context.
!
!  WC - Word Context to close
!
!*****************************************************************************

Subroutine CloseWord(WC)
  Type (Word_Context), Intent(INOUT) :: WC

  Integer :: i

  Do i = WC%StartUnit, WC%CurrUnit
    Close(i)
  End Do

  DeAllocate(WC%Buffer)

  Return
End Subroutine


!*****************************************************************************
!
!  gettoken - Gets a token delimited by DelimiterStr from unit Currfd
!
!  WC      - Word Context to close
!  token   - Token Returned
!  endchar - End Delimiter character returned
!
!  Return Values :
!     Gettoken returns the number of characters in the token
!
!     W_ERROR is always set to getc's value. This means that EOL and
!     EOF are returned via W_ERROR. EOL and EOF are always returned even 
!     if no token is encountered. Also if EOL or EOF is encountered then
!     ENDCHAR's value is junk.
!
!*****************************************************************************

Integer Function GetToken(WC, token, endchar)
  Type (Word_Context), Intent(INOUT) :: wc
  Character(*),        Intent(OUT) :: token
  Character(*),        Intent(OUT) :: EndChar

  Character  :: c
  Integer    :: pos


  pos = 0
  Token = " "

                     !*** Pick off delimiters ***
  Call GetC(WC, c)
  Do While ((Scan(WC%DelimiterStr, c) /= 0) .AND. (WC%LiteralChar /= c) &
    .AND. (W_ERROR == W_OK))
    Call GetC(WC, c)
  End Do

!##write(*,*) 'gt: c=',c
                    
  if (W_ERROR == W_OK) then        !*** Get The token ***
     if (WC%LiteralChar == c) then
       Call GetC(WC, c) 
       Do While ((WC%LiteralChar /= c) .AND. (W_ERROR == W_OK))
          pos = pos + 1
          Token(pos:pos) = c
          Call GetC(WC, c)
       End Do
     else 
       Do While ((Scan(WC%DelimiterStr, c) == 0) .AND. (W_ERROR == W_OK))
	 pos = pos + 1
         token(pos:pos) = c              ! Store Prev char
  	 Call GetC(WC, c)                ! Get New Char
!##write(*,*) 'gt: c2=',c
      End Do
    end If
  End If


  EndChar = c
  GetToken = Pos
  Return
End Function


!******************************************************************************
!
! nextword - Gets the next word from the input file.
!            Handles "Poping" Nested input files.
!
! WC     - Word Context to use
! token  - Next word returned
! tlen   - Token Length
!
!
!******************************************************************************

Subroutine NextWord(WC, token, tlen)
  Type (Word_Context), Intent(INOUT) :: WC
  Character(*),          Intent(OUT) :: Token
  Integer,               Intent(OUT) :: tlen

  Character     :: EndChar
  Logical       :: Finished


!##write(*,*) 'nw: Cu = ',currunit
  Finished = .FALSE.
  W_ERROR = W_OK
  tlen = 0
  Do While (.NOT. Finished)
    Do While ((tlen == 0) .AND. (W_ERROR /= W_EOF))
      tlen = GetToken(WC, token, EndChar)
!##write(*,*) 'nw: Tok = ',token, ' * tlen = ',tlen, ' * W_e = ',w_error
    End Do

    if ((tlen == 0) .AND. (W_ERROR == W_EOF)) then
       if (WC%CurrUnit > WC%StartUnit) then       !*** Close Include File ***
          WC%DebugNExtLine = .FALSE.
          Close(WC%CurrUnit)
          WC%CurrUnit = WC%CurrUnit - 1

	  tlen = GetToken(WC, token, EndChar)
       else if (WC%UsingKbd .AND. (WC%CurrUnit == WC%StartUnit)) then
          Close(WC%CurrUnit)
          WC%CurrUnit = 6
          
          tlen = GetToken(WC, token, EndChar)
       else 
	  Finished = .TRUE.
       End If
    else
       Finished = .TRUE.
    End If
  End Do

  Return
End Subroutine


!******************************************************************************
!
! GetWord -  Gets the next word from the input file.
!            Handles "Pushing" Nested include files.
!            The IncludeStr is NOT case sensitive.
!
!            NOTE : Returns on EOL
!
! WC     - Word Context to use
! token  - Next word returned
! tlen   - Token Length
!
!
!******************************************************************************

Subroutine GetWord(WC, token, tlen)
   Type (Word_Context), Intent(INOUT) :: WC
   Character(*),        Intent(OUT)   :: Token
   Integer,             Intent(OUT)   :: tlen

   Character(LEN(token))   :: temp
   Character               :: c
   Integer                 :: i,k
   Logical                 :: Finished

   Call NextWord(WC, Token, tlen)

   if (W_Error /= W_EOF) then

      Finished = .FALSE.

      Do While (.NOT. Finished)
        Finished = .TRUE.
        temp = Token
        Call UpperCase(temp)

        if (temp == WC%IncludeStr) then                   !*** Include File ***
          Call NextWord(WC, temp,tlen)

          if (WC%CurrUnit == 6) then
             WC%CurrUnit = WC%StartUnit
          else
	     WC%CurrUnit = WC%CurrUnit + 1
          End If
 
          Open(WC%CurrUnit, FILE=temp, IOSTAT=W_ERROR)
            if( W_ERROR /= 0) then
              write(*,*) 'GetWord: Error opening include file', temp
              stop
            endif

          i = WC%CurrUnit - WC%StartUnit + 1
          if (WC%UsingKbd) i = i + 1

          WC%Buffer(i)%Line = 0
          WC%Buffer(i)%TextLen = 0
          WC%Buffer(i)%Column = 0
          WC%Buffer(i)%Text = "                                               "
          WC%Buffer(i)%FileName = temp

	  Call NextWord(WC, Token, tlen)
	  Finished = .FALSE.
        End If

        if (Index(Token, WC%CommentChar) /= 0) then   !*** Comment ****
          Do While (Index(Token, WC%CommentChar) /= 0)
            Do While (W_Error == W_OK)
               Call GetC(WC, c)
            End Do

            k = Index(Token, WC%CommentChar)
	    if (k < 2) then
               Call NextWord(WC, Token, tlen)
            else
	       Token(k:tlen) = " "
               tlen = tlen - k
            End If

            Finished = .FALSE.
          End Do
        End If    
      End Do
   End if

   Return
End Subroutine


!******************************************************************************
!
! GetNextWord -  Gets the next word from the input file.
!                Handles "Pushing" Nested include files.
!                The IncludeStr is NOT case sensitive.
!
!                NOTE : DOES NOT return on EOL
!
! WC     - Word Context to use
! token  - Next word returned
! tlen   - Token Length
!
!
!******************************************************************************

Subroutine GetNextWord(WC, token, tlen)
   Type (Word_Context), Intent(INOUT) :: WC
   Character(*),        Intent(OUT)   :: Token
   Integer,             Intent(OUT)   :: tlen

   Call GetWord(WC, token, tlen)
   Do While ((tlen == 0) .AND. (W_ERROR /= W_EOF))
      Call GetWord(WC, Token, Tlen)
   End Do

   if (W_ERROR == W_EOL) W_Error = W_Ok

   Return
End Subroutine


!*****************************************************************************
!
! Word_GetText - Returns a structure containing the current filename,
!                   line of text, file line number and column number.
!
! WC        - Word Context to use
! Debuginfo - Structure to store the info in 
!
!*****************************************************************************

Subroutine Word_GetText(WC, DebugInfo)
   Type (Word_Context),  Intent(IN)  :: WC
   Type (buffer_Struct), Intent(OUT) :: DebugInfo

   Integer :: i

   i = WC%CurrUnit - WC%StartUnit + 1
 
   if (WC%UsingKbd) then
     if (WC%CurrUnit == 6) then
        i = 1
     else
        i = i + 1
     End If
   End If
 
   DebugInfo = WC%Buffer(i)

   Return
End Subroutine

!*****************************************************************************
!
! Word_PrintText - Prints a Buffer_Struct to the specified device
!
! fd        - Output Device 
! IDText    - Text to prepend to the beginning of output to locate prog pos
! DebugInfo - Debug Info to print
! 
!*****************************************************************************

Subroutine Word_PrintText(fd, IDText, DebugInfo)
   Integer,             Intent(IN)  :: fd
   Character*(*),       Intent(IN)  :: IDText
   Type (buffer_Struct), Intent(IN) :: DebugInfo

   Character*200 :: temp
   Integer       :: c

   c = DebugInfo%Column
   Temp = Repeat('-', c)
   temp(c:c) = "^"

   Write(fd, *) Trim(IDText), 'File: ',Trim(DebugInfo%FileName)
   Write(fd, *) Trim(IDText), 'Line and Column : ',    &
                DebugInfo%Line,',', DebugInfo%Column
   Write(fd, *) Trim(IDText), 'Text:'
   Write(fd, *) Trim(IDText), DebugInfo%Text(1:DebugInfo%TextLen)
   Write(fd, *) Trim(IDText), Trim(Temp)

   Return
End Subroutine

!*****************************************************************************
!
! Word_GetAndPrint - Gets and Prints a Buffer_Struct to
!                    the specified device
!
! WC        - Word Context to use
! fd        - Output Device 
! IDText    - Text to prepend to the beginning of output to locate prog pos
!
!*****************************************************************************

Subroutine Word_GetAndPrint(WC, fd, IDText)
   Type (Word_Context), Intent(IN)  :: WC
   Integer,             Intent(IN) :: fd
   Character*(*),       Intent(IN) :: IDText

   Type (Buffer_Struct) :: DebugInfo   
   
   Call Word_GetText(WC, DebugInfo)
   Call Word_PrintText(fd, IDText, DebugInfo)

   Return
End Subroutine

!*****************************************************************************
!
! GetIntSingle - Get A single integer from the input stream
!
! WC    - Word Context to use
! num   - Integer returned
!
!*****************************************************************************

Subroutine GetIntSingle(WC, num)
  Type (Word_Context), Intent(INOUT)  :: WC
  Integer,             Intent(OUT)    :: num

  Character(50) :: token
  Integer       :: tlen


  Call GetNextWord(WC, token, tlen)


  if (tlen > 0)  Read(Token, *, IOSTAT=W_ERROR) num

  Return
End Subroutine


!*****************************************************************************
!
! GetIntArray - Get an array of integers from the input stream
!
! WC    - Word Context to use
! a     - Integers returned
! num   - Number of integers to get
!
!*****************************************************************************

Subroutine GetIntArray(wc, a,num)
  Type (Word_Context), Intent(INOUT) :: WC
  Integer,             Intent(OUT)   :: a(:)
  Integer, OPTIONAL,   Intent(IN)    :: num

  Integer :: i, n

  if (Present(NUM)) then
    N = Num
  else
    N = Size(A)
  end If

  i = 2
  Call GetIntSingle(WC, a(1))
  Do While ((W_Error == W_OK) .AND. (i<=N))
    Call GetIntSingle(WC, a(i))
    i = i + 1
  End Do
  
  Return
End Subroutine


!*****************************************************************************
!
! GetRealSingle - Get A single real from the input stream
!
! WC    - Word Context to use
! num   - Real returned
!
!*****************************************************************************

Subroutine GetRealSingle(WC, num)
  Type (Word_Context), Intent(INOUT)  :: WC
  Real*4, Intent(OUT)                 :: num

  Character(50) :: token
  Integer       :: tlen

  Call GetNextWord(WC, token, tlen)

  If (Index(token, '.') == 0) token = trim(token) // '.0'

  if (tlen > 0) Read(Token, *, IOSTAT=W_ERROR) num

  Return
End Subroutine


!*****************************************************************************
!
! GetRealArray - Get an array of Reals from the input stream
!
! WC    - Word Context to use
! a     - Reals returned
! num   - Number of reals to get
!
!*****************************************************************************

Subroutine GetRealArray(WC, a,num)
  Type (Word_Context), Intent(INOUT) :: WC
  Real*4,              Intent(OUT)   :: a(:)
  Integer, OPTIONAL,   Intent(IN)    :: num

  Integer :: i, n

  if (Present(NUM)) then
    N = Num
  else
    N = Size(A)
  end If

  i = 2
  Call GetRealSingle(WC, a(1))
  Do While ((W_Error == W_OK) .AND. (i<=N))
    Call GetRealSingle(WC, a(i))
    i = i + 1
  End Do
  
  Return
End Subroutine


!*****************************************************************************
!
! GetDBLSingle - Get A Double precison number from the input stream
!
! WC    - Word Context to use
! num   - Double returned
!
!*****************************************************************************

Subroutine GetDBLSingle(WC, num)
  Type (Word_Context), Intent(INOUT) :: WC
  Real*8,              Intent(OUT)   :: num

  Character(50) :: token
  Integer       :: tlen

  Call GetNextWord(WC, token, tlen)

  If (Index(token, '.') == 0) token = trim(token) // '.0'

  if (tlen > 0) Read(Token, *, IOSTAT=W_ERROR) num

  Return
End Subroutine


!*****************************************************************************
!
! GetDBLArray - Get an array of Doubles from the input stream
!
! WC    - Word Context to use
! a     - Doubles returned
! num   - Number of Doubles to get(OPTIONAL)
!
!*****************************************************************************

Subroutine GetDBLArray(WC, a,num)
  Type (Word_Context), Intent(INOUT) :: WC
  Real*8,              Intent(OUT)   :: a(:)
  Integer, OPTIONAL,   Intent(IN)    :: num

  Integer :: i, n

  if (Present(NUM)) then
    N = Num
  else
    N = Size(A)
  end If

  i = 2
  Call GetDBLSingle(WC, a(1))

  Do While ((W_Error == W_OK) .AND. (i<=N))
    Call GetDBLSingle(WC, a(i))
    i = i + 1
  End Do
  
  Return
End Subroutine


!*****************************************************************************
!
! GetCMPLXSingle - Get A single COMPLEX*8 from the input stream
!
! WC    - Word Context to use
! num   - Complex number returned returned
!
!*****************************************************************************

Subroutine GetCMPLXSingle(WC, num)
  Type (Word_Context), Intent(INOUT) :: WC
  Complex*8,           Intent(OUT)   :: num

  Character(50) :: token
  Integer       :: tlen
  Real*4        :: r, i

  Call GetNextWord(WC, token, tlen)
  If (Index(token, '.') == 0) token = trim(token) // '.0'
  if (tlen > 0) Read(Token, *, IOSTAT=W_ERROR) r

  if (W_Error == W_OK) then
    Call GetNextWord(WC, token, tlen)
    If (Index(token, '.') == 0) token = trim(token) // '.0'
    if (tlen > 0) Read(Token, *, IOSTAT=W_ERROR) i
  End If

  Num = CMPLX(r,i, KIND=KIND(NUM))

  Return
End Subroutine


!*****************************************************************************
!
! GetCMPLXArray - Get an array of complex*8 from the input stream
!
! WC    - Word Context to use
! a     - Complex nums returned
! num   - Number of reals to get(OPTIONAL)
!
!*****************************************************************************

Subroutine GetCMPLXArray(WC, a,num)
  Type (Word_Context), Intent(INOUT) :: WC
  Complex*8,           Intent(OUT)   :: a(:)
  Integer, OPTIONAL,   Intent(IN)    :: num

  Integer :: i, n

  if (Present(NUM)) then
    N = Num
  else
    N = Size(A)
  end If

  i = 2
  Call GetCMPLXSingle(WC, a(1))
  Do While ((W_Error == W_OK) .AND. (i<=N))
    Call GetCMPLXSingle(WC, a(i))
    i = i + 1
  End Do
  
  Return
End Subroutine

!*****************************************************************************
!
! GetZCMPLXSingle - Get A single COMPLEX*16 from the input stream
!
! WC    - Word Context to use
! num   - Complex number returned returned
!
!*****************************************************************************

Subroutine GetZCMPLXSingle(WC, num)
  Type (Word_Context), Intent(INOUT) :: WC
  Complex*16,          Intent(OUT)   :: num

  Character(50) :: token
  Integer       :: tlen
  Real*8        :: r, i

  Call GetNextWord(WC, token, tlen)
  If (Index(token, '.') == 0) token = trim(token) // '.0'
  if (tlen > 0) Read(Token, *, IOSTAT=W_ERROR) r

  if (W_Error == W_OK) then
    Call GetNextWord(WC, token, tlen)
    If (Index(token, '.') == 0) token = trim(token) // '.0'
    if (tlen > 0) Read(Token, *, IOSTAT=W_ERROR) i
  End If

  Num = CMPLX(r,i, KIND=KIND(num))

  Return
End Subroutine


!*****************************************************************************
!
! GetZCMPLXArray - Get an array of complex*16 from the input stream
!
! WC    - Word Context to use
! a     - Complex nums returned
! num   - Number of reals to get(OPTIONAL)
!
!*****************************************************************************

Subroutine GetZCMPLXArray(WC, a,num)
  Type (Word_Context), Intent(INOUT) :: WC
  Complex*16,          Intent(OUT)   :: a(:)
  Integer, OPTIONAL,   Intent(IN)    :: num

  Integer :: i, n

  if (Present(NUM)) then
    N = Num
  else
    N = Size(A)
  end If

  i = 2
  Call GetZCMPLXSingle(WC, a(1))
  Do While ((W_Error == W_OK) .AND. (i<=N))
    Call GetZCMPLXSingle(WC, a(i))
    i = i + 1
  End Do
  
  Return
End Subroutine


End Module

