!*****************************************************************************
!
! File : ylm_fact.f90
!   by : Alan Tackett
!   on : 04/19/1999
!  for : PW-PAW
!
!  Contains the routines for creating and storing the Ylm  functions
!
!******************************************************************************

Module ylm_fact

Use lrulib
Use gpoints
Use work_mgr
Use atom_data
Use crystal_data
Use spherical_harmonic

implicit none

Type (COMPLEX_LRU_Context) :: LRU_Ylm

Integer, PRIVATE, Pointer :: L_Start(:)
Integer, PRIVATE          :: L_Max, MaxPerKpnt

!******************************************************************************
Contains
!******************************************************************************

!******************************************************************************
!
!  GetYlm - Gets the specified Ylm from disk
!
!    Atom  - Specific Atom to get
!    Phase - Pointer to the data(Returned)
!
!******************************************************************************

Subroutine GetYlm( Kpnt, L, m, Ylm)
  Integer,         Intent(IN) :: Kpnt
  Integer,         Intent(IN) :: L
  Integer,         Intent(IN) :: m
  Complex,         Pointer    :: Ylm(:)

  Integer :: index 

  index = (Kpnt-1)*MaxPerKpnt + L_Start(L) + abs(m)

  Call LRU_GetRec(LRU_Ylm,  index, Ylm)
 
!**   write(6,'("GetYLM:",1p2e15.7,3i4)') Dot_Product(Ylm,ylm),L,m,kpnt
  Return

End Subroutine

!******************************************************************************
!
!  StoreYlm - Stores the Ylm grids to disk
!
!******************************************************************************

Subroutine StoreYlm
  Integer       :: L, m, j, Kpnt,  N, DI
  Real  :: G(3), Kvec(3), KG(3), R(3), THeta
  Complex, Pointer :: Ylm(:)
  Complex :: Ylm_list(13)
  Character*200 :: token

Write(Log_Unit,*) 'StoreYlm: Start!!!!!!!!!!!!!!!!'
!RETURN !**REMOVE**

   Call GetBuffer( Ylm)
   N = Gpnt_Size(G_Low) - 1

   DI = 0
   Do kpnt=1, Size(BZ%Ku(1,:))
      Kvec = BZ%Ku(:,kpnt)

      Do L=0, L_Max
         Do m=0, L
            Ylm = 0
      
            Ylm_List = Spharm(Kvec(1), Kvec(2), Kvec(3), L, .TRUE.)
            Ylm(1) = Ylm_list(L+m+1)

            Do j=2, Gpnt_Size(G_LOW)  !** Skip G=0 term
               G = Kvec + Gpnt(1:3,j)
               Ylm_List = Spharm(G(1), G(2), G(3), L, .TRUE.)
               Ylm(j) = Ylm_List(L+m+1)

               G = Kvec - Gpnt(1:3,j)
               Ylm_List = Spharm(G(1), G(2), G(3), L, .TRUE.)
               Ylm(j+N) = Ylm_List(L+m+1)
            End Do
      

            DI = DI + 1
            Write(token,*) 'StoreYlm: Error Writing Data! :', &
                 ' * L=',L, ' * m=',m, ' * DI=',DI

            Call FilterValue(Ylm)
            Call LRU_PutRec(LRU_Ylm,  DI, Ylm)
!            Call WriteFile_LOW(LRU_Ylm%FD_Base, Di,  Ylm, token)
         End Do
      End Do
   End Do

   call FreeBuffer(Ylm)
Write(Log_Unit,*) 'StoreYlm: Completed store!'
   Return
End Subroutine



!******************************************************************************
!
! InitYlmFact - Initializes the Ylm factor routines
!
!   FD_Base - Base File unit
!   MaxMem  - Max amout of memory to use for structure factors(Mbytes)
!   MaxL    - Max L value
!
!******************************************************************************

Subroutine InitYlmFact(FD_Base, MaxMem, MaxL)
  Integer,       Intent(IN) :: FD_Base
  Real,          Intent(IN) :: MaxMem
  Integer,       Intent(IN) :: MaxL

  Integer :: L, m, MaxRec

!RETURN !**REMOVE**

  !*** First count the number of records needed and init L_start***
  L_Max = MaxL
  Allocate(L_start(0:L_Max))
  MaxRec = 0
  Do L=0, L_Max
     L_Start(L) = MaxRec+1
     Do m=0, L
        MaxRec = MaxRec + 1
     End Do
  End Do

  MaxPerKpnt = MaxRec

  MaxRec = Size(BZ%Ku(1,:))*MaxRec

Write(Log_Unit,*) 'InitYlmFact:  MaxPerKpnt=',MaxPerKpnt, ' * MaxRec=',MaxRec

  Call LRU_InitContext(LRU_Ylm, 'Ylm', FD_Base, MaxMem, &
       MaxRec,  Gall_Size(G_LOW), .TRUE.)

  Call StoreYlm   !** Create the Ylm factors

  Return
End Subroutine


End Module
